<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

use Symfony\Component\Config\Loader\LoaderInterface;
use Symfony\Component\HttpKernel\Kernel;

class AppKernel extends Kernel
{

    /**
     * @param string $environment
     * @param bool $debug
     */
    public function __construct($environment, $debug)
    {
        $this->environment = $environment;
        $this->debug = (Boolean)$debug;
        $this->rootDir = $this->getRootDir();
        $this->name = $this->getName();

        if ($this->debug) {
            $this->startTime = microtime(true);
        }


        if (PHP_MAJOR_VERSION >= 7) {
            set_error_handler(function ($errno, $errstr) {
                return strpos($errstr, 'Declaration of') === 0;
            }, E_WARNING);
        }

    }
    /**
     * @return array|\Symfony\Component\HttpKernel\Bundle\BundleInterface[]
     */
    public function registerBundles()
    {
        $bundles = [
            new Symfony\Bundle\FrameworkBundle\FrameworkBundle(),
            new Symfony\Bundle\SecurityBundle\SecurityBundle(),
            new Symfony\Bundle\MonologBundle\MonologBundle(),
            new Doctrine\Bundle\DoctrineBundle\DoctrineBundle(),
            new Doctrine\Bundle\MigrationsBundle\DoctrineMigrationsBundle(),
            new Doctrine\Bundle\FixturesBundle\DoctrineFixturesBundle(),
            new Sensio\Bundle\FrameworkExtraBundle\SensioFrameworkExtraBundle(),
            new Ambta\DoctrineEncryptBundle\AmbtaDoctrineEncryptBundle(),
            // start CS stuff
            new Cybersponse\CoreBundle\CybersponseCoreBundle(),
            new Cybersponse\SearchBundle\CybersponseSearchBundle(),
            new Cybersponse\BackupBundle\CybersponseBackupBundle(),
            new Cybersponse\AuthenticationBundle\CybersponseAuthenticationBundle(),
            new Cybersponse\AuthorizationBundle\CybersponseAuthorizationBundle(),
            new Cybersponse\TenantBundle\CybersponseTenantBundle(),
            new Cybersponse\ViewTemplateBundle\CybersponseViewTemplateBundle(),
            new Cybersponse\WorkflowBundle\CybersponseWorkflowBundle(),
            new Cybersponse\FilesBundle\CybersponseFilesBundle(),
            new Cybersponse\ActorsBundle\CybersponseActorsBundle(),
            new Cybersponse\QueryBundle\CybersponseQueryBundle(),
            new Cybersponse\CustomerTenantBundle\CybersponseCustomerTenantBundle(),
            new Dunglas\ApiBundle\DunglasApiBundle(),

            new Cybersponse\MetadataBundle\CybersponseMetadataBundle(),
            new Cybersponse\SettingsBundle\CybersponseSettingsBundle(),
            new Cybersponse\ProxyBundle\CybersponseProxyBundle(),

            // end CS stuff
            new Snc\RedisBundle\SncRedisBundle(),
            new Knp\Bundle\PaginatorBundle\KnpPaginatorBundle(),
            new Lexik\Bundle\JWTAuthenticationBundle\LexikJWTAuthenticationBundle(),
            new OldSound\RabbitMqBundle\OldSoundRabbitMqBundle()
        ];

        if (in_array($this->getEnvironment(), ['dev', 'test'])) {
            $bundles[] = new Sensio\Bundle\DistributionBundle\SensioDistributionBundle();
            $bundles[] = new Sensio\Bundle\GeneratorBundle\SensioGeneratorBundle();
            $bundles[] = new Hautelook\AliceBundle\HautelookAliceBundle();
        }

        return $bundles;
    }

    protected function getKernelParameters()
    {
        $parameters = parent::getKernelParameters();
        $parameters['kernel.uc_environment'] = ucfirst($this->getEnvironment());
        return $parameters;
    }

    /**
     * @param LoaderInterface $loader
     * @throws Exception
     */
    public function registerContainerConfiguration(LoaderInterface $loader)
    {
        $loader->load(sprintf('%s/config/config_%s.yml', $this->getRootDir(), $this->getEnvironment()));
    }
}
