<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Application\Migrations;

use Doctrine\DBAL\Migrations\AbstractMigration;
use Doctrine\DBAL\Schema\Schema;
use Cybersponse\WorkflowBundle\Entity\WorkflowStepType;
use Cybersponse\WorkflowBundle\Entity\WorkflowStep;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
class Version20160428194711 extends AbstractMigration implements ContainerAwareInterface
{
    /** @var ContainerInterface */
    protected $container;

    protected $compatMap = [
        'workflow.tasks.add' => 'add',
        'workflow.tasks.add_one' => 'add_one',
        'workflow.tasks.cond' => 'cond',
        'workflow.tasks.if' => 'if',
        'workflow.tasks.If' => 'if',
        'workflow.tasks.delay' => 'delay',
        'workflow.tasks.no_op' => 'no_op',
        'workflow.tasks.failed_workflow_task' => 'panic',
        'workflow.tasks.pause' => 'pause',
        'workflow.tasks.resume' => 'resume',
        'workflow.tasks.setvariable' => 'set_multiple',
        'workflow.tasks.set_variable' => 'set_multiple',
        'workflow.tasks.update_settings' => 'update_settings',
        'workflow.tasks.workflowreference' => 'call',
        'workflow_reference' => 'call',
        'workflow.tasks.workflow_reference' => 'call',
        'start_periodic_workflow' => 'start_periodic',
        'pro_shop.tasks.start_periodic_workflow' => 'start_periodic',
        'stop_periodic_workflow' => 'stop_periodic',
        'pro_shop.tasks.stop_periodic_workflow' => 'stop_periodic',
        'pro_shop.tasks.apicall' => 'api_call',
        'pro_shop.tasks.api_call' => 'api_call',
        'pro_shop.tasks.crudhub_crud' => 'crudhub_crud',
        'pro_shop.tasks.fetch_email' => 'fetch_email',
        'pro_shop.tasks.sendemailtask' => 'send_email',
        'pro_shop.tasks.sendemail' => 'send_email',
        'pro_shop.tasks.send_email' => 'send_email',
        'pro_shop.tasks.insertdatatask' => 'insert_data',
        'pro_shop.tasks.insert_data' => 'insert_data',
        'pro_shop.tasks.map_json' => 'map_json',
        'map_workflow' => 'map',
        'pro_shop.tasks.mapworkflow' => 'map',
        'pro_shop.tasks.map_workflow' => 'map',
        'pro_shop.tasks.updatedata' => 'update_data',
        'pro_shop.tasks.updatedatatask' => 'update_data',
        'pro_shop.tasks.update_data' => 'update_data',
        'pro_shop.tasks.xml_to_dictionary' => 'xml_to_dictionary'
    ];

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return 'Migrates old-style Sealab script IDs to new-style IDs';
    }

    /**
     * Slices out the resource identifier of an IRI, minus any trailing slash
     *
     * @param string $iri
     * @return string
     */
    protected function iriToId($iri) {
        $iriParts = explode('/', $iri);
        $id = array_pop($iriParts);

        if (! $id) {
            // Cut off any trailing slash
            return array_pop($iriParts);
        }

        return $id;
    }

    /**
     * Converts the script value of arguments in WorkflowStep and
     * WorkflowStepType according to the provided script mapping
     *
     * @param mixed $entityClass
     * @param array $scriptMapping
     */
    protected function convertScriptNames($entityClass, $scriptMapping) {
        $scriptKey = 'script';

        /** @var ManagerRegistry $managerRegistry */
        $managerRegistry = $this->container->get('doctrine');
        $entityManager = $managerRegistry->getManagerForClass($entityClass);
        $entityRepository = $entityManager->getRepository($entityClass);

        $entityTypes = $entityRepository->findAll();
        foreach ($entityTypes as $entityType) {
            $arguments = $entityType->getArguments();

            if (array_key_exists($scriptKey, $arguments)) {
                $oldScriptId = strtolower($this->iriToId($arguments[$scriptKey]));

                if (array_key_exists($oldScriptId, $scriptMapping)) {
                    $newScriptId = $scriptMapping[$oldScriptId];
                    $arguments[$scriptKey] = '/wf/workflow/tasks/' . $newScriptId;
                }
            }

            $entityType->setArguments($arguments);
            $entityManager->persist($entityType);
        }

        $entityManager->flush();
    }

    /**
     * @param Schema $schema
     */
    public function up(Schema $schema)
    {
        $this->convertScriptNames(WorkflowStepType::class, $this->compatMap);
        $this->convertScriptNames(WorkflowStep::class, $this->compatMap);
    }

    /**
     * @param Schema $schema
     */
    public function down(Schema $schema)
    {
        $appendSlash = function($scriptId) {
            return $scriptId . '/';
        };

        $scriptMapping = array_map($appendSlash, array_flip($this->compatMap));
        $this->convertScriptNames(WorkflowStepType::class, $scriptMapping);
        $this->convertScriptNames(WorkflowStep::class, $scriptMapping);
    }
}