<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Application\Migrations;

use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\DBAL\Migrations\AbstractMigration;
use Doctrine\DBAL\Schema\Schema;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
class Version20180521080348 extends AbstractMigration implements ContainerAwareInterface
{
    /** @var ContainerInterface */
    private $container;

    private $resolveTask = '{
          "@type": "Workflow",
          "triggerLimit": null,
          "name": "Resolve Alert",
          "description": "Marks the Security Alert as Closed",
          "collection": "/api/3/workflow_collections/6368061c-c6ea-462b-b33f-75437efb0e3b",
          "tag": "#system",
          "isActive": true,
          "singleRecordExecution": false,
          "parameters": [],
          "synchronous": false,
          "triggerStep": "/api/3/workflow_steps/25790c68-bc00-44e7-9216-a8e6b8e164b6",
          "steps": [
            {
              "@type": "WorkflowStep",
              "name": "Update status",
              "arguments": {
                "message": {
                  "content": "<p>The Alert status is updated to <strong><span style=\"color: #00ff00;\">Closed</span></strong></p>",
                  "records": "[\'{{vars.request.data.records[0][\'@id\']}}\']"
                },
                "resource": {
                  "status": {
                    "id": 6802,
                    "@id": "/api/3/picklists/fac53e73-8d16-4189-98d5-95fbd1555232",
                    "icon": null,
                    "@type": "Picklist",
                    "color": null,
                    "display": "Closed",
                    "listName": "/api/3/picklist_names/33e964a9-d607-49f2-813c-7ce46141815a",
                    "itemValue": "Closed",
                    "orderIndex": 2
                  },
                  "closurereason": "<p>{{vars.request.data.closingReason}}</p>"
                },
                "_showJson": false,
                "collection": "{{vars.request.data.records[0][\'@id\']}}",
                "collectionType": "/api/3/alerts",
                "step_variables": []
              },
              "status": null,
              "left": "170",
              "top": "120",
              "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928722",
              "uuid": "7c324cd6-69bd-4b37-a370-ef384e275ca4"
            },
            {
              "@type": "WorkflowStep",
              "name": "Start",
              "arguments": {
                "route": "a8d3c49a-2225-4353-9064-88801b3bb799",
                "title": "Resolve",
                "resources": [
                  "alerts"
                ],
                "inputVariables": [
                  {
                    "name": "closingReason",
                    "type": "string",
                    "title": "Rich Text",
                    "usable": true,
                    "formType": "richtext",
                    "collection": false,
                    "searchable": true,
                    "templateUrl": "app/components/form/fields/richtext.html",
                    "allowedEncryption": true,
                    "allowedGridColumn": true
                  }
                ],
                "step_variables": [],
                "singleRecordExecution": true
              },
              "status": null,
              "left": "20",
              "top": "20",
              "stepType": "/api/3/workflow_step_types/f414d039-bb0d-4e59-9c39-a8f1e880b18a",
              "uuid": "25790c68-bc00-44e7-9216-a8e6b8e164b6"
            }
          ],
          "routes": [
            {
              "@type": "WorkflowRoute",
              "uuid": "bc0d62cd-93dc-4375-9632-b872f320ff73",
              "name": "Start -> Update status",
              "isExecuted": false,
              "targetStep": "/api/3/workflow_steps/7c324cd6-69bd-4b37-a370-ef384e275ca4",
              "sourceStep": "/api/3/workflow_steps/25790c68-bc00-44e7-9216-a8e6b8e164b6"
            }
          ],
          "uuid": "54866017-8228-4674-a9f5-fd1f1304d1db"
        }';

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    private function addWorkflow($workflow){
        $workflowResource = $this->container->get('resource.workflow');
        $context = $workflowResource->getDenormalizationContext();
        $context["ignore_field_authorization"] = true;
        $context["ignore_authorization"] = true;
        $workflowObject = $this->container->get('serializer')->deserialize(
            $workflow,
            $workflowResource->getEntityClass(),
            'json-ld',
            $context
        );
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
        $workflowManager->persist($workflowObject);
        $workflowManager->flush();
    }

    private function deleteWorkflow($workflowId){
        /** @var ResourceInterface $workflowResource */
        $workflowResource = $this->container->get('resource.workflow');

        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
        $workflowToDelete = $workflowManager->find($workflowResource->getEntityClass(), $workflowId);
        if($workflowToDelete==null or sizeof($workflowToDelete)==0){
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }

    /**
     * @param Schema $schema
     */
    public function up(Schema $schema)
    {
        // this up() migration is auto-generated, please modify it to your needs
        $this->addWorkflow($this->resolveTask);
    }

    /**
     * @param Schema $schema
     */
    public function down(Schema $schema)
    {

        $workflowIds = [
            '54866017-8228-4674-a9f5-fd1f1304d1db',
        ];

        // this down() migration is auto-generated, please modify it to your needs
        foreach($workflowIds as $workflowIId ){
            error_log($workflowIId);
            $this->deleteWorkflow($workflowIId);
        }
    }
}
