<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Application\Migrations;

use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\DBAL\Migrations\AbstractMigration;
use Doctrine\DBAL\Schema\Schema;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
class Version20180524105302 extends AbstractMigration implements ContainerAwareInterface
{
    /** @var ContainerInterface */
    private $container;

    private $taskResume = '{
    "@type": "Workflow",
    "triggerLimit": null,
    "name": "Task Resume",
    "description": "This is a playbook is triggered for manual task triggers for revoking the approval.",
    "collection": "/api/3/workflow_collections/a9816e15-5e42-4785-a0ee-23fe8ddb19b6",
    "tag": null,
    "isActive": true,
    "singleRecordExecution": false,
    "parameters": [],
    "synchronous": false,
    "triggerStep": "/api/3/workflow_steps/04988c73-8fa0-4555-aac5-8f340f7a36d8",
    "steps": [
      {
        "@type": "WorkflowStep",
        "name": "Start",
        "arguments": {
          "resource": "tasks",
          "step_variables": {
            "record": "{{vars.request.data}}"
          },
          "fieldbasedtrigger": {
            "limit": 30,
            "logic": "OR",
            "filters": [
              {
                "type": "object",
                "field": "status",
                "value": "/api/3/picklists/343f4b67-e929-4205-bf95-ba5b70545fed",
                "_value": {
                  "itemValue": "Completed"
                },
                "operator": "eq"
              },
              {
                "type": "object",
                "field": "status",
                "value": "/api/3/picklists/7bf6c03d-8e04-46d0-b392-febd78d72bad",
                "_value": {
                  "display": "Skipped",
                  "itemValue": "Skipped"
                },
                "operator": "eq"
              }
            ]
          }
        },
        "status": null,
        "left": "487",
        "top": "20",
        "stepType": "/api/3/workflow_step_types/9300bf69-5063-486d-b3a6-47eb9da24872",
        "uuid": "04988c73-8fa0-4555-aac5-8f340f7a36d8"
      },
      {
        "@type": "WorkflowStep",
        "name": "Revoke requesting playbook",
        "arguments": {
          "when": "{{vars.record.workflowid}}",
          "script": "/wf/workflow/tasks/crudhub_crud",
          "arguments": {
            "method": "POST",
            "resource": "{\"status\": {{ vars.record.status}} }",
            "collection": "/api/wf/api/workflows/{{vars.record.workflowid}}/resume/?format=json"
          },
          "step_variables": []
        },
        "status": null,
        "left": "488",
        "top": "140",
        "stepType": "/api/3/workflow_step_types/ee73e569-2188-43fe-a7f0-1964ba82a4de",
        "uuid": "d273bcf0-e6a6-48b6-9e5d-f25580c6a6e3"
      }
    ],
    "routes": [
      {
        "@type": "WorkflowRoute",
        "uuid": "2c20704e-ab43-4db4-ad09-33cff8762f95",
        "name": "Start -> Revoke requesting playbook",
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/d273bcf0-e6a6-48b6-9e5d-f25580c6a6e3",
        "sourceStep": "/api/3/workflow_steps/04988c73-8fa0-4555-aac5-8f340f7a36d8"
      }
    ],
    "uuid": "5fecc103-aa4b-4626-b69d-5e38697d89e3"
  }';

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    private function addWorkflow($workflow){
        $workflowResource = $this->container->get('resource.workflow');
        $context = $workflowResource->getDenormalizationContext();
        $context["ignore_field_authorization"] = true;
        $context["ignore_authorization"] = true;
        $workflowObject = $this->container->get('serializer')->deserialize(
            $workflow,
            $workflowResource->getEntityClass(),
            'json-ld',
            $context
        );
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
        $workflowManager->persist($workflowObject);
        $workflowManager->flush();
    }

    private function deleteWorkflow($workflowId){
        /** @var ResourceInterface $workflowResource */
        $workflowResource = $this->container->get('resource.workflow');

        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
        $workflowToDelete = $workflowManager->find($workflowResource->getEntityClass(), $workflowId);
        if($workflowToDelete==null or sizeof($workflowToDelete)==0){
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }

    /**
     * @param Schema $schema
     */
    public function up(Schema $schema)
    {
        // this up() migration is auto-generated, please modify it to your needs
        $this->deleteWorkflow('5fecc103-aa4b-4626-b69d-5e38697d89e3');
    }

    /**
     * @param Schema $schema
     */
    public function down(Schema $schema)
    {
        $this->addWorkflow($this->taskResume);
    }
}
