<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Application\Migrations;

use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\DBAL\Migrations\AbstractMigration;
use Doctrine\DBAL\Schema\Schema;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
class Version20190327124630 extends AbstractMigration implements ContainerAwareInterface
{

    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    private function addWorkflow($workflow){
        $workflowResource = $this->container->get('resource.workflow');
        $context = $workflowResource->getDenormalizationContext();
        $context["ignore_field_authorization"] = true;
        $context["ignore_authorization"] = true;
        $workflowObject = $this->container->get('serializer')->deserialize(
            $workflow,
            $workflowResource->getEntityClass(),
            'json-ld',
            $context
        );
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
        $workflowManager->persist($workflowObject);
        $workflowManager->flush();
    }

    private function deleteWorkflow($workflowId){
        /** @var ResourceInterface $workflowResource */
        $workflowResource = $this->container->get('resource.workflow');
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
        $workflowToDelete = $workflowManager->find($workflowResource->getEntityClass(), $workflowId);
        if($workflowToDelete==null or sizeof($workflowToDelete)==0){
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }

    private $audit_log_purge = '
    {
        "@type": "Workflow",
        "triggerLimit": null,
        "name": "AuditLog Cleanup",
        "description": "Workflow for Scheduling AuditLog Cleanup",
        "collection": "/api/3/workflow_collections/82ea5860-0308-4fd3-90cd-3a4893781ee2",
        "tag": "#system",
        "isActive": true,
        "singleRecordExecution": false,
        "remoteExecutableFlag": false,
        "parameters": [],
        "synchronous": false,
        "triggerStep": "/api/3/workflow_steps/3e2200bf-c7de-4ebb-a7ab-6490f4f0bc51",
        "steps": [
          {
            "@type": "WorkflowStep",
            "name": "Delete AuditLogs",
            "description": null,
            "arguments": {
              "params": {
                "iri": "/gateway/audit/activities/delete/",
                "body": "{\"uptoDate\": {{arrow.utcnow().timestamp - vars.ttl_config.ttl}} }",
                "method": "POST"
              },
              "version": "2.4.0",
              "connector": "cyops_utilities",
              "operation": "make_cyops_request",
              "operationTitle": "CyOPs: Make CyOPs API Call",
              "step_variables": []
            },
            "status": null,
            "left": "20",
            "top": "260",
            "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
            "uuid": "bda18bfb-5263-4054-abd3-027a74e33394"
          },
          {
            "@type": "WorkflowStep",
            "name": "Fetch AuditLog TTL Settings",
            "description": null,
            "arguments": {
              "params": {
                "iri": "/api/3/system_settings",
                "body": "",
                "method": "GET"
              },
              "version": "2.4.0",
              "connector": "cyops_utilities",
              "operation": "make_cyops_request",
              "operationTitle": "CyOPs: Make CyOPs API Call",
              "step_variables": {
                "ttl_config": "{{vars.result.data[\"hydra:member\"][0][\"publicValues\"][\"auditlog_ttl_config\"]}}"
              }
            },
            "status": null,
            "left": "20",
            "top": "140",
            "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
            "uuid": "3d410571-ecc9-4813-93c7-64e84f1d0e53"
          },
          {
            "@type": "WorkflowStep",
            "name": "Start",
            "description": null,
            "arguments": {
              "step_variables": []
            },
            "status": null,
            "left": "20",
            "top": "20",
            "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
            "uuid": "3e2200bf-c7de-4ebb-a7ab-6490f4f0bc51"
          }
        ],
        "routes": [
          {
            "@type": "WorkflowRoute",
            "uuid": "4b2cfe4d-34c5-4638-9faf-9ee5b9512601",
            "name": "Fetch PB Log TTL Settings -> cleanup",
            "label": null,
            "isExecuted": false,
            "targetStep": "/api/3/workflow_steps/bda18bfb-5263-4054-abd3-027a74e33394",
            "sourceStep": "/api/3/workflow_steps/3d410571-ecc9-4813-93c7-64e84f1d0e53"
          },
          {
            "@type": "WorkflowRoute",
            "uuid": "da298044-4153-419c-a438-65aa72b66c0f",
            "name": "Start -> Fetch AuditLog TTL Settings",
            "label": null,
            "isExecuted": false,
            "targetStep": "/api/3/workflow_steps/3d410571-ecc9-4813-93c7-64e84f1d0e53",
            "sourceStep": "/api/3/workflow_steps/3e2200bf-c7de-4ebb-a7ab-6490f4f0bc51"
          }
        ],
        "versions": null,
        "uuid": "b9144644-d69c-4b98-bf33-337c3b1a6cfa"
      }
    ';

    /**
     * @param Schema $schema
     */
    public function up(Schema $schema)
    {
        $workflowIds = [
            'ebda6780-4eaf-47de-9a68-20a50b1ae8a2',
            '059d1d15-4ba3-4ee2-8426-2431ec4cbf63',
            '5beda203-0dbf-481f-afdd-a7e07f1f93fd',
            'a507278b-b8ee-4b0f-9765-845e384dbb2a',
            'ebfc713a-3ed3-4ccf-a53d-a8e2a369d242',
            '6477a44b-26c5-4dd8-9b18-88fd49396a81',
        ];
        foreach($workflowIds as $workflowIId ){
            error_log($workflowIId);
            $this->deleteWorkflow($workflowIId);
        }
        $this->addWorkflow($this->audit_log_purge);

    }

    /**
     * @param Schema $schema
     */
    public function down(Schema $schema)
    {
        $workflowIds = [
            'b9144644-d69c-4b98-bf33-337c3b1a6cfa'
        ];
        foreach($workflowIds as $workflowIId ){
            error_log($workflowIId);
            $this->deleteWorkflow($workflowIId);
        }

    }
}
