<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Application\Migrations;


use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\DBAL\Migrations\AbstractMigration;
use Doctrine\DBAL\Schema\Schema;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
class Version20190412080011 extends AbstractMigration implements ContainerAwareInterface
{

  private $container;

  public function setContainer(ContainerInterface $container = null)
  {
    $this->container = $container;
  }

  private function deleteWorkflow($workflowId)
  {
    /** @var ResourceInterface $workflowResource */
    $workflowResource = $this->container->get('resource.workflow');
    $managerRegistry = $this->container->get('doctrine');
    $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
    $workflowToDelete = $workflowManager->find($workflowResource->getEntityClass(), $workflowId);
    if ($workflowToDelete == null or sizeof($workflowToDelete) == 0) {
      $workflowManager->flush();
      return;
    }
    $workflowManager->remove($workflowToDelete);
    $workflowManager->flush();
  }


  private function addWorkflow($workflow)
  {
    $workflowResource = $this->container->get('resource.workflow');
    $context = $workflowResource->getDenormalizationContext();
    $context["ignore_field_authorization"] = true;
    $context["ignore_authorization"] = true;
    $workflowObject = $this->container->get('serializer')->deserialize(
      $workflow,
      $workflowResource->getEntityClass(),
      'json-ld',
      $context
    );
    /** @var ManagerRegistry $objectManager */
    $managerRegistry = $this->container->get('doctrine');
    $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
    $workflowManager->persist($workflowObject);
    $workflowManager->flush();
  }

  private $schedule_reporting = '
    {
      "@type": "Workflow",
      "triggerLimit": null,
      "name": "Generate Report from Schedule",
      "description": "Generate report using scheduler in CyOPs",
      "collection": "/api/3/workflow_collections/3af01eb9-0887-415b-b7fd-4d45a9d2760f",
      "tag": "#CyOPs-Utilities",
      "isActive": true,
      "singleRecordExecution": false,
      "remoteExecutableFlag": false,
      "parameters": [],
      "synchronous": false,
      "triggerStep": "/api/3/workflow_steps/944d9e56-0201-4cf5-9229-586440259b0b",
      "steps": [
        {
          "@type": "WorkflowStep",
          "name": "Start",
          "description": null,
          "arguments": {
            "step_variables": []
          },
          "status": null,
          "left": "20",
          "top": "20",
          "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
          "uuid": "944d9e56-0201-4cf5-9229-586440259b0b"
        },
        {
          "@type": "WorkflowStep",
          "name": "Save Response",
          "description": null,
          "arguments": {
            "response": "{{ vars.result.data }}",
            "inputParameters": "{{vars.__schedule.inputParameters}}"
          },
          "status": null,
          "left": "380",
          "top": "211",
          "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
          "uuid": "58de668d-7806-4a54-b07e-2e0399a756e4"
        },
        {
          "@type": "WorkflowStep",
          "name": "Generate Report By Scheduler",
          "description": null,
          "arguments": {
            "name": "CyOPs Schedule Report",
            "params": {
              "scheduler": "{{vars.__schedule}}"
            },
            "version": "1.0.0",
            "connector": "cyops-schedule-report",
            "operation": "schedule_report",
            "operationTitle": "Generate Report By Scheduler",
            "step_variables": []
          },
          "status": null,
          "left": "220",
          "top": "128",
          "stepType": "/api/3/workflow_step_types/0bfed618-0316-11e7-93ae-92361f002671",
          "uuid": "36d90c6c-f636-4ca9-905a-aa30197c89f4"
        },
        {
          "@type": "WorkflowStep",
          "name": "Send Email",
          "description": null,
          "arguments": {
            "when": "{{vars.__schedule.email}}",
            "config": "88c3d39c-2fa9-4731-b00d-29815008f17c",
            "params": {
              "cc": "{{vars.result.response.createUser.email}}",
              "to": "{{vars.__schedule.inputParameters.query.email}}",
              "bcc": "",
              "from": "no-reply@cybersponse.com",
              "content": "This is an automatically generated email. Please do not reply to it.",
              "subject": "CyOPs: {{vars.__schedule.name}}",
              "iri_list": "[\'{{vars.response.file[\'@id\']}}\']",
              "file_name": "{{vars.response.file.name}}",
              "file_path": "",
              "content_type": "text/plain"
            },
            "version": "2.3.0",
            "from_str": "soc@cybersponse.com",
            "connector": "smtp",
            "operation": "send_email",
            "operationTitle": "Send Email",
            "step_variables": []
          },
          "status": null,
          "left": "520",
          "top": "292",
          "stepType": "/api/3/workflow_step_types/4c0019b2-055c-44d0-968c-678a0c2d762e",
          "uuid": "e895980b-64d1-43fb-b809-00589fc15f6f"
        }
      ],
      "routes": [
        {
          "@type": "WorkflowRoute",
          "uuid": "278bf937-39ca-401d-a867-6a950d10c47e",
          "name": "Save Report in CyOPs -> Save Response",
          "label": null,
          "isExecuted": false,
          "targetStep": "/api/3/workflow_steps/58de668d-7806-4a54-b07e-2e0399a756e4",
          "sourceStep": "/api/3/workflow_steps/36d90c6c-f636-4ca9-905a-aa30197c89f4"
        },
        {
          "@type": "WorkflowRoute",
          "uuid": "3db005c5-e460-48b4-94a7-ffb160164f4a",
          "name": "Start -> Generate Report By Manual Trigger",
          "label": null,
          "isExecuted": false,
          "targetStep": "/api/3/workflow_steps/36d90c6c-f636-4ca9-905a-aa30197c89f4",
          "sourceStep": "/api/3/workflow_steps/944d9e56-0201-4cf5-9229-586440259b0b"
        },
        {
          "@type": "WorkflowRoute",
          "uuid": "d817c17c-2dca-42c9-b24a-3ebf712dacbc",
          "name": "Save Response -> Send Email",
          "label": null,
          "isExecuted": false,
          "targetStep": "/api/3/workflow_steps/e895980b-64d1-43fb-b809-00589fc15f6f",
          "sourceStep": "/api/3/workflow_steps/58de668d-7806-4a54-b07e-2e0399a756e4"
        }
      ],
      "versions": null,
      "uuid": "f4a7957e-8512-439b-9084-24620271ea43",
      "isPrivate": false,
      "owners": []
    }
    ';

  /**
   * @param Schema $schema
   */
  public function up(Schema $schema)
  {
    $this->addWorkflow($this->schedule_reporting);
  }


  /**
   * @param Schema $schema
   */
  public function down(Schema $schema)
  {
    $workflowId = 'f4a7957e-8512-439b-9084-24620271ea43';
    $this->deleteWorkflow($workflowId);
  }
}
