<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Application\Migrations;


use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\DBAL\Migrations\AbstractMigration;
use Doctrine\DBAL\Schema\Schema;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
class Version20190726080839 extends AbstractMigration implements ContainerAwareInterface
{

  private $container;

  public function setContainer(ContainerInterface $container = null)
  {
    $this->container = $container;
  }

  private function deleteWorkflow($workflowId)
  {
    /** @var ResourceInterface $workflowResource */
    $workflowResource = $this->container->get('resource.workflow');
    $managerRegistry = $this->container->get('doctrine');
    $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
    $workflowToDelete = $workflowManager->find($workflowResource->getEntityClass(), $workflowId);
    if ($workflowToDelete == null or sizeof($workflowToDelete) == 0) {
      $workflowManager->flush();
      return;
    }
    $workflowManager->remove($workflowToDelete);
    $workflowManager->flush();
  }


  private function addWorkflow($workflow)
  {
    $workflowResource = $this->container->get('resource.workflow');
    $context = $workflowResource->getDenormalizationContext();
    $context["ignore_field_authorization"] = true;
    $context["ignore_authorization"] = true;
    $workflowObject = $this->container->get('serializer')->deserialize(
      $workflow,
      $workflowResource->getEntityClass(),
      'json-ld',
      $context
    );
    /** @var ManagerRegistry $objectManager */
    $managerRegistry = $this->container->get('doctrine');
    $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
    $workflowManager->persist($workflowObject);
    $workflowManager->flush();
  }

  private $resume_task = '
  {
    "@type": "Workflow",
    "triggerLimit": null,
    "name": "Manual Task > Resume Playbook",
    "description": "Resumes the paused workflow after a manual task is completed.",
    "collection": "/api/3/workflow_collections/a9816e15-5e42-4785-a0ee-23fe8ddb19b6",
    "tag": "#system",
    "isActive": true,
    "singleRecordExecution": false,
    "remoteExecutableFlag": false,
    "parameters": [],
    "synchronous": false,
    "triggerStep": "/api/3/workflow_steps/04988c73-8fa0-4555-aac5-8f340f7a36d8",
    "steps": [
      {
        "@type": "WorkflowStep",
        "name": "Start",
        "description": null,
        "arguments": {
          "resource": "tasks",
          "step_variables": {
            "record": "{{vars.request.data}}"
          },
          "fieldbasedtrigger": {
            "limit": 30,
            "logic": "OR",
            "filters": [
              {
                "type": "object",
                "field": "status",
                "value": "/api/3/picklists/343f4b67-e929-4205-bf95-ba5b70545fed",
                "_value": {
                  "itemValue": "Completed"
                },
                "operator": "eq"
              },
              {
                "type": "object",
                "field": "status",
                "value": "/api/3/picklists/7bf6c03d-8e04-46d0-b392-febd78d72bad",
                "_value": {
                  "display": "Skipped",
                  "itemValue": "Skipped"
                },
                "operator": "eq"
              }
            ]
          }
        },
        "status": null,
        "left": "487",
        "top": "20",
        "stepType": "/api/3/workflow_step_types/9300bf69-5063-486d-b3a6-47eb9da24872",
        "uuid": "04988c73-8fa0-4555-aac5-8f340f7a36d8"
      },
      {
        "@type": "WorkflowStep",
        "name": "Revoke required Playbooks",
        "description": null,
        "arguments": {
          "when": "{{vars.record.workflowid and (vars.workflow_status == \'awaiting\')}}",
          "params": {
            "iri": "/api/wf/api/workflows/{{vars.record.workflowid}}/resume/?format=json",
            "body": "{\'status\': {{ vars.record.status}}, \'task_data\':  {{vars.record}}, \'step_id\':{{vars.record.stepid}}}",
            "method": "POST"
          },
          "version": "2.7.0",
          "connector": "cyops_utilities",
          "operation": "make_cyops_request",
          "operationTitle": "CyOPs: Make CyOPs API Call",
          "step_variables": []
        },
        "status": null,
        "left": "280",
        "top": "332",
        "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
        "uuid": "594c2818-c6bb-4548-ad3a-0366f515fdd5"
      },
      {
        "@type": "WorkflowStep",
        "name": "Get Playbook Status",
        "description": null,
        "arguments": {
          "params": {
            "iri": "/api/wf/api/workflows/{{vars.input.records[0].workflowid}}/?fields=status",
            "body": "",
            "method": "GET"
          },
          "version": "2.7.0",
          "connector": "cyops_utilities",
          "operation": "make_cyops_request",
          "operationTitle": "CyOPs: Make CyOPs API Call",
          "step_variables": {
            "workflow_status": "{{vars.result.data.status}}"
          }
        },
        "status": null,
        "left": "428",
        "top": "160",
        "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
        "uuid": "bf817073-fb12-40f8-bed7-1c5406aae5af"
      }
    ],
    "routes": [
      {
        "@type": "WorkflowRoute",
        "uuid": "94cc5feb-1ee1-4f17-9d1e-3bf33207da7d",
        "name": "Get Playbook Status -> Revoke required Playbooks",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/594c2818-c6bb-4548-ad3a-0366f515fdd5",
        "sourceStep": "/api/3/workflow_steps/bf817073-fb12-40f8-bed7-1c5406aae5af"
      },
      {
        "@type": "WorkflowRoute",
        "uuid": "db65e877-d2c4-4fed-9abe-9c979489edb0",
        "name": "Start -> Get Playbook Status",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/bf817073-fb12-40f8-bed7-1c5406aae5af",
        "sourceStep": "/api/3/workflow_steps/04988c73-8fa0-4555-aac5-8f340f7a36d8"
      }
    ],
    "versions": null,
    "uuid": "5fecc103-aa4b-4626-b69d-5e38697d89e3",
    "isPrivate": false,
    "owners": []
  }
    ';

  private $notifyOwnersPlaybook = '
  {
    "@type": "Workflow",
    "triggerLimit": null,
    "name": "Approval > Notify Owners",
    "description": "Triggers whenever an approval is requested from a playbook.",
    "collection": "/api/3/workflow_collections/a9816e15-5e42-4785-a0ee-23fe8ddb19b6",
    "tag": "#system",
    "isActive": true,
    "singleRecordExecution": false,
    "remoteExecutableFlag": false,
    "parameters": [],
    "synchronous": false,
    "triggerStep": "/api/3/workflow_steps/78d6f291-119e-4140-94c2-9a5cec6f8e27",
    "steps": [
      {
        "@type": "WorkflowStep",
        "name": "Fetch relations of the record",
        "description": null,
        "arguments": {
          "script": "/wf/workflow/tasks/crudhub_crud",
          "arguments": {
            "method": "GET",
            "resource": "",
            "collection": "{{ vars.record[\'@id\'] }}?$relationships=true"
          },
          "step_variables": {
            "team_uuid": "{% set parts = vars.result[\'owners\'][0][\'@id\'].split(\'/\') %}{{ parts[-1] }}"
          }
        },
        "status": null,
        "left": "400",
        "top": "351",
        "stepType": "/api/3/workflow_step_types/ee73e569-2188-43fe-a7f0-1964ba82a4de",
        "uuid": "503343ca-56a1-4efa-aefe-23a9e95b34a7"
      },
      {
        "@type": "WorkflowStep",
        "name": "Check for assigned to",
        "description": null,
        "arguments": {
          "conditions": [
            {
              "step_iri": "/api/3/workflow_steps/1adcae3c-49b8-45a5-9d0b-afd898fa9afe",
              "condition": "{{ vars.record.assignedTo != None }}"
            },
            {
              "step_iri": "/api/3/workflow_steps/503343ca-56a1-4efa-aefe-23a9e95b34a7",
              "condition": "{{ vars.record.assignedTo == None }}"
            }
          ]
        },
        "status": null,
        "left": "400",
        "top": "240",
        "stepType": "/api/3/workflow_step_types/12254cf5-5db7-4b1a-8cb1-3af081924b28",
        "uuid": "dfc48c0d-6307-43fc-9170-f3e4124a66ec"
      },
      {
        "@type": "WorkflowStep",
        "name": "Get emails of the team members",
        "description": null,
        "arguments": {
          "script": "/wf/workflow/tasks/crudhub_crud",
          "arguments": {
            "method": "GET",
            "resource": "",
            "collection": "/api/3/people?teams={{ vars.team_uuid }}"
          },
          "step_variables": {
            "emails": "{{ vars.result[\"hydra:member\"] | json_query(\'[*].email\')}}"
          }
        },
        "status": null,
        "left": "400",
        "top": "460",
        "stepType": "/api/3/workflow_step_types/ee73e569-2188-43fe-a7f0-1964ba82a4de",
        "uuid": "b505879a-3b26-4509-a4b1-cbbaef4c6f3c"
      },
      {
        "@type": "WorkflowStep",
        "name": "emails join",
        "description": null,
        "arguments": {
          "email_join": "{{vars.emails | join(\',\')}}"
        },
        "status": null,
        "left": "400",
        "top": "560",
        "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
        "uuid": "c9fd401a-0487-41a3-9243-18d87ee65572"
      },
      {
        "@type": "WorkflowStep",
        "name": "Start",
        "description": null,
        "arguments": {
          "resource": "approvals",
          "step_variables": {
            "host": "https://{{vars.request.data.approvalhost}}",
            "record": "{{ vars.request.data }}",
            "approval_id": "{{vars.request.data[\"@id\"].strip().split(\'/\')[-1]}}"
          },
          "fieldbasedtrigger": {
            "limit": 30,
            "logic": "AND",
            "filters": [
              {
                "type": "object",
                "field": "method",
                "value": "/api/3/picklists/2a817063-e12a-496d-934d-381d22a95b3b",
                "_value": {
                  "display": "Email",
                  "itemValue": "Email"
                },
                "operator": "eq"
              }
            ]
          }
        },
        "status": null,
        "left": "400",
        "top": "20",
        "stepType": "/api/3/workflow_step_types/ea155646-3821-4542-9702-b246da430a8d",
        "uuid": "78d6f291-119e-4140-94c2-9a5cec6f8e27"
      },
      {
        "@type": "WorkflowStep",
        "name": "Send an Email to Assigned users",
        "description": null,
        "arguments": {
          "name": "SMTP",
          "config": "6d8eb856-3f98-4e7d-84b9-79c9a8a7ad60",
          "params": {
            "cc": "",
            "to": "{{vars.record.assignedTo.email}}",
            "bcc": "",
            "from": "no-reply@cybersponse.com",
            "content": "<p>Greetings,</p>\n<p>This is an auto-generated email for your approval. {{ vars.record.approvaldescription }} Please follow the following link to approve/reject the request.</p>\n<p>{{ vars.host }}/approvals/{{ vars.record.workflowid }}/{{vars.approval_id}}/{{vars.record.stepid}}</p>\n<p>Thanks</p>",
            "subject": "AUTO: Email for approval",
            "iri_list": "",
            "file_name": "",
            "file_path": ""
          },
          "version": "2.3.0",
          "connector": "smtp",
          "operation": "send_richtext_email",
          "operationTitle": "Send Rich Text Email",
          "step_variables": []
        },
        "status": null,
        "left": "20",
        "top": "360",
        "stepType": "/api/3/workflow_step_types/0bfed618-0316-11e7-93ae-92361f002671",
        "uuid": "1adcae3c-49b8-45a5-9d0b-afd898fa9afe"
      },
      {
        "@type": "WorkflowStep",
        "name": "Send an Email to the assigned Team",
        "description": null,
        "arguments": {
          "name": "SMTP",
          "config": "6d8eb856-3f98-4e7d-84b9-79c9a8a7ad60",
          "params": {
            "cc": "",
            "to": "{{vars.email_join}}",
            "bcc": "",
            "from": "no-reply@cybersponse.com",
            "content": "<p>Greetings,</p>\n<p>This is an auto-generated email for your approval. {{ vars.record.approvaldescription }} Please follow the following link to approve/reject the request.</p>\n<p>{{ vars.host }}/approvals/{{vars.record.workflowid}}/{{vars.approval_id}}/{{vars.record.stepid}}</p>\n<p>Thanks</p>",
            "subject": "AUTO: Email for approval",
            "iri_list": "",
            "file_name": "",
            "file_path": ""
          },
          "version": "2.3.0",
          "connector": "smtp",
          "operation": "send_richtext_email",
          "operationTitle": "Send Rich Text Email",
          "step_variables": []
        },
        "status": null,
        "left": "800",
        "top": "560",
        "stepType": "/api/3/workflow_step_types/0bfed618-0316-11e7-93ae-92361f002671",
        "uuid": "ca82503d-2eae-4442-9f27-2779826296f9"
      }
    ],
    "routes": [
      {
        "@type": "WorkflowRoute",
        "uuid": "9e1c01aa-17bd-49da-8281-f3d046d8970c",
        "name": "Fetch relations of the record -> Get emails of the team members",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/b505879a-3b26-4509-a4b1-cbbaef4c6f3c",
        "sourceStep": "/api/3/workflow_steps/503343ca-56a1-4efa-aefe-23a9e95b34a7"
      },
      {
        "@type": "WorkflowRoute",
        "uuid": "d3ed0d01-20ec-401c-a3a9-9006a94219a8",
        "name": "Check for assigned to -> Fetch relations of the record",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/503343ca-56a1-4efa-aefe-23a9e95b34a7",
        "sourceStep": "/api/3/workflow_steps/dfc48c0d-6307-43fc-9170-f3e4124a66ec"
      },
      {
        "@type": "WorkflowRoute",
        "uuid": "a2a7eeb4-7216-4c19-ace3-2a9ddda95eb1",
        "name": "Get emails of the team members -> emails join",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/c9fd401a-0487-41a3-9243-18d87ee65572",
        "sourceStep": "/api/3/workflow_steps/b505879a-3b26-4509-a4b1-cbbaef4c6f3c"
      },
      {
        "@type": "WorkflowRoute",
        "uuid": "cc55101d-1bb9-48d7-8e9c-131f795a5105",
        "name": "Start -> Check for assigned to",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/dfc48c0d-6307-43fc-9170-f3e4124a66ec",
        "sourceStep": "/api/3/workflow_steps/78d6f291-119e-4140-94c2-9a5cec6f8e27"
      },
      {
        "@type": "WorkflowRoute",
        "uuid": "571c8573-befe-4ac0-bd1b-604e47dff068",
        "name": "Check for assigned to -> Send an Email to Assigned users",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/1adcae3c-49b8-45a5-9d0b-afd898fa9afe",
        "sourceStep": "/api/3/workflow_steps/dfc48c0d-6307-43fc-9170-f3e4124a66ec"
      },
      {
        "@type": "WorkflowRoute",
        "uuid": "ad9f60fc-9fa6-4871-a5e5-e208b03b41b4",
        "name": "emails join -> Send an Email to the assigned Team",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/ca82503d-2eae-4442-9f27-2779826296f9",
        "sourceStep": "/api/3/workflow_steps/c9fd401a-0487-41a3-9243-18d87ee65572"
      }
    ],
    "versions": null,
    "uuid": "dbe2606d-e5fe-4737-96a7-5d5631eaf723",
    "isPrivate": false,
    "owners": []
  }';

  private $notifyUpdatedOwnersPlaybook = '
  {
    "@type": "Workflow",
    "triggerLimit": null,
    "name": "Approval > Notify Updated Owners",
    "description": "Triggers whenever an approval change is requested from a playbook.",
    "collection": "/api/3/workflow_collections/a9816e15-5e42-4785-a0ee-23fe8ddb19b6",
    "tag": "#system",
    "isActive": true,
    "singleRecordExecution": false,
    "remoteExecutableFlag": false,
    "parameters": [],
    "synchronous": false,
    "triggerStep": "/api/3/workflow_steps/b249e047-2346-4fe5-908c-78b03ec22930",
    "steps": [
      {
        "@type": "WorkflowStep",
        "name": "Fetch relations of the record",
        "description": null,
        "arguments": {
          "script": "/wf/workflow/tasks/crudhub_crud",
          "arguments": {
            "method": "GET",
            "resource": "",
            "collection": "{{ vars.record[\'@id\'] }}?$relationships=true"
          },
          "step_variables": {
            "team_uuid": "{% set parts = vars.result[\'owners\'][0][\'@id\'].split(\'/\') %}{{ parts[-1] }}"
          }
        },
        "status": null,
        "left": "400",
        "top": "351",
        "stepType": "/api/3/workflow_step_types/ee73e569-2188-43fe-a7f0-1964ba82a4de",
        "uuid": "f2b71bbf-222d-446c-9877-f3a150554bd4"
      },
      {
        "@type": "WorkflowStep",
        "name": "Check for assigned to",
        "description": null,
        "arguments": {
          "conditions": [
            {
              "step_iri": "/api/3/workflow_steps/17eec8bd-068f-4dd5-a7f3-efaed7f9efc7",
              "condition": "{{ vars.record.assignedTo != None }}"
            },
            {
              "step_iri": "/api/3/workflow_steps/f2b71bbf-222d-446c-9877-f3a150554bd4",
              "condition": "{{ vars.record.assignedTo == None }}"
            }
          ]
        },
        "status": null,
        "left": "400",
        "top": "240",
        "stepType": "/api/3/workflow_step_types/12254cf5-5db7-4b1a-8cb1-3af081924b28",
        "uuid": "a78bbf05-b408-46d9-a1cc-b326369bb284"
      },
      {
        "@type": "WorkflowStep",
        "name": "Get emails of the team members",
        "description": null,
        "arguments": {
          "script": "/wf/workflow/tasks/crudhub_crud",
          "arguments": {
            "method": "GET",
            "resource": "",
            "collection": "/api/3/people?teams={{ vars.team_uuid }}"
          },
          "step_variables": {
            "emails": "{{ vars.result[\"hydra:member\"] | json_query(\'[*].email\')}}"
          }
        },
        "status": null,
        "left": "400",
        "top": "460",
        "stepType": "/api/3/workflow_step_types/ee73e569-2188-43fe-a7f0-1964ba82a4de",
        "uuid": "b458e711-bd33-4caf-9f44-ba66ce24e419"
      },
      {
        "@type": "WorkflowStep",
        "name": "emails join",
        "description": null,
        "arguments": {
          "email_join": "{{vars.emails | join(\',\')}}"
        },
        "status": null,
        "left": "400",
        "top": "560",
        "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
        "uuid": "95cd53ef-40d1-49e4-b8e2-c9b345e1772e"
      },
      {
        "@type": "WorkflowStep",
        "name": "Send an Email to Assigned users",
        "description": null,
        "arguments": {
          "name": "SMTP",
          "config": "6d8eb856-3f98-4e7d-84b9-79c9a8a7ad60",
          "params": {
            "cc": "",
            "to": "{{vars.record.assignedTo.email}}",
            "bcc": "",
            "from": "no-reply@cybersponse.com",
            "content": "<p>Greetings,</p>\n<p>This is an auto-generated email for your approval. {{ vars.record.approvaldescription }} Please follow the following link to approve/reject the request.</p>\n<p>{{ vars.host }}/approvals/{{ vars.record.workflowid }}/{{vars.approval_id}}/{{vars.record.stepid}}</p>\n<p>Thanks</p>",
            "subject": "AUTO: Email for approval",
            "iri_list": "",
            "file_name": "",
            "file_path": ""
          },
          "version": "2.3.0",
          "connector": "smtp",
          "operation": "send_richtext_email",
          "operationTitle": "Send Rich Text Email",
          "step_variables": []
        },
        "status": null,
        "left": "20",
        "top": "360",
        "stepType": "/api/3/workflow_step_types/0bfed618-0316-11e7-93ae-92361f002671",
        "uuid": "17eec8bd-068f-4dd5-a7f3-efaed7f9efc7"
      },
      {
        "@type": "WorkflowStep",
        "name": "Start",
        "description": null,
        "arguments": {
          "resource": "approvals",
          "step_variables": {
            "host": "https://{{vars.request.data.approvalhost}}",
            "record": "{{ vars.request.data }}",
            "approval_id": "{{vars.request.data[\"@id\"].strip().split(\'/\')[-1]}}"
          },
          "fieldbasedtrigger": {
            "limit": 30,
            "logic": "AND",
            "filters": [
              {
                "type": "object",
                "field": "method",
                "value": "/api/3/picklists/2a817063-e12a-496d-934d-381d22a95b3b",
                "_value": {
                  "itemValue": "Email"
                },
                "operator": "eq"
              },
              {
                "type": "object",
                "field": "assignedTo",
                "value": null,
                "_value": {
                  "display": "",
                  "itemValue": ""
                },
                "operator": "changed"
              },
              {
                "type": "object",
                "field": "assignedTo",
                "value": "false",
                "_value": [],
                "operator": "isnull"
              }
            ]
          }
        },
        "status": null,
        "left": "400",
        "top": "20",
        "stepType": "/api/3/workflow_step_types/9300bf69-5063-486d-b3a6-47eb9da24872",
        "uuid": "b249e047-2346-4fe5-908c-78b03ec22930"
      },
      {
        "@type": "WorkflowStep",
        "name": "Send an Email to the assigned Team",
        "description": null,
        "arguments": {
          "name": "SMTP",
          "config": "6d8eb856-3f98-4e7d-84b9-79c9a8a7ad60",
          "params": {
            "cc": "",
            "to": "{{vars.email_join}}",
            "bcc": "",
            "from": "no-reply@cybersponse.com",
            "content": "<p>Greetings,</p>\n<p>This is an auto-generated email for your approval. {{ vars.record.approvaldescription }} Please follow the following link to approve/reject the request.</p>\n<p>{{ vars.host }}/approvals/{{vars.record.workflowid}}/{{vars.approval_id}}/{{vars.record.stepid}}</p>\n<p>Thanks</p>",
            "subject": "AUTO: Email for approval",
            "iri_list": "",
            "file_name": "",
            "file_path": ""
          },
          "version": "2.3.0",
          "connector": "smtp",
          "operation": "send_richtext_email",
          "operationTitle": "Send Rich Text Email",
          "step_variables": []
        },
        "status": null,
        "left": "800",
        "top": "560",
        "stepType": "/api/3/workflow_step_types/0bfed618-0316-11e7-93ae-92361f002671",
        "uuid": "42bd61b3-9978-4516-92c3-816a0b35fcd7"
      }
    ],
    "routes": [
      {
        "@type": "WorkflowRoute",
        "uuid": "1d048061-3eaf-4ba6-b121-0721d1834d50",
        "name": "Fetch relations of the record -> Get emails of the team members",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/b458e711-bd33-4caf-9f44-ba66ce24e419",
        "sourceStep": "/api/3/workflow_steps/f2b71bbf-222d-446c-9877-f3a150554bd4"
      },
      {
        "@type": "WorkflowRoute",
        "uuid": "9518ba07-3d7a-4e9c-bc70-9b12b181f4a0",
        "name": "Check for assigned to -> Fetch relations of the record",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/f2b71bbf-222d-446c-9877-f3a150554bd4",
        "sourceStep": "/api/3/workflow_steps/a78bbf05-b408-46d9-a1cc-b326369bb284"
      },
      {
        "@type": "WorkflowRoute",
        "uuid": "f1897096-e97d-4b66-96f8-d8da1a6cf5be",
        "name": "Get emails of the team members -> emails join",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/95cd53ef-40d1-49e4-b8e2-c9b345e1772e",
        "sourceStep": "/api/3/workflow_steps/b458e711-bd33-4caf-9f44-ba66ce24e419"
      },
      {
        "@type": "WorkflowRoute",
        "uuid": "4fea87d5-68f5-441a-9e2c-2532d6690707",
        "name": "Check for assigned to -> Send an Email to Assigned users",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/17eec8bd-068f-4dd5-a7f3-efaed7f9efc7",
        "sourceStep": "/api/3/workflow_steps/a78bbf05-b408-46d9-a1cc-b326369bb284"
      },
      {
        "@type": "WorkflowRoute",
        "uuid": "622af3c4-c7ec-4cdd-8a77-906e30bd8318",
        "name": "emails join -> Send an Email to the assigned Team",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/42bd61b3-9978-4516-92c3-816a0b35fcd7",
        "sourceStep": "/api/3/workflow_steps/95cd53ef-40d1-49e4-b8e2-c9b345e1772e"
      },
      {
        "@type": "WorkflowRoute",
        "uuid": "7dd43716-8160-4a3c-b410-206b4af537b1",
        "name": "Start -> Check for assigned to",
        "label": null,
        "isExecuted": false,
        "targetStep": "/api/3/workflow_steps/a78bbf05-b408-46d9-a1cc-b326369bb284",
        "sourceStep": "/api/3/workflow_steps/b249e047-2346-4fe5-908c-78b03ec22930"
      }
    ],
    "versions": null,
    "uuid": "9a00d4f9-ba1d-4e9c-9737-a16d49818029",
    "isPrivate": false,
    "owners": []
  }';

  /**
   * @param Schema $schema
   */
  public function up(Schema $schema)
  {
    $this->addWorkflow($this->resume_task);
    $this->addWorkflow($this->notifyOwnersPlaybook);
    $this->addWorkflow($this->notifyUpdatedOwnersPlaybook);
  }


  /**
   * @param Schema $schema
   */
  public function down(Schema $schema)
  {
      $workflowId = '5fecc103-aa4b-4626-b69d-5e38697d89e3';
      $this->deleteWorkflow($workflowId);

      $notifyOwnersWorkflowId = 'dbe2606d-e5fe-4737-96a7-5d5631eaf723';
      $this->deleteWorkflow($notifyOwnersWorkflowId);

      $notifyUpdatedOwnersWorkflowId = '9a00d4f9-ba1d-4e9c-9737-a16d49818029';
      $this->deleteWorkflow($notifyUpdatedOwnersWorkflowId);
  }
}
