<?php

namespace Application\Migrations;

use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\DBAL\Migrations\AbstractMigration;
use Doctrine\DBAL\Schema\Schema;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
class Version20200507124517 extends AbstractMigration implements ContainerAwareInterface
{
    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    private function deleteWorkflow($workflowId)
    {
        /** @var ResourceInterface $workflowResource */
        $workflowResource = $this->container->get('resource.workflow');
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
        $workflowToDelete = $workflowManager->find($workflowResource->getEntityClass(), $workflowId);
        if ($workflowToDelete == null or sizeof($workflowToDelete) == 0) {
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }


    private function addWorkflow($workflow)
    {
        $workflowResource = $this->container->get('resource.workflow');
        $context = $workflowResource->getDenormalizationContext();
        $context["ignore_field_authorization"] = true;
        $context["ignore_authorization"] = true;
        $workflowObject = $this->container->get('serializer')->deserialize(
            $workflow,
            $workflowResource->getEntityClass(),
            'json-ld',
            $context
        );
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
        $workflowManager->persist($workflowObject);
        $workflowManager->flush();
    }

    private $agent_heartbeat = '{
        "@type": "Workflow",
        "triggerLimit": null,
        "name": "Agent > Trigger Health Check",
        "aliasName": null,
        "description": "Scheduled Workflow that initiates a heartbeat request from all agents on the instance",
        "collection": "/api/3/workflow_collections/82ea5860-0308-4fd3-90cd-3a4893781ee2",
        "tag": null,
        "isActive": false,
        "singleRecordExecution": false,
        "remoteExecutableFlag": false,
        "parameters": [],
        "synchronous": false,
        "triggerStep": "/api/3/workflow_steps/981ac967-df97-49ba-8551-cf1fbd41f77b",
        "steps": [
          {
            "@type": "WorkflowStep",
            "name": "Start",
            "description": null,
            "arguments": {
              "step_variables": []
            },
            "status": null,
            "left": "40",
            "top": "40",
            "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
            "uuid": "981ac967-df97-49ba-8551-cf1fbd41f77b"
          },
          {
            "@type": "WorkflowStep",
            "name": "Initiate HeartBeat",
            "description": null,
            "arguments": {
              "params": {
                "iri": "/api/integration/agent-heartbeat/all/",
                "body": "",
                "method": "GET"
              },
              "version": "3.0.3",
              "connector": "cyops_utilities",
              "operation": "make_cyops_request",
              "operationTitle": "FSR: Make FortiSOAR API Call",
              "step_variables": []
            },
            "status": null,
            "left": "236",
            "top": "183",
            "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
            "uuid": "1c09225f-4c26-4108-80bf-f1b7b61c65e4"
          }
        ],
        "routes": [
          {
            "@type": "WorkflowRoute",
            "uuid": "d60badb9-e2ac-4b11-bc8e-97b6ee6e5ed0",
            "name": "Start -> Initiate HearthBeat",
            "label": null,
            "isExecuted": false,
            "targetStep": "/api/3/workflow_steps/1c09225f-4c26-4108-80bf-f1b7b61c65e4",
            "sourceStep": "/api/3/workflow_steps/981ac967-df97-49ba-8551-cf1fbd41f77b"
          }
        ],
        "versions": null,
        "lastModifyDate": 1588854494,
        "uuid": "ced3c4f0-0bb3-4862-91e3-ec044e8afc8c",
        "recordTags": [
          "system"
        ],
        "isPrivate": false,
        "owners": []
      }';

      private $agent_unreachable = '{
        "@type": "Workflow",
        "triggerLimit": null,
        "name": "Agent > Check For Missed Heartbeats",
        "aliasName": null,
        "description": "Scheduled Workflow that checks if there has been no heartbeat from agents since a configured interval and marks their status as Failed",
        "collection": "/api/3/workflow_collections/82ea5860-0308-4fd3-90cd-3a4893781ee2",
        "tag": null,
        "isActive": false,
        "singleRecordExecution": false,
        "remoteExecutableFlag": false,
        "parameters": [],
        "synchronous": false,
        "triggerStep": "/api/3/workflow_steps/6fb7d5dc-1b37-4a51-8253-820ce42f6eaf",
        "steps": [
          {
            "@type": "WorkflowStep",
            "name": "Start",
            "description": null,
            "arguments": {
              "step_variables": []
            },
            "status": null,
            "left": "60",
            "top": "13",
            "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
            "uuid": "6fb7d5dc-1b37-4a51-8253-820ce42f6eaf"
          },
          {
            "@type": "WorkflowStep",
            "name": "Find unreachable nodes",
            "description": null,
            "arguments": {
              "query": {
                "sort": [],
                "limit": 30,
                "logic": "AND",
                "filters": [
                  {
                    "type": "primitive",
                    "field": "role",
                    "value": "self",
                    "operator": "neq",
                    "_operator": "neq"
                  },
                  {
                    "type": "primitive",
                    "field": "healthCheckLastSync",
                    "value": "{{vars.max_time_for_marking_agent_unreachbale}}",
                    "operator": "lte",
                    "_operator": "lte"
                  },
                  {
                    "type": "primitive",
                    "field": "active",
                    "value": true,
                    "operator": "eq",
                    "_operator": "eq"
                  },
                  {
                    "type": "object",
                    "field": "configurationHealth",
                    "value": "/api/3/picklists/b5566a53-7af5-48f6-9dbd-5bf5fc9f432b",
                    "_value": {
                      "@id": "/api/3/picklists/b5566a53-7af5-48f6-9dbd-5bf5fc9f432b",
                      "display": "Awaiting Remote Node Connection",
                      "itemValue": "Awaiting Remote Node Connection"
                    },
                    "operator": "neq"
                  }
                ]
              },
              "module": "agents?$limit=100",
              "step_variables": []
            },
            "status": null,
            "left": "200",
            "top": "231",
            "stepType": "/api/3/workflow_step_types/b593663d-7d13-40ce-a3a3-96dece928770",
            "uuid": "5d4caed3-e789-46e6-8686-ecbe78689d64"
          },
          {
            "@type": "WorkflowStep",
            "name": "Configuration",
            "description": null,
            "arguments": {
              "update_agents": "[]",
              "max_time_for_marking_agent_unreachbale": "{{arrow.utcnow().shift(minutes=-30) }}"
            },
            "status": null,
            "left": "132",
            "top": "120",
            "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
            "uuid": "eba18c2f-5395-4fe1-9343-13678d584cc1"
          },
          {
            "@type": "WorkflowStep",
            "name": "Prepare bulk update payload",
            "description": null,
            "arguments": {
              "update_payload": "{% for each_item in vars.steps.Find_unreachable_nodes %} \n{{vars.update_agents.append({\"@id\": each_item[\"@id\"], \"configurationHealth\": \"135030fa-1b0b-453d-9fed-df9be5d4397f\"} ) }} \n{% endfor %}"
            },
            "status": null,
            "left": "273",
            "top": "340",
            "stepType": "/api/3/workflow_step_types/04d0cf46-b6a8-42c4-8683-60a7eaa69e8f",
            "uuid": "c11612c3-8d19-48ab-b2c3-01f1058cc558"
          },
          {
            "@type": "WorkflowStep",
            "name": "Mark agent unreachable",
            "description": null,
            "arguments": {
              "when": "{{vars.update_agents | length > 0}}",
              "params": {
                "iri": "/api/3/update/agents",
                "body": "{\"data\": {{vars.update_agents}} }",
                "method": "PUT"
              },
              "version": "3.0.3",
              "connector": "cyops_utilities",
              "operation": "make_cyops_request",
              "operationTitle": "FSR: Make FortiSOAR API Call",
              "step_variables": []
            },
            "status": null,
            "left": "346",
            "top": "450",
            "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
            "uuid": "7b99c417-d0a7-4ba7-99ab-a43cb51e2bc0"
          }
        ],
        "routes": [
          {
            "@type": "WorkflowRoute",
            "uuid": "456e8884-d0f0-412d-b6f0-b5d2fabd2902",
            "name": "Configurations -> Find unreachable nodes",
            "label": null,
            "isExecuted": false,
            "targetStep": "/api/3/workflow_steps/5d4caed3-e789-46e6-8686-ecbe78689d64",
            "sourceStep": "/api/3/workflow_steps/eba18c2f-5395-4fe1-9343-13678d584cc1"
          },
          {
            "@type": "WorkflowRoute",
            "uuid": "94faf49a-6480-43aa-a7be-9723e7ea57fd",
            "name": "Find unreachable nodes -> prepare bulk update payload",
            "label": null,
            "isExecuted": false,
            "targetStep": "/api/3/workflow_steps/c11612c3-8d19-48ab-b2c3-01f1058cc558",
            "sourceStep": "/api/3/workflow_steps/5d4caed3-e789-46e6-8686-ecbe78689d64"
          },
          {
            "@type": "WorkflowRoute",
            "uuid": "20ae55ce-e2f3-43c2-aba4-e35916980590",
            "name": "prepare bulk update payload -> Mark agent unreachable",
            "label": null,
            "isExecuted": false,
            "targetStep": "/api/3/workflow_steps/7b99c417-d0a7-4ba7-99ab-a43cb51e2bc0",
            "sourceStep": "/api/3/workflow_steps/c11612c3-8d19-48ab-b2c3-01f1058cc558"
          },
          {
            "@type": "WorkflowRoute",
            "uuid": "404c15e9-f9ee-4f05-ac0c-43fa58eb0353",
            "name": "Start -> Configurations",
            "label": null,
            "isExecuted": false,
            "targetStep": "/api/3/workflow_steps/eba18c2f-5395-4fe1-9343-13678d584cc1",
            "sourceStep": "/api/3/workflow_steps/6fb7d5dc-1b37-4a51-8253-820ce42f6eaf"
          }
        ],
        "versions": null,
        "lastModifyDate": 1591694342,
        "uuid": "c022b98c-20ac-4a77-896b-1e2d6862da7d",
        "recordTags": [
          "system"
        ],
        "isPrivate": false,
        "owners": []
      }';

    /**
     * @param Schema $schema
     */
    public function up(Schema $schema)
    {
        $this->addWorkflow($this->agent_heartbeat);
        $this->addWorkflow($this->agent_unreachable);
    }

    /**
     * @param Schema $schema
     */
    public function down(Schema $schema)
    {
        $this->deleteWorkflow($this->agent_heartbeat);
    }
}
