<?php

namespace Application\Migrations;

use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\DBAL\Migrations\AbstractMigration;
use Doctrine\DBAL\Schema\Schema;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
class Version20200731110940 extends AbstractMigration implements ContainerAwareInterface
{

    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    private function deleteWorkflow($workflowId)
    {
        /** @var ResourceInterface $workflowResource */
        $workflowResource = $this->container->get('resource.workflow');
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
        $workflowToDelete = $workflowManager->find($workflowResource->getEntityClass(), $workflowId);
        if ($workflowToDelete == null) {
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }


    private function addWorkflow($workflow)
    {
        $workflowResource = $this->container->get('resource.workflow');
        $context = $workflowResource->getDenormalizationContext();
        $context["ignore_field_authorization"] = true;
        $context["ignore_authorization"] = true;
        $workflowObject = $this->container->get('serializer')->deserialize(
            $workflow,
            $workflowResource->getEntityClass(),
            'json-ld',
            $context
        );
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
        $workflowManager->persist($workflowObject);
        $workflowManager->flush();
    }
    
    private $audit_purging = '{
        "@type": "Workflow",
        "triggerLimit": null,
        "name": "AuditLog Cleanup",
        "description": "Workflow for Scheduling AuditLog Cleanup",
        "collection": "/api/3/workflow_collections/82ea5860-0308-4fd3-90cd-3a4893781ee2",
        "tag": "#system",
        "recordTags":"/api/3/tags/system",
        "isActive": true,
        "singleRecordExecution": false,
        "remoteExecutableFlag": false,
        "parameters": [],
        "synchronous": false,
        "triggerStep": "/api/3/workflow_steps/3e2200bf-c7de-4ebb-a7ab-6490f4f0bc51",
        "steps": [
          {
            "@type": "WorkflowStep",
            "name": "Delete AuditLogs",
            "description": null,
            "arguments": {
              "params": {
                "iri": "/gateway/audit/activities/delete/",
                "body": "{\"uptoDate\": {{arrow.utcnow().timestamp*1000 - vars.ttl_config.ttl*24*60*60*1000}} }",
                "method": "POST"
              },
              "version": "3.0.1",
              "connector": "cyops_utilities",
              "operation": "make_cyops_request",
              "operationTitle": "FSR: Make FortiSOAR API Call",
              "step_variables": []
            },
            "status": null,
            "left": "20",
            "top": "260",
            "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
            "uuid": "bda18bfb-5263-4054-abd3-027a74e33394"
          },
          {
            "@type": "WorkflowStep",
            "name": "Fetch AuditLog TTL Settings",
            "description": null,
            "arguments": {
              "params": {
                "iri": "/api/3/system_settings",
                "body": "",
                "method": "GET"
              },
              "version": "3.0.1",
              "connector": "cyops_utilities",
              "operation": "make_cyops_request",
              "operationTitle": "FSR: Make FortiSOAR API Call",
              "step_variables": {
                "ttl_config": "{{vars.result.data[\"hydra:member\"][0][\"publicValues\"][\"auditlog_ttl_config\"]}}"
              }
            },
            "status": null,
            "left": "20",
            "top": "140",
            "stepType": "/api/3/workflow_step_types/0109f35d-090b-4a2b-bd8a-94cbc3508562",
            "uuid": "3d410571-ecc9-4813-93c7-64e84f1d0e53"
          },
          {
            "@type": "WorkflowStep",
            "name": "Start",
            "description": null,
            "arguments": {
              "step_variables": []
            },
            "status": null,
            "left": "20",
            "top": "20",
            "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
            "uuid": "3e2200bf-c7de-4ebb-a7ab-6490f4f0bc51"
          }
        ],
        "routes": [
          {
            "@type": "WorkflowRoute",
            "uuid": "4b2cfe4d-34c5-4638-9faf-9ee5b9512601",
            "name": "Fetch PB Log TTL Settings -> cleanup",
            "label": null,
            "isExecuted": false,
            "targetStep": "/api/3/workflow_steps/bda18bfb-5263-4054-abd3-027a74e33394",
            "sourceStep": "/api/3/workflow_steps/3d410571-ecc9-4813-93c7-64e84f1d0e53"
          },
          {
            "@type": "WorkflowRoute",
            "uuid": "da298044-4153-419c-a438-65aa72b66c0f",
            "name": "Start -> Fetch AuditLog TTL Settings",
            "label": null,
            "isExecuted": false,
            "targetStep": "/api/3/workflow_steps/3d410571-ecc9-4813-93c7-64e84f1d0e53",
            "sourceStep": "/api/3/workflow_steps/3e2200bf-c7de-4ebb-a7ab-6490f4f0bc51"
          }
        ],
        "versions": null,
        "uuid": "b9144644-d69c-4b98-bf33-337c3b1a6cfa"
      }';

    /**
     * @param Schema $schema
     */
    public function up(Schema $schema)
    {
        $this->deleteWorkflow("b9144644-d69c-4b98-bf33-337c3b1a6cfa");
        $this->addWorkflow($this->audit_purging);
    }

    /**
     * @param Schema $schema
     */
    public function down(Schema $schema)
    {
        $this->deleteWorkflow("b9144644-d69c-4b98-bf33-337c3b1a6cfa");

    }
}
