<?php

namespace Application\Migrations;

use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\DBAL\Migrations\AbstractMigration;
use Doctrine\DBAL\Schema\Schema;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
class Version20200817054651 extends AbstractMigration implements ContainerAwareInterface
{
    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    private function deleteWorkflow($workflowId)
    {
        /** @var ResourceInterface $workflowResource */
        $workflowResource = $this->container->get('resource.workflow');
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
        $workflowToDelete = $workflowManager->find($workflowResource->getEntityClass(), $workflowId);
        if ($workflowToDelete == null) {
            $workflowManager->flush();
            return;
        }
        $workflowManager->remove($workflowToDelete);
        $workflowManager->flush();
    }


    private function addWorkflow($workflow)
    {
        $workflowResource = $this->container->get('resource.workflow');
        $context = $workflowResource->getDenormalizationContext();
        $context["ignore_field_authorization"] = true;
        $context["ignore_authorization"] = true;
        $workflowObject = $this->container->get('serializer')->deserialize(
            $workflow,
            $workflowResource->getEntityClass(),
            'json-ld',
            $context
        );
        /** @var ManagerRegistry $objectManager */
        $managerRegistry = $this->container->get('doctrine');
        $workflowManager = $managerRegistry->getManagerForClass($workflowResource->getEntityClass());
        $workflowManager->persist($workflowObject);
        $workflowManager->flush();
    }

    private $integration_purge_schedule = '{
          "@type": "Workflow",
          "triggerLimit": null,
          "name": "Purge Integration Logs",
          "aliasName": null,
          "description": "Scheduled playbook to purge integration logs",
          "collection": "/api/3/workflow_collections/82ea5860-0308-4fd3-90cd-3a4893781ee2",
          "tag": null,
          "isActive": true,
          "singleRecordExecution": false,
          "remoteExecutableFlag": false,
          "parameters": [],
          "synchronous": false,
          "triggerStep": "/api/3/workflow_steps/07478e52-9bd4-468f-92c3-02143ddf6305",
          "steps": [
            {
              "@type": "WorkflowStep",
              "name": "Start",
              "description": null,
              "arguments": {
                "step_variables": []
              },
              "status": null,
              "left": "40",
              "top": "40",
              "stepType": "/api/3/workflow_step_types/b348f017-9a94-471f-87f8-ce88b6a7ad62",
              "uuid": "07478e52-9bd4-468f-92c3-02143ddf6305"
            },
            {
              "@type": "WorkflowStep",
              "name": "Call Purge API",
              "description": null,
              "arguments": {
                "name": "Utilities",
                "params": {
                  "iri": "/api/integration/action-log/purge/?format=json",
                  "body": "{\"keep_days\": 1}",
                  "method": "POST"
                },
                "version": "3.0.4",
                "connector": "cyops_utilities",
                "operation": "make_cyops_request",
                "operationTitle": "FSR: Make FortiSOAR API Call",
                "pickFromTenant": false,
                "step_variables": []
              },
              "status": null,
              "left": "40",
              "top": "160",
              "stepType": "/api/3/workflow_step_types/0bfed618-0316-11e7-93ae-92361f002671",
              "uuid": "b1215378-258d-443d-8631-2fe65029ff73"
            }
          ],
          "routes": [
            {
              "@type": "WorkflowRoute",
              "uuid": "25544b8f-a312-4c2a-8239-1624c5ed5eed",
              "name": "Start -> Call Purge API",
              "label": null,
              "isExecuted": false,
              "targetStep": "/api/3/workflow_steps/b1215378-258d-443d-8631-2fe65029ff73",
              "sourceStep": "/api/3/workflow_steps/07478e52-9bd4-468f-92c3-02143ddf6305"
            }
          ],
          "versions": null,
          "lastModifyDate": 1597393442,
          "priority": null,
          "uuid": "0ee017e5-9484-4501-8ddc-66e7c7fba96d",
          "recordTags": [
            "system"
          ],
          "isPrivate": false,
          "owners": []
        }';

    /**
     * @param Schema $schema
     */
    public function up(Schema $schema)
    {
        $this->addWorkflow($this->integration_purge_schedule);
    }

    /**
     * @param Schema $schema
     */
    public function down(Schema $schema)
    {
        $this->deleteWorkflow("0ee017e5-9484-4501-8ddc-66e7c7fba96d");
    }
}
