<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ActorsBundle\Controller;


use Cybersponse\ActorsBundle\Entity\Actor;
use Cybersponse\CoreBundle\Controller\AuthorizationResourceController;
use Dunglas\ApiBundle\Api\ResourceCollectionInterface;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class ActorsController extends AuthorizationResourceController
{
    /** @var Actor */
    protected $user;

    public function getAction(Request $request)
    {
        $resource = parent::getResource($request);
        $object = $this->getCurrentUser();
        return parent::getSuccessResponse($resource, $object, 200, [], ['ignore_authorization' => true, 'ignore_field_authorization' => true, 'ignore_relationship_authorization' => true]);
    }

    public function putAction(Request $request)
    {
        return parent::putAction($request, null);
    }

    /**
     * @inheritdoc
     */
    public function findOrThrowNotFound(ResourceInterface $resource, $id)
    {
        return $this->getCurrentUser();
    }

    /**
     * @inheritdoc
     */
    protected function getResource(Request $request)
    {
        $user = $this->getCurrentUser();
        $entityClass = get_class($user);
        /** @var ResourceCollectionInterface $resourceCollection */
        $resourceCollection = $this->get('api.resource_collection');
        return $resourceCollection->getResourceForEntity($entityClass);
    }

    /**
     * @inheritdoc
     */
    protected function denyAccessUnlessGranted($attributes, $object = null, $message = 'Access Denied.')
    {
        // do nothing -- event subscribers should handle authorization for this controller
    }

    /**
     * @return Actor|mixed
     */
    protected function getCurrentUser()
    {
        if (!$this->user) {
            /** @var TokenStorageInterface $tokenStorage */
            $tokenStorage = $this->container->get('security.token_storage');
            $token = $tokenStorage->getToken();
            $this->user = $token->getUser();
        }

        return $this->user;
    }
}
