<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ActorsBundle\Controller;

use Dunglas\ApiBundle\JsonLd\Response;
use Cybersponse\ActorsBundle\Entity\Actor;
use Cybersponse\ActorsBundle\Entity\Avatar;
use Symfony\Component\HttpFoundation\Request;
use Cybersponse\ActorsBundle\Provider\AvatarProvider;
use Cybersponse\FilesBundle\Controller\ImagesController;
use Symfony\Component\HttpKernel\Exception\HttpException;


/**
 * Class AvatarController
 *
 * Todo: To do this feature effectively, it would be better to allow for permissions per action.
 * Todo: For example, GET Avatar should require application.read and POST/DELETE/PUT Avatar should require security.update/security.delete
 * @package Cybersponse\ActorsBundle\Controller
 */
class AvatarController extends ImagesController
{
    public function getAction(Request $request, $id)
    {
        //Ignore permissions because this is a public asset
        $resource = $this->getResource($request);
        /** @var Avatar $avatar */
        $avatar = $this->findOrThrowNotFound($resource, $id);
        $response = $this->getSuccessResponse($resource, $avatar, Response::HTTP_OK, [], ['ignore_authorization' => true]);
        $response->headers->set('Content-Type', $avatar->getMimeType());
        $response->setContent($avatar->getFile()->getBytes());
        return $response;
    }

    public function uploadCurrentAction(Request $request)
    {
        /** @var AvatarProvider $avatarProvider */
        $avatarProvider = $this->container->get('cybersponse.actor.provider.avatar');
        /** @var Avatar $avatar */
        $avatar = $avatarProvider->getDataForCurrentUser();
        if ($avatar) {
            $fileProvider = $this->get('cybersponse.files.provider.files');
            $fileProvider->removeFiles($avatar);
        }

        /** @var Actor $user */
        $user = $this->getUser();
        $request->attributes->set('uuid', $user->getUuid());

        $avataObj = parent::uploadFile($request);
        /** @var Actor $user */
        $user = $this->getUser();
        $upload = $request->files->get('file');
        $size = getimagesize($upload->getPathname());
        $avataObj['id'] = $user->getUuid();
        $avataObj['width'] = $size[0];
        $avataObj['height'] = $size[1];
        $securityToken = $this->container->get('security.token_storage')->getToken();
        $iriConverter = $this->container->get('api.iri_converter');

        $userIri = $iriConverter->getIriFromItem($securityToken->getUser());

        $avataObj['uuid'] =  $user->getUuid();
        $avataObj['uploadUser'] = $userIri;
        return parent::cpostAction($request, $avataObj);
    }

    /**
     * @inheritDoc
     */
    protected function denyAccessUnlessGranted($resource, $actions, $object = null)
    {
        // do nothing -- event subscribers should handle authorization for this controller
    }

    public function downloadCurrentAction(Request $request)
    {
        $resource = $this->getResource($request);
        /** @var AvatarProvider $avatarProvider */
        $avatarProvider = $this->container->get('cybersponse.actor.provider.avatar');
        /** @var Avatar $avatar */
        $avatar = $avatarProvider->getDataForCurrentUser();
        return $this->buildResponseForAvatar($request, $avatar, $resource);
    }

    public function downloadAction(Request $request, $id)
    {
        $resource = $this->getResource($request);
        $request->attributes->set('id', $id);
        /** @var Avatar $avatar */
        $avatar = $this->findOrThrowNotFound($resource, $id);
        return $this->buildResponseForAvatar($request, $avatar, $resource);
    }

    public function deleteCurrentAction(Request $request)
    {
        /** @var AvatarProvider $avatarProvider */
        $avatarProvider = $this->container->get('cybersponse.actor.provider.avatar');
        /** @var Avatar $avatar */
        $avatar = $avatarProvider->getDataForCurrentUser();
        $fileProvider = $this->get('cybersponse.files.provider.files');
        $fileProvider->removeFiles($avatar);
        return new Response(null, Response::HTTP_NO_CONTENT);
    }

    public function notImplementedAction()
    {
        throw new HttpException(Response::HTTP_NOT_IMPLEMENTED, 'Not Implemented. Post to /api/avatars/current');
    }

    /**
     * @param Request $request
     * @param $avatar
     * @param $resource
     * @return \Dunglas\ApiBundle\JsonLd\Response
     */
    private function buildResponseForAvatar(Request $request, $avatar, $resource)
    {
        if (!$avatar) {
            throw $this->createNotFoundException();
        }
        $response = $this->getSuccessResponse(
            $resource,
            $avatar,
            Response::HTTP_OK,
            [],
            ['ignore_authorization' => true]
        );
        $accept = $request->headers->get('Accept', 'application/json');
        // fnmatch is a simple wildcard matcher, typically used for filename shell matching.
        if (fnmatch($accept, $avatar->getMimeType())) {
            $response->headers->set('Content-Type', $avatar->getMimeType());
            $response->setContent($avatar->getFile()->getBytes());
        }
        return $response;
    }
}
