<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ActorsBundle\Entity;

use Cybersponse\AuthorizationBundle\Entity\Role;
use Cybersponse\AuthorizationBundle\Entity\Team;
use Cybersponse\CoreBundle\Entity\BaseEntity;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Intl\Exception\MethodNotImplementedException;
use Symfony\Component\Security\Core\User\UserInterface;

/**
 * @ORM\Table(name="actors")
 * @ORM\Entity
 * @ORM\InheritanceType("SINGLE_TABLE")
 * @ORM\DiscriminatorColumn(name="record_type", type="string", length=20)
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Operations\Operations(delete=false)
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ name }}{{ firstname }} {{ lastname }}")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="actors")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="people")
 */
class Actor extends BaseEntity implements UserInterface
{
    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="Cybersponse\AuthorizationBundle\Entity\Role", cascade={"persist"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Roles")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="roles")
     * @Doctrine\ORM\Mapping\JoinTable(name="actor_role",
     *   joinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="actor_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   },
     *   inverseJoinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="role_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   }
     * )
     */
    protected $roles;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="Cybersponse\AuthorizationBundle\Entity\Team", mappedBy="actors", cascade={"persist"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Teams")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="teams")
     */
    protected $teams;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="user_id", type="string", length=36, unique=true)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Visibility\Visibility(visibility=false)
     */
    protected $userId;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="avatar", type="string", length=255, nullable=true, unique=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="hidden")
     */
    protected $avatar;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="user_type", type="string", length=36, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Visibility\Visibility(visibility=false)
     */
    protected $userType;

    function __construct()
    {
        $this->roles = new \Doctrine\Common\Collections\ArrayCollection();
        $this->teams = new \Doctrine\Common\Collections\ArrayCollection();
    }


    /**
     * @inheritdoc
     * @codeCoverageIgnore
     */
    public function getPassword()
    {
        throw new MethodNotImplementedException(__METHOD__);
    }

    /**
     * @inheritdoc
     * @codeCoverageIgnore
     */
    public function getSalt()
    {
        throw new MethodNotImplementedException(__METHOD__);
    }

    /**
     * @inheritdoc
     */
    public function getUsername()
    {
        return $this->getUserId();
    }

    /**
     * @inheritdoc
     * @codeCoverageIgnore
     */
    public function eraseCredentials()
    {
        // Nothing to see here, move along.
    }

    /**
     * Add roles
     *
     * @param Role $role
     * @return self
     */
    public function addRole(Role $role)
    {
        if (!$this->roles->contains($role)) {
            $this->roles[] = $role;
        }

        return $this;
    }

    /**
     * Remove roles
     *
     * @param Role $role
     */
    public function removeRole(Role $role)
    {
        $this->roles->removeElement($role);
    }

    /**
     * Get roles
     *
     * TODO this method is expected by other consumers...had to do something crazy to make it backward compatible :|
     * @param bool $asArray
     * @return \Cybersponse\AuthorizationBundle\Entity\Role[]
     */
    public function getRoles($asArray = true)
    {
        return $asArray ? $this->roles->toArray() : $this->roles;
    }

    /**
     * Add teams
     *
     * @param Team $team
     * @return self
     */
    public function addTeam(Team $team)
    {
        if (!$this->teams->contains($team)) {
            $this->teams[] = $team;
            $team->addActor($this);
        }

        return $this;
    }

    /**
     * Remove teams
     *
     * @param Team $teams
     */
    public function removeTeam(Team $teams)
    {
        $this->teams->removeElement($teams);
        $teams->removeActor($this);
    }

    /**
     * Get teams
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getTeams()
    {
        return $this->teams;
    }

    /**
     * @param string $userId
     */
    public function setUserId($userId)
    {
        $this->userId = $userId;
    }

    /**
     * @return string
     */
    public function getUserId()
    {
        return $this->userId;
    }

    /**
     * @param string $userType
     */
    public function setUserType($userType)
    {
        $this->userType = $userType;
    }

    /**
     * @return string
     */
    public function getUserType()
    {
        return $this->userType;
    }

    /**
     * @return string|null
     */
    public function getAvatar()
    {
        return $this->avatar;
    }

    /**
     * @param string $avatar
     */
    public function setAvatar($avatar)
    {
        $this->avatar = $avatar;
    }

    /**
     * @return mixed
     */
    public function getIsActive()
    {
        return $this->isActive;
    }

    /**
     * @return mixed
     */
    public function getIsLocked()
    {
        return $this->isLocked;
    }
}
