<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ActorsBundle\Serializer;


use Cybersponse\ActorsBundle\Entity\Avatar;
use Cybersponse\ActorsBundle\Entity\Actor;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryProvider;
use Cybersponse\SettingsBundle\Provider\SettingsProvider;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class CurrentActorNormalizer extends ActorNormalizer
{
    /** @var SettingsProvider */
    protected $settingsProvider;

    /** @var PermissionsDictionaryProvider */
    protected $permissionsProvider;

    /** @var Actor $user */
    protected $user;

    /** @var TokenStorageInterface $tokenStorage */
    protected $tokenStorage;

    /**
     * @inheritdoc
     */
    public function supportsNormalization($data, $format = null)
    {
        return self::FORMAT === $format && ($data instanceof Actor) && $data === $this->getCurrentUser();
    }

    /**
     * @inheritdoc
     */
    public function supportsDenormalization($data, $type, $format = null)
    {
        return is_subclass_of($type, Actor::class);
    }

    /**
     * @param Actor $actor
     * @inheritdoc
     */
    public function normalize($actor, $format = null, array $context = [])
    {
        // Symfony's normalizer will create a cache of Class to Normalizer meaning it will always choose this one.
        if ($actor !== $this->getCurrentUser()) {
            return parent::normalize($actor, $format, $context);
        }

        $context['ignore_authorization'] = true;
        $context['ignore_field_authorization'] = true;
        $context['ignore_relationship_authorization'] = false;

        $result = parent::normalize($actor, $format, $context);

        // Only show expanded settings/permissions on the top level, never in lists
        $inSubLevel = array_key_exists('json_ld_sub_level', $context) && $context['json_ld_sub_level'];

        if (is_array($result) && !$inSubLevel) {
            $permissions = $this->permissionsProvider->getDataForActor($actor);
            $result['@permissions'] = $permissions;

            list($userSettings, $systemSettings) = $this->settingsProvider->getDataForActor($actor);
            $result['@settings'] = $this->settingsProvider->getMergedSettings($userSettings, $systemSettings);
        }


        return $result;
    }

    public function denormalize($data, $class, $format = null, array $context = [])
    {
        if (array_key_exists('object_to_populate', $context) && $context['object_to_populate'] === $this->getCurrentUser()) {
            $context['ignore_field_authorization'] = true;
        }

        return parent::denormalize($data, $class, $format, $context);
    }

    /**
     * @param SettingsProvider $settingsProvider
     */
    public function setSettingsProvider($settingsProvider)
    {
        $this->settingsProvider = $settingsProvider;
    }

    /**
     * @param PermissionsDictionaryProvider $permissionsProvider
     */
    public function setPermissionsProvider($permissionsProvider)
    {
        $this->permissionsProvider = $permissionsProvider;
    }

    /**
     * @param TokenStorageInterface $tokenStorage
     */
    public function setTokenStorage($tokenStorage)
    {
        $this->tokenStorage = $tokenStorage;
    }

    /**
     * @return Actor
     */
    protected function getCurrentUser()
    {
        if (!$this->user) {
            $token = $this->tokenStorage->getToken();

            $this->user = null;
            if ($token) {
                $user = $token->getUser();
                $this->user = $user;
            }
        }

        return $this->user;
    }
}
