<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ActorsBundle\Serializer;


use Cybersponse\ActorsBundle\Entity\Avatar;
use Cybersponse\ActorsBundle\Provider\AvatarProvider;

class CurrentAvatarNormalizer extends AvatarNormalizer
{
    /** @var Avatar $avatar */
    protected $user;

    /** @var AvatarProvider */
    protected $avatarProvider;

    /**
     * @inheritdoc
     */
    public function supportsNormalization($data, $format = null)
    {
        return self::FORMAT === $format && ($data instanceof Avatar) && $data === $this->getCurrentAvatar();
    }

    /**
     * @inheritdoc
     */
    public function supportsDenormalization($data, $type, $format = null)
    {
        return is_subclass_of($type, Avatar::class);
    }

    /**
     * @param Avatar $avatar
     * @inheritdoc
     */
    public function normalize($avatar, $format = null, array $context = [])
    {
        // Symfony's normalizer will create a cache of Class to Normalizer meaning it will always choose this one.
        if ($avatar !== $this->getCurrentAvatar()) {
            return parent::normalize($avatar, $format, $context);
        }

        $context['ignore_authorization'] = true;
        $context['ignore_field_authorization'] = true;
        $context['ignore_relationship_authorization'] = false;

        return parent::normalize($avatar, $format, $context);
    }

    public function denormalize($data, $class, $format = null, array $context = [])
    {
        if (array_key_exists('object_to_populate', $context) && $context['object_to_populate'] === $this->getCurrentAvatar()) {
            $context['ignore_field_authorization'] = true;
        }

        return parent::denormalize($data, $class, $format, $context);
    }

    /**
     * @param AvatarProvider $avatarProvider
     */
    public function setAvatarProvider($avatarProvider)
    {
        $this->avatarProvider = $avatarProvider;
    }

    /**
     * @return Avatar
     */
    protected function getCurrentAvatar()
    {
        return $this->avatarProvider->getDataForCurrentUser();
    }
}
