<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ActorsBundle\Subscriber;


use Cybersponse\ActorsBundle\Entity\Actor;
use Cybersponse\AuthorizationBundle\AuthorizationProvider;
use Cybersponse\AuthorizationBundle\Entity\Role;
use Cybersponse\AuthorizationBundle\Entity\Team;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Doctrine\ORM\PersistentCollection;
use Dunglas\ApiBundle\Api\ResourceCollectionInterface;
use Dunglas\ApiBundle\Event\Events;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class ActorSecuritySubscriber implements EventSubscriberInterface
{
    /** @var AuthorizationProvider */
    protected $authorizationProvider;

    /** @var ResourceCollectionInterface */
    protected $resourceCollection;

    /**
     * @param AuthorizationProvider $authorizationProvider
     */
    public function setAuthorizationProvider($authorizationProvider)
    {
        $this->authorizationProvider = $authorizationProvider;
    }

    /**
     * @param ResourceCollectionInterface $resourceCollection
     */
    public function setResourceCollection($resourceCollection)
    {
        $this->resourceCollection = $resourceCollection;
    }

    public static function getSubscribedEvents()
    {
        return [
            Events::PRE_UPDATE => ['handlePreUpdate', 999],
        ];
    }

    /**
     * @param DataEvent $event
     * @throws \Exception
     */
    public function handlePreUpdate(DataEvent $event)
    {
        $object = $event->getData();

        if ($object instanceof Actor) {
            // TODO Remove this. This authorization must happen prior to denormalizing the actor!

            /** @var PersistentCollection $roles */
            $roles = $object->getRoles(false);

            if (count($roles->getInsertDiff()) > 0 || count($roles->getDeleteDiff()) > 0) {
                $rolesResource = $this->resourceCollection->getResourceForEntity(Role::class);
                $this->authorizationProvider->denyAccessUnlessGranted($rolesResource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);
            }

            /** @var PersistentCollection $teams */
            $teams = $object->getTeams();

            if (count($teams->getInsertDiff()) > 0 || count($teams->getDeleteDiff()) > 0) {
                $teamsResource = $this->resourceCollection->getResourceForEntity(Team::class);
                $this->authorizationProvider->denyAccessUnlessGranted($teamsResource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);
            }
        }
    }
}
