<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ActorsBundle\Subscriber;


use Cybersponse\ActorsBundle\Entity\Avatar;
use Cybersponse\ActorsBundle\Entity\Actor;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Doctrine\Common\Persistence\ObjectRepository;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityRepository;
use Dunglas\ApiBundle\Api\IriConverter;
use Dunglas\ApiBundle\Event\Events;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class AvatarSubscriber implements EventSubscriberInterface
{
    /** @var EntityManager */
    protected $actorManager;

    /** @var ObjectRepository */
    protected $actorRepository;

    /** @var IriConverter */
    protected $iriConverter;

    public static function getSubscribedEvents()
    {
        return [
            Events::PRE_CREATE => ['handleUpsert', 1000],
            Events::PRE_UPDATE => ['handleUpsert', 1000],
            Events::PRE_DELETE => ['handleDelete', 1000],
        ];
    }

    /**
     * @param DataEvent $event
     * @throws \Exception
     */
    public function handleUpsert(DataEvent $event)
    {
        $avatar = $event->getData();

        if ($avatar instanceof Avatar && $this->actorRepository) {
            $avatarIri = $this->iriConverter->getIriFromItem($avatar);
            $this->associateAvatarToActor($avatar, $avatarIri);
        }
    }

    /**
     * @param DataEvent $event
     * @throws \Exception
     */
    public function handleDelete(DataEvent $event)
    {
        $avatar = $event->getData();

        if ($avatar instanceof Avatar && $this->actorRepository) {
            $this->associateAvatarToActor($avatar, null);
        }
    }

    /**
     * @param EntityRepository $actorManager
     */
    public function setActorManager($actorManager)
    {
        $this->actorManager = $actorManager;
        $this->actorRepository = $this->actorManager->getRepository(Actor::class);
    }

    /**
     * @param IriConverter $iriConverter
     */
    public function setIriConverter($iriConverter)
    {
        $this->iriConverter = $iriConverter;
    }

    /**
     * @param Avatar $avatar
     * @param string $avatarIri
     */
    private function associateAvatarToActor($avatar, $avatarIri)
    {
        $actorId = $avatar->getId();

        /** @var Actor $actor */
        $actor = $this->actorRepository->find($actorId);
        $actor->setAvatar($avatarIri);

        $this->actorManager->persist($actor);
        $this->actorManager->flush();
    }
}
