<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ActorsBundle\Tests\Functional\ActorImplementation;


use Cybersponse\TestBundle\Functional\ApiTestCase;

class CreateItemTest extends ApiTestCase
{
    protected function setUp()
    {
        parent::setUp();
        $this->requestVerb = 'POST';
    }

    public function getRequestUri()
    {
        return '/api/3/mock_actors';
    }

    public function test_GivenInvalidPermissions_ShouldReturn403()
    {
        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $this->assertEquals(403, $response->getStatusCode());
    }

    public function test_GivenInvalidPermissions_ShouldReturnAccessDeniedHydraError()
    {
        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $deserializedContent = json_decode($response->getContent(), true);
        $this->assertEquals($deserializedContent['hydra:description'], 'Access Denied.');
    }

    public function test_GivenValidPermissions_GivenValidRequiredFields_ShouldReturn201()
    {
        $this->setRoles([
            $this->createModulePermission('mock_actors', true)
        ]);

        $this->setJsonRequestContent([
            'userId' => 'asdf'
        ]);

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $this->assertEquals(201, $response->getStatusCode());
    }

    public function test_GivenValidPermissions_GivenValidRequiredFields_ShouldReturnCreatedFields()
    {
        $this->setRoles([
            $this->createModulePermission('mock_actors', true)
        ]);

        $expectedUserId = 'asdf';
        $this->setJsonRequestContent([
            'userId' => $expectedUserId
        ]);

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $deserializedContent = json_decode($response->getContent(), true);
        $this->assertEquals($deserializedContent['userId'], $expectedUserId);
    }

    public function test_GivenValidPermissions_GivenValidRequiredFields_GivenRelationshipsTrue_ShouldReturnCreatedFields()
    {
        $this->setRoles([
            $this->createModulePermission('mock_actors', true)
        ]);

        $this->setJsonRequestContent([
            'userId' => 'asdf'
        ]);

        $this->requestParameters = ['$relationships' => true];

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $deserializedContent = json_decode($response->getContent(), true);
        $this->assertArrayHasKey('teams', $deserializedContent);
    }

    public function test_GivenValidPermissions_GivenValidRequiredFields_GivenValidItemId_ShouldReturnIriInId()
    {
        $this->setRoles([
            $this->createModulePermission('mock_actors', true)
        ]);

        $expectedUuid = 'TRUST-ME-I-AM-VALID';
        $this->setJsonRequestContent([
            'uuid' => $expectedUuid,
            'userId' => 'asdf'
        ]);

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $deserializedContent = json_decode($response->getContent(), true);
        $expectedIri = sprintf('%s/%s', $this->getRequestUri(), $expectedUuid);
        $this->assertEquals($deserializedContent['@id'], $expectedIri);
    }
}