<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ActorsBundle\Tests\Functional\ActorImplementation;


use Cybersponse\TestBundle\Functional\ApiTestCase;

class ReadCollectionTest extends ApiTestCase
{
    protected function setUp()
    {
        parent::setUp();
        $this->requestVerb = 'GET';
    }

    public function getRequestUri()
    {
        return '/api/3/mock_actors';
    }

    public function test_GivenInvalidPermissions_ShouldReturn403()
    {
        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $this->assertEquals(403, $response->getStatusCode());
    }

    public function test_GivenInvalidPermissions_ShouldReturnAccessDeniedHydraError()
    {
        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $deserializedContent = json_decode($response->getContent(), true);
        $this->assertEquals($deserializedContent['hydra:description'], 'Access Denied.');
    }

    public function test_GivenValidPermissions_ShouldReturn200()
    {
        $this->setRoles([
            $this->createModulePermission('mock_actors', false, true)
        ]);

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $this->assertEquals(200, $response->getStatusCode());
    }

    public function test_GivenValidPermissions_GivenRelationshipsTrue_ShouldReturnRelationshipFields()
    {
        $this->setRoles([
            $this->createModulePermission('mock_actors', false, true)
        ]);

        $this->requestParameters = ['$relationships' => true];

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $deserializedContent = json_decode($response->getContent(), true);
        $firstDeserializedItem = current($deserializedContent['hydra:member']);
        $this->assertArrayHasKey('teams', $firstDeserializedItem);
    }
}