<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ActorsBundle\Tests\Functional\ActorImplementation;


use Cybersponse\TestBundle\Functional\ApiTestCase;

class ReadItemTest extends ApiTestCase
{
    const NON_EXISTENT_MOCK_ACTOR_UUID = 'non-existent-uuid-brah';
    const VALID_MOCK_ACTOR_UUID = '608c6815-9300-44c8-9d7d-28dc97ded44e';

    /** @var string */
    protected $itemId = self::NON_EXISTENT_MOCK_ACTOR_UUID;

    protected function setUp()
    {
        parent::setUp();
        $this->requestVerb = 'GET';
    }

    public function getRequestUri()
    {
        return sprintf('/api/3/mock_actors/%s', $this->itemId);
    }

    public function test_GivenInvalidPermissions_ShouldReturn403()
    {
        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $this->assertEquals(403, $response->getStatusCode());
    }

    public function test_GivenInvalidPermissions_ShouldReturnAccessDeniedHydraError()
    {
        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $deserializedContent = json_decode($response->getContent(), true);
        $this->assertEquals($deserializedContent['hydra:description'], 'Access Denied.');
    }

    public function test_GivenValidPermissions_GivenNonexistentItemId_ShouldReturn404()
    {
        $this->itemId = self::NON_EXISTENT_MOCK_ACTOR_UUID;

        $this->setRoles([
            $this->createModulePermission('mock_actors', false, true)
        ]);

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $this->assertEquals(404, $response->getStatusCode());
    }

    public function test_GivenValidPermissions_GivenNonexistentItemId_ShouldReturnNotFoundHydraError()
    {
        $this->itemId = self::NON_EXISTENT_MOCK_ACTOR_UUID;

        $this->setRoles([
            $this->createModulePermission('mock_actors', false, true)
        ]);

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $deserializedContent = json_decode($response->getContent(), true);
        $this->assertEquals($deserializedContent['hydra:description'], 'Not Found');
    }

    public function test_GivenValidPermissions_GivenValidItemId_ShouldReturn200()
    {
        $this->itemId = self::VALID_MOCK_ACTOR_UUID;

        $this->setRoles([
            $this->createModulePermission('mock_actors', false, true)
        ]);

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $this->assertEquals(200, $response->getStatusCode());
    }

    public function test_GivenValidPermissions_GivenValidItemId_ShouldReturnIriInId()
    {
        $this->itemId = self::VALID_MOCK_ACTOR_UUID;

        $this->setRoles([
            $this->createModulePermission('mock_actors', false, true)
        ]);

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $deserializedContent = json_decode($response->getContent(), true);
        $this->assertEquals($deserializedContent['@id'], $this->getRequestUri());
    }

    public function test_GivenValidPermissions_GivenValidItemId_GivenRelationshipsTrue_ShouldReturnRelationshipFields()
    {
        $this->itemId = self::VALID_MOCK_ACTOR_UUID;

        $this->setRoles([
            $this->createModulePermission('mock_actors', false, true)
        ]);

        $this->requestParameters = ['$relationships' => true];

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $deserializedContent = json_decode($response->getContent(), true);
        $this->assertArrayHasKey('teams', $deserializedContent);
    }
}
