<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ActorsBundle\Tests\Functional;


use Cybersponse\TestBundle\Functional\ApiTestCase;

class ActorsControllerTest extends ApiTestCase
{
    protected function setUp()
    {
        parent::setUp(); // TODO: Change the autogenerated stub
        $this->requestUri = '/api/3/actors/current';
    }

    public function test_ShouldReturn200()
    {
        $this->executeSystemUnderTest();
        $response = $this->client->getResponse();

        $this->assertEquals(200, $response->getStatusCode());
    }

    public function test_GivenSpecificModulePermission_ShouldReturnModuleInPermissions()
    {
        $expectedModuleName = 'awesome';
        $expectedPermission = $this->createModulePermission($expectedModuleName);

        $this->setRoles([
            $expectedPermission
        ]);

        $this->executeSystemUnderTest();
        $response = $this->client->getResponse();

        $currentActor = json_decode($response->getContent(), true);
        $permissions = $currentActor['@permissions'];
        $this->assertArrayHasKey($expectedModuleName, $permissions);
    }

    public function test_GivenSpecificPermissionAction_ShouldReturnActionInPermissions()
    {
        $expectedModuleName = 'awesome';

        $this->setRoles([
            $this->createModulePermission($expectedModuleName, true)
        ]);

        $this->executeSystemUnderTest();
        $response = $this->client->getResponse();

        $currentActor = json_decode($response->getContent(), true);
        $permissions = $currentActor['@permissions'];
        $this->assertTrue($permissions[$expectedModuleName]['create']);
    }
}