<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthenticationBundle\Hmac;

use Cybersponse\ProxyBundle\Provider\FingerprintProvider;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Security\Core\Authentication\Provider\AuthenticationProviderInterface;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Exception\AuthenticationException;
use Symfony\Component\Security\Core\User\UserProviderInterface;

class HmacProvider implements AuthenticationProviderInterface
{
    /** @var RequestStack */
    protected $requestStack;

    /** @var DasHmacSecretKeyProvider */
    protected $dasHmacSecretKeyProvider;

    /** @var UserProviderInterface */
    protected $userProvider;

    /**
     * @param UserProviderInterface $userProvider
     */
    public function __construct(UserProviderInterface $userProvider)
    {
        $this->userProvider = $userProvider;
    }

    /**
     * @param DasHmacSecretKeyProvider $dasHmacSecretKeyProvider
     */
    public function setDasHmacSecretKeyProvider($dasHmacSecretKeyProvider)
    {
        $this->dasHmacSecretKeyProvider = $dasHmacSecretKeyProvider;
    }

    /**
     * @param RequestStack $requestStack
     */
    public function setRequestStack($requestStack)
    {
        $this->requestStack = $requestStack;
    }

    /**
     * {@inheritdoc}
     */
    public function authenticate(TokenInterface $token)
    {
        if (!$this->validateHmacFingerprint($token->getCredentials())) {
            throw new AuthenticationException('Invalid HMAC Fingerprint');
        }

        $userId = $this->dasHmacSecretKeyProvider->getUserId();
        $user = $this->userProvider->loadUserByUsername($userId);

        $authToken = new HmacUserToken($user->getRoles());
        $authToken->setUser($user);
        $authToken->setPayload($token->getCredentials());

        return $authToken;
    }

    /**
     * {@inheritdoc}
     */
    public function supports(TokenInterface $token)
    {
        return $token instanceof HmacUserToken;
    }

    /**
     * @param string $hmacFingerprint
     * @return bool
     */
    protected function validateHmacFingerprint($hmacFingerprint)
    {
        $request = $this->requestStack->getCurrentRequest();
        $success = FingerprintProvider::validateFingerprint($hmacFingerprint, $this->dasHmacSecretKeyProvider, $request);
        return $success;
    }
}
