<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthenticationBundle\Jwt;


use Namshi\JOSE\JWS;

/**
 * Class CorrectSimpleJWS
 *
 * This class is a copy of the SimpleJWS without the 'iat' property.
 *
 * @package Cybersponse\AuthenticationBundle\Jwt
 */
class CorrectSimpleJWS extends JWS
{

    /**
     * Checks that the JWS has been signed with a valid private key by verifying it with a public $key
     * and the token is not expired.
     *
     * @param resource|string $key
     * @param string $algo The algorithms this JWS should be signed with. Use it if you want to restrict which algorithms you want to allow to be validated.
     *
     * @return bool
     */
    public function isValid($key, $algo = null)
    {
        $payload = $this->getPayload();
        return $this->verify($key, $algo) && !$this->isExpired($payload) && !$this->twoFactorToken($payload);
    }


    public function twoFactorToken($payload) {
        if (isset($payload['status']) && $payload['status']=='2fa') {
            return true;
        }
        return false;

    }

    /**
     * @param $payload
     * @return bool
     */
    public function isExpired($payload)
    {
        if (isset($payload['exp']) && is_numeric($payload['exp'])) {
            $now = new \DateTime('now');
            return ($now->format('U') - $payload['exp']) > 0;
        }
        return false;
    }
}
