<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Controller;


use Doctrine\Common\Util\Inflector;

use Dunglas\ApiBundle\JsonLd\Response;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\Serializer\Serializer;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Dunglas\ApiBundle\Controller\ResourceController;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Cybersponse\AuthorizationBundle\AuthorizationProvider;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

class UserRoleTeamController extends ResourceController
{
   
    /**
     * @param string $moduleName
     * @param string $action
     * @return JsonResponse
     */
    public function getAction(Request $request)
    {
        $authorizationChecker = $this->container->get('security.authorization_checker');
        if (!$authorizationChecker->isGranted(['perm:application.read'])) {
            throw new AccessDeniedException();
        }
        $teams = [];
        $request->query->set("\$relationships", true);
        $resource = $this->identifyResourceByShortName('people');
        $normalizedTeamData = $this->get('serializer')->normalize($this->getUser(), 'json-ld', $resource->getNormalizationContext() + ['ignore_authorization' => true, 'ignore_field_authorization' => true, 'ignore_relationship_authorization' => false]);

        $data = [];
        $iriConverter = $this->container->get('api.iri_converter');
        foreach ($normalizedTeamData['teams'] as $key => $value) {
            array_push($teams, $iriConverter->getItemFromIri($value));
        }

        $normalizedData = $this->get('serializer')->normalize(
            $teams,
            'json-ld',
            $resource->getNormalizationContext() + ['ignore_authorization' => true, 'ignore_field_authorization' => true, 'ignore_relationship_authorization' => false]
        );
        $data["teams"] = $normalizedData["hydra:member"];
        $data["roles"] = [];
        foreach ($normalizedTeamData['roles'] as $key => $value) {
            $roleItem = $iriConverter->getItemFromIri($value);
            $normalizedData = $this->get('serializer')->normalize(
                $roleItem,
                'json-ld',
                $resource->getNormalizationContext() + ['ignore_authorization' => true, 'ignore_field_authorization' => true, 'ignore_relationship_authorization' => false]
            );
            $normalizedData['modulePermissions'] = [];
            array_push($data["roles"], $normalizedData);
        }
        $resource = $this->identifyResourceByShortName('people');
        return $this->getSuccessResponse($resource, $data, Response::HTTP_OK, [], ['request_uri' => $request->getRequestUri()]);
    }


    /**
     * @param ResourceInterface $resource
     * @param string|array $actions
     * @param object|null $object
     */
    protected function denyAccessUnlessGranted(ResourceInterface $resource, $actions, $object = null)
    {
        /** @var AuthorizationProvider $authorizationProvider */
        $authorizationProvider = $this->get('cybersponse.authorization.authorization_provider');
        $authorizationProvider->denyAccessUnlessGranted($resource, $actions, $object);
    }

    /**
     * @param $moduletype
     * @return null
     */
    public function identifyResourceByShortName($moduleType)
    {
        $resourceClassName = Inflector::singularize(Inflector::classify($moduleType));
        $resource = $this->get('api.resource_collection')->getResourceForShortName($resourceClassName);
        if ($resource) {
            return $resource;
        }
        $resourceClassName = Inflector::classify($moduleType);
        $resource = $this->get('api.resource_collection')->getResourceForShortName($resourceClassName);
        if ($resource) {
            return $resource;
        }
        $model = $this->getClassNameByType($moduleType);
        $singular = $model->getDescriptions()['singular'];
        $escapedSingular = preg_replace('/[^a-z0-9\s]+/i', '', $singular);
        $resourceName = \Doctrine\Common\Util\Inflector::classify($escapedSingular);
        $resourceCollection = $this->container->get('api.resource_collection');
        return $resourceCollection->getResourceForShortName($resourceName);
    }

}
