<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Entity;


use Cybersponse\CoreBundle\Entity\BaseEntity;
use Cybersponse\CoreBundle\Entity\Module;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;

/**
 * ModulePermission
 *
 * @Doctrine\ORM\Mapping\Table(name="module_permissions")
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="module_permissions")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="security")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ action | uppercase }} {{ module.name }}")
 */
class ModulePermission extends BaseEntity
{
    /**
     * @var boolean
     *
     * @Doctrine\ORM\Mapping\Column(name="canCreate", type="boolean", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     */
    protected $canCreate = false;

    /**
     * @var boolean
     *
     * @Doctrine\ORM\Mapping\Column(name="canExecute", type="boolean", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     */
    protected $canExecute = false;

    /**
     * @var boolean
     *
     * @Doctrine\ORM\Mapping\Column(name="canRead", type="boolean", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     */
    protected $canRead = false;

    /**
     * @var boolean
     *
     * @Doctrine\ORM\Mapping\Column(name="canUpdate", type="boolean", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     */
    protected $canUpdate = false;

    /**
     * @var boolean
     *
     * @Doctrine\ORM\Mapping\Column(name="canDelete", type="boolean", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     */
    protected $canDelete = false;

    /**
     * @var Module
     *
     * @Doctrine\ORM\Mapping\ManyToOne(targetEntity="Cybersponse\CoreBundle\Entity\Module", cascade={"persist"})
     * @Doctrine\ORM\Mapping\JoinColumns({
     *   @Doctrine\ORM\Mapping\JoinColumn(name="module_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     * })
     * @Normalize()
     * @Denormalize()
     */
    protected $module;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="Cybersponse\AuthorizationBundle\Entity\FieldPermission", cascade={"persist", "remove"})
     * @Doctrine\ORM\Mapping\JoinTable(name="modulepermission_fieldpermission",
     *   joinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="modulepermission_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   },
     *   inverseJoinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="fieldpermission_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   }
     * )
     * @Normalize()
     * @Denormalize()
     */
    protected $fieldPermissions;


    function __construct()
    {
        $this->fieldPermissions = new ArrayCollection();
    }

    /**
     * Set canCreate
     *
     * @param boolean $canCreate
     * @return ModulePermission
     */
    public function setCanCreate($canCreate)
    {
        $this->canCreate = $canCreate;

        return $this;
    }

    /**
     * Get canCreate
     *
     * @return boolean
     */
    public function getCanCreate()
    {
        return $this->canCreate;
    }

    /**
     * Set canExecute
     *
     * @param boolean $canExecute
     * @return ModulePermission
     */
    public function setCanExecute($canExecute)
    {
        $this->canExecute = $canExecute;

        return $this;
    }

    /**
     * Get canExecute
     *
     * @return boolean
     */
    public function getCanExecute()
    {
        return $this->canExecute;
    }

    /**
     * Set canRead
     *
     * @param boolean $canRead
     * @return ModulePermission
     */
    public function setCanRead($canRead)
    {
        $this->canRead = $canRead;

        return $this;
    }

    /**
     * Get canRead
     *
     * @return boolean
     */
    public function getCanRead()
    {
        return $this->canRead;
    }

    /**
     * Set canUpdate
     *
     * @param boolean $canUpdate
     * @return ModulePermission
     */
    public function setCanUpdate($canUpdate)
    {
        $this->canUpdate = $canUpdate;

        return $this;
    }

    /**
     * Get canUpdate
     *
     * @return boolean
     */
    public function getCanUpdate()
    {
        return $this->canUpdate;
    }

    /**
     * Set canDelete
     *
     * @param boolean $canDelete
     * @return ModulePermission
     */
    public function setCanDelete($canDelete)
    {
        $this->canDelete = $canDelete;

        return $this;
    }

    /**
     * Get canDelete
     *
     * @return boolean
     */
    public function getCanDelete()
    {
        return $this->canDelete;
    }

    /**
     * Set module
     *
     * @param Module $module
     * @return ModulePermission
     */
    public function setModule(Module $module = null)
    {
        $this->module = $module;

        return $this;
    }

    /**
     * Get module
     *
     * @return Module
     */
    public function getModule()
    {
        return $this->module;
    }

    /**
     * Add fieldPermissions
     *
     * @param FieldPermission $fieldPermission
     * @return ModulePermission
     */
    public function addFieldPermission(FieldPermission $fieldPermission)
    {
        $this->fieldPermissions[] = $fieldPermission;

        return $this;
    }

    /**
     * Remove fieldPermissions
     *
     * @param FieldPermission $fieldPermission
     */
    public function removeFieldPermission(FieldPermission $fieldPermission)
    {
        $this->fieldPermissions->removeElement($fieldPermission);
    }

    /**
     * Get fieldPermissions
     *
     * @return \Doctrine\Common\Collections\Collection|FieldPermission[]
     */
    public function getFieldPermissions()
    {
        return $this->fieldPermissions;
    }
}
