<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Entity;


use Cybersponse\CoreBundle\Entity\BaseEntity;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Security\Core\Role\RoleInterface;

/**
 * Role
 *
 * @Doctrine\ORM\Mapping\Table(name="roles")
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="roles")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="security")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ name }}")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="Role",plural="Roles")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DefaultSort\DefaultSort(defaultSort={{"field":"name","direction":"asc"}})
 */
class Role extends BaseEntity implements RoleInterface
{
    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="description", type="text", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Description")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="textarea")
     */
    protected $description;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="name", type="string", unique=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Name")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     * @Symfony\Component\Validator\Constraints\NotBlank()
     */
    protected $name;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="Cybersponse\AuthorizationBundle\Entity\ModulePermission", cascade={"persist", "remove"})
     * @Doctrine\ORM\Mapping\JoinTable(name="role_modulepermission",
     *   joinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="role_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   },
     *   inverseJoinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="modulepermission_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   }
     * )
     * @Normalize()
     * @Denormalize()
     */
    protected $modulePermissions;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->modulePermissions = new \Doctrine\Common\Collections\ArrayCollection();
    }

    /**
     * Returns the role.
     *
     * This method returns a string representation whenever possible.
     *
     * When the role cannot be represented with sufficient precision by a
     * string, it should return null.
     *
     * @return string|null A string representation of the role, or null
     */
    public function getRole()
    {
        return 'ROLE_' . $this->getName();
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set description
     *
     * @param string $description
     * @return Role
     */
    public function setDescription($description)
    {
        $this->description = $description;

        return $this;
    }

    /**
     * Get description
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Add modulePermissions
     *
     * @param \Cybersponse\AuthorizationBundle\Entity\ModulePermission $modulePermission
     * @return Role
     */
    public function addModulePermission(\Cybersponse\AuthorizationBundle\Entity\ModulePermission $modulePermission)
    {
        $this->modulePermissions[] = $modulePermission;

        return $this;
    }

    /**
     * Remove modulePermissions
     *
     * @param \Cybersponse\AuthorizationBundle\Entity\ModulePermission $modulePermission
     */
    public function removeModulePermission(\Cybersponse\AuthorizationBundle\Entity\ModulePermission $modulePermission)
    {
        $this->modulePermissions->removeElement($modulePermission);
    }

    /**
     * Get modulePermissions
     *
     * @return \Doctrine\Common\Collections\Collection|ModulePermission[]
     */
    public function getModulePermissions()
    {
        return $this->modulePermissions;
    }
}
