<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Entity;


use Cybersponse\CoreBundle\Entity\BaseEntity;
use Cybersponse\ActorsBundle\Entity\Actor;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize;
use Doctrine\ORM\Mapping as ORM;

/**
 * Team
 *
 * @Doctrine\ORM\Mapping\Table(name="teams")
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="teams")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="security")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ name }}")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DefaultSort\DefaultSort(defaultSort={{"field":"name","direction":"asc"}})
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="Team",plural="Teams")
 * @Cybersponse\AuthorizationBundle\Validation\NotSelfReferentialConstraint()
 * @Cybersponse\AuthorizationBundle\Validation\NonIntersectingCollectionFieldsConstraint()
 */
class Team extends BaseEntity
{
    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="description", type="text", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Description")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="textarea")
     */
    protected $description;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="name", type="string", unique=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Team Name")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     * @Symfony\Component\Validator\Constraints\NotBlank()
     */
    protected $name;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Actors")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="actors")
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="Cybersponse\ActorsBundle\Entity\Actor", inversedBy="teams", cascade={"persist"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="manyToMany")
     * @Doctrine\ORM\Mapping\JoinTable(name="team_actor",
     *   joinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="team_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   },
     *   inverseJoinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="actor_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   }
     * )
     */
    protected $actors;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="Cybersponse\AuthorizationBundle\Entity\Team", mappedBy="children", cascade={"persist"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="manyToMany")
     */
    protected $parents;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="Cybersponse\AuthorizationBundle\Entity\Team", inversedBy="siblings", cascade={"persist"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="manyToMany")
     * @Doctrine\ORM\Mapping\JoinTable(name="team_sibling",
     *   joinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="sibling_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   },
     *   inverseJoinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="team_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   }
     * )
     */
    protected $siblings;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="Cybersponse\AuthorizationBundle\Entity\Team", inversedBy="parents", cascade={"persist"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="manyToMany")
     * @Doctrine\ORM\Mapping\JoinTable(name="team_child",
     *   joinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="child_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   },
     *   inverseJoinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="team_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   }
     * )
     */
    protected $children;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->actors = new \Doctrine\Common\Collections\ArrayCollection();
        $this->parents = new \Doctrine\Common\Collections\ArrayCollection();
        $this->siblings = new \Doctrine\Common\Collections\ArrayCollection();
        $this->children = new \Doctrine\Common\Collections\ArrayCollection();
    }

    /**
     * Set name
     *
     * @param string $name
     * @return Team
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set description
     *
     * @param string $description
     * @return Team
     */
    public function setDescription($description)
    {
        $this->description = $description;

        return $this;
    }

    /**
     * Get description
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Add actors
     *
     * @param \Cybersponse\ActorsBundle\Entity\Actor $actor
     * @return Team
     */
    public function addActor(Actor $actor)
    {
        if (!$this->actors->contains($actor)) {
            $this->actors[] = $actor;
        }

        return $this;
    }

    /**
     * Remove actors
     *
     * @param \Cybersponse\ActorsBundle\Entity\Actor $actor
     */
    public function removeActor(Actor $actor)
    {
        $this->actors->removeElement($actor);
    }

    /**
     * Get actors
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getActors()
    {
        return $this->actors;
    }

    /**
     * Add parent
     *
     * @param Team $parent
     * @return Team
     */
    public function addParent(Team $parent)
    {
        if (!$this->parents->contains($parent)) {
            $this->parents[] = $parent;
        }

        if (!in_array($this, $parent->getChildren()->toArray())) {
            $parent->addChild($this);
        }

        return $this;
    }

    /**
     * Remove parent
     *
     * @param Team $parent
     */
    public function removeParent(Team $parent)
    {
        $this->parents->removeElement($parent);

        if (in_array($this, $parent->getChildren()->toArray())) {
            $parent->removeChild($this);
        }
    }

    /**
     * Get parents
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getParents()
    {
        return $this->parents;
    }

    /**
     * Add sibling
     *
     * @param Team $sibling
     * @return Team
     */
    public function addSibling(Team $sibling)
    {
        if (!$this->siblings->contains($sibling)) {
            $this->siblings[] = $sibling;
        }

        if (!in_array($this, $sibling->getSiblings()->toArray())) {
            $sibling->addSibling($this);
        }

        return $this;
    }

    /**
     * Remove sibling
     *
     * @param Team $sibling
     */
    public function removeSibling(Team $sibling)
    {
        $this->siblings->removeElement($sibling);

        if (in_array($this, $sibling->getSiblings()->toArray())) {
            $sibling->removeSibling($this);
        }
    }

    /**
     * Get siblings
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getSiblings()
    {
        return $this->siblings;
    }

    /**
     * Add child
     *
     * @param Team $child
     * @return Team
     */
    public function addChild(Team $child)
    {
        if (!$this->children->contains($child)) {
            $this->children[] = $child;
        }

        if (!in_array($this, $child->getParents()->toArray())) {
            $child->addParent($this);
        }

        return $this;
    }

    /**
     * Remove child
     *
     * @param Team $child
     */
    public function removeChild(Team $child)
    {
        $this->children->removeElement($child);

        if (in_array($this, $child->getParents()->toArray())) {
            $child->removeParent($this);
        }
    }

    /**
     * Get children
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getChildren()
    {
        return $this->children;
    }
}
