<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Listener;


use Cybersponse\AuthorizationBundle\Ownership\OwnershipProvider;
use Cybersponse\AuthorizationBundle\Ownership\TeamsProvider;
use Cybersponse\AuthorizationBundle\Traits\ConditionalOwnable;
use Doctrine\Common\Persistence\ManagerRegistry;
use Dunglas\ApiBundle\Api\IriConverterInterface;
use Dunglas\ApiBundle\Event\DataEvent;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

class AssignConditionalOwnershipListener
{
    /** @var ManagerRegistry */
    protected $managerRegistry;

    /** @var OwnershipProvider */
    protected $ownershipProvider;

    /** @var TeamsProvider */
    protected $teamProvider;

    /** @var IriConverterInterface */
    protected $iriConverter;

    /**
     * @param OwnershipProvider $ownershipProvider
     */
    public function setOwnershipProvider(OwnershipProvider $ownershipProvider)
    {
        $this->ownershipProvider = $ownershipProvider;
    }

    /**
     * @param TeamsProvider $teamProvider
     */
    public function setTeamProvider($teamProvider)
    {
        $this->teamProvider = $teamProvider;
    }

    /**
     * @param IriConverterInterface $iriConverter
     */
    public function setIriConverter($iriConverter)
    {
        $this->iriConverter = $iriConverter;
    }

    /**
     * @param ManagerRegistry $managerRegistry
     */
    public function setManagerRegistry($managerRegistry)
    {
        $this->managerRegistry = $managerRegistry;
    }

    public function add_owners_to_record(DataEvent $event)
    {
        $resourceClass = $event->getResource()->getEntityClass();
        $managerClass = $this->managerRegistry->getManagerForClass($resourceClass);
        if (is_null($managerClass)) {
            return;
        }
        if ($this->ownershipProvider->isClassConditionalOwnable($resourceClass)) {
            /** @var ConditionalOwnable $item */
            $item = $event->getData();
            $owners = $item->getOwners() ? $item->getOwners(): [];
            $is_private = $item->getIsPrivate() ? $item->getIsPrivate(): false;

            if (count($owners) < 1 && $is_private) {
                $teams = $this->teamProvider->getDataForCurrentUser();
                if (count($teams) === 0) {
                    throw new AccessDeniedException('You cannot add zero owner to a record. A record must always have at least one owner.');
                }
                foreach ($teams as $team) {
                    $item->addOwner($team);
                }
            } elseif (count($owners) > 0 && $is_private == false) {
                foreach ($owners as $team) {
                    $item->removeOwner($team);
                }
            }
        }
    }

    /**
     * @param DataEvent $event
     */
    public function onBeforeCreate(DataEvent $event)
    {
        $this->add_owners_to_record($event);
    }

    /**
     * @param DataEvent $event
     */
    public function onBeforeUpdate(DataEvent $event)
    {
        $this->add_owners_to_record($event);
    }
}
