<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Listener;


use Cybersponse\AuthorizationBundle\Ownership\OwnershipProvider;
use Cybersponse\AuthorizationBundle\Ownership\TeamsProvider;
use Cybersponse\AuthorizationBundle\Traits\Ownable;
use Doctrine\Common\Persistence\ManagerRegistry;
use Dunglas\ApiBundle\Api\IriConverterInterface;
use Dunglas\ApiBundle\Event\DataEvent;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

class AssignOwnershipListener
{
    /** @var ManagerRegistry */
    protected $managerRegistry;

    /** @var OwnershipProvider */
    protected $ownershipProvider;

    /** @var TeamsProvider */
    protected $teamProvider;

    /** @var IriConverterInterface */
    protected $iriConverter;

    /**
     * @param OwnershipProvider $ownershipProvider
     */
    public function setOwnershipProvider(OwnershipProvider $ownershipProvider)
    {
        $this->ownershipProvider = $ownershipProvider;
    }

    /**
     * @param TeamsProvider $teamProvider
     */
    public function setTeamProvider($teamProvider)
    {
        $this->teamProvider = $teamProvider;
    }

    /**
     * @param IriConverterInterface $iriConverter
     */
    public function setIriConverter($iriConverter)
    {
        $this->iriConverter = $iriConverter;
    }

    /**
     * @param ManagerRegistry $managerRegistry
     */
    public function setManagerRegistry($managerRegistry)
    {
        $this->managerRegistry = $managerRegistry;
    }

    /**
     * @param DataEvent $event
     */
    public function onBeforeCreate(DataEvent $event)
    {
        $resourceClass = $event->getResource()->getEntityClass();

        $managerClass = $this->managerRegistry->getManagerForClass($resourceClass);

        if (is_null($managerClass)) {
            return;
        }

        if ($this->ownershipProvider->isClassOwnable($resourceClass)) {
            /** @var Ownable $item */
            $item = $event->getData();
            $owners = $item->getOwners();
            // TODO: This allows users to create things that are owned by other teams. Not sure if this is ideal!
            if ($owners == null || count($owners) < 1) {
                $teams = $this->teamProvider->getDataForCurrentUser();
                if (count($teams) === 0) {
                    throw new AccessDeniedException('A user must be on at least one team in order to create records.');
                }
                foreach ($teams as $team) {
                    $item->addOwner($team);
                }
            }
        }
    }
}
