<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Listener;


use Cybersponse\AuthorizationBundle\Traits\UserOwnable;
use Doctrine\Common\Persistence\Event\LifecycleEventArgs;
use Doctrine\Common\Persistence\ManagerRegistry;
use Dunglas\ApiBundle\Event\DataEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class UserOwnableListener
{

    /** @var ManagerRegistry */
    protected $managerRegistry;

    /** @var TokenStorageInterface */
    protected $tokenStorage;

    /** @var ContainerInterface */
    protected $container;

    /**
     * @param TokenStorageInterface $tokenStorage
     */
    public function setTokenStorage($tokenStorage)
    {
        $this->tokenStorage = $tokenStorage;
    }

    /**
     * @param ContainerInterface $container
     */
    public function setContainer($container)
    {
        $this->container = $container;
    }

    /**
     * @param ManagerRegistry $managerRegistry
     */
    public function setManagerRegistry($managerRegistry)
    {
        $this->managerRegistry = $managerRegistry;
    }

    public function prePersist(LifecycleEventArgs $eventArgs)
    {
        if (in_array(UserOwnable::class, class_uses($object = $eventArgs->getObject()))) {
            /** @var UserOwnable $object */
            $this->handleEvent($object);
        }
    }

    /**
     * @param UserOwnable $object
     */
    protected function handleEvent($object)
    {
        if ($token = $this->tokenStorage->getToken()) {
            if ($user = $token->getUser()) {
                $iriConverter = $this->container->get('api.iri_converter');
                $userIri = $iriConverter->getIriFromItem($user);
                $object->setAssignee($userIri);
            }
        }
    }
}
