<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Ownership;


use Cybersponse\ActorsBundle\Entity\Actor;
use Cybersponse\AuthorizationBundle\Entity\Team;
use Cybersponse\CoreBundle\Provider\CacheableActorDataProvider;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;

class AccessibleTeamIdsProvider extends CacheableActorDataProvider
{

    public function appendChildTeams($accessibleTeamIds, $childTeam)
    {
        $accessibleTeamIds[$childTeam->getUuid()] = true;
        return $accessibleTeamIds;
    }

    public function getAllChild($childTeams, $accessibleTeamIds)
    {
        foreach ($childTeams as $childTeam) {
            if (array_key_exists($childTeam->getUuid(), $accessibleTeamIds) == false) {
                $accessibleTeamIds = $this->appendChildTeams($accessibleTeamIds, $childTeam);
                if (count($childTeam->getChildren())) {
                    $accessibleTeamIds = $this->getAllChild($childTeam->getChildren(), $accessibleTeamIds);
                    return $accessibleTeamIds;
                }
            }
        }
        return $accessibleTeamIds;
    }

    public function getAllParent($parentTeams, $accessibleTeamIds)
    {
        foreach ($parentTeams as $parentTeam) {
            if (array_key_exists($parentTeam->getUuid(), $accessibleTeamIds) == false) {
                $accessibleTeamIds = $this->appendChildTeams($accessibleTeamIds, $parentTeam);
                if (count($parentTeam->getParents())) {
                    $accessibleTeamIds = $this->getAllParent($parentTeam->getParents(), $accessibleTeamIds);
                    return $accessibleTeamIds;
                }
            }
        }
        return $accessibleTeamIds;
    }

    /**
     * @return array
     */
    public function getParentDataForTeam($teams)
    {
        $accessibleTeamIds = [];
        foreach ($teams as $team) {
            $accessibleTeamIds[$team->getUuid()] = true;

            /** @var Team[] $parentTeams */
            $parentTeams = $team->getParents();
            $accessibleTeamIds = $this->getAllParent($parentTeams, $accessibleTeamIds);

            /** @var Team[] $siblingTeams */
            $siblingTeams = $team->getSiblings();
            foreach ($siblingTeams as $siblingTeam) {
                $accessibleTeamIds[$siblingTeam->getUuid()] = true;
            }
        }
        return array_keys($accessibleTeamIds);
    }

    /**
     * @inheritdoc
     * @return array
     */
    public function getDataForActor(Actor $actor)
    {
        $username = $actor->getUsername();

        if ($this->cache->cacheContains($username)) {
            $accessibleTeamIds = $this->cache->getFromCache($username);
        } else {
            $accessibleTeamIds = [];
            /** @var Team[] $teams */
            $teams = $actor->getTeams()->toArray();

            foreach ($teams as $team) {
                $accessibleTeamIds[$team->getUuid()] = true;

                /** @var Team[] $childTeams */
                $childTeams = $team->getChildren();
                $accessibleTeamIds = $this->getAllChild($childTeams, $accessibleTeamIds);

                /** @var Team[] $siblingTeams */
                $siblingTeams = $team->getSiblings();
                foreach ($siblingTeams as $siblingTeam) {
                    $accessibleTeamIds[$siblingTeam->getUuid()] = true;
                }
            }

            $accessibleTeamIds = array_keys($accessibleTeamIds);

            $this->cache->persistToCache($username, $accessibleTeamIds, $this->cacheLifetime);
        }
        return $accessibleTeamIds;
    }

    /**
     * @inheritdoc
     * @return array
     */
    public function getDataForToken(TokenInterface $token)
    {
        return parent::getDataForToken($token);
    }

    /**
     * @inheritdoc
     * @return array
     */
    public function getDataForCurrentUser()
    {
        return parent::getDataForCurrentUser();
    }
}
