<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Permissions;


class PermissionResolver
{
    /** @var PermissionsDictionaryProvider */
    protected $permissionsDictionaryProvider;

    /**
     * PermissionResolver constructor.
     * @param PermissionsDictionaryProvider $permissionsDictionaryProvider
     */
    public function __construct(PermissionsDictionaryProvider $permissionsDictionaryProvider)
    {
        $this->permissionsDictionaryProvider = $permissionsDictionaryProvider;
    }

    /**
     * @param string $permission Either {module}.{create|read|update|delete} or {module}.{field}.{read|write}
     * @return bool
     */
    public function resolvePermission($permission)
    {
        $permissionsDictionary = $this->permissionsDictionaryProvider->getDataForCurrentUser();

        $isGranted = false;
        list($module, $action) = explode('.', $permission, 2);

        if (!array_key_exists($module, $permissionsDictionary)) {
            return false;
        }

        $modulePermissions = $permissionsDictionary[$module];
        $delimiterCount = substr_count($action, '.');

        if ($delimiterCount === 0) {
            // CRUD action
            $isGranted = $modulePermissions[$action];
        } else if ($delimiterCount === 1) {
            list($field, $fieldAction) = explode('.', $action);
            $fieldsDictionary = $modulePermissions['fields'];
            $fieldPermissions = array_key_exists($field, $fieldsDictionary) ? $fieldsDictionary[$field] : [];
            $isGranted = (!array_key_exists($fieldAction, $fieldPermissions) || $fieldPermissions[$fieldAction]) && $this->hasModulePermission($modulePermissions, $fieldAction);
        }

        return $isGranted;
    }

    /**
     * @param $modulePermssions
     * @param $action
     * @return bool
     */
    private function hasModulePermission(array $modulePermssions, $action)
    {
        if ($modulePermssions === null || empty($modulePermssions) || $action === "" || $action === null) {
            return false;
        }
        if ($action === PermissionsDictionaryBuilder::PERM_FIELD_WRITE) {
            return (array_key_exists(PermissionsDictionaryBuilder::PERM_CRUD_CREATE, $modulePermssions) ? $modulePermssions[PermissionsDictionaryBuilder::PERM_CRUD_CREATE] : false) || (array_key_exists(PermissionsDictionaryBuilder::PERM_CRUD_UPDATE, $modulePermssions) ? $modulePermssions[PermissionsDictionaryBuilder::PERM_CRUD_UPDATE] : false) || (array_key_exists(PermissionsDictionaryBuilder::PERM_CRUD_DELETE, $modulePermssions) ? $modulePermssions[PermissionsDictionaryBuilder::PERM_CRUD_DELETE] : false);
        } else {
            return array_key_exists(PermissionsDictionaryBuilder::PERM_CRUD_READ, $modulePermssions) ? $modulePermssions[PermissionsDictionaryBuilder::PERM_CRUD_READ] : false;
        }
    }
}