<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Permissions;


use Cybersponse\ActorsBundle\Entity\Actor;
use Cybersponse\CoreBundle\Provider\CacheableActorDataProvider;
use Cybersponse\AuthorizationBundle\Entity\Role;

class PermissionsDictionaryProvider extends CacheableActorDataProvider
{
    /** @var PermissionsDictionaryBuilder */
    protected $permissionsDictionaryBuilder;

    /** @var array */
    protected $permissionsByUsername = [];

    /**
     * @inheritdoc
     */
    public function getDataForActor(Actor $actor)
    {
        $username = $actor->getUsername();

        if (array_key_exists($username, $this->permissionsByUsername)) {
            return $this->permissionsByUsername[$username];
        }

        if ($this->cache->cacheContains($username)) {
            $permissionsDictionary = $this->cache->getFromCache($username);
        } else {
            /** @var Role[] $roles */
            $roles = $actor->getRoles();

            $permissionsDictionary = $this->permissionsDictionaryBuilder->buildPermissionsDictionary($roles);

            $this->cache->persistToCache($username, $permissionsDictionary, $this->cacheLifetime);
        }
        $this->permissionsByUsername[$username] = $permissionsDictionary;

        return $permissionsDictionary;
    }

    /**
     * @param PermissionsDictionaryBuilder $permissionsDictionaryBuilder
     */
    public function setPermissionsDictionaryBuilder($permissionsDictionaryBuilder)
    {
        $this->permissionsDictionaryBuilder = $permissionsDictionaryBuilder;
    }
}