<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Serialization;


use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\Serializer\Serializer;
use Cybersponse\AuthorizationBundle\Entity\Tag;
use Cybersponse\CoreBundle\Serializer\EfficientItemNormalizer;

class TagNormalizer extends EfficientItemNormalizer
{
    const FORMAT_JSON_LD = 'json-ld';

    /** @var Serializer */
    protected $serializer;

    /** @var ResourceInterface */
    protected $tagResource;

    /**
     * {@inheritdoc}
     */
    public function supportsNormalization($data, $format = null)
    {
        return self::FORMAT_JSON_LD === $format && $data instanceof Tag;
    }

    /**
     * @inheritdoc
     */
    public function supportsDenormalization($data, $type, $format = null)
    {
        return is_subclass_of($type, Tag::class);
    }

    /**
     * {@inheritdoc}
     */
    public function denormalize($data, $class, $format = null, array $context = [])
    {
        $context['ignore_authorization']                = true;
        $context['ignore_relationship_authorization']   = true;

        return parent::denormalize($data, $class, $format, $context);
    }
    
    /**
     * {@inheritdoc}
     */
    public function normalize($object, $format = null, array $context = [])
    {
        $context['ignore_authorization']                = true;
        $context['ignore_relationship_authorization']   = true;

        $normalizationContext = $this->tagResource->getNormalizationContext();

        return parent::normalize(
            $object,
            $format,
            $context + $normalizationContext
        );
    }

    /**
     * @param ResourceInterface $tagResource
     */
    public function setTagResource($tagResource)
    {
        $this->tagResource = $tagResource;
    }
}
