<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Subscriber;


use Cybersponse\AuthorizationBundle\Entity\Tag;
use Dunglas\ApiBundle\JsonLd\Response;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Doctrine\Common\Persistence\ObjectRepository;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityRepository;
use Dunglas\ApiBundle\Event\Events;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Exception\HttpException;

class TagSubscriber implements EventSubscriberInterface
{

    const INVALID_CHAR_IN_TAG = "Special characters '#', '?', ',', '\'', '\"' and '/' are not allowed in tag.";

    /** @var EntityManager */
    protected $tagManager;

    /** @var ObjectRepository */
    protected $tagRepository;

    public static function getSubscribedEvents()
    {
        return [
            Events::PRE_CREATE => ['handleCreate', 1000],
            Events::PRE_UPDATE => ['handleUpdate', 1000]
        ];
    }

    public function validateAndGetTags($data)
    {
        $tags = array();
        foreach ($data->getRecordTags() as $tag) {
          if (preg_match("/[#,?'\"]/", $tag->getUuid())) {
                throw new HttpException(Response::HTTP_BAD_REQUEST, self::INVALID_CHAR_IN_TAG);
            }
            $tags[] = $tag->getUuid();
        }
        return $tags;
    }

    /**
     * @param DataEvent $event
     * @throws \Exception
     */
    public function handleCreate(DataEvent $event)
    {
        $data = $event->getData();
        if (method_exists($data, 'getRecordTags') && $data->getRecordTags()) {
            $tags = $this->validateAndGetTags($data);
            $this->tagRepository->UpsertTag($tags);
        }
    }

    /**
     * @param DataEvent $event
     * @throws \Exception
     */
    public function handleUpdate(DataEvent $event)
    {
        $data = $event->getData();
        $previous = $event->getPrevious();
        if (method_exists($data, 'getRecordTags') && $data->getRecordTags()) {
            $previous_tags = array();
            $tags = $this->validateAndGetTags($data);
            if (isset($previous['recordTags'])) {
                foreach ($previous['recordTags'] as $tag) {
                    if (is_string($tag)) {
                        $exploded_string = explode("/", $tag);
                        $previous_tags[] = end($exploded_string);
                    }
                }
            }
            
            $diff_tags = array_diff($tags, $previous_tags);
            if (count($diff_tags)) {
                $this->tagRepository->UpsertTag($diff_tags);
            }
        }
    }

    /**
     * @param EntityRepository $tagManager
     */
    public function setTagManager($tagManager)
    {
        $this->tagManager = $tagManager;
        $this->tagRepository = $this->tagManager->getRepository(Tag::class);
    }
}
