<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Subscriber;

use Cybersponse\UtilsBundle\Utils;
use Dunglas\ApiBundle\Event\Events;
use Dunglas\ApiBundle\Event\DataEvent;
use Cybersponse\ActorsBundle\Entity\Actor;
use Cybersponse\CoreBundle\Provider\CacheableInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class UserCacheSubscriber implements EventSubscriberInterface
{
    /** @var CacheableInterface[] */
    protected $caches = [];

    /**
     * @param CacheableInterface $cache
     */
    public function addCache($cache)
    {
        $this->caches[] = $cache;
    }

    /** @var ContainerInterface */
    protected $container;

    /**
     * @param ContainerInterface $container
     */
    public function setContainer($container)
    {
        $this->container = $container;
    }
    
    /**
     * @inheritdoc
     */
    public static function getSubscribedEvents()
    {
        return [
            Events::POST_UPDATE => ['clearUserPermissionsCache', 0],
            Events::POST_DELETE => ['clearUserPermissionsCache', 0]
        ];
    }

    /**
     * @param DataEvent $dataEvent
     */
    public function clearUserPermissionsCache(DataEvent $dataEvent)
    {
        $object = $dataEvent->getData();

        if ($object instanceof Actor && !is_null($object->getUserId())) {
            foreach ($this->caches as $cache) {
                $cache->deleteFromCache($object->getUserId());
            }
            $util = new Utils();
            $rootDir = $this->container->getParameter('kernel.root_dir');
            $cmd =sprintf('sudo -u nginx php %s/console --env=prod cybersponse:util -r', $rootDir);
            $util->broadcastToHA($this->container, 'cluster_redis_clear', $cmd);
        }
    }
}
