<?php

namespace Cybersponse\AuthorizationBundle\Subscriber;

use Dunglas\ApiBundle\Event\Events;
use Cybersponse\UtilsBundle\ARCUtils;
use Cybersponse\FilesBundle\Entity\PgFile;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class WidgetSubscriber implements EventSubscriberInterface
{
    const WIDGET_PATH = '/opt/cyops-ui/widgets';

    /** @var ContainerInterface */
    protected $container;

    /**
    * @param ContainerInterface $container
    */
    public function setContainer($container)
    {
        $this->container = $container;
    }

    public static function getSubscribedEvents()
    {
        return [

            Events::PRE_UPDATE => ['handleUpdate', 100],
            Events::PRE_CREATE => ['handleCreate', 100],
            Events::POST_DELETE => ['handleDelete', 100]
        ];
    }

    /**
     * @param DataEvent $event
     * @throws \Exception
     */
    public function handleUpdate(DataEvent $event)
    {
        if ($event->getResource()->getModelType() == 'widgets') {
            $data = $event->getData();
            $util = new ARCUtils();
            if (!$data->getDraft() && $data->getInstalled()) {
                $util->buildWidgetFiles($data, self::WIDGET_PATH, $this->container->get('doctrine')->getManagerForClass(PgFile::class));
                $this->broadcastBuild($data->getId());
                $util->removeWidgetPreviewFiles($data->getName());
            } elseif ($data->getDraft() && !$data->getInstalled()) {
                $util->removeWidgetFiles($data);
                $this->broadcastBuild(null, $data->getId());
            }
        }
    }

    public function handleCreate(DataEvent $event)
    {
        if ($event->getResource()->getModelType() == 'widgets') {
            $data = $event->getData();
            $util = new ARCUtils();
            if (!$data->getDraft()) {
                $util->buildWidgetFiles($data, self::WIDGET_PATH, $this->container->get('doctrine')->getManagerForClass(PgFile::class));
                $this->broadcastBuild($data->getId());
            }
        }
    }

    public function handleDelete(DataEvent $event)
    {
        if ($event->getResource()->getModelType() == 'widgets') {
            $data = $event->getData();
            $util = new ARCUtils();
            if (!$data->getDraft()) {
                $util->removeWidgetFiles($data);
                $this->broadcastBuild(null, $data->getId());
            }
        }
    }
    
    private function broadcastBuild($wid=null, $deleteWidget=null)
    {
        $dasUri = $this->container->getParameter('das.uri');
        $proxyProvider = $this->container->get('cybersponse.core.provider.proxy');
        $dasClient = $proxyProvider->getDasClient();
        $cmd = "";
        if ($wid) {
            $cmd = $cmd."sudo -u nginx php /opt/cyops-api/app/console --env=prod cybersponse:widgets:build -w ".$wid;
        } elseif ($deleteWidget) {
            $cmd = $cmd."sudo -u nginx php /opt/cyops-api/app/console --env=prod cybersponse:widgets:build -d ".$wid;
        }

        $objectData = array(
            "commandId" => "cluster_publish_build_widget",
            "command" => $cmd
        );
        $headers = array(
            "Content-type" => "application/json"
        );
        $dasRequests = new GuzzleRequest('post', $dasUri . '/clustercommand/', $headers, json_encode($objectData));
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $resCode = $dasResponse->getStatusCode();
            if ($resCode == 200) {
                return 0;
            }
        } catch (\Exception $error) {
            error_log($error->getMessage());
        }
        return true;
    }
}
