<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Tests\Unit\Ownership;


use Cybersponse\AuthorizationBundle\Ownership\OwnershipProvider;
use Cybersponse\AuthorizationBundle\Tests\Unit\Ownership\Mocks\NonOwnableClass;
use Cybersponse\AuthorizationBundle\Tests\Unit\Ownership\Mocks\OwnableClass;

class OwnershipProviderTest extends \PHPUnit_Framework_TestCase
{
    /** @var OwnershipProvider */
    protected $ownershipProvider;

    protected function setUp()
    {
        $this->ownershipProvider = new OwnershipProvider();
    }

    public function test_isObjectOwnable_givenOwnableObject_shouldBeTrue()
    {
        $ownableObject = new OwnableClass();
        $isOwnable = $this->ownershipProvider->isObjectOwnable($ownableObject);
        $this->assertTrue($isOwnable);
    }

    public function test_isObjectOwnable_givenNonOwnableObject_shouldBeFalse()
    {
        $ownableObject = new NonOwnableClass();
        $isOwnable = $this->ownershipProvider->isObjectOwnable($ownableObject);
        $this->assertFalse($isOwnable);
    }

    public function test_isClassOwnable_givenOwnableClass_shouldBeTrue()
    {
        $isOwnable = $this->ownershipProvider->isClassOwnable(OwnableClass::class);
        $this->assertTrue($isOwnable);
    }

    public function test_isClassOwnable_givenNonOwnableClass_shouldBeFalse()
    {
        $isOwnable = $this->ownershipProvider->isClassOwnable(NonOwnableClass::class);
        $this->assertFalse($isOwnable);
    }

}
 