<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Tests\Unit\Permissions;


use Cybersponse\AuthorizationBundle\Permissions\PermissionResolver;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryProvider;

class PermissionResolverTest extends \PHPUnit_Framework_TestCase
{
    /** @var \PHPUnit_Framework_MockObject_MockObject|PermissionsDictionaryProvider */
    protected $mockPermissionsDictionaryProvider;

    protected function setUp()
    {
        parent::setUp();
        $this->mockPermissionsDictionaryProvider = $this->getMockBuilder(PermissionsDictionaryProvider::class)->disableOriginalConstructor()->getMock();
    }

    protected function executeSystemUnderTest($permissionsDictionary, $permission)
    {
        $this->mockPermissionsDictionaryProvider->method('getDataForCurrentUser')->willReturn($permissionsDictionary);
        $permissionsResolver = new PermissionResolver($this->mockPermissionsDictionaryProvider);
        return $permissionsResolver->resolvePermission($permission);
    }

    public function test_GivenModuleReadPermissionTrue_GivenModuleReadPermission_ShouldReturnTrue()
    {
        $module = 'incidents';
        $action = 'read';
        $permissionsDictionary = [
            $module => [
                $action => true
            ]
        ];
        $permission = "$module.$action";

        $result = $this->executeSystemUnderTest($permissionsDictionary, $permission);

        $this->assertTrue($result);
    }

    public function test_GivenModuleReadPermissionFalse_GivenModuleReadPermission_ShouldReturnFalse()
    {
        $module = 'incidents';
        $action = 'read';
        $permissionsDictionary = [
            $module => [
                $action => false
            ]
        ];
        $permission = "$module.$action";

        $result = $this->executeSystemUnderTest($permissionsDictionary, $permission);

        $this->assertFalse($result);
    }

    public function test_GivenNonexistentModule_GivenModuleReadPermission_ShouldReturnFalse()
    {
        $module = 'incidents';
        $action = 'read';
        $permissionsDictionary = [];
        $permission = "$module.$action";

        $result = $this->executeSystemUnderTest($permissionsDictionary, $permission);

        $this->assertFalse($result);
    }

    public function test_GivenFieldReadPermissionTrue_GivenFieldReadPermission_ShouldReturnFalse()
    {
        $module = 'incidents';
        $field = 'owners';
        $action = 'read';
        $permissionsDictionary = [
            $module => [
                'fields' => [
                    $field => [
                        $action => true
                    ]
                ]
            ]
        ];
        $permission = "$module.$field.$action";

        $result = $this->executeSystemUnderTest($permissionsDictionary, $permission);
        //should return false as no module level permission defined
        $this->assertFalse($result);
    }

    public function test_GivenFieldReadPermissionFalse_GivenFieldReadPermission_ShouldReturnFalse()
    {
        $module = 'incidents';
        $field = 'owners';
        $action = 'read';
        $permissionsDictionary = [
            $module => [
                'fields' => [
                    $field => [
                        $action => false

                    ]
                ]
            ]
        ];
        $permission = "$module.$field.$action";

        $result = $this->executeSystemUnderTest($permissionsDictionary, $permission);

        $this->assertFalse($result);
    }

    public function test_GivenNonexistentFieldPermission_GivenFieldReadPermission_ShouldReturnFalse()
    {
        $module = 'incidents';
        $field = 'owners';
        $action = 'read';
        $permissionsDictionary = [
            $module => [
                'fields' => []
            ]
        ];
        $permission = "$module.$field.$action";

        $result = $this->executeSystemUnderTest($permissionsDictionary, $permission);

        $this->assertFalse($result);
    }
}
