<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Tests\Unit\Permissions\PermissionsDictionaryBuilder;


use Cybersponse\AuthorizationBundle\Entity\FieldPermission;
use Cybersponse\AuthorizationBundle\Entity\ModulePermission;
use Cybersponse\AuthorizationBundle\Entity\Role;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;
use Cybersponse\CoreBundle\Entity\Module;

class BuildPermissionsDictionaryMethodTest extends \PHPUnit_Framework_TestCase
{
    protected function executeSystemUnderTest($roles)
    {
        $serializer = new PermissionsDictionaryBuilder();
        return $serializer->buildPermissionsDictionary($roles);
    }

    public function test_GivenSinglePermTrue_ShouldReturnTrue()
    {
        $moduleType = 'meh';

        $roles = [
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, true)
            )
        ];
        $permissionsDictionary = $this->executeSystemUnderTest($roles);

        $this->assertTrue($permissionsDictionary[$moduleType]['create']);
    }

    public function test_GivenSinglePermFalse_ShouldReturnFalse()
    {
        $moduleType = 'meh';

        $roles = [
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, false)
            )
        ];
        $permissionsDictionary = $this->executeSystemUnderTest($roles);

        $this->assertFalse($permissionsDictionary[$moduleType]['create']);
    }

    public function test_GivenSinglePermNull_ShouldNotReturnField()
    {
        $moduleType = 'meh';
        $field = 'blah';

        $roles = [
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, true, true, true, true, [$this->createFieldPermissions($field, null)])
            )
        ];
        $permissionsDictionary = $this->executeSystemUnderTest($roles);

        $this->assertNotContains($field, $permissionsDictionary[$moduleType]['fields']);
    }

    public function test_GivenFirstPermTrue_GivenSecondPermFalse_ShouldReturnTrue()
    {
        $moduleType = 'meh';

        $roles = [
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, true)
            ),
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, false)
            )
        ];
        $permissionsDictionary = $this->executeSystemUnderTest($roles);

        $this->assertTrue($permissionsDictionary[$moduleType]['create']);
    }

    public function test_GivenFirstPermFalse_GivenSecondPermTrue_ShouldReturnTrue()
    {
        $moduleType = 'meh';

        $roles = [
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, false)
            ),
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, true)
            )
        ];
        $permissionsDictionary = $this->executeSystemUnderTest($roles);

        $this->assertTrue($permissionsDictionary[$moduleType]['create']);
    }

    public function test_GivenFirstFieldPermFalse_GivenSecondFieldPermTrue_ShouldReturnTrue()
    {
        $moduleType = 'meh';
        $field = 'blah';

        $roles = [
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, true, true, true, true, [$this->createFieldPermissions($field, false)])
            ),
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, true, true, true, true, [$this->createFieldPermissions($field, true)])
            )
        ];
        $permissionsDictionary = $this->executeSystemUnderTest($roles);

        $this->assertTrue($permissionsDictionary[$moduleType]['fields'][$field]['read']);
    }

    public function test_GivenFirstFieldPermTrue_GivenSecondFieldPermFalse_ShouldReturnTrue()
    {
        $moduleType = 'meh';
        $field = 'blah';

        $roles = [
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, true, true, true, true, [$this->createFieldPermissions($field, true)])
            ),
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, true, true, true, true, [$this->createFieldPermissions($field, false)])
            )
        ];
        $permissionsDictionary = $this->executeSystemUnderTest($roles);

        $this->assertTrue($permissionsDictionary[$moduleType]['fields'][$field]['read']);
    }

    public function test_GivenFirstRoleWithModuleRead_GivenSecondRoleWithModuleRead_GivenSecondRoleWithFieldReadFalse_ShouldNotReturnFieldRestriction()
    {
        $moduleType = 'meh';
        $field = 'blah';

        $roles = [
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, true, true)
            ),
            $this->createRoleFromPermissions(
                $this->createModulePermissions($moduleType, true, true, true, true, [$this->createFieldPermissions($field, false)])
            )
        ];
        $permissionsDictionary = $this->executeSystemUnderTest($roles);

        $this->assertArrayNotHasKey($field, $permissionsDictionary[$moduleType]['fields']);
    }

    protected function createRoleFromPermissions()
    {
        $role = $this->getMockBuilder(Role::class)->disableOriginalConstructor()->getMock();
        $role->method('getModulePermissions')->willReturn(func_get_args());
        return $role;
    }

    protected function createModulePermissions($type, $create = false, $read = false, $update = false, $delete = false, $fieldPermissions = [])
    {
        $module = $this->getMockBuilder(Module::class)->disableOriginalConstructor()->getMock();
        $module->method('getType')->willReturn($type);

        $modulePermission = $this->getMockBuilder(ModulePermission::class)->disableOriginalConstructor()->getMock();
        $modulePermission->method('getModule')->willReturn($module);
        $modulePermission->method('getCanCreate')->willReturn($create);
        $modulePermission->method('getCanRead')->willReturn($read);
        $modulePermission->method('getCanUpdate')->willReturn($update);
        $modulePermission->method('getCanDelete')->willReturn($delete);
        $modulePermission->method('getFieldPermissions')->willReturn($fieldPermissions);

        return $modulePermission;
    }

    protected function createFieldPermissions($field, $read = null, $write = null)
    {
        $fieldPermission = $this->getMockBuilder(FieldPermission::class)->disableOriginalConstructor()->getMock();
        $fieldPermission->method('getField')->willReturn($field);
        $fieldPermission->method('getCanRead')->willReturn($read);
        $fieldPermission->method('getCanWrite')->willReturn($write);

        return $fieldPermission;
    }
}
