<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Tests\Unit\Permissions\PermissionsDictionaryProvider;


use Cybersponse\ActorsBundle\Entity\Actor;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryProvider;
use Cybersponse\CoreBundle\Provider\CacheableInterface;

class GetDataForActorMethodTest extends \PHPUnit_Framework_TestCase
{
    /** @var \PHPUnit_Framework_MockObject_MockObject|Actor */
    protected $mockActor;

    /** @var \PHPUnit_Framework_MockObject_MockObject|CacheableInterface */
    protected $mockCache;

    /** @var \PHPUnit_Framework_MockObject_MockObject|PermissionsDictionaryBuilder */
    protected $mockPermissionsDictionaryBuilder;

    protected function setUp()
    {
        parent::setUp();
        $this->mockActor = $this->getMockBuilder(Actor::class)->disableOriginalConstructor()->getMock();
        $this->mockCache = $this->getMockBuilder(CacheableInterface::class)->getMock();
        $this->mockPermissionsDictionaryBuilder = $this->getMockBuilder(PermissionsDictionaryBuilder::class)->getMock();
    }

    protected function executeSystemUnderTest()
    {
        $provider = new PermissionsDictionaryProvider();
        $provider->setPermissionsDictionaryBuilder($this->mockPermissionsDictionaryBuilder);
        $provider->setCache($this->mockCache);
        return $provider->getDataForActor($this->mockActor);
    }

    public function test_GivenNonCachedPermissions_ShouldBuildActorRoles()
    {
        $expectedActorRoles = ['role1', 'role2'];
        $this->mockCache->method('cacheContains')->willReturn(false);
        $this->mockActor->method('getRoles')->willReturn($expectedActorRoles);
        $this->mockPermissionsDictionaryBuilder
            ->expects($this->once())
            ->method('buildPermissionsDictionary')
            ->with($this->equalTo($expectedActorRoles));

        $this->executeSystemUnderTest();
    }

    public function test_GivenNonCachedPermissions_ShouldPersistBuilderResultToCache()
    {
        $expectedBuildPermissions = ['built' => 'dict'];
        $this->mockCache->method('cacheContains')->willReturn(false);
        $this->mockPermissionsDictionaryBuilder->method('buildPermissionsDictionary')->willReturn($expectedBuildPermissions);
        $this->mockCache
            ->expects($this->once())
            ->method('persistToCache')
            ->with($this->anything(), $this->equalTo($expectedBuildPermissions), $this->anything());

        $this->executeSystemUnderTest();
    }

    public function test_GivenNonCachedPermissions_ShouldReturnBuilderResult()
    {
        $expectedBuildPermissions = ['built' => 'dict'];
        $this->mockCache->method('cacheContains')->willReturn(false);
        $this->mockPermissionsDictionaryBuilder->method('buildPermissionsDictionary')->willReturn($expectedBuildPermissions);

        $permissionsDictionary = $this->executeSystemUnderTest();

        $this->assertEquals($expectedBuildPermissions, $permissionsDictionary);
    }

    public function test_GivenCachedPermissions_ShouldReturnCachedPermissions()
    {
        $expectedPermissionsDictionary = ['i' => 'am', 'expect' => 'ed'];
        $this->mockCache->method('cacheContains')->willReturn(true);
        $this->mockCache->method('getFromCache')->willReturn($expectedPermissionsDictionary);

        $permissionsDictionary = $this->executeSystemUnderTest();

        $this->assertEquals($expectedPermissionsDictionary, $permissionsDictionary);
    }
}
