<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Tests\Unit\Voters;


use Cybersponse\AuthorizationBundle\Permissions\PermissionResolver;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryProvider;
use Cybersponse\AuthorizationBundle\Voters\PermissionVoter;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Authorization\Voter\VoterInterface;

class PermissionVoterTest extends \PHPUnit_Framework_TestCase
{
    const FORCE_FALSE = 'perm:false';
    const FORCE_TRUE = 'perm:true';
    const FAKE_PERMS_DICT = '';

    /** @var \PHPUnit_Framework_MockObject_MockObject|PermissionResolver */
    protected $mockPermissionResolver;

    protected function setUp()
    {
        parent::setUp();
        $this->mockPermissionResolver = $this->getMockBuilder(PermissionResolver::class)->disableOriginalConstructor()->getMock();
        $this->mockPermissionResolver
            ->method('resolvePermission')
            ->willReturnMap([
                ['true', true],
                ['false', false]
            ]);
    }

    protected function executeSystemUnderTest($attributes)
    {
        /** @var TokenInterface $mockToken */
        $mockToken = $this->getMock(TokenInterface::class);

        $permissionVoter = new PermissionVoter();
        $permissionVoter->setPermissionResolver($this->mockPermissionResolver);
        return $permissionVoter->vote($mockToken, null, $attributes);
    }

    public function test_GivenFalseAttribute_GivenTrueAttribute_ShouldReturnGranted()
    {
        $attributes = [static::FORCE_FALSE, static::FORCE_TRUE];

        $result = $this->executeSystemUnderTest($attributes);

        $this->assertEquals(VoterInterface::ACCESS_GRANTED, $result);
    }

    public function test_GivenTrueAttribute_GivenFalseAttribute_ShouldReturnGranted()
    {
        $attributes = [static::FORCE_TRUE, static::FORCE_FALSE];

        $result = $this->executeSystemUnderTest($attributes);

        $this->assertEquals(VoterInterface::ACCESS_GRANTED, $result);
    }

    public function test_GivenTrueAttribute_GivenTrueAttribute_ShouldReturnGranted()
    {
        $attributes = [static::FORCE_TRUE, static::FORCE_TRUE];

        $result = $this->executeSystemUnderTest($attributes);

        $this->assertEquals(VoterInterface::ACCESS_GRANTED, $result);
    }

    public function test_GivenFalseAttribute_GivenFalseAttribute_ShouldReturnDenied()
    {
        $attributes = [static::FORCE_FALSE, static::FORCE_FALSE];

        $result = $this->executeSystemUnderTest($attributes);

        $this->assertEquals(VoterInterface::ACCESS_DENIED, $result);
    }

    public function test_GivenUnsupportedAttribute_ShouldReturnAbstained()
    {
        $attributes = ['not:valid.perm'];

        $result = $this->executeSystemUnderTest($attributes);

        $this->assertEquals(VoterInterface::ACCESS_ABSTAIN, $result);
    }

    public function test_GivenEmptyAttributes_ShouldReturnAbstained()
    {
        $attributes = [];

        $result = $this->executeSystemUnderTest($attributes);

        $this->assertEquals(VoterInterface::ACCESS_ABSTAIN, $result);
    }
}
