<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Traits;

use Cybersponse\AuthorizationBundle\Entity\Team;
use Doctrine\Common\Collections\Collection;

trait ConditionalOwnable
{
    /**
     * @var Team[]|Collection
     *
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="Cybersponse\AuthorizationBundle\Entity\Team", cascade={"persist"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Owners")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="teams")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="manyToMany")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     * @Symfony\Component\Validator\Constraints\Count(min="0")
     */
    protected $owners;

    /**
     * @ORM\Column(type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Is Private")"
     */
    protected $isPrivate = false;

    /**
     * Set isPrivate
     *
     * @param boolean $isPrivate
     * @return self
     */
    public function setIsPrivate($isPrivate)
    {
        $this->isPrivate = $isPrivate;

        return $this;
    }

    /**
     * Get isActive
     *
     * @return boolean
     */
    public function getIsPrivate()
    {
        return $this->isPrivate;
    }

    /**
     * Add owner
     *
     * @param Team $owner
     * @return self
     */
    public function addOwner($owner)
    {
        if (is_array($this->owners) && empty($this->owners)) {
            unset($this->owners);
            $this->owners = [];
        }
        $this->owners[] = $owner;
        return $this;
    }

    /**
     * Remove owner
     *
     * @param Team $owner
     */
    public function removeOwner($owner)
    {
        if (is_array($this->owners) && ($key = array_search($owner, $this->owners)) !== false) {
            unset($this->owners[$key]);
            return;
        }
        $this->owners->removeElement($owner);
    }

    /**
     * Get owners
     *
     * @return Team[]
     */
    public function getOwners()
    {
        return $this->owners;
    }
}
