<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */
namespace Cybersponse\AuthorizationBundle\Traits;

use Cybersponse\AuthorizationBundle\Entity\Team;
use Doctrine\Common\Collections\Collection;

trait Ownable
{
    /**
     * @var Team[]|Collection
     *
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="Cybersponse\AuthorizationBundle\Entity\Team", cascade={"persist"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Owners")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="teams")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="manyToMany")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     * @Symfony\Component\Validator\Constraints\Count(min="1", minMessage="owners: Unable to perform requested operation. Ensure the record has at least one owner assigned.")
     */
    protected $owners;

    /**
     * Add owner
     *
     * @param Team $owner
     * @return self
     */
    public function addOwner($owner)
    {
        if (is_array($this->owners) && empty($this->owners)) {
            unset($this->owners);
            $this->owners = [];
        }
        if (!empty($this->owners)) {
            if (!is_array($this->getOwners()) && isset($this->owners) && get_class($this->owners) == "Doctrine\ORM\PersistentCollection") {
                if (!in_array($owner, $this->owners->getValues())) {
                    $this->owners[] = $owner;
                }
            } elseif (!in_array($owner, $this->owners)) {
                $this->owners[] = $owner;
            }
        } else {
            $this->owners[] = $owner;
        }
        return $this;
    }

    /**
     * Remove owner
     *
     * @param Team $owner
     */
    public function removeOwner($owner)
    {
        if (is_array($this->owners) && ($key = array_search($owner, $this->owners)) !== false) {
            unset($this->owners[$key]);
            return;
        }
        $this->owners->removeElement($owner);
    }

    /**
     * Get owners
     *
     * @return Team[]
     */
    public function getOwners()
    {
        return $this->owners;
    }
}
