<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Validation;


use Cybersponse\AuthorizationBundle\Entity\Team;
use Doctrine\Common\Collections\Collection;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;

class NonIntersectingCollectionFieldsConstraintValidator extends ConstraintValidator
{
    /**
     * @param Team $value
     * @param Constraint $constraint
     */
    public function validate($value, Constraint $constraint)
    {
        if (!$constraint instanceof NonIntersectingCollectionFieldsConstraint) {
            throw new UnexpectedTypeException($constraint, __NAMESPACE__ . '\NotSelfReferentialConstraint');
        }

        $visitedTeams = [];

        return $this->containsVisitedTeam($value->getChildren(), $visitedTeams)
            || $this->containsVisitedTeam($value->getParents(), $visitedTeams)
            || $this->containsVisitedTeam($value->getSiblings(), $visitedTeams);
    }

    /**
     * @param Collection $teams
     * @param string[] $visitedTeams
     * @return bool|void
     */
    protected function containsVisitedTeam($teams, &$visitedTeams)
    {
        /** @var Team $team */
        foreach ($teams as $team) {
            $name = $team->getName();
            if (array_key_exists($name, $visitedTeams)) {
                $this->addNonIntersectingConstraintViolation();
                return true;
            }

            $visitedTeams[$name] = true;
        }

        return false;
    }

    protected function addNonIntersectingConstraintViolation()
    {
        $this->context->addViolation(NonIntersectingCollectionFieldsConstraint::MESSAGE);
    }
}
