<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Validation;


use Cybersponse\AuthorizationBundle\Entity\Team;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;

class NotSelfReferentialConstraintValidator extends ConstraintValidator
{
    /**
     * @param Team $value
     * @param Constraint $constraint
     */
    public function validate($value, Constraint $constraint)
    {
        if (!$constraint instanceof NotSelfReferentialConstraint) {
            throw new UnexpectedTypeException($constraint, __NAMESPACE__ . '\NotSelfReferentialConstraint');
        }

        /** @var Team[] $children */
        $children = $value->getChildren();
        if ($this->containsSelfReferences($children, $value)) {
            $this->addSelfReferentialViolation();
            return;
        }

        /** @var Team[] $parents */
        $parents = $value->getParents();
        if ($this->containsSelfReferences($parents, $value)) {
            $this->addSelfReferentialViolation();
            return;
        }

        /** @var Team[] $siblings */
        $siblings = $value->getSiblings();
        if ($this->containsSelfReferences($siblings, $value)) {
            $this->addSelfReferentialViolation();
            return;
        }
    }

    protected function addSelfReferentialViolation()
    {
        $this->context->addViolation(NotSelfReferentialConstraint::MESSAGE);
    }

    /**
     * @param Team[] $teams
     * @param Team $currentTeam
     * @return bool
     */
    protected function containsSelfReferences($teams, $currentTeam)
    {
        foreach ($teams as $team) {
            if ($team->getName() === $currentTeam->getName()) {
                return true;
            }
        }

        return false;
    }
}
