<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Voters;


use Cybersponse\AuthorizationBundle\Entity\Team;
use Cybersponse\AuthorizationBundle\Ownership\AccessibleTeamIdsProvider;
use Cybersponse\AuthorizationBundle\Ownership\OwnershipProvider;
use Doctrine\Common\Collections\AbstractLazyCollection;
use Dunglas\ApiBundle\Api\IriConverterInterface;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Authorization\Voter\VoterInterface;

class OwnershipVoter implements VoterInterface
{
    /** @var OwnershipProvider */
    protected $ownershipProvider;

    /** @var AccessibleTeamIdsProvider */
    protected $accessibleTeamsProvider;

    /** @var IriConverterInterface */
    protected $iriConverter;

    /** @var ResourceInterface */
    protected $teamResource;

    /**
     * @param OwnershipProvider $ownershipProvider
     */
    public function setOwnershipProvider($ownershipProvider)
    {
        $this->ownershipProvider = $ownershipProvider;
    }

    /**
     * @param AccessibleTeamIdsProvider $accessibleTeamsProvider
     */
    public function setAccessibleTeamsProvider($accessibleTeamsProvider)
    {
        $this->accessibleTeamsProvider = $accessibleTeamsProvider;
    }

    /**
     * @param IriConverterInterface $iriConverter
     */
    public function setIriConverter($iriConverter)
    {
        $this->iriConverter = $iriConverter;
    }

    /**
     * @param ResourceInterface $teamResource
     */
    public function setTeamResource($teamResource)
    {
        $this->teamResource = $teamResource;
    }

    /**
     * Ownership has
     * @inheritdoc
     */
    public function supportsAttribute($attribute)
    {
        return true;
    }

    /**
     * @inheritdoc
     */
    public function supportsClass($class)
    {
        return $this->ownershipProvider->isClassOwnable($class);
    }

    /**
     * @inheritdoc
     */
    public function vote(TokenInterface $token, $object, array $attributes)
    {
        $result = VoterInterface::ACCESS_ABSTAIN;

        if ($this->ownershipProvider->isObjectOwnable($object)) {
            $result = VoterInterface::ACCESS_DENIED;

            $teamBaseIri = $this->iriConverter->getIriFromResource($this->teamResource);

            $accessibleTeamIds = $this->accessibleTeamsProvider->getDataForCurrentUser();
            $accessibleTeamIris = array_map(function ($teamId) use ($teamBaseIri) {
                return sprintf('%s/%s', $teamBaseIri, $teamId);
            }, $accessibleTeamIds);

            $ownerIris = $object->getOwners();
            if ($ownerIris instanceof AbstractLazyCollection) {
                $ownerIris = array_map(function (Team $team) {
                    return $this->iriConverter->getIriFromItem($team);
                }, $ownerIris->toArray());
            } elseif (is_array($ownerIris) && $this->isOfTeamArray($ownerIris)) {
                $ownerIris = array_map(function (Team $team) {
                    return $this->iriConverter->getIriFromItem($team);
                }, $ownerIris);
            }

            $intersect = array_intersect($accessibleTeamIris, $ownerIris);

            if (count($intersect) > 0) {
                $result = VoterInterface::ACCESS_GRANTED;
            }
        }

        return $result;
    }

    //This function is temporary fix for regression of pre_create_trigger bug
    //bug#13491
    private function isOfTeamArray($ownerIris)
    {
        foreach ($ownerIris as $owner) {
            if (!($owner instanceof Team)) return false;
        }
        return true;
    }
}
