<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\AuthorizationBundle\Voters;


use Cybersponse\AuthorizationBundle\Permissions\PermissionResolver;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryProvider;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Authorization\Voter\VoterInterface;

class PermissionVoter implements VoterInterface
{
    const PREFIX = 'perm:';

    /** @var PermissionsDictionaryProvider */
    protected $permissionsProvider;

    /** @var PermissionResolver */
    protected $permissionResolver;

    /**
     * @inheritdoc
     */
    public function supportsAttribute($attribute)
    {
        return 0 === strpos($attribute, self::PREFIX);
    }

    /**
     * @inheritdoc
     */
    public function supportsClass($class)
    {
        return true;
    }

    /**
     * @inheritdoc
     */
    public function vote(TokenInterface $token, $object, array $attributes)
    {
        $result = VoterInterface::ACCESS_ABSTAIN;

        foreach ($attributes as $attribute) {
            if (!$this->supportsAttribute($attribute)) {
                continue;
            }

            $permission = substr($attribute, strlen(static::PREFIX));

            $isGranted = $this->permissionResolver->resolvePermission($permission);

            if ($isGranted) {
                return VoterInterface::ACCESS_GRANTED;
            }

            $result = VoterInterface::ACCESS_DENIED;
        }

        return $result;
    }

    /**
     * @param PermissionResolver $permissionResolver
     */
    public function setPermissionResolver($permissionResolver)
    {
        $this->permissionResolver = $permissionResolver;
    }
}
