<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\BackupBundle\Core;


use Cybersponse\BackupBundle\Processor\AbstractProcessor;
use RecursiveIteratorIterator;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Finder\Finder;
use Symfony\Component\Finder\Iterator\RecursiveDirectoryIterator;
use Symfony\Component\Finder\SplFileInfo;

class FolderBackupProcessor extends AbstractProcessor
{
    /** @var FileSystem */
    protected $fileSystem;

    /**
     * {@inheritdoc}
     */
    function backup()
    {
        $folderToBackup = $this->getFolderToBackup();

        $folderName = pathinfo($folderToBackup, PATHINFO_BASENAME);

        $timestamp = time();

        $backupFolderLocation = $this->getBackupFolderLocation();

        $backupFolderPath = sprintf('%s/%s_%s', $backupFolderLocation, $folderName, $timestamp);

        $this->copy($folderToBackup, $backupFolderPath);

        $this->fileSystem->remove($folderToBackup);
        $this->fileSystem->mkdir($folderToBackup);
    }

    /**
     * {@inheritdoc}
     */
    function restore()
    {
        $folderToBackup = $this->getFolderToBackup();
        $backupFolderLocation = $this->getBackupFolderLocation();

        $finder = $this->buildFinder();

        /** @var SplFileInfo $file */
        foreach ($finder->directories()->in($backupFolderLocation) as $file) {
            //Only move latest directory
            $this->copy($file->getRealPath(), $folderToBackup);
            $this->fileSystem->remove($file->getRealPath());
            break;
        }
    }

    /**
     * {@inheritdoc}
     */
    function onSuccess()
    {
        $backupFolderLocation = $this->getBackupFolderLocation();
        $finder = $this->buildFinder();

        /** @var SplFileInfo $file */
        foreach ($finder->directories()->in($backupFolderLocation) as $file) {

            //Remove all backups
            $this->fileSystem->remove($file->getRealPath());
        }
    }

    /**
     * @return string
     */
    public function getFolderToBackup()
    {
        return $this->configuration['location'];
    }

    /**
     * @param string $environment
     */
    public function setEnvironment($environment)
    {
        $this->environment = $environment;
    }

    /**
     * @param Filesystem $fileSystem
     */
    public function setFileSystem($fileSystem)
    {
        $this->fileSystem = $fileSystem;
    }

    /**
     * @return string
     */
    private function getBackupFolderLocation()
    {
        return $this->configuration['backup_location'];
    }

    /**
     * @return Finder
     */
    protected function buildFinder()
    {
        $finder = new Finder();

        $finder->sort(function (SplFileInfo $a, SplFileInfo $b) {
            return strcmp($b->getRealPath(), $a->getRealPath());
        });
        $finder->depth(0);
        return $finder;
    }

    /**
     * Deep copy
     *
     * @param $source
     * @param $target
     */
    private function copy($source, $target)
    {
        if (file_exists($target)) {
            $this->fileSystem->remove($target);
        }

        $this->fileSystem->mkdir($target);

        $directoryIterator = new RecursiveDirectoryIterator($source, RecursiveDirectoryIterator::SKIP_DOTS);
        $iterator = new RecursiveIteratorIterator($directoryIterator, RecursiveIteratorIterator::SELF_FIRST);
        foreach ($iterator as $item) {
            if ($item->isDir()) {
                $this->fileSystem->mkdir($target . DIRECTORY_SEPARATOR . $iterator->getSubPathName());
            } else {
                $this->fileSystem->copy($item, $target . DIRECTORY_SEPARATOR . $iterator->getSubPathName());
            }
        }
    }
}