<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\BackupBundle\Core;


use Cybersponse\BackupBundle\Processor\AbstractProcessor;

class RedisFlushProcessor extends AbstractProcessor
{
    /** @var \Predis\Client | \Snc\RedisBundle\Client\Phpredis\Client */
    protected $redisClient;

    /**
     * {@inheritdoc}
     */
    function backup()
    {
        $keys = $this->redisClient->keys('*');
        $items = [];
        foreach ($keys as $key) {
            $items[$key] = $this->redisClient->get($key);
        }

        file_put_contents($this->getRedisDumpFileName(), json_encode($items));
    }

    /**
     * {@inheritdoc}
     */
    function restore()
    {
        $this->redisClient->flushdb();

        $items = json_decode(file_get_contents($this->getRedisDumpFileName()));
        foreach ($items as $key => $value) {
            $this->redisClient->set($key, $value);
        }

        $this->cleanup();
    }

    /**
     * {@inheritdoc}
     */
    function onSuccess()
    {
        $this->cleanup();
    }

    /**
     * Remove the backup file
     */
    private function cleanup()
    {
        unlink($this->getRedisDumpFileName());
    }

    /**
     * @param \Predis\Client|\Snc\RedisBundle\Client\Phpredis\Client $redisClient
     */
    public function setRedisClient($redisClient)
    {
        $this->redisClient = $redisClient;
    }

    /**
     * @return string
     */
    private function getRedisDumpFileName()
    {
        $backupLocation = $this->configuration['backup_location'];
        if (!is_dir($backupLocation)) {
            mkdir($backupLocation);
        }

        return $backupLocation . 'redis.json';
    }
}