<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\BackupBundle\DependencyInjection;


use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Reference;

class ProcessorCompilerPass implements CompilerPassInterface
{
    /**
     * You can modify the container here before it is dumped to PHP code.
     *
     * @param ContainerBuilder $container
     * @throws \Exception
     */
    public function process(ContainerBuilder $container)
    {
        $processorsConfiguration = $container->getParameter('cybersponse.backup.processors');

        $processorReferences = [];

        foreach ($processorsConfiguration as $processorName => $processorConfiguration) {
            if (!$processorConfiguration['enabled']) {
                continue;
            }

            $definitionName = sprintf('cybersponse.backup.processor.%s', $processorName);

            if (!$container->hasDefinition($definitionName)) {
                throw new \Exception(sprintf('Definition: %s was not found in the container.', $definitionName));
            }

            $processorDefinition = $container->getDefinition($definitionName);
            $processorDefinition->addMethodCall('setConfiguration', [$processorConfiguration['config']]);

            $container->setDefinition($definitionName, $processorDefinition);

            $processorReferences[] = new Reference($definitionName);
        }

        $processorManagerDefinition = $container->getDefinition('cybersponse.backup.manager.processor');
        $processorManagerDefinition->addMethodCall('setProcessors', [$processorReferences]);
    }
}
