<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\BackupBundle\Manager;

use Cybersponse\BackupBundle\Processor\ProcessorInterface;

class ProcessorManager implements ProcessorInterface
{
    /** @var ProcessorInterface[] */
    protected $processors;

    /**
     * @param ProcessorInterface[] $processors
     */
    public function setProcessors($processors)
    {
        $this->processors = $processors;
    }

    /**
     * @return bool
     *
     * @throws \Exception
     */
    function backup()
    {
        foreach ($this->processors as $processor) {
            if (!$processor instanceof ProcessorInterface) {
                throw new \Exception(sprintf('Given processor: %s, is not an instance of ProcessorInterface.'), get_class($processor));
            }

            $result = $processor->backup();

            if ($result) {
                throw new \Exception(sprintf('Failed to backup using processor: %s'), get_class($processor));
            }
        }

        return true;
    }

    /**
     * @return bool
     *
     * @throws \Exception
     */
    function restore()
    {
        foreach ($this->processors as $processor) {
            if (!$processor instanceof ProcessorInterface) {
                throw new \Exception(sprintf('Given processor: %s, is not an instance of ProcessorInterface.'), get_class($processor));
            }

            $result = $processor->restore();

            if ($result) {
                throw new \Exception(sprintf('Failed to restore using processor: %s'), get_class($processor));
            }
        }

        return true;
    }

    /**
     * @return bool
     *
     * @throws \Exception
     */
    function onSuccess()
    {
        foreach ($this->processors as $processor) {
            if (!$processor instanceof ProcessorInterface) {
                throw new \Exception(sprintf('Given processor: %s, is not an instance of ProcessorInterface.'), get_class($processor));
            }

            $result = $processor->onSuccess();

            if ($result) {
                throw new \Exception(sprintf('Failed to cleanup using processor: %s'), get_class($processor));
            }
        }

        return true;
    }

    /**
     * @param array $configuration
     */
    function setConfiguration($configuration)
    {
    }
}