<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\CacheWarmer;

use Cybersponse\CoreBundle\Provider\CacheableInterface;
use Cybersponse\CoreBundle\Service\ApiDocumentationBuilder;
use Cybersponse\CoreBundle\Service\ContextBuilder;
use Symfony\Component\HttpKernel\CacheClearer\CacheClearerInterface;
use Symfony\Component\HttpKernel\CacheWarmer\CacheWarmerInterface;

class MetadataCacheWarmer implements CacheWarmerInterface, CacheClearerInterface
{
    /** @var CacheableInterface */
    protected $cache;

    /** @var ContextBuilder */
    protected $contextBuilder;

    /** @var ApiDocumentationBuilder */
    protected $apiDocumentationBuilder;

    /**
     * Checks whether this warmer is optional or not.
     *
     * Optional warmers can be ignored on certain conditions.
     *
     * A warmer should return true if the cache can be
     * generated incrementally and on-demand.
     *
     * @return bool true if the warmer is optional, false otherwise
     */
    public function isOptional()
    {
        return true;
    }

    /**
     * Warms up the cache.
     *
     * @param string $cacheDir The cache directory
     */
    public function warmUp($cacheDir)
    {
        $this->contextBuilder->getContexts();
        $this->apiDocumentationBuilder->getApiDocumentationAsMap();
    }

    /**
     * Clears any caches necessary.
     *
     * @param string $cacheDir The cache directory.
     */
    public function clear($cacheDir)
    {
        $this->cache->clearCache();
    }

    /**
     * @param ContextBuilder $contextBuilder
     */
    public function setContextBuilder($contextBuilder)
    {
        $this->contextBuilder = $contextBuilder;
    }

    /**
     * @param ApiDocumentationBuilder $apiDocumentationBuilder
     */
    public function setApiDocumentationBuilder($apiDocumentationBuilder)
    {
        $this->apiDocumentationBuilder = $apiDocumentationBuilder;
    }

    /**
     * @param CacheableInterface $cache
     */
    public function setCache($cache)
    {
        $this->cache = $cache;
    }
}
