<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Controller;

use Cybersponse\UtilsBundle\Utils;
use Doctrine\Common\Util\Inflector;
use Dunglas\ApiBundle\Event\Events;
use Cybersponse\UtilsBundle\ARCUtils;
use Dunglas\ApiBundle\JsonLd\Response;
use Doctrine\ORM\Mapping\ClassMetadata;
use Cybersponse\FilesBundle\Entity\File;
use Cybersponse\CoreBundle\Api\ApiEvents;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\Serializer\Serializer;
use Cybersponse\CoreBundle\Entity\BaseEntity;
use Symfony\Component\HttpFoundation\Request;
use Doctrine\Common\Persistence\ManagerRegistry;
use Cybersponse\AuthorizationBundle\Traits\Ownable;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Dunglas\ApiBundle\Exception\ExceptionInterface;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Dunglas\ApiBundle\Controller\ResourceController;
use Dunglas\ApiBundle\Api\ResourceCollectionInterface;
use Dunglas\ApiBundle\Exception\DeserializationException;
use Cybersponse\MetadataBundle\Entity\StagingModelMetadata;
use Cybersponse\MetadataBundle\Mapping\ModelMetadataFactory;
use Cybersponse\AuthorizationBundle\AuthorizationProvider;
use Cybersponse\CoreBundle\Exception\NotImplementedException;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\Validator\ConstraintViolationListInterface;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Cybersponse\AuthorizationBundle\Ownership\AccessibleTeamIdsProvider;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;
use Symfony\Component\Serializer\Exception\ExceptionInterface as SerializerExceptionInterface;

class AuthorizationResourceController extends ResourceController
{
    const DELTA_DATA = 'data';
    const HTTP_METHOD = 'operation';
    const REMOTE_ADDR = 'source';
    const WEB_SOCKET_SESSION_ID = 'websocketId';
    const COMPONENT = 'component';
    const ENTITY_TYPE = 'entityType';
    const UUID = 'entityUuid';
    const USER_UUID = 'userId';
    const USER = 'user';
    const TRANSACTION_DATE = 'transactionDate';
    const PLAYBOOK_IRI = 'playbookIri';
    const PLAYBOOK_NAME = 'playbookName';
    const TITLE = 'title';
    const API_RESOURCE_COLLECTION = 'api.resource_collection';
    const SELECTIVE_FIELDS = '__selectFields';
    const EVENT_DISPATCHER = 'event_dispatcher';
    const PICKLIST_NAMES_CLASS = 'PicklistNames';
    const PICKLIST_NAMES_RESOURCE = 'picklist_names';
    const NORMALIZER_METHOD = 'json-ld';
    const DISALLOWED_MODULES = ['tenants', 'routers', 'agents', 'comments', 'actors', 'files', 'images'];
    const HYDRA_ERROR = 'hydra-error';
    const TENANT_ROLE_NAME = 'tenant';
    const ELASTIC_SEARCH_OPERATION_KEY = 'esoperation';
    const SYSTEM_MODULE_FOR_REPLICATION = ['StagingModelMetadata', 'Workflow', 'WorkflowCollection'];
    const MESSAGE_MODIFICATION_NOT_ALLOWED = 'Modification of this comments is not permitted.';
    const COMMENT_DELETE_NOT_ALLOWED = 'Deletion of comments not allowed.';
    const COMMENT_DELETE_PERMISSION_REQUIRED = 'Delete comment permission required for comment delete';
    const GENERIC_TEMPLATE_CLASS =  ['Cybersponse\ViewTemplateBundle\Entity\GenericTemplate', 'Cybersponse\ViewTemplateBundle\Entity\Dashboard', 'Cybersponse\ViewTemplateBundle\Entity\Reporting'];

    /**
     * @param $moduleType
     * @return mixed
     */
    public function identifyResourceByShortName($moduleType)
    {
        $resourceClassName = Inflector::singularize(Inflector::classify($moduleType));
        $resource = $this->get(self::API_RESOURCE_COLLECTION)->getResourceForShortName($resourceClassName);
        if ($resource) {
            return $resource;
        }
        $resourceClassName = Inflector::classify($moduleType);
        $resource = $this->get(self::API_RESOURCE_COLLECTION)->getResourceForShortName($resourceClassName);
        if ($resource) {
            return $resource;
        }
        $model = $this->getClassNameByType($moduleType);
        $singular = $model->getDescriptions()['singular'];
        $escapedSingular = preg_replace('/[^a-z0-9\s]+/i', '', $singular);
        $resourceName = \Doctrine\Common\Util\Inflector::classify($escapedSingular);
        $resourceCollection = $this->container->get(self::API_RESOURCE_COLLECTION);
        return $resourceCollection->getResourceForShortName($resourceName);
    }

    /**
     * @param $type
     * @return object
     */
    protected function getClassNameByType($type)
    {
        /** @var ManagerRegistry $managerRegistry */
        $managerRegistry = $this->container->get('doctrine');
        $queryManager = $managerRegistry->getManagerForClass(StagingModelMetadata::class);
        $queryRepository = $queryManager->getRepository(StagingModelMetadata::class);
        return $queryRepository->findOneBy(array('type' => $type));
    }

    /**
     * @param $objectData
     * @param $resource
     * @param $context
     * @return null
     * @throws DeserializationException
     */
    public function createObject($objectData, $resource, $context)
    {
        $object = null;
        $serializer = $this->get('serializer');
        try {
            $object = $serializer->deserialize(
                json_encode($objectData),
                $resource->getEntityClass(),
                self::NORMALIZER_METHOD,
                $context
            );
        } catch (ExceptionInterface $e) {
            throw new DeserializationException($e->getMessage(), $e->getCode(), $e);
        } catch (SerializerExceptionInterface $e) {
            throw new DeserializationException($e->getMessage(), $e->getCode(), $e);
        } catch (\Exception $e) {
            throw new HttpException(Response::HTTP_BAD_REQUEST, $e->getMessage(), $e);
        }
        return $object;
    }

    public function ValidateOwnerShip($request, $item, $resource = null)
    {
        if ($resource) {
            $className = $resource->getEntityClass();
            if (in_array($className, self::GENERIC_TEMPLATE_CLASS)) {
                $permissionType = PermissionsDictionaryBuilder::PERM_CRUD_READ;
                $currentUser = $this->getUser()->getUserID();
                if ($item->getCreateUser()) {
                    $createdUser = $item->getCreateUser()->getUserId();
                }
                if (strcmp($currentUser, $createdUser) == 0 || $this->checkAuthorized($permissionType) || $item->getUuid() == '82bc8209-ef8d-463d-bcd8-db3ac08882cb-463d') {
                    return true;
                } elseif (count($item->getRoles())) {
                    $userRoleIds =  $this->getCurrentUserRoleIds();
                    $recordRoles = $this->getRecordRoleUuid($item);
                    $this->checkTeamIntersection($userRoleIds, $recordRoles);
                    return true;
                } else {
                    throw new AccessDeniedException('Access Denied as record ownership/assignment mishmatch');
                }
            }
        }
        if ($request && $item) {
            $accessibleRecordTeamUuid = $this->getRecordTeamUuid($item);
            $loggedInUserTeamUuid = $this->getLoggedInUserTeamUuid($this->getUser());
            // Check if Record is ownable
            $this->checkTeamIntersection($accessibleRecordTeamUuid, $loggedInUserTeamUuid);
        }
    }

    /**
     * @param ResourceInterface $resource
     * @param int|string $id
     * @param null $request
     * @return object
     */
    public function findOrThrowNotFound(ResourceInterface $resource, $id, $request = null)
    {
        // This will return the entire database object including relationship.
        // Whenever an object is retrieved from DB entire object along with relationship is retrieved.
        $item = parent::findOrThrowNotFound($resource, $id);
        $this->ValidateOwnerShip($request, $item, $resource);
        return $item;
    }
    protected function getRecordTeamUuid($object)
    {
        $record_team = [];
        if (method_exists($object, 'getOwners') && $object->getOwners() != null && $object->getOwners() != []) {
            $acc = new AccessibleTeamIdsProvider();
            $record_team = $acc->getParentDataForTeam($object->getOwners());
        }
        return $record_team;
    }

    protected function getRecordRoleUuid($object)
    {
        $record_roles = [];
        if (method_exists($object, 'getRoles') && $object->getRoles() != null && $object->getRoles()->toArray() != []) {
            foreach ($object->getRoles()->toArray() as $key => $value) {
                array_push($record_roles, $value->getUuid());
            }
        }
        return $record_roles;
    }
    public function getCurrentUserRoleIds()
    {
        $roles = $this->getUser()->getRoles();
        $roleIds = array();
        if (!is_null($roles)) {
            foreach ($roles as $role) {
                array_push($roleIds, $role->getUuid());
            }
        }
        return $roleIds;
    }

    private function checkAuthorized($permissionType)
    {
        $authorizationChecker = $this->container->get('security.authorization_checker');
        if (!$authorizationChecker->isGranted(['perm:security.' . $permissionType])) {
            return false;
        }
        return true;
    }

    protected function getLoggedInUserTeamUuid($object)
    {
        $record_team = [];
        if (method_exists($object, 'getTeams') && $object->getTeams() != null && $object->getTeams() != []) {
            foreach ($object->getTeams() as $team) {
                array_push($record_team, $team->getUuid());
            }
        }
        return $record_team;
    }

    /**
     * @param $item
     * @param $normalizedTeamData
     * @param $normalizedItemData
     */
    protected function checkTeamIntersection($accessibleRecordTeamUuid, $loggedInUserTeamUuid)
    {
        if (count($accessibleRecordTeamUuid) && count($loggedInUserTeamUuid)) {
            // Checking if Record owning teams and logged in user teams have common teams
            if (!count(array_intersect($accessibleRecordTeamUuid, $loggedInUserTeamUuid))) {
                throw $this->createAccessDeniedException();
            }
        }
    }

    /**
     * @param $updatedObject
     * @param $previousObject
     * @param $method
     * @param array $changeData
     * @return array
     */
    protected function prepareChangeData(
        $updatedObject,
        $previousObject,
        $method,
        $changeData = [],
        $title = null
    ) {
        $delta_data = [];
        $delta_data['oldData'] = $previousObject;
        $delta_data['newData'] = $updatedObject;
        $delta_data['changeData'] = $changeData;
        $userName = $this->extractUserName();
        $entityType = ARCUtils::extractEntityType($previousObject, $updatedObject, "@type");
        $entityIId = explode('/', ARCUtils::extractEntityType($previousObject, $updatedObject, "@id"));
        $entityId = end($entityIId);
        return [
            self::DELTA_DATA => $delta_data,
            self::HTTP_METHOD => $method,
            self::REMOTE_ADDR => $_SERVER['REMOTE_ADDR'],
            self::WEB_SOCKET_SESSION_ID => isset($_SERVER['HTTP_WEBSOCKET_SESSIONID']) ? $_SERVER['HTTP_WEBSOCKET_SESSIONID'] : null,
            self::COMPONENT => 'crudhub',
            self::ENTITY_TYPE => $entityType,
            self::UUID => $entityId,
            self::USER_UUID => $this->getUser()->getUuid(),
            self::TRANSACTION_DATE => (int) ($_SERVER['REQUEST_TIME_FLOAT'] * 1000),
            self::USER => $userName,
            self::PLAYBOOK_IRI => isset($_SERVER['HTTP_X_CYOPS_PLAYBOOKIRI']) ? $_SERVER['HTTP_X_CYOPS_PLAYBOOKIRI'] : null,
            self::PLAYBOOK_NAME => isset($_SERVER['HTTP_X_CYOPS_PLAYBOOKNAME']) ? $_SERVER['HTTP_X_CYOPS_PLAYBOOKNAME'] : null,
            self::TITLE => $title
        ];
    }

    /**
     * @return string
     */
    private function extractUserName()
    {
        if (method_exists($this->getUser(), "getfirstname")) {
            if (method_exists($this->getUser(), "getlastname")) {
                $userName = $this->getUser()->getfirstname() . ' ' . $this->getUser()->getlastname();
            } else {
                $userName = $this->getUser()->getfirstname();
            }
        } else {
            $userName = $this->getUser()->getname();
        }
        return $userName;
    }

    /**
     * @param $updatedObject
     * @param $resource
     * @param $request_body
     */
    public function publishPostData($updatedObject, $resource, $request_body)
    {
        $utilsObj = new Utils();
        //Send Data to PostMan For Replication
        $diffObj = $utilsObj->prepareReplicationDiffData($updatedObject);
        $utilsObj->replicateData($this, $resource, $request_body, [$diffObj], 'POST');

        // Sending Data to Audit Log and ES
        $data = $this->prepareChangeData($updatedObject, (object) null, 'Create');
        $data[self::ELASTIC_SEARCH_OPERATION_KEY] = 'index';
        $utilsObj->sendData($this, [$data]);
    }

    /**
     * @param $updatedObject
     * @param $preUpdatedObject
     * @param $resource
     * @param $request_body
     * @param $isDataConflict
     * @param $objectData
     */
    public function publishPutData($updatedObject, $preUpdatedObject, $resource, $request_body, $isDataConflict, $objectData)
    {
        $utilsObj = new Utils();
        $data = $this->prepareChangeData($updatedObject, $preUpdatedObject, 'Update', $objectData);
        $data[self::ELASTIC_SEARCH_OPERATION_KEY] = 'modify';
        $utilsObj->sendData($this, [$data]);

        $diffObj = $utilsObj->prepareReplicationDiffData($updatedObject, $preUpdatedObject);
        $utilsObj->replicateData($this, $resource, $request_body, [$diffObj], 'PUT', $isDataConflict);
    }

    /**
     * @param $resource
     * @param $object
     * @param $request
     * @return bool|string
     */
    protected function isPicklistAlreadyUsed($resource, $object, $request)
    {
        $picklistName = "";
        if ($resource->getModelType() == self::PICKLIST_NAMES_RESOURCE) {
            $picklistName = $object->getName();
        } elseif ($resource->getModelType() == "picklists") {
            $jsonObject = json_decode($request->getContent(), true);
            $picklistName = $jsonObject['parent-picklist-title'];
        }

        if ($picklistName == "") {
            return false;
        }

        /** @var ModelMetadataFactory $modelMetadataFactory */
        $modelMetadataFactory = $this->container->get('cybersponse.metadata.model_metadata_factory');
        $modulesWithPicklist = $modelMetadataFactory->getModulesForPicklist($picklistName);
        if (count($modulesWithPicklist) > 0) {
            $moduleNames = "";
            foreach ($modulesWithPicklist as $moduleName) {
                $moduleNames .= $moduleName . " , ";
            }
            return substr($moduleNames, 0, -3);
        }
        return false;
    }

    /**
     * @param ResourceInterface $resource
     * @param string $field
     * @param string|array $actions
     */
    protected function denyFieldAccessUnlessGranted($resource, $field, $actions)
    {
        /** @var AuthorizationProvider $authorizationProvider */
        $authorizationProvider = $this->get('cybersponse.authorization.authorization_provider');
        $authorizationProvider->denyFieldAccessUnlessGranted($resource, $field, $actions);
    }

    /**
     * @param ResourceInterface $resource
     * @param string|array $actions
     * @param object|null $object
     */
    protected function denyAccessUnlessGranted(ResourceInterface $resource, $actions, $object = null)
    {
        /** @var AuthorizationProvider $authorizationProvider */
        $authorizationProvider = $this->get('cybersponse.authorization.authorization_provider');
        $authorizationProvider->denyAccessUnlessGranted($resource, $actions, $object);
    }

    /**
     * @param $request_body
     * @param $resource
     * @param $request
     * @param $object
     * @return Response
     */
    protected function createRecord($resource, $request, $object, $request_body)
    {
        /** @var Serializer $serializer */
        $serializer = $this->get('serializer');
        $utilsObj = new Utils();
        $utilsObj->setTenantObject($this, $resource, $request_body, $object);
        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_CREATE, new DataEvent($resource, $object));

        $existing_set_asJson = ARCUtils::setAsJson($request, false);
        $existing_relationship = ARCUtils::setRelationShip($request, false);
        $updatedObject = $serializer->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext());
        ARCUtils::setRelationShip($request, $existing_relationship);
        ARCUtils::setAsJson($request, $existing_set_asJson);
        $this->publishPostData($updatedObject, $resource, $request_body);
        return $object;
    }

    /**
     * @inheritdoc
     */
    public function cpostAction(Request $request, $file = null)
    {
        $request_body = ARCUtils::extractRequestBody($request, $file);
        ARCUtils::validateRequestData($request_body);
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_CREATE);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_CREATE, new DataEvent($resource, null));
        if (array_key_exists("__link", $request_body) || array_key_exists("__unlink", $request_body)) {
            $request_body = ARCUtils::mergePostRequest($request_body);
        }
        $object = $this->createObject($request_body, $resource, $resource->getDenormalizationContext());
        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_CREATE_VALIDATION, new DataEvent($resource, $object));
        $violations = $this->get('validator')->validate($object, null, $resource->getValidationGroups());
        if (0 === count($violations)) {
            $object = $this->createRecord($resource, $request, $object, $request_body);
            return $this->getSuccessResponse($resource, $object, Response::HTTP_CREATED);
        }
        return $this->getErrorResponse($violations);
    }

    /**
     * @param ResourceInterface $resource
     * @param array|object $data
     * @param int $status
     * @param array $headers
     * @param array $additionalContext
     * @param array $__selectFields
     * @return Response
     */
    protected function getSuccessResponse(
        ResourceInterface $resource,
        $data,
        $status = 200,
        array $headers = [],
        array $additionalContext = [],
        array $__selectFields = [],
        $passwordBlock = true
    ) {
        if ($__selectFields) {
            $__selectFields = array_unique(array_merge($__selectFields, ['id', '@id', '@type']), SORT_REGULAR);
        }
        $normalized_data = $this->get('serializer')->normalize(
            $data,
            self::NORMALIZER_METHOD,
            $resource->getNormalizationContext() + $additionalContext + [
                'required_attributes' => $__selectFields,
                'block_password' => $passwordBlock
            ]
        );

        $response = new Response(
            $normalized_data,
            $status,
            $headers
        );

        if ($status == Response::HTTP_CREATED) {
            $iri = $this->get('api.iri_converter')->getIriFromItem($data);
            $response->headers->set('Location', $iri);
        }

        return $response;
    }

    /**
     * @param array $resData
     * @param array $__selectFields
     * @return array trimData
     */
    public function trimOutput($resData, $__selectFields)
    {
        if (isset($resData['hydra:member'])) {
            foreach ($resData['hydra:member'] as $outCount => $member) {
                $temp = array_intersect_key($member, array_flip($__selectFields));
                $resData['hydra:member'][$outCount] = $temp;
            }
        } else {
            return array_intersect_key($resData, array_flip($__selectFields));
        }
        return $resData;
    }

    /**
     * @inheritdoc
     */
    protected function getErrorResponse(ConstraintViolationListInterface $violations, $throwError = false)
    {
        $clientErrorCode = Response::HTTP_BAD_REQUEST;

        foreach ($violations as $violation) {
            if ($violation->getConstraint() instanceof UniqueEntity) {
                $clientErrorCode = Response::HTTP_CONFLICT;
            }
        }
        if ($throwError) {
            throw new HttpException($violations);
        }
        return new Response($this->get('serializer')->normalize($violations, self::HYDRA_ERROR), $clientErrorCode);
    }

    /**
     * @inheritdoc
     */
    public function cgetAction(Request $request ,$permission = [PermissionsDictionaryBuilder::PERM_CRUD_READ],$context=[])
    {
        $__selectFields = [];
        if (array_key_exists('$' . self::SELECTIVE_FIELDS, $_GET)) {
            $__selectFields = explode(',', $_GET['$' . self::SELECTIVE_FIELDS]);
            $request->query->remove(self::SELECTIVE_FIELDS);
        }
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, $permission);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_RETRIEVE_LIST, new DataEvent($resource, null));
        $blockPassword = Utils::isPasswordBlock($request);
        $data = $this->getCollectionData($resource, $request);

        if (
            $request->get($this->container->getParameter('api.collection.pagination.page_parameter_name')) &&
            0 === count($data)
        ) {
            throw $this->createNotFoundException();
        }

        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::RETRIEVE_LIST, new DataEvent($resource, $data));
        $context['request_uri'] = $request->getRequestUri();
        $context['permission'] = $permission;
        return $this->getSuccessResponse(
            $resource,
            $data,
            Response::HTTP_OK,
            [],
            $context,
            $__selectFields,
            $blockPassword
        );
    }

    /**
     * @inheritdoc
     */
    public function getAction(Request $request, $id)
    {
        $__selectFields = [];
        if (array_key_exists(self::SELECTIVE_FIELDS, $_GET)) {
            $__selectFields = explode(',', $_GET[self::SELECTIVE_FIELDS]);
            $request->query->remove(self::SELECTIVE_FIELDS);
        }
        $blockPassword = Utils::isPasswordBlock($request);
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
        $object = $this->findOrThrowNotFound($resource, $id, $request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ, $object);
        return $this->getSuccessResponse(
            $resource,
            $object,
            200,
            [],
            [],
            $__selectFields,
            $blockPassword
        );
    }

    protected function get_object_keys($objectData)
    {
        $keys = array_keys($objectData);
        if (isset($objectData['__link'])) {
            foreach ($objectData['__link'] as $key => $link) {
                array_push($keys, $key);
            }
        }
        if (isset($objectData['__unlink'])) {
            foreach ($objectData['__unlink'] as $key => $link) {
                array_push($keys, $key);
            }
        }
        array_push($keys, "tags");
        return $keys;
    }

    protected function updateRecord($request, $resource, $object, $objectData, $request_body)
    {
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE, $object);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_UPDATE_DESERIALIZE, new DataEvent($resource, $object));

        $context = $resource->getDenormalizationContext();
        $context['object_to_populate'] = $object;

        $entityClass = explode('\\', $resource->getEntityClass());
        if (end($entityClass) != self::PICKLIST_NAMES_CLASS) {
            $existing_set_asJson = ARCUtils::setAsJson($request, false);
        }

        /** @var Serializer $serializer */
        $serializer = $this->get('serializer');
        $__selectedFields = $this->get_object_keys($objectData);
        $preUpdatedObject = $serializer->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext() + ['required_m2m_attributes' => $__selectedFields]);
        $utilsObj = new Utils();
        $isDataConflict = $utilsObj->checkForConflicts($this, $objectData, $preUpdatedObject, isset($request_body->{'__replicateData'}));

        if (array_key_exists("__link", $objectData) || array_key_exists("__unlink", $objectData)) {
            $objectData = ARCUtils::mergeRequest($objectData, $preUpdatedObject);
        }
        if ($isDataConflict) {
            $objectData = array('conflict' => false);
        }

        $object = $this->createObject($objectData, $resource, $context);

        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_UPDATE_VALIDATION, new DataEvent($resource, $object));

        $violations = $this->get('validator')->validate($object, null, $resource->getValidationGroups());
        if (0 === count($violations)) {
            // Validation succeed
            if (end($entityClass) != self::PICKLIST_NAMES_CLASS) {
                $existing_set_asJson = ARCUtils::setAsJson($request, $existing_set_asJson);
            }

            $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_UPDATE, new DataEvent($resource, $object, $preUpdatedObject));

            if (end($entityClass) != self::PICKLIST_NAMES_CLASS) {
                $existing_set_asJson = ARCUtils::setAsJson($request, $existing_set_asJson);
            }
            //Fix for bug#15804
            //Before sending verify that its related to attachment owners
            $updatedObject = $serializer->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext() + ['required_m2m_attributes' => $__selectedFields]);
            $owners = (object) null;
            if (method_exists($object, 'getOwners')) {
                $owners = $object->getOwners();
            }
            $this->updateFileFromAttachment($resource, $updatedObject, $owners);

            if ($isDataConflict) {
                $preUpdatedObject['conflict'] = true;
            }
            $this->publishPutData($updatedObject, $preUpdatedObject, $resource, $request_body, $isDataConflict, $objectData);
            if (end($entityClass) != self::PICKLIST_NAMES_CLASS) {
                ARCUtils::setAsJson($request, $existing_set_asJson);
            }
            return $object;
        }
        return $violations;
    }

    /**
     * @inheritdoc
     */
    public function putAction(Request $request, $id, $returnObjectOnly = false)
    {
        $resource = $this->getResource($request);
        $request_body = json_decode($request->getContent());
        ARCUtils::updateReferenceToIriAndCheckForRelation($request_body);
        ARCUtils::validateRequestData($request_body);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);
        $objectData = json_decode(json_encode($request_body), true);
        $object = $this->findOrThrowNotFound($resource, $id, $request);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_UPDATE, new DataEvent($resource, null));
        $object = $this->updateRecord($request, $resource, $object, $objectData, $request_body);
        if (get_class($object) === "Symfony\Component\Validator\ConstraintViolationList") {
            return $this->getErrorResponse($object);
        }
        if ($returnObjectOnly) {
            return $object;
        }
        return $this->getSuccessResponse($resource, $object);
    }

    /**
     * @param $resource
     * @param $object
     * @param $owners
     * @return array
     */
    protected function updateFileFromAttachment($resource, $object, $owners)
    {
        $fileId = null;
        $fileObj = [];
        foreach ($object as $key => $value) {
            if (isset($value['@type']) && $value['@type'] == "File") {
                $fileId = substr(strrchr($value['@id'], '/'), 1);
                if (!empty($fileId) && $fileId !== null) {
                    $managerRegistry = $this->container->get('doctrine');
                    $manager = $managerRegistry->getManagerForClass(\Cybersponse\FilesBundle\Entity\File::class);
                    $fileProvider = $manager->getRepository(\Cybersponse\FilesBundle\Entity\File::class);
                    $fileObject = $fileProvider->find($fileId);
                    $ownerArray = array();
                    foreach ($fileObject->getOwners()->getvalues() as $objectOwner) {
                        array_push($ownerArray, $objectOwner->getUuid());
                    }
                    foreach ($owners as $owner) {
                        if (in_array($owner->getUUID(), $ownerArray)) {
                            $pos = array_keys($ownerArray, $owner->getUUID());
                            unset($ownerArray[$pos[0]]);
                        } else {
                            $fileObject->addOwner($owner);
                        }
                    }
                    foreach ($fileObject->getOwners()->getvalues() as $objectOwner) {
                        if (in_array($objectOwner->getUUID(), $ownerArray)) {
                            $fileObject->removeOwner($objectOwner);
                        }
                    }
                    $manager->persist($fileObject);
                    $manager->flush();
                    return $fileObject;
                }
            }
        }
        return $fileObj;
    }

    /**
     * @inheritdoc
     */
    public function deleteAction(Request $request, $id)
    {
        $request_body = json_decode($request->getContent());
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE);
        $object = $this->findOrThrowNotFound($resource, $id, $request);
        $moduleNames = $this->isPicklistAlreadyUsed($resource, $object, $request);

        if ($moduleNames) {
            $picklistDeleteMessage = $this->container->getParameter('picklist_cannot_be_deleted');
            return new Response(sprintf($picklistDeleteMessage, $moduleNames), Response::HTTP_FORBIDDEN);
        }

        /** @var Serializer $serializer */
        $serializer = $this->get('serializer');
        $existing_relationship = ARCUtils::setRelationShip($request, false);
        $existing_set_asJson = ARCUtils::setAsJson($request, false);
        $previousObject = $serializer->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext());
        ARCUtils::setRelationShip($request, $existing_relationship);
        ARCUtils::setAsJson($request, $existing_set_asJson);

        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_DELETE, new DataEvent($resource, null));
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE, $object);
        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_DELETE, new DataEvent($resource, $object));

        $this->removeFileFromAttachment($resource, $previousObject);
        $data = $this->prepareChangeData((object) null, $previousObject, 'Delete');
        $utilsObj = new Utils();
        $data[self::ELASTIC_SEARCH_OPERATION_KEY] = 'delete';
        $utilsObj->sendData($this, [$data]);

        // MSSP logic to replicate data
        $diffObj = $utilsObj->prepareReplicationDiffData([], $previousObject);
        $utilsObj->replicateData($this, $resource, $request_body, [$diffObj], 'DELETE');

        //purposefully removed multi-picklist after sending notifications to audit and notification server
        //as we do not want it to be recorded in logs

        return new Response(null, Response::HTTP_NO_CONTENT);
    }

    /**
     * @param $resource
     * @param $object
     * @return mixed
     */
    protected function removeFileFromAttachment($resource, $object)
    {
        $metadataEntityManager = $this->container->get('doctrine.orm.entity_manager');
        $fileRepo = $metadataEntityManager->getRepository(File::class);
        foreach ($object as $key => $value) {
            if (isset($value['@type']) && $value['@type'] == "File") {
                $fileId = substr(strrchr($value['@id'], '/'), 1);
                if (!empty($fileId) && $fileId !== null) {
                    $file = $fileRepo->findOneBy(array("uuid" => $fileId));
                    if ($file) {
                        $this->get('cybersponse.files.provider.files')->removeFiles($file);
                    }
                    return $object;
                }
            } 
        }
        return $object;
    }

    /**
     * @param Request $request
     * @param string $id
     * @param string $field
     * @return Response
     */
    public function getFieldAction(Request $request, $id, $field)
    {
        // Extract selective field if any
        $__selectFields = [];
        if (array_key_exists(self::SELECTIVE_FIELDS, $_GET)) {
            $__selectFields = explode(',', $_GET[self::SELECTIVE_FIELDS]);
            $request->query->remove(self::SELECTIVE_FIELDS);
        }

        // Identify if the requested resource is owners
        $ignore_rbac = $field === 'owners';

        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_RETRIEVE, new DataEvent($resource, null));

        /** @var BaseEntity $object */
        $object = $this->findOrThrowNotFound($resource, $id, $request);

        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ, $object);
        // Check Field Level Permissions
        $this->denyFieldAccessUnlessGranted($resource, $field, PermissionsDictionaryBuilder::PERM_FIELD_READ);

        /** @var ManagerRegistry $managerRegistry */
        $managerRegistry = $this->container->get('doctrine');
        $manager = $managerRegistry->getManagerForClass($resource->getEntityClass());
        $classMetadata = $manager->getClassMetadata($resource->getEntityClass());
        if ($classMetadata->hasAssociation($field)) {
            //todo check if ORM ClassMetadata?
            /** @var ClassMetadata $classMetadata */
            $association = $classMetadata->associationMappings[$field];
            if (in_array($association['type'], [ClassMetadata::MANY_TO_MANY, ClassMetadata::ONE_TO_MANY, ClassMetadata::TO_MANY])) {
                if ($association['isOwningSide']) {
                    $inversedField = $association['inversedBy'];
                } else {
                    $inversedField = $association['mappedBy'];
                }

                $targetEntity = $association['targetEntity'];
                $associationResource = $this->get(self::API_RESOURCE_COLLECTION)->getResourceForEntity($targetEntity);

                $request->query->set($inversedField, $object->getUuid());
                //RBAC Issue: 26935
                if ($this->container->hasParameter('ignore_rbac_for_related_record') && $this->container->getParameter('ignore_rbac_for_related_record')) {
                    $GLOBALS['__associatedFieldRequest__'] = true;
                }
                $request->overrideGlobals();
                $associatedEntities = $this->getCollectionData($associationResource, $request);
                $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
                $this->get(self::EVENT_DISPATCHER)->dispatch(Events::RETRIEVE_LIST, new DataEvent($associationResource, $associatedEntities));
                return $this->getSuccessResponse($associationResource, $associatedEntities, Response::HTTP_OK, [], ['request_uri' => $request->getRequestUri(), 'required_attributes' => $__selectFields, 'ignore_field_authorization' => $ignore_rbac, 'ignore_authorization' => $ignore_rbac]);
            }
        }

        throw $this->createNotFoundException();
    }

    /**
     * @param Request $request
     * @param string $id
     * @param string $field
     * @return Response
     * @throws DeserializationException
     */
    public function postFieldAction(Request $request, $id, $field)
    {
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);

        /** @var BaseEntity $object */
        $object = $this->findOrThrowNotFound($resource, $id, $request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE, $object);

        /** @var ManagerRegistry $managerRegistry */
        $managerRegistry = $this->container->get('doctrine');
        $modelManager = $managerRegistry->getManagerForClass($resource->getEntityClass());
        $classMetadata = $modelManager->getClassMetadata($resource->getEntityClass());
        if ($classMetadata->hasAssociation($field)) {
            /** @var ClassMetadata $classMetadata */
            $association = $classMetadata->associationMappings[$field];
            if (in_array($association['type'], [ClassMetadata::MANY_TO_MANY, ClassMetadata::ONE_TO_MANY, ClassMetadata::TO_MANY])) {
                $targetEntity = $association['targetEntity'];
                /** @var ResourceCollectionInterface $resourceCollection */
                $resourceCollection = $this->get(self::API_RESOURCE_COLLECTION);
                $associatedResource = $resourceCollection->getResourceForEntity($targetEntity);

                $this->denyAccessUnlessGranted($associatedResource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);
                try {
                    $associatedObject = $this->get('serializer')->deserialize(
                        $request->getContent(),
                        $associatedResource->getEntityClass(),
                        self::NORMALIZER_METHOD,
                        $associatedResource->getDenormalizationContext()
                    );
                } catch (ExceptionInterface $e) {
                    throw new DeserializationException($e->getMessage(), $e->getCode(), $e);
                } catch (SerializerExceptionInterface $e) {
                    throw new DeserializationException($e->getMessage(), $e->getCode(), $e);
                }

                /** @var Ownable $associatedObject */
                if (in_array(Ownable::class, class_uses($associatedResource->getEntityClass())) && empty($associatedObject->getOwners())) {
                    /** @var Ownable $object */
                    if (in_array(Ownable::class, class_uses($resource->getEntityClass()))) {
                        foreach ($object->getOwners() as $owner) {
                            $associatedObject->addOwner($owner);
                        }
                    } else {
                        $user = $this->getUser();
                        if ($user) {
                            foreach ($user->getTeams()->toArray() as $owner) {
                                $associatedObject->addOwner($owner);
                            }
                        }
                    }
                }

                $this->denyAccessUnlessGranted($associatedResource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE, $associatedObject);

                $fieldName = $association['fieldName'];
                $adderMethod = Inflector::singularize('add' . Inflector::classify($fieldName));
                $object->$adderMethod($associatedObject);

                $modelManager->flush();

                return $this->getSuccessResponse($associatedResource, $associatedObject, Response::HTTP_CREATED);
            }
        }

        throw $this->createNotFoundException();
    }

    /**
     * @param Request $request
     * @param string $id
     * @param string $field
     * @param string $fieldId
     * @return Response
     */
    public function deleteFieldAction(Request $request, $id, $field, $fieldId)
    {
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);

        /** @var BaseEntity $object */
        $object = $this->findOrThrowNotFound($resource, $id, $request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE, $object);

        /** @var ManagerRegistry $managerRegistry */
        $managerRegistry = $this->container->get('doctrine');
        $modelManager = $managerRegistry->getManagerForClass($resource->getEntityClass());
        $classMetadata = $modelManager->getClassMetadata($resource->getEntityClass());
        if ($classMetadata->hasAssociation($field)) {
            //todo check if ORM ClassMetadata?
            /** @var ClassMetadata $classMetadata */
            $association = $classMetadata->associationMappings[$field];
            if (in_array($association['type'], [ClassMetadata::MANY_TO_MANY, ClassMetadata::ONE_TO_MANY, ClassMetadata::TO_MANY])) {
                $targetEntity = $association['targetEntity'];
                /** @var ResourceCollectionInterface $resourceCollection */
                $resourceCollection = $this->get(self::API_RESOURCE_COLLECTION);
                $associatedResource = $resourceCollection->getResourceForEntity($targetEntity);

                $this->denyAccessUnlessGranted($associatedResource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);

                $associatedObject = $this->findOrThrowNotFound($associatedResource, $fieldId);
                $this->denyAccessUnlessGranted($associatedResource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE, $associatedObject);

                $fieldName = $association['fieldName'];
                $methodName = Inflector::singularize('remove' . Inflector::classify($fieldName));
                $object->$methodName($associatedObject);

                $modelManager->flush();

                return new Response(null, Response::HTTP_NO_CONTENT);
            }
        }

        throw $this->createNotFoundException();
    }

    /**
     * Returns a NotImplementedException.
     *
     * This will result in a 501 response code. Usage example:
     *
     *     throw $this->createNotImplementedException('Not Implemented');
     *
     * @param string $message A message
     * @param \Exception|null $previous The previous exception
     *
     * @return NotImplementedException
     */
    public function createNotImplementedException($message = 'Not Implemented', \Exception $previous = null)
    {
        return new NotImplementedException($message, $previous);
    }
}
