<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Controller;

use Cybersponse\UtilsBundle\ARCUtils;
use Dunglas\ApiBundle\JsonLd\Response;
use Symfony\Component\HttpFoundation\Request;
use Cybersponse\SettingsBundle\Entity\SystemSettings;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;

class CommentsController extends AuthorizationResourceController
{

    const RELATIONSHIP_KEY = "\$relationships";
    const ENABLED = 'enabled';

    public function updateComment($request, $id, $resource)
    {
        $request->query->set(self::RELATIONSHIP_KEY, true);
        $object = parent::putAction($request, $id, true);
        $request->query->remove(self::RELATIONSHIP_KEY, true);
        return parent::getSuccessResponse($resource, $object);
    }

    public function getRecordCommentAction(Request $request, $module, $record_id, $id )
    {
        $associatedResource = parent::identifyResourceByShortName($module);
        $this->denyAccessUnlessGranted($associatedResource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
        $moduleRecord = $this->findOrThrowNotFound($associatedResource, $record_id,$request);
        if (count($moduleRecord) == 0){
            throw new HttpException(Response::HTTP_NOT_FOUND);
        }
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
        $commentRecord = $this->findOrThrowNotFound($resource, $id);
        return $this->getSuccessResponse($resource, $commentRecord, Response::HTTP_OK, [], ['ignore_authorization' => true]);
    }

    public function updateAction(Request $request, $id)
    {
        $resource = $this->getResource($request);

        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);
        $managerRegistry = $this->container->get('doctrine');
        $manager = $managerRegistry->getManagerForClass(SystemSettings::class);
        $systemRepository = $manager->getRepository(SystemSettings::class);
        $systemSetting = $systemRepository->findAll();

        $commentSettings  = $systemSetting[0]->getPublicValues()['commentModification'];

        if (!array_key_exists(self::ENABLED, $commentSettings) || !$commentSettings[self::ENABLED]) {
            throw new HttpException(Response::HTTP_NOT_IMPLEMENTED, 'Modification of comments not supported.');
        }
        $object = $this->findOrThrowNotFound($resource, $id, $request);

        if ($object->getIsDeleted()) {
            throw new HttpException(Response::HTTP_FORBIDDEN, self::MESSAGE_MODIFICATION_NOT_ALLOWED);
        }

        $user = $this->getUser()->getUserId();
        if ($object->getCreateUser()->getUserId() == $user && ($object->getModifyDate()->getTimestamp() > $_SERVER['REQUEST_TIME'] - (int) $commentSettings['timeWindow'])) {
            return $this->updateComment($request, $id, $resource);
        } else {
            $authorizationChecker = $this->container->get('security.authorization_checker');
            if (!$authorizationChecker->isGranted(['perm:security.update'])) {
                throw new HttpException(Response::HTTP_FORBIDDEN, self::MESSAGE_MODIFICATION_NOT_ALLOWED);
            } else {
                return $this->updateComment($request, $id, $resource);
            }
        }
    }


    public function deleteAction(Request $request, $id)
    {
        $managerRegistry = $this->container->get('doctrine');
        $manager = $managerRegistry->getManagerForClass(SystemSettings::class);
        $systemRepository = $manager->getRepository(SystemSettings::class);
        $systemSetting = $systemRepository->findAll();
        $commentSettings  = $systemSetting[0]->getPublicValues()['commentModification'];
        if (!array_key_exists(self::ENABLED, $commentSettings) || !$commentSettings[self::ENABLED]) {
            throw new HttpException(Response::HTTP_NOT_IMPLEMENTED, self::COMMENT_DELETE_NOT_ALLOWED);
        }

        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE);

        $object = $this->findOrThrowNotFound($resource, $id, $request);
        $user = $this->getUser()->getUserId();
        if (array_key_exists('softDelete', $commentSettings) && !$commentSettings['softDelete']) {
            if ($object->getCreateUser()->getUserId() == $user && ($object->getModifyDate()->getTimestamp() > $_SERVER['REQUEST_TIME'] - (int) $commentSettings['timeWindow'])) {
                return parent::deleteAction($request, $id);
            } else {
                $authorizationChecker = $this->container->get('security.authorization_checker');
                if (!$authorizationChecker->isGranted(['perm:security.delete'])) {
                    throw new HttpException(Response::HTTP_FORBIDDEN, self::COMMENT_DELETE_PERMISSION_REQUIRED);
                } else {
                    return parent::deleteAction($request, $id);
                }
            }
        } else {

            $serverParam = $request->server->all();
            $serverParam['REQUEST_METHOD'] = 'PUT';
            $request->initialize(
                $request->query->all(),
                $request->request->all(),
                $request->attributes->all(),
                $request->cookies->all(),
                $request->files->all(),
                $serverParam,
                '{"isDeleted":true, "recordTags": []}'
            );
            ARCUtils::setRelationShip($request, false);
            if ($object->getCreateUser()->getUserId() == $user && ($object->getModifyDate()->getTimestamp() > $_SERVER['REQUEST_TIME'] - (int) $commentSettings['timeWindow'])) {
                return parent::putAction($request, $id);
            } else {
                $authorizationChecker = $this->container->get('security.authorization_checker');
                if (!$authorizationChecker->isGranted(['perm:security.delete'])) {
                    throw new HttpException(Response::HTTP_FORBIDDEN, self::COMMENT_DELETE_PERMISSION_REQUIRED);
                } else {
                    return parent::putAction($request, $id);
                }
            }
        }
    }
}
