<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Controller;


use Symfony\Component\HttpFoundation\JsonResponse;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;
use Cybersponse\CoreBundle\Api\ApiEvents;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Dunglas\ApiBundle\Event\Events;
use Dunglas\ApiBundle\Exception\DeserializationException;
use Dunglas\ApiBundle\Exception\ExceptionInterface;
use Dunglas\ApiBundle\JsonLd\Response;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Serializer\Exception\ExceptionInterface as SerializerExceptionInterface;
use Symfony\Component\Serializer\Exception\UnexpectedValueException;

class EtlController extends AuthorizationResourceController
{

    public function batchInsertAction(Request $request)
    {
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_CREATE);
        $this->get('event_dispatcher')->dispatch(ApiEvents::PRE_API_CREATE, new DataEvent($resource, null));

        try {
            $data = $this->denormalize(
                $request->getContent(),
                $resource->getEntityClass(),
                'json-ld',
                $resource->getDenormalizationContext()
            );
        } catch (ExceptionInterface $e) {
            throw new DeserializationException($e->getMessage(), $e->getCode(), $e);
        } catch (SerializerExceptionInterface $e) {
            throw new DeserializationException($e->getMessage(), $e->getCode(), $e);
        }
        $result = array();
        foreach ($data as $key => $val) {
            $this->get('event_dispatcher')->dispatch(Events::PRE_CREATE_VALIDATION, new DataEvent($resource, $val));
            $violations = $this->get('validator')->validate($val, null, $resource->getValidationGroups());
            if (0 === count($violations)) {
                $this->get('event_dispatcher')->dispatch(Events::PRE_CREATE, new DataEvent($resource, $val));
                $item = array();
                $item['status'] = Response::HTTP_CREATED;
                $item['data'] = $this->get('serializer')->normalize(
                    $val,
                    'json-ld',
                    $resource->getNormalizationContext()
                );
                $result[] = $item;
            } else {
                $item = array();
                $item['status'] = Response::HTTP_BAD_REQUEST;
                $item['data'] = $this->get('serializer')->normalize($violations, 'hydra-error');
                $result[] = $item;
            }
        }
        return JsonResponse::create($result);
    }

    protected function getResource(Request $request)
    {
        $data = json_decode($request->getContent(), true);
        return $this->get('api.resource_collection')->getResourceForShortName(preg_replace('/\s+/', '', $data['_resource']));
    }

    private function denormalize($data, $type, $format, array $context = array())
    {
        $serializer = $this->get('serializer');
        if (!$serializer->supportsDecoding($format)) {
            throw new UnexpectedValueException(sprintf('Deserialization for the format %s is not supported', $format));
        }
        $data = $serializer->decode($data, $format, $context);
        $result = array();
        foreach ($data['items'] as $key => $val) {
            $result[$key] = $serializer->denormalize($val, $type, $format, $context);
        }
        return $result;
    }
}
