<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Controller;

use Symfony\Component\HttpFoundation\Request;
use Cybersponse\UtilsBundle\ARCUtils;
use Dunglas\ApiBundle\JsonLd\Response;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;

class GraphNodeController extends AuthorizationResourceController
{
    const MAX_COUNT = 100;
    const MAX_DEPTH = 4;
    protected $visitedNodes = [];
    protected $maxRelation = self::MAX_COUNT;
    protected $depth = self::MAX_DEPTH;
    protected $nodeCounter = 1;
    protected $graph=[];


    public function getNodesAction(Request $request, $moduletype, $id)
    {
        $resource = $this->identifyResourceByShortName($moduletype);
        $request_body = json_decode($request->getContent(), true);
        ARCUtils::setRelationShip($request, false);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
        $object = $this->findOrThrowNotFound($resource, $id, $request);
        $context['relationships'] = True;
        //todo :optimize normalizer , try to remove relationship true
        $normalizedObject = $this->get('serializer')->normalize($object, 'json-ld', $context);
        $this->maxRelation = isset($request_body['count']) ? $request_body['count'] : self::MAX_COUNT;
        $this->depth = isset($request_body['depth']) ? $request_body['depth'] : self::MAX_DEPTH;
        unset($request_body['depth']);
        unset($request_body['count']);
        $this->graph['nodes'] = [];
        $this->graph['edges'] = [];
        $rootnode = [];
        $rootnode['id'] = $normalizedObject['@id'];
        $rootnode['size'] = 30;
        array_push($this->visitedNodes, $normalizedObject['@id']);
        if (isset($request_body[$moduletype]['display_field'])) {
            $rootnode['title'] = (string)$normalizedObject[$request_body[$moduletype]['display_field']];
        } else {
            $rootnode['title'] = (string)$normalizedObject["id"];
        }
        $rootnode['label'] = strlen($rootnode['title']) > 20 ? substr($rootnode['title'],0,20)."..." : $rootnode['title'];
        if (isset($request_body[$moduletype])) {
            $rootnode['data'] = $request_body[$moduletype];
        }
        if (isset($request_body[$moduletype]['picklist']) && isset($normalizedObject[$request_body[$moduletype]['picklist']])) {
            $rootnode['color'] = $normalizedObject[$request_body[$moduletype]['picklist']]['color'];
            unset($rootnode['data']['picklist']);
        } elseif (isset($request_body[$moduletype]['color']) && $request_body[$moduletype]['color'] !== '') {
            $rootnode['color'] = $request_body[$moduletype]['color'];
            unset($rootnode['data']['color']);
        }
        $rootnode['moduleType'] = $moduletype;
        $rootnode['group'] = $moduletype;
        $nodesRelation = [];
        if (isset($rootnode['data']['relationships'])) {
            $nodesRelation = $rootnode['data']['relationships'];
        }
        unset($rootnode['data']['relationships']);
        $rootnode['adjacencies'] = [];
        foreach ($nodesRelation as $rkey => $rvalue) {
            $rvalue = $rvalue['field'];
            if (isset($normalizedObject[$rvalue])) {
                // if no read permission on related field then only 2 fields (@id, type) is available 
                if (is_array($normalizedObject[$rvalue]) && count($normalizedObject[$rvalue]) && count($normalizedObject[$rvalue][0]) > 2) {
                    foreach ($normalizedObject[$rvalue] as $innerkey => $innervalue) {
                        if ($innerkey < $this->maxRelation) {
                            $connectedNode = [];
                            $connectedNode['from'] = $normalizedObject['@id'];
                            $connectedNode['to'] = $innervalue['@id'];
                            $this->addToEdge($connectedNode);
                            $this->getNodes($connectedNode, $request_body, 1);
                            
                        } else {
                            $connectedNode = [];
                            $connectedNode['from'] = $normalizedObject['@id'];
                            $connectedNode['to'] = $this->nodeCounter;
                            $this->nodeCounter = $this->nodeCounter + 1;
                            $connectedNode['moduleTo'] = $innervalue['@id'];
                            $connectedNode['type'] = 'countNode';
                            $connectedNode['showCountNode'] = count($normalizedObject[$rvalue]) - $innerkey;
                            array_push($this->graph['edges'], $connectedNode);
                            $this->getNodes($connectedNode, $request_body, 1);
                            break;
                        }
                    }
                }
            }
        }
        array_push($this->graph['nodes'], $rootnode);
        return new Response($this->graph);
    }


    public function getNodes($connectedNode, $request_body, $level)
    {
        if (isset($connectedNode['type']) && $connectedNode['type'] == 'countNode') {
            $explod = explode('/', $connectedNode['moduleTo']);
            $id = end($explod);
            $rootnode = [];
            $resultArry = [];
            $resource = $this->identifyResourceByShortName($explod[3]);
            $object = $this->findOrThrowNotFound($resource, $id);
            $normalizedObject = $this->get('serializer')->normalize($object, 'json-ld');
            $rootnode['id'] = $connectedNode['to'];
            $rootnode['moduleType'] = $explod[3];
            $rootnode['group'] = $explod[3];
            $rootnode['title'] = strtoupper($explod[3]) . "-(" . (string)$connectedNode['showCountNode'] . "+ ".strtoupper($explod[3])." ) ";
            $rootnode['label'] = strlen($rootnode['title']) > 20 ? substr($rootnode['title'],0,20)."..." : $rootnode['title'];
            if (isset($request_body[$explod[3]])) {
                $rootnode['data'] = $request_body[$explod[3]];
                unset($rootnode['data']['relationships']);
            }
            if (isset($request_body[$explod[3]]['picklist']) && isset($normalizedObject[$request_body[$explod[3]]['picklist']])) {
                $rootnode['color'] = $normalizedObject[$request_body[$explod[3]]['picklist']]['color'];
                unset($rootnode['data']['picklist']);
            } elseif (isset($request_body[$explod[3]]['color']) && $request_body[$explod[3]]['color'] !== '') {
                $rootnode['color'] = $request_body[$explod[3]]['color'];
                unset($rootnode['data']['color']);
            }
            $rootnode['type'] = 'countNode';
            $rootnode['data']['tooltip'] = "There are ".(string)$connectedNode['showCountNode']." more records, however only ".$this->maxRelation." are shown in this view";

            array_push($resultArry, $rootnode);
            return $resultArry;
        } else 
        {
            $explod = explode('/', $connectedNode['to']);
            $id = end($explod);
            $rootnode = [];
            $resultArry = [];
            $resource = $this->identifyResourceByShortName($explod[3]);
            //todo : remove $this->get('cybersponse.authorization.authorization_provider')->isGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ)
            if ($this->get('cybersponse.authorization.authorization_provider')->isGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ)) {
                ++$level;
                $object = $this->findOrThrowNotFound($resource, $id);
                $normalizedObject = $this->get('serializer')->normalize($object, 'json-ld');
                $rootnode['id'] = $normalizedObject['@id'];
                $rootnode['moduleType'] = $explod[3];
                $rootnode['group'] = $explod[3];
                if (isset($request_body[$explod[3]]['display_field'])) {
                    $rootnode['title'] = (string)$normalizedObject[$request_body[$explod[3]]['display_field']];
                } else {
                    $rootnode['title'] = (string)$normalizedObject["id"];
                }
                $rootnode['label'] = strlen($rootnode['title']) > 20 ? substr($rootnode['title'],0,20)."..." : $rootnode['title'];
                if (isset($request_body[$explod[3]])) {
                    $rootnode['data'] = $request_body[$explod[3]];
                    unset($rootnode['data']['relationships']);
                }
                if (isset($request_body[$explod[3]]['picklist']) && isset($normalizedObject[$request_body[$explod[3]]['picklist']])) {
                    $rootnode['color'] = $normalizedObject[$request_body[$explod[3]]['picklist']]['color'];
                    unset($rootnode['data']['picklist']);
                } elseif (isset($request_body[$explod[3]]['color']) && $request_body[$explod[3]]['color'] !== '') {
                    $rootnode['color'] = $request_body[$explod[3]]['color'];
                    unset($rootnode['data']['color']);
                }

                if (isset($request_body[$explod[3]]) && $level < $this->depth) {
                    foreach ($request_body[$explod[3]]['relationships'] as $rvalue) {
                        $rvalue = $rvalue['field'];
                        if (isset($normalizedObject[$rvalue])) {
                            if (is_array($normalizedObject[$rvalue]) && count($normalizedObject[$rvalue])) {
                                foreach ($normalizedObject[$rvalue] as $innerkey => $innervalue) {
                                    if ($innerkey < $this->maxRelation) {
                                        $connectedNode = [];
                                        $connectedNode['from'] = $normalizedObject['@id'];
                                        $connectedNode['to'] = $innervalue['@id'];
                                        $this->addToEdge($connectedNode);
                                        $this->getNodes($connectedNode, $request_body, $level);
                                        
                                    } else {
                                        $connectedNode = [];
                                        $connectedNode['from'] = $normalizedObject['@id'];
                                        $connectedNode['to'] = $this->nodeCounter;
                                        $this->nodeCounter = $this->nodeCounter + 1;
                                        $connectedNode['moduleTo'] = $innervalue['@id'];
                                        $connectedNode['type'] = 'countNode';
                                        $connectedNode['showCountNode'] = count($normalizedObject[$rvalue]) - $innerkey;
                                        array_push($this->graph['edges'], $connectedNode);
                                        $this->getNodes($connectedNode, $request_body, $level);
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }

                if(!in_array($rootnode['id'], $this->visitedNodes)){
                    array_push($this->graph['nodes'], $rootnode);
                    array_push($this->visitedNodes, $rootnode['id']);
                }
            }
        }
    }

    public function addToEdge($edge){
        $found = false;
        foreach ($this->graph['edges'] as $value) {
            if(($value['from'] == $edge['from'] && $value['to'] == $edge['to']) || ($value['from'] == $edge['to'] && $value['to'] == $edge['from'])){
                $found = true;
                break; 
            }
        }
        if(!$found){
            array_push($this->graph['edges'], $edge);
        }
    }
}
