<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

/**
 * Created by PhpStorm.
 * User: Santosh-Diwase
 * Date: 22/12/17
 * Time: 2:54 PM
 */
namespace Cybersponse\CoreBundle\Controller;

use Cybersponse\UtilsBundle\Utils;
use Cybersponse\UtilsBundle\ARCUtils;
use Dunglas\ApiBundle\Event\Events;
use Dunglas\ApiBundle\JsonLd\Response;
use Cybersponse\CoreBundle\Api\ApiEvents;
use Symfony\Component\HttpFoundation\Request;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Cybersponse\SettingsBundle\Entity\SystemSettings;
use Symfony\Component\Config\Definition\Exception\Exception;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;

class MiscApiController extends AuthorizationResourceController
{

    const CLASSNAME = 'workflow';
    const MODULE_TYPE = 'workflow_collections';

    public function getPreviousObject($resource, $request, $serializer, $object)
    {
        $existing_relationship = ARCUtils::setRelationShip($request, false);
        $existing_set_asJson = ARCUtils::setAsJson($request, false);
        $previousObject = $serializer->normalize($object, 'json-ld', $resource->getNormalizationContext());
        ARCUtils::setRelationShip($request, $existing_relationship);
        ARCUtils::setAsJson($request, $existing_set_asJson);
        $this->get('event_dispatcher')->dispatch(ApiEvents::PRE_API_DELETE, new DataEvent($resource, null));
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE, $object);
        $this->get('event_dispatcher')->dispatch(Events::PRE_DELETE, new DataEvent($resource, $object));
        return $previousObject;
    }

    public function sendPublishData($utilsObj, $auditing_data, $replicate_data, $resource, $request_body)
    {
        $utilsObj->sendData($this, $auditing_data);
        $utilsObj->replicateData($this, $resource, $request_body, $replicate_data, 'DELETE');
    }

    protected function getWorkflowOwnersUuid($workflow)
    {
        $owners = [];
        foreach ($workflow->getOwners()->getValues() as $owner) {
            array_push($owners, $owner->getUuid());
        }
        return $owners;
    }

    protected function isPlaybookAccessibleByUser($privatePlaybooks, $visibleTeam)
    {
        foreach ($privatePlaybooks as $workflow) {
            $workflowOwners = $this->getWorkflowOwnersUuid($workflow);
            if (!count(array_intersect($workflowOwners, $visibleTeam))) {
                return false;
            }
        }
        return true;
    }

    public function isCollectionDeleteable($workflowCollectionId, $visibleTeam)
    {
        $utilsObj = new Utils();
        $workflowResource = $this->identifyResourceByShortName(self::CLASSNAME);
        $playbookFilter = ['collection' => $workflowCollectionId, 'isPrivate' => true];
        $privatePlaybooks = $utilsObj->findWithWhere($this, $workflowResource, $playbookFilter);
        if ($privatePlaybooks) {
            return $this->isPlaybookAccessibleByUser($privatePlaybooks, $visibleTeam);
        } else {
            return true;
        }
    }

    /**
     * @return AccessibleTeamIdProvider
     */
    protected function accessibleTeamIdProvider()
    {
        return $this->get('cybersponse.authorization.ownership.accessible_team_ids_provider');
    }

    public function bulkDeleteAction(Request $request, $moduletype)
    {
        $resource = $this->identifyResourceByShortName($moduletype);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE);

        $request_body = json_decode($request->getContent());
        $serializer = $this->get('serializer');
        $ids = $request_body->ids;
        $partial = false;
        $auditing_data = array();
        $replicate_data = array();
        $utilsObj = new Utils();
        if (sizeof($ids) < 1) {
            return new Response("Deleted", Response::HTTP_OK);
        }
        foreach ($ids as $id) {
            $object = null;
            try {
                $object = $this->findOrThrowNotFound($resource, $id, $request);
            } catch (\Exception $e) {
                error_log("Exception occurred while deleting" . $id);
                error_log($e->getMessage());
                $partial = true;
                continue;
            }
            $moduleNames = $this->isPicklistAlreadyUsed($resource, $object, $request);
            if ($moduleNames) {
                return new Response("Unable to delete picklist : being used in " . $moduleNames . " module.", Response::HTTP_FORBIDDEN);
            }

            $previousObject = $this->getPreviousObject($resource, $request, $serializer, $object);
            $this->removeFileFromAttachment($resource, $previousObject);
            $data = $this->prepareChangeData((object)null, $previousObject, 'Delete');
            $data[self::ELASTIC_SEARCH_OPERATION_KEY] = 'delete';
            array_push($auditing_data, $data);

            $diffObj = $utilsObj->prepareReplicationDiffData([], $previousObject);
            array_push($replicate_data, $diffObj);
        }
        $utilsObj->sendData($this, $auditing_data);
        $utilsObj->replicateData($this, $resource, $request_body, $replicate_data, 'DELETE');
        if ($partial) {
            return new Response("Partial Deleted", Response::HTTP_OK);
        }
        return new Response("Deleted", Response::HTTP_OK);
    }

    public function bulkDeleteWorkflowCollectionAction(Request $request)
    {
        $resource = $this->identifyResourceByShortName(self::MODULE_TYPE);
        $accessibleTeamIdprovider = $this->accessibleTeamIdProvider();
        $visibleTeam = $accessibleTeamIdprovider->getDataForCurrentUser();
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE);
        $request_body = json_decode($request->getContent());
        $serializer = $this->get('serializer');
        $ids = $request_body->ids;
        $partial = false;
        $auditing_data = array();
        $replicate_data = array();
        $utilsObj = new Utils();

        if (sizeof($ids) < 1) {
            return new Response("Deleted", Response::HTTP_OK);
        }
        foreach ($ids as $id) {
            $isCollectionDeleteable = $this->isCollectionDeleteable($id, $visibleTeam);
            if (!$isCollectionDeleteable) {
                $partial = true;
                continue;
            }
            $object = null;
            try {
                $object = $this->findOrThrowNotFound($resource, $id, $request);
            } catch (\Exception $e) {
                error_log($e->getMessage());
                $partial = true;
                continue;
            }
            $previousObject = $this->getPreviousObject($resource, $request, $serializer, $object);
            $data = $this->prepareChangeData((object)null, $previousObject, 'Delete');
            $data[self::ELASTIC_SEARCH_OPERATION_KEY] = 'delete';
            array_push($auditing_data, $data);
            $diffObj = $utilsObj->prepareReplicationDiffData([], $previousObject);
            array_push($replicate_data, $diffObj);
        }
        $this->sendPublishData($utilsObj, $auditing_data, $replicate_data, $resource, $request_body);
        if ($partial) {
            return new Response("WorkflowCollection Partial Deleted", Response::HTTP_OK);
        }
        return new Response(Response::HTTP_NO_CONTENT);
    }

    public function bulkDeleteCommentsAction(Request $request)
    {
        $request_body = json_decode($request->getContent(), true);
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE);

        $ids = $request_body['ids'];
        $partial = false;
        $auditing_data = array();
        $replicate_data = array();
        $utilsObj = new Utils();

        if (sizeof($ids) < 1) {
            return new Response("Deleted", Response::HTTP_OK);
        }

        $managerRegistry = $this->container->get('doctrine');
        $manager = $managerRegistry->getManagerForClass(SystemSettings::class);
        $systemRepository = $manager->getRepository(SystemSettings::class);
        $systemSetting = $systemRepository->findAll();
        $commentSettings  = $systemSetting[0]->getPublicValues()['commentModification'];

        if (!array_key_exists('enabled', $commentSettings) || !$commentSettings['enabled']) {
            throw new HttpException(Response::HTTP_NOT_IMPLEMENTED, self::COMMENT_DELETE_NOT_ALLOWED);
        }

        $user = $this->getUser()->getUserId();
        $authorizationChecker = $this->container->get('security.authorization_checker');
        $serializer = $this->get('serializer');

        foreach ($ids as $id) {
            $object = null;
            try {
                $object = $this->findOrThrowNotFound($resource, $id, $request);

                if (array_key_exists('softDelete', $commentSettings) && !$commentSettings['softDelete']) {
                    if ($object->getCreateUser()->getUserId() == $user && ($object->getModifyDate()->getTimestamp() > $_SERVER['REQUEST_TIME'] - (int)$commentSettings['timeWindow'])) {
                        $previousObject = $this->getPreviousObject($resource, $request, $serializer, $object);
                    } else {
                        if (!$authorizationChecker->isGranted(['perm:security.delete'])) {
                            error_log("Deletion of (" . $id . ") this comments not permitted");
                            continue;
                        } else {
                            $previousObject = $this->getPreviousObject($resource, $request, $serializer, $object);
                        }
                    }
                    $data = $this->prepareChangeData((object)null, $previousObject, 'Delete');
                    $data[self::ELASTIC_SEARCH_OPERATION_KEY] = 'delete';
                    array_push($auditing_data, $data);
                    $diffObj = $utilsObj->prepareReplicationDiffData([], $previousObject);
                    array_push($replicate_data, $diffObj);
                } else {

                    $serverParam = $request->server->all();
                    $serverParam['REQUEST_METHOD'] = 'PUT';
                    $request->initialize(
                        $request->query->all(),
                        $request->request->all(),
                        $request->attributes->all(),
                        $request->cookies->all(),
                        $request->files->all(),
                        $serverParam,
                        '{"isDeleted":true}'
                    );
                    if ($object->getCreateUser()->getUserId() == $user && ($object->getModifyDate()->getTimestamp() > $_SERVER['REQUEST_TIME'] - (int)$commentSettings['timeWindow'])) {
                        return parent::putAction($request, $id);
                    } else {
                        if (!$authorizationChecker->isGranted(['perm:security.delete'])) {
                            throw new HttpException(Response::HTTP_FORBIDDEN, self::COMMENT_DELETE_PERMISSION_REQUIRED);
                        } else {
                            return parent::putAction($request, $id);
                        }
                    }
                }
            } catch (Exception $exception) {
                error_log($exception->getMessage());
                $partial = true;
                continue;
            }
        }
        if (array_key_exists('softDelete', $commentSettings) && !$commentSettings['softDelete']) {
            $this->sendPublishData($utilsObj, $auditing_data, $replicate_data, $resource, $request_body);
        }
        if ($partial) {
            return new Response("Comments Partial Deleted", Response::HTTP_OK);
        }
        return new Response(Response::HTTP_NO_CONTENT);
    }
}
