<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */
namespace Cybersponse\CoreBundle\Controller;

use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;
use Cybersponse\UtilsBundle\Utils;
use Cybersponse\UtilsBundle\ARCUtils;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Dunglas\ApiBundle\Event\Events;
use Dunglas\ApiBundle\JsonLd\Response;
use Symfony\Component\Config\Definition\Exception\Exception;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Serializer\Serializer;
use Symfony\Component\HttpKernel\Exception\HttpException;

class MiscApiCreateController extends AuthorizationResourceController
{
    private $successObj = array();
    private $failedObj = array();
    private $badReqObj = array();
    private $auditArray = array();
    private $replicationDataArray = array();
    const OPTION_KEY = 'options';
    const RELATIONSHIP_KEY = 'relationships';
    const CLONE_KEY = 'cloneField';

    public function bulkCreateAction(Request $request, $moduletype)
    {
        $resource = $this->identifyResourceByShortName($moduletype);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_CREATE);
        $objectData = json_decode($request->getContent());
        $selectedFields = $objectData->{self::SELECTIVE_FIELDS} ? $objectData->{self::SELECTIVE_FIELDS}: [];

        if (array_key_exists("data", $objectData)) {
            foreach ($objectData->data as $arr) {
                ARCUtils::updateReferenceToIriAndCheckForRelation($arr);
                $this->createRecord($arr, $resource, $request);
                $arr = json_decode(json_encode($arr), true);
            }
        } elseif (array_key_exists("cloneIds", $objectData)) {
            $objectData = json_decode($request->getContent(), true);
            if (array_key_exists(self::OPTION_KEY, $objectData) && array_key_exists(self::CLONE_KEY, $objectData[self::OPTION_KEY]) && ($objectData[self::OPTION_KEY][self::CLONE_KEY] !== null)) {
                $cloneFieldName = $objectData[self::OPTION_KEY][self::CLONE_KEY];
            }
            foreach ($objectData['cloneIds'] as $cId) {
                if (isset($cId)) {
                    $explodeId = explode('/', $cId);
                    $id = end($explodeId);
                    $object = null;
                    try {
                        $object = $this->findOrThrowNotFound($resource, $id, $request);
                        if (!$object) {
                            array_push($this->failedObj, $id);
                        } else {
                            if (array_key_exists(self::OPTION_KEY, $objectData) && array_key_exists(self::RELATIONSHIP_KEY, $objectData[self::OPTION_KEY]) && $objectData[self::OPTION_KEY][self::RELATIONSHIP_KEY]) {
                                $request->query->set("\$relationships", true);
                            }
                            /** @var Serializer $serializer */
                            $normalized_data = $this->get('serializer')->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext());
                            if (array_key_exists($cloneFieldName, $normalized_data) && $normalized_data[$cloneFieldName]) {
                                $normalized_data[$cloneFieldName] = "copy of " . $normalized_data[$cloneFieldName];
                            } elseif (array_key_exists('name', $normalized_data)) {
                                $normalized_data['name'] = "copy of " . $normalized_data['name'];
                            }
                            unset($normalized_data['id']);
                            if (isset($normalized_data['tenantRecordId'])) {
                                unset($normalized_data['tenantRecordId']);
                            }
                            unset($normalized_data['@id']);
                            if (array_key_exists(self::OPTION_KEY, $objectData) && array_key_exists(self::RELATIONSHIP_KEY, $objectData[self::OPTION_KEY]) && $objectData[self::OPTION_KEY][self::RELATIONSHIP_KEY]) {
                                $request->query->remove("\$relationships", true);
                            }
                            $this->createRecord($normalized_data, $resource, $request);
                        }
                    } catch (Exception $exception) {
                        error_log("Exception occurred while Inserting" . $id);
                        error_log($exception->getMessage());
                        $err = [$exception->getMessage()];
                        array_push($err, $cId);
                        array_push($this->badReqObj, $err);
                        continue;
                    }
                } else {
                    array_push($this->failedObj, $cId);
                }
            }
        } else {
            throw new HttpException(Response::HTTP_BAD_REQUEST, 'Invalid query parameters.');
        }

        $this->publishToRabbitMqChannel($resource, $request);

        $partialUpdatedObject = array();
        $partialUpdatedObject["success"] = $this->successObj;
        $partialUpdatedObject["failure"] = $this->failedObj;
        $partialUpdatedObject["badData"] = $this->badReqObj;
        if ((count($this->badReqObj) == 0) && (count($this->failedObj) == 0)) {
            return $this->getSuccessResponse($resource, $this->successObj, 200, [], [], $selectedFields);
        } elseif (count($this->successObj) == 0) {
            throw new HttpException(Response::HTTP_BAD_REQUEST, 'Invalid query parameters.');
        } else {
            $normalized_success_data = $this->get('serializer')->normalize(
                $this->successObj,
                self::NORMALIZER_METHOD,
                $resource->getNormalizationContext()
            );
            $partialUpdatedObject["success"] = $normalized_success_data;
            return new Response($partialUpdatedObject, Response::HTTP_MULTI_STATUS);
        }
    }

    public function publishToRabbitMqChannel($resource, $request)
    {
        $utilsObj = new Utils();
        $utilsObj->sendData($this, $this->auditArray);

        if (count($this->replicationDataArray) > 0) {
            $utilsObj->replicateData($this, $resource, json_decode($request->getContent()), $this->replicationDataArray, 'POST');
        }
    }

    public function appendToReplicateData($normalized_data)
    {
        $utilsObj = new Utils();
        $data = $this->prepareChangeData($normalized_data, (object)null, 'Create');
        $data[self::ELASTIC_SEARCH_OPERATION_KEY] = 'index';
        array_push($this->auditArray, $data);
        $diffObj = $utilsObj->prepareReplicationDiffData($normalized_data);
        array_push($this->replicationDataArray, $diffObj);
    }

    public function createRecord($insertData, $resource, Request $request)
    {
        $object = null;
        if (array_key_exists("__link", $insertData) || array_key_exists("__unlink", $insertData)) {
            $insertData = ARCUtils::mergePostRequest($insertData);
        }
        try {
            $object = $this->createObject($insertData, $resource, $resource->getDenormalizationContext());
        } catch (Exception $e) {
            $err = [$e->getMessage()];
            array_push($err, $e->getCode());
            array_push($err, $e);
            array_push($this->badReqObj, $err);
            return false;
        }
        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_CREATE_VALIDATION, new DataEvent($resource, $object));
        $violations = $this->get('validator')->validate($object, null, $resource->getValidationGroups());
        $utilsObj = new Utils();
        if (0 === count($violations)) {
            $utilsObj->setTenantObject($this, $resource, $insertData, $object);
            $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_CREATE, new DataEvent($resource, $object));
            array_push($this->successObj, $object);

            $set_relation = ARCUtils::setRelationShip($request, false);
            $existing_set_asJson = ARCUtils::setAsJson($request, false);

            $normalized_data = $this->get('serializer')->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext());
            $this->appendToReplicateData($normalized_data);

            ARCUtils::setRelationShip($request, $set_relation);
            ARCUtils::setAsJson($request, $existing_set_asJson);
            return true;
        }
        array_push($this->failedObj, $insertData);
        return false;
    }
}
