<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

/**
 * Created by PhpStorm.
 * User: Ajahar Inamdar
 * Date: 8/2/18
 */

namespace Cybersponse\CoreBundle\Controller;

use Cybersponse\UtilsBundle\Utils;
use Dunglas\ApiBundle\Event\Events;
use Cybersponse\UtilsBundle\ARCUtils;
use Dunglas\ApiBundle\JsonLd\Response;
use Cybersponse\CoreBundle\Api\ApiEvents;
use Symfony\Component\Serializer\Serializer;
use Symfony\Component\HttpFoundation\Request;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Cybersponse\SettingsBundle\Entity\SystemSettings;
use Symfony\Component\Config\Definition\Exception\Exception;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;

class MiscApiUpdateController extends AuthorizationResourceController
{

    public function extractObjId($obj)
    {
        if (isset($obj['@id'])) {
            $objId = explode('/', $obj['@id']);
            return end($objId);
        } elseif (isset($obj['uuid'])) {
            return $obj['uuid'];
        } else {
            return false;
        }
    }

    /**
     * @param Request $request
     * @param $moduletype
     * @return Response
     * @throws \Dunglas\ApiBundle\Exception\DeserializationException
     */

    public function bulkUpdateAction(Request $request, $moduletype)
    {
        $request_body = json_decode($request->getContent());
        return $this->bulkUpdate($request, $moduletype, $request_body);
    }

    public function bulkUpdate(Request $request, $moduletype, $request_body, $partial = [])
    {
        $updatedBulkObject = array();
        $successStatus = array();
        $failureStatus = array();
        $badReqStatus = array();
        $auditing_data = array();
        $replicationData = array();
        $conflictingData = array();
        $utilsObj = new Utils();

        $resource = $this->identifyResourceByShortName($moduletype);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);
        $context = $resource->getDenormalizationContext();

        $entityClass = explode('\\', $resource->getEntityClass());
        $request_body = json_decode($request->getContent());
        $BulkObj = $request_body->data;
        $selectedFields = $request_body->{self::SELECTIVE_FIELDS} ? $request_body->{self::SELECTIVE_FIELDS} : [];

        /** @var Serializer $serializer */
        $serializer = $this->get('serializer');
        foreach ($BulkObj as $processed_data) {
            ARCUtils::updateReferenceToIriAndCheckForRelation($processed_data);
            $processed_data = json_decode(json_encode($processed_data), true);
            $id = $this->extractObjId($processed_data);
            if ($id) {
                unset($processed_data['@id']);
                try {
                    $object = $this->findOrThrowNotFound($resource, $id, $request);
                    $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE, $object);
                    $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_UPDATE_DESERIALIZE, new DataEvent($resource, $object));
                    $context['object_to_populate'] = $object;

                    if (end($entityClass) != self::PICKLIST_NAMES_CLASS) {
                        $existing_set_asJson = ARCUtils::setAsJson($request, false);
                    }
                    $__selectedFields = $this->get_object_keys($processed_data);
                    $preUpdatedObject = $serializer->normalize($object, 'json-ld', $resource->getNormalizationContext() + ['required_m2m_attributes' => $__selectedFields]);

                    if (array_key_exists("__link", $processed_data) || array_key_exists("__unlink", $processed_data)) {
                        $processed_data = ARCUtils::mergeRequest($processed_data, $preUpdatedObject);
                    }

                    $isDataConflict = $utilsObj->checkForConflicts($this, $processed_data, $preUpdatedObject, isset($request_body->{'__replicateData'}));
                    if ($isDataConflict) {
                        $processed_data = array('conflict' => false);
                    }
                    $object = $this->createObject($processed_data, $resource, $context);
                } catch (Exception $exception) {
                    error_log("Exception occurred while updating" . $id);
                    error_log($exception->getMessage());
                    array_push($failureStatus, $object);
                    continue;
                }
                $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_UPDATE_VALIDATION, new DataEvent($resource, $object));
                $violations = $this->get('validator')->validate($object, null, $resource->getValidationGroups());
                if (0 === count($violations)) {
                    // Validation succeed
                    if (end($entityClass) != self::PICKLIST_NAMES_CLASS) {
                        $existing_set_asJson = ARCUtils::setAsJson($request, $existing_set_asJson);
                    }

                    $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_UPDATE, new DataEvent($resource, $object, $preUpdatedObject));

                    if (end($entityClass) != self::PICKLIST_NAMES_CLASS) {
                        $existing_set_asJson = ARCUtils::setAsJson($request, $existing_set_asJson);
                    }

                    $owners = (object) null;
                    if (method_exists($object, 'getOwners')) {
                        $owners = $object->getOwners();
                    }
                    $updatedObject = $serializer->normalize($object, 'json-ld', $resource->getNormalizationContext() + ['required_m2m_attributes' => $__selectedFields]);
                    $this->updateFileFromAttachment($resource, $updatedObject, $owners);
                    if (end($entityClass) != 'PicklistNames') {
                        ARCUtils::setAsJson($request, $existing_set_asJson);
                    }

                    array_push($successStatus, $object);
                    $data = $this->prepareChangeData($updatedObject, $preUpdatedObject, 'Update', $processed_data);
                    $data[self::ELASTIC_SEARCH_OPERATION_KEY] = 'modify';
                    array_push($auditing_data, $data);

                    if (isset($updatedObject['tenant']) || in_array(end($entityClass), self::SYSTEM_MODULE_FOR_REPLICATION) ) {
                        if ($isDataConflict) {
                            $preUpdatedObject['conflict'] = true;
                        }
                        $diffObj = $utilsObj->prepareReplicationDiffData($updatedObject, $preUpdatedObject);
                        if ($isDataConflict) {
                            array_push($conflictingData, $diffObj);
                        } else {
                            array_push($replicationData, $diffObj);
                        }
                    }
                } else {
                    $processed_data['error'] = $violations[0]->getMessage();
                    array_push($failureStatus, $processed_data);
                }
            } else {
                array_push($badReqStatus, $processed_data);
            }
        }

        $this->publishData($auditing_data, $conflictingData, $replicationData, $resource, $request_body);

        $updatedBulkObject["success"] = $successStatus;
        $partialUpdatedObject = array();
        $partialUpdatedObject["failure"] = $failureStatus;
        if (count($partial)) {
            foreach ($partial as $key => $value) {
                array_push($partialUpdatedObject["failure"], $value);
            }
        }
        $partialUpdatedObject["badData"] = $badReqStatus;
        $partialUpdate = count($BulkObj) - count($updatedBulkObject["success"]) - count($partialUpdatedObject["badData"]);
        $failureUpdate = count($BulkObj) - count($partialUpdatedObject["failure"]);
        if (count($partialUpdatedObject["badData"]) || $partialUpdate) {
            return new Response($partialUpdatedObject, Response::HTTP_MULTI_STATUS);
        } elseif ($failureUpdate == 0) {
            return $this->getErrorResponse($partialUpdatedObject["failure"]);
        } else {
            return $this->getSuccessResponse($resource, $updatedBulkObject["success"], 200, [], [], $selectedFields);
        }
    }

    public function publishData($auditing_data, $conflictingData, $replicationData, $resource, $request_body)
    {
        $utilsObj = new Utils();
        $utilsObj->sendData($this, $auditing_data);
        if (count($conflictingData) > 0) {
            $replicationData = $conflictingData;
            $utilsObj->replicateData($this, $resource, $request_body, $replicationData, 'PUT', true);
        } else {
            $utilsObj->replicateData($this, $resource, $request_body, $replicationData, 'PUT');
        }
    }


    public function bulkCommentUpdateAction(Request $request)
    {
        $request_body = json_decode($request->getContent(), true);
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);
        $managerRegistry = $this->container->get('doctrine');
        $manager = $managerRegistry->getManagerForClass(SystemSettings::class);
        $systemRepository = $manager->getRepository(SystemSettings::class);
        $systemSetting = $systemRepository->findAll();
        $commentSettings  = $systemSetting[0]->getPublicValues()['commentModification'];
        $partial = [];
        $user = $this->getUser()->getUserId();
        if (!array_key_exists('enabled', $commentSettings) || !$commentSettings['enabled']) {
            throw new HttpException(Response::HTTP_NOT_IMPLEMENTED, 'Modification of comments not supported.');
        }
        $modifidedRequest_body['data'] = [];
        foreach ($request_body['data'] as $key => $value) {
            $id = $this->extractObjId($value);
            if ($id) {
                try {
                    $object = $this->findOrThrowNotFound($resource, $id, $request);
                    if ($object->getIsDeleted()) {
                        array_push($partial, $value);
                    }
                    if ($object->getCreateUser()->getUserId() == $user && ($object->getModifyDate()->getTimestamp() > $_SERVER['REQUEST_TIME'] - (int) $commentSettings['timeWindow'])) {
                        array_push($modifidedRequest_body['data'], $value);
                    } else {
                        $authorizationChecker = $this->container->get('security.authorization_checker');
                        if (!$authorizationChecker->isGranted(['perm:security.update'])) {
                            array_push($partial, $value);
                        } else {
                            array_push($modifidedRequest_body['data'], $value);
                        }
                    }
                } catch (\Exception $e) {
                    array_push($partial, $value);
                }
            }
        }
        return $this->bulkUpdate($request, 'comments', json_encode($modifidedRequest_body), $partial);
    }
}
