<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Controller;

use Cybersponse\ActorsBundle\Entity\Actor;
use Dunglas\ApiBundle\JsonLd\Response;
use Symfony\Component\Yaml\Yaml;
use Cybersponse\UtilsBundle\Utils;
use Symfony\Component\HttpFoundation\Request;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;


class PasswordResetController extends AuthorizationResourceController
{

    /**
     * @return string
     */
    private function extractUserId()
    {
        return $this->getUser()->getUserId();
    }

    private function extractEmail($userId)
    {
        return $this->getUser()->getEmail($userId);
    }

    private function extractUuid()
    {
        return $this->getUser()->getUuid();
    }

    public function checkAuthorization($method)
    {
        $authorizationChecker = $this->container->get('security.authorization_checker');
        if ($method === 'POST') {
            if (!$authorizationChecker->isGranted(['perm:security.create']) && !$authorizationChecker->isGranted(['perm:security.update'])) {
                throw new AccessDeniedException();
            }
        }
    }

    public function sendMail($templateId, $dataToReplace, $mailDetails, $integrationClient, $defaultMessage)
    {
        $resource = $this->identifyResourceByShortName('email_template');
        $email_template = [];
        $headers = array(
            "Content-type" => "application/json"
        );
        try {
            $object = $this->findOrThrowNotFound($resource, $templateId);
        } catch (\Exception $error) {
            $object = false;
        }

        if ($object) {
            /** @var Serializer $serializer */
            $email_template = $this->get('serializer')->normalize($object, 'json-ld', $resource->getNormalizationContext());
        } else {
            $email_options = Yaml::parse(file_get_contents('/opt/cyops/configs/cyops-api/mailtemplate.yml'));
            $email_template = $email_options[$defaultMessage];
        }

        if (strpos($email_template['content'],'https://{{@Server_fqhn}}')){
            $email_template['content'] = str_replace('https://{{@Server_fqhn}}', '{{@Server_fqhn}}', $email_template['content']);
        }
        if (strpos($email_template['content'],'https://{{@cyops_hostname}}')){
            $email_template['content'] = str_replace('https://{{@cyops_hostname}}', '{{@cyops_hostname}}', $email_template['content']);
        }
        foreach ($dataToReplace as $key => $value) {
            $email_template['content'] = str_replace($key, $value, $email_template['content']);
        }

        $smtpObject = '{
                            "connector": "smtp",
                            "operation": "send_richtext_email",
                            "params": {
                                "ccc": "",
                                "bcc": "",
                                "content_type":"text/html"
                            }
                        }';

        $jsonDecodedSmtp = json_decode($smtpObject, true);
        $jsonDecodedSmtp['params']['subject'] = $email_template['subject'];
        $jsonDecodedSmtp['params']['content'] = $email_template['content'];
        $jsonDecodedSmtp['params']['to'] = $mailDetails['to'];
        $smtpUri = $this->container->getParameter('integrations.uri');
        $smtpUri = $smtpUri . '/execute/?format=json';
        $smtpRequest = new GuzzleRequest('post', $smtpUri, $headers, json_encode($jsonDecodedSmtp));
        try {
            $integrationClient->send($smtpRequest);
        } catch (\Exception $exception) {
            error_log($exception->getMessage());
            error_log($exception->getCode());
            $errMsg = json_decode($exception->getResponse()->getBody()->read(2048), true);
            return JsonResponse::create($errMsg);
        }
        return true;
    }

    public function resetPasswordAction(Request $request)
    {

        $this->checkAuthorization($request->getMethod());
        $dasUri = $this->container->getParameter('das.uri');
        $proxyProvider = $this->get('cybersponse.core.provider.proxy');
        $dasClient = $proxyProvider->getDasClient();
        $objectData = json_decode($request->getContent(), true);
        $headers = array(
            "Content-type" => "application/json"
        );
        $dasRequests = new GuzzleRequest('post', $dasUri . '/password', $headers, $request->getContent());

        try {
            $dasResponse = $dasClient->send($dasRequests);
            $resCode = $dasResponse->getStatusCode();
        } catch (\Exception $error) {
            error_log($error->getMessage());
            $errVar = json_decode($error->getResponse()->getBody()->read(2048), true);
            $errMsg['message'] = $errVar['Error'];
            $errCode = $error->getCode();
            return JsonResponse::create($errMsg, $errCode);
        }

        if ($resCode < 300) {
            if ($objectData['sendEmail']) {
                $integrationClient = $proxyProvider->getIntegrationClient();
                $dataToReplace = [];
                $mailDetails = [];
                $mailDetails['to'] = $objectData['email'];
                $defaultMessage = 'adminresetpassword';
                $resource = $this->identifyResourceByShortName('people');
                $object = $this->findOrThrowNotFound($resource, $objectData['currentUUID']);
                $item = $this->get('serializer')->normalize($object, 'json-ld', []);
                $title = "Password for " . $item['firstname'] . " " . $item['lastname'] . "[" . $objectData['loginId'] . "] Changed";
                $updateObj = [];
                $updateObj['@type'] = $item['@type'];
                $updateObj['@id'] = $item['@id'];
                $data = parent::prepareChangeData($updateObj, (object)null, 'Event', [], $title);
                $utilsObj = new Utils();
                $utilsObj->sendData($this, [$data]);
                $this->sendMail('2218f5b9-0b17-4f21-91c9-5404696d1908', $dataToReplace, $mailDetails, $integrationClient, $defaultMessage);
            }
            return JsonResponse::create("Password Changed sucessfully");
        }
    }

    public function changePasswordAction(Request $request)
    {

        $dasUri = $this->container->getParameter('das.uri');
        $proxyProvider = $this->get('cybersponse.core.provider.proxy');
        $dasClient = $proxyProvider->getDasClient();
        $objectData = json_decode($request->getContent(), true);
        $headers = array(
            "Content-type" => "application/json"
        );
        $userId = $this->extractUserId();
        $userUuid = $this->extractUuid();
        if ($userUuid === $objectData['uuid']) {
            $dasRequests = new GuzzleRequest('put', $dasUri . '/password', $headers, $request->getContent());
            try {
                $dasResponse = $dasClient->send($dasRequests);
                $resCode = $dasResponse->getStatusCode();
            } catch (\Exception $error) {
                error_log($error->getMessage());
                $errVar = json_decode($error->getResponse()->getBody()->read(2048), true);
                $errMsg['message'] = $errVar['Error'];
                $errCode = $error->getCode();
                return JsonResponse::create($errMsg, $errCode);
            }

            if ($resCode < 300) {
                $sealabClient = $proxyProvider->getProxyClient();
                $dataToReplace = [];
                $mailDetails = [];
                $mailDetails['to'] = $this->extractEmail($userId);
                $defaultMessage = 'changepassword';
                $mailResponse = $this->sendMail('e71ba2df-e4ed-4823-9c28-b86423e9a7eb', $dataToReplace, $mailDetails, $sealabClient, $defaultMessage);
                if ($mailResponse === true) {
                    $sucessMsg['message'] = "Password Changed sucessfully";
                    return JsonResponse::create($sucessMsg);
                } else {
                    $sucessMsg['message'] = "Password Changed sucessfully,mail not sent due to smtp connector failure";
                    return JsonResponse::create($sucessMsg);
                }
            }
        } else {
            error_log("user Token mismatch");
            return JsonResponse::create("user Token mismatch", 400);
        }
    }

    public function sendPasswordMailAction(Request $request)
    {

        $dasUri = $this->container->getParameter('das.uri');
        $proxyProvider = $this->get('cybersponse.core.provider.proxy');
        $dasClient = $proxyProvider->getDasClient();
        $objectData = json_decode($request->getContent(), true);
        $headers = array(
            "Content-type" => "application/json"
        );
        $keyFile = file_get_contents('/opt/cyops-api/app/keys/daspublic.key', true);
        $idKey = base64_encode($keyFile);
        $objectData['idKey'] = $idKey;
        $dasRequests = new GuzzleRequest('post', $dasUri . '/token?uuid=' . $objectData['uuid'], $headers, json_encode($objectData));
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $dasResponseData = $dasResponse->getBody()->getContents();
            $jsonDecodeResponse = json_decode($dasResponseData, true);
            $resCode = $dasResponse->getStatusCode();
        } catch (\Exception $error) {
            error_log($error->getMessage());
            $errVar = json_decode($error->getResponse()->getBody()->read(2048), true);
            $errMsg['message'] = $errVar['Error'];
            $errCode = $error->getCode();
            return JsonResponse::create($errMsg, $errCode);
        }
        $em = $this->getDoctrine()->getManager();
        $actorRepo = $em->getRepository(Actor::class);
        $actor = $actorRepo->findBy(array("userId" => $objectData['uuid']));
        if ($actor) {
            $actorEmail = $actor[0]->getEmail();
        } else {
            throw new HttpException(Response::HTTP_FORBIDDEN, "Actor with user_id '". $objectData['uuid'] ."' does Not exist");
        }

        if ($resCode < 300) {
            $integrationClient = $proxyProvider->getIntegrationClient();
            $dataToReplace = [];
            $dataToReplace['{{vars.userId}}'] = $objectData['loginId'];
            $dataToReplace['{{vars.token}}'] = $jsonDecodeResponse['token'];
            $dataToReplace['{{vars.request.baseUri}}'] = $request->getScheme() . "://" . $request->getHttpHost();
            $dataToReplace['{{@Server_fqhn}}'] = $request->getScheme() . "://" . $request->getHttpHost();
            $dataToReplace['{{@cyops_hostname}}'] = $request->getScheme() . "://" . $request->getHttpHost();
            $mailDetails = [];
            $mailDetails['to'] = $actorEmail;
            $defaultMessage = 'newpassword';
            $mailResponse = $this->sendMail('12f23d8d-505c-4b4b-8911-6595c99b07f0', $dataToReplace, $mailDetails, $integrationClient, $defaultMessage);
            if ($mailResponse === true) {
                $sucessMsg['message'] = "Reset token sent to user";
                return JsonResponse::create($sucessMsg);
            } else {
                $sucessMsg['message'] = "Reset token generated,mail not sent due to smtp connector failure";
                return JsonResponse::create($sucessMsg, 400);
            }
        }
    }
}
