<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Controller;

use Cybersponse\CoreBundle\Entity\ExportJob;
use Cybersponse\CoreBundle\Entity\ExportTemplate;
use Cybersponse\CoreBundle\Entity\ImportJob;
use Doctrine\ORM\EntityManager;
use Dunglas\ApiBundle\JsonLd\Response;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Ramsey\Uuid\Uuid;

class PortingController extends Controller
{

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws DeserializationException
     */
    public function exportStartAction(Request $request)
    {
        $authorizationChecker = $this->container->get('security.authorization_checker');

        if (!$authorizationChecker->isGranted(['perm:application.update']) || !$authorizationChecker->isGranted(['perm:security.update'])) {
            throw new AccessDeniedException();
        }
        $tokenStorage = $this->container->get('security.token_storage');
        $token = $tokenStorage->getToken();
        /** @var EntityManager $entityManager */
        $entityManager = $this->container->get('doctrine')->getManagerForClass(ExportTemplate::class);
        $repository = $entityManager->getRepository(ExportTemplate::class);
        $uuid = $request->get('template');
        $fileName = $request->get('fileName');
        if (!Uuid::isValid($uuid)) {
            throw new \Exception(sprintf('Invalid uuid: %s', $uuid));
        }
        /** @var ExportTemplate $ExportTemplate */
        $exportTemplate = $repository->find($uuid);
        
        $rootDir = $this->container->getParameter('kernel.root_dir');
        $cmd = '';
        if ($exportTemplate) {
            $exportJob = new ExportJob();
            $exportJob->setTemplate($exportTemplate);
            $exportJob->setFileName($fileName);
            $jobUuid = Uuid::uuid4()->toString();
            $exportJob->setUuid($jobUuid);
            $entityManager->persist($exportJob);
            $entityManager->flush();
            $tokenPart = '';
            $tokenType = 'HMAC';
            if (method_exists($this->getUser(), "getfirstname")) {
                $tokenType = 'JWT';
            }
            if (get_class($token->getUser()) == 'Cybersponse\TenantBundle\Entity\Prod\Person') {
                $tokenPart = sprintf('--user-token=%s --token-type=%s', $token->getCredentials(), $tokenType);
            }
            $cmd = sprintf('/usr/bin/php %s/console --env=prod fortisoar:jobs:export --job-uuid=%s %s', $rootDir, $jobUuid, $tokenPart);
        } else {
            $msg = "Export template does not exist - Uuid = " . $uuid;
            throw new HttpException(Response::HTTP_NOT_FOUND, $msg);
        }
        // error_log(print_r($cmd, True), 3, "/var/log/cyops/cyops-api/test.log");
        chdir($rootDir . '/../');
        exec(sprintf('bash -c "exec nohup setsid %s > /var/log/cyops/cyops-api/last_config_export.log 2>&1 &"', $cmd));

        return new JsonResponse([
            '@type' => 'ExportJob',
            'jobUuid' => $jobUuid
        ]);
    }

    /**
     * @param Request $request
     * @param string $id
     * @return JsonResponse
     * @throws DeserializationException
     */
    public function importStartAction(Request $request, $id)
    {
        return $this->runCommand($id, false, [
            "result" => "Started import",
            "progress" => 0
        ]);
    }
    
    /**
     * @param string $id
     * @return JsonResponse
     * @throws DeserializationException
     */
    private function runCommand($id, $dryRun, $successResponse)
    {
        $authorizationChecker = $this->container->get('security.authorization_checker');

        if (!$authorizationChecker->isGranted(['perm:application.update']) || !$authorizationChecker->isGranted(['perm:security.update'])) {
            throw new AccessDeniedException();
        }
        if (!Uuid::isValid($id)) {
            throw new \Exception(sprintf('Invalid uuid: %s', $id));
        }
        $tokenStorage = $this->container->get('security.token_storage');
        $token = $tokenStorage->getToken();
        /** @var EntityManager $entityManager */
        $entityManager = $this->container->get('doctrine')->getManagerForClass(ImportJob::class);
        $repository = $entityManager->getRepository(ImportJob::class);
        /** @var ImportJob $importJob */
        $importJob = $repository->find($id);
        $tokenType = 'HMAC';
        if (method_exists($this->getUser(), "getfirstname")) {
            $tokenType = 'JWT';
        }
        $cmd = '';

        if ($importJob) {
            $tokenPart = '';
            if ($token->getCredentials()) {
                $tokenPart = sprintf('--user-token=%s --token-type=%s', $token->getCredentials(), $tokenType);
            }
            $cmd = sprintf('/usr/bin/php /opt/cyops-api/app/console --env=prod fortisoar:jobs:import --job-uuid=%s %s %s', $id, ($dryRun ? '-o' : ''), $tokenPart);
            // error_log(print_r($cmd, True), 3, "/var/log/cyops/cyops-api/test.log");
        } else {
            $msg = "Import job does not exist - Uuid = " . $id;
            throw new HttpException(Response::HTTP_NOT_FOUND, $msg);
        }
        chdir($this->container->getParameter('kernel.root_dir') . '/../');
        exec(sprintf('bash -c "exec nohup setsid %s > /var/log/cyops/cyops-api/last_config_import.log 2>&1 &"', $cmd));

        return new JsonResponse([
            '@type' => 'ImportJob',
            'status' => $successResponse
        ]);
    }
    
    /**
     * @param Request $request
     * @param string $id
     * @return JsonResponse
     * @throws DeserializationException
     */
    public function importParamsAction(Request $request, $id)
    {
        return $this->runCommand($id, true, [
            "result" => "Import job options populated"
        ]);
    }
}
