<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */
namespace Cybersponse\CoreBundle\Controller;

use Cybersponse\AuthorizationBundle\AuthorizationProvider;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;
use Cybersponse\AuthorizationBundle\Traits\ConditionalOwnable;
use Cybersponse\AuthorizationBundle\Traits\Ownable;
use Cybersponse\CoreBundle\Api\Bulk\BulkApiResults;
use Cybersponse\CoreBundle\Api\Bulk\BulkManagerError;
use Cybersponse\CoreBundle\Api\Bulk\DenormalizationResult;
use Cybersponse\CoreBundle\Serializer\TypeAgnosticLinkedDataSerializer;
use Cybersponse\UtilsBundle\Utils;
use Cybersponse\UtilsBundle\ARCUtils;
use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\Common\Inflector\Inflector;
use Doctrine\Common\Persistence\ObjectManager;
use Dunglas\ApiBundle\Api\ResourceCollectionInterface;
use Dunglas\ApiBundle\Event\DataEvent;
use Dunglas\ApiBundle\Event\Events;
use Dunglas\ApiBundle\Exception\DeserializationException;
use Cybersponse\CoreBundle\Traits\PeerReplicable;
use Dunglas\ApiBundle\JsonLd\Response;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\Serializer\Serializer;
use Symfony\Component\Validator\Validator\ValidatorInterface;
use Symfony\Component\Serializer\Exception\ExceptionInterface as SerializerExceptionInterface;
class TypeAgnosticResourceController extends Controller
{
    const UNAUTHORIZED_COLLECTION_NAME = 'unauthorized';
    const DELTA_DATA = 'data';
    const HTTP_METHOD = 'operation';
    const REMOTE_ADDR = 'source';
    const WEB_SOCKET_SESSION_ID = 'websocketId';
    const COMPONENT = 'component';
    const ENTITY_TYPE = 'entityType';
    const UUID = 'entityUuid';
    const USER_UUID = 'userId';
    const USER = 'user';
    const TRANSACTION_DATE = 'transactionDate';
    const PLAYBOOK_IRI = 'playbookIri';
    const PLAYBOOK_NAME = 'playbookName';
    const REASON = 'reason';
    const CODE = 'code';
    const ACCESS_DENIED_ERROR_FORMAT = 'User must have CREATE permission for %s';
    const OPERATION = 'operation';
    /**
     * @param $previousObject
     * @param $updatedObject
     * @return null
     */
    private function extractEntityType($previousObject, $updatedObject, $identity)
    {
        if (array_key_exists($identity, $updatedObject)) {
            return $updatedObject[$identity];
        } else if (array_key_exists($identity, $previousObject)) {
            return $previousObject[$identity];
        }
        return null;
    }

    /**
     * @param $resourceShortName
     * @return ResourceInterface|null
     */
    public function identifyResourceByShortName($resourceShortName)
    {
        $resourceClassName = Inflector::singularize(Inflector::classify($resourceShortName));
        /** @var ResourceCollectionInterface $resourceCollection */
        $resourceCollection = $this->container->get('api.resource_collection');
        $resource = $resourceCollection->getResourceForShortName($resourceClassName);

        if ($resource === null) {
            $model = $this->getClassNameByType($resourceShortName);
            $singular = $model->getDescriptions()['singular'];
            $escapedSingular = preg_replace('/[^a-z0-9\s]+/i', '', $singular);
            $resourceName = \Doctrine\Common\Util\Inflector::classify($escapedSingular);
            $resource = $resourceCollection->getResourceForShortName($resourceName);
        }
        return $resource;
    }

    /**
     * @param string $type
     * @return StagingModelMetadata|null
     */
    public function getClassNameByType($type)
    {
        /** @var ManagerRegistry $managerRegistry */
        $managerRegistry = $this->container->get('doctrine');
        $queryManager = $managerRegistry->getManagerForClass(StagingModelMetadata::class);
        $queryRepository = $queryManager->getRepository(StagingModelMetadata::class);
        return $queryRepository->findOneBy(array('type' => $type));
    }

    public function isPeerReplicable($entity_class)
    {
        return in_array(PeerReplicable::class, class_uses($entity_class));
    }

    public function isConditionalOwnable($entity_class)
    {
        return in_array(ConditionalOwnable::class, class_uses($entity_class));
    }

    public function isOwnable($entity_class)
    {
        return in_array(Ownable::class, class_uses($entity_class));
    }

    /**
     * @return string
     */
    private function extractUserName()
    {
        if (method_exists($this->getUser(), "getfirstname")) {
            if (method_exists($this->getUser(), "getlastname")) {
                $userName = $this->getUser()->getfirstname() . ' ' . $this->getUser()->getlastname();
            } else {
                $userName = $this->getUser()->getfirstname();
            }
        } else {
            $userName = $this->getUser()->getname();
        }
        return $userName;
    }
    /**
     * @param $updatedObject
     * @param $previousObject
     * @param $method
     * @param array $changeData
     * @return array
     */
    protected function prepareChangeData(
        $updatedObject,
        $previousObject,
        $method,
        $changeData = []
    ) {
        $delta_data = [];
        $delta_data['oldData'] = $previousObject;
        $delta_data['newData'] = $updatedObject;
        $delta_data['changeData'] = $changeData;
        $userName = $this->extractUserName();
        $entityType = ARCUtils::extractEntityType($previousObject, $updatedObject, "@type");
        $entityIId = explode('/', ARCUtils::extractEntityType($previousObject, $updatedObject, "@id"));
        $entityId = end($entityIId);
        return [
            self::DELTA_DATA            => $delta_data,
            self::OPERATION             => 'index',
            self::HTTP_METHOD           => $method,
            self::REMOTE_ADDR           => $_SERVER['REMOTE_ADDR'],
            self::WEB_SOCKET_SESSION_ID => isset($_SERVER['HTTP_WEBSOCKET_SESSIONID'])?$_SERVER['HTTP_WEBSOCKET_SESSIONID']: Null,
            self::COMPONENT             => 'crudhub',
            self::ENTITY_TYPE           => $entityType,
            self::UUID                  => $entityId,
            self::USER_UUID             => $this->getUser()->getUuid(),
            self::TRANSACTION_DATE      => (int)($_SERVER['REQUEST_TIME_FLOAT']*1000),
            self::USER                  => $userName,
            self::PLAYBOOK_IRI          => isset($_SERVER['HTTP_X_CYOPS_PLAYBOOKIRI'])?$_SERVER['HTTP_X_CYOPS_PLAYBOOKIRI']: Null,
            self::PLAYBOOK_NAME         => isset($_SERVER['HTTP_X_CYOPS_PLAYBOOKNAME'])?$_SERVER['HTTP_X_CYOPS_PLAYBOOKNAME']: Null
        ];
    }
    /**
     * @param Request $request
     * @return \Symfony\Component\HttpFoundation\Response|static
     * @throws DeserializationException
     *
     * Authorization and Model Validation are all-or-nothing (if any one model fails authorization or validation, the whole request fails)
     */
    public function createAction(Request $request)
    {
        $auditData = array();
        /** @var Serializer $serializer */
        $serializer = $this->get('serializer');
        /** @var ManagerRegistry $ormManagerRegistry */
        $ormManagerRegistry = $this->container->get('doctrine');
        /** @var ResourceCollectionInterface $resourceCollection */
        $resourceCollection = $this->container->get('api.resource_collection');
        /** @var AuthorizationProvider $authorizationProvider */
        $authorizationProvider = $this->get('cybersponse.authorization.authorization_provider');
        /** @var EventDispatcherInterface $eventDispatcher */
        $eventDispatcher = $this->get('event_dispatcher');
        /** @var ValidatorInterface $validator */
        $validator = $this->get('validator');
        /** @var TypeAgnosticLinkedDataSerializer $typeAgnosticLinkedDataDenormalizer */
        $typeAgnosticLinkedDataDenormalizer = $this->container->get('cybersponse.core.serializer.type_agnostic_linked_data');
        $request->attributes->add(['_resource' => null]); // because Dunglas RequestExceptionListener...
        try {
            $unlinkedDenormalizationResults = $typeAgnosticLinkedDataDenormalizer->deserializeUnlinked($request->getContent());
        } catch (SerializerExceptionInterface $serializerException) {
            throw new DeserializationException($serializerException->getMessage(), $serializerException->getCode(), $serializerException);
        }
        $this->persistResults($ormManagerRegistry, $unlinkedDenormalizationResults);
        $linkedDenormalizationResults = $this->linkDenormalizationResults($serializer, $resourceCollection, $unlinkedDenormalizationResults);
        $unauthorizedResources = $this->checkAuthorization($linkedDenormalizationResults, $resourceCollection, $authorizationProvider);
        if (count($unauthorizedResources) > 0) {
            $errorMessage = sprintf(self::ACCESS_DENIED_ERROR_FORMAT, implode(', ', $unauthorizedResources));
            throw new AccessDeniedHttpException($errorMessage);
        }
        $violations = $this->validateResults($linkedDenormalizationResults, $resourceCollection, $eventDispatcher, $validator);
        if (count($violations) > 0) {
            return new Response($serializer->normalize($violations, 'hydra-error'), Response::HTTP_BAD_REQUEST);
        }
        $objectManagerMap = $this->buildObjectManagerMap($linkedDenormalizationResults, $resourceCollection, $ormManagerRegistry);
        $persistResults = $this->tryFlushObjects($objectManagerMap);
        $responseCode = Response::HTTP_OK;
        if (count($persistResults->getPersisted()) > 0) {
            $responseCode = Response::HTTP_CREATED;
        }
        if (count($persistResults->getFailed()) > 0) {
            // TODO: Determine when to set 409 (or anything else) instead of 400
            $responseCode = Response::HTTP_BAD_REQUEST;
        }
        $request->query->set("\$asJson", true);
        $request->query->set("\$relationships", true);
        $response = $serializer->normalize($persistResults, 'json-ld');
        # Sending Change Data to
        foreach ($response['persisted'] as $item) {
            $preparedData = $this->prepareChangeData($item, (object)null, 'Create');
            $preparedData['esoperation'] = 'index';
            array_push($auditData, $preparedData);
        }
        $utilsObj = new Utils();
        $utilsObj->sendData($this, $auditData);
        return new Response($response, $responseCode);
    }
    /**
     * @param ManagerRegistry $ormManagerRegistry
     * @param DenormalizationResult[] $unlinkedDenormalizationResults
     */
    protected function persistResults(ManagerRegistry $ormManagerRegistry, $unlinkedDenormalizationResults)
    {
        array_map(function (DenormalizationResult $denormalizationResult) use ($ormManagerRegistry) {
            $objectManager = $ormManagerRegistry->getManagerForClass(get_class($denormalizationResult->getResult()));
            $objectManager->persist($denormalizationResult->getResult());
        }, $unlinkedDenormalizationResults);
    }
    /**
     * @param Serializer $serializer
     * @param ResourceCollectionInterface $resourceCollection
     * @param DenormalizationResult[] $unlinkedDenormalizationResults
     * @return DenormalizationResult[]
     */
    protected function linkDenormalizationResults(Serializer $serializer, ResourceCollectionInterface $resourceCollection, $unlinkedDenormalizationResults)
    {
        $linkedDenormalizationResults = array_map(function (DenormalizationResult $denormalizationResult) use ($serializer, $resourceCollection) {
            $resource = $resourceCollection->getResourceForEntity($denormalizationResult->getResult());
            $linkedObject = $serializer->denormalize(
                $denormalizationResult->getFrom(),
                $resource->getEntityClass(),
                'json-ld',
                $resource->getDenormalizationContext() + ['object_to_populate' => $denormalizationResult->getResult()]
            );
            return new DenormalizationResult($denormalizationResult->getFrom(), $linkedObject);
        }, $unlinkedDenormalizationResults);
        return $linkedDenormalizationResults;
    }
    /**
     * @param DenormalizationResult[] $linkedDenormalizationResults
     * @param ResourceCollectionInterface $resourceCollection
     * @param AuthorizationProvider $authorizationProvider
     * @return array
     */
    protected function checkAuthorization($linkedDenormalizationResults, ResourceCollectionInterface $resourceCollection, AuthorizationProvider $authorizationProvider)
    {
        $unauthorizedResources = array_reduce($linkedDenormalizationResults, function ($unauthorizedResources, DenormalizationResult $denormalizationResult) use ($resourceCollection, $authorizationProvider) {
            $resource = $resourceCollection->getResourceForEntity($denormalizationResult->getResult());
            // TODO Think about a small authorization cache here
            if (!$authorizationProvider->isGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_CREATE)) {
                $unauthorizedResources[] = $resource->getShortName();
            }
            return $unauthorizedResources;
        }, []);
        return $unauthorizedResources;
    }
    /**
     * @param DenormalizationResult[] $linkedDenormalizationResults
     * @param ResourceCollectionInterface $resourceCollection
     * @param EventDispatcherInterface $eventDispatcher
     * @param ValidatorInterface $validator
     * @return mixed
     */
    protected function validateResults($linkedDenormalizationResults, ResourceCollectionInterface $resourceCollection, EventDispatcherInterface $eventDispatcher, ValidatorInterface $validator)
    {
        $violations = array_reduce($linkedDenormalizationResults, function ($allViolations, DenormalizationResult $denormalizationResult) use ($resourceCollection, $eventDispatcher, $validator) {
            $resource = $resourceCollection->getResourceForEntity($denormalizationResult->getResult());
            $eventDispatcher->dispatch(Events::PRE_CREATE_VALIDATION, new DataEvent($resource, $denormalizationResult->getResult()));
            $violations = $validator->validate($denormalizationResult->getResult(), null, $resource->getValidationGroups());
            if (0 !== count($violations)) {
                $allViolations[] = [
                    'object' => $denormalizationResult->getFrom(),
                    'violations' => $violations
                ];
            }
            return $allViolations;
        }, []);
        return $violations;
    }
    /**
     * @param DenormalizationResult[] $linkedDenormalizationResults
     * @param ManagerRegistry $ormManagerRegistry
     * @return array
     */
    protected function buildObjectManagerMap($linkedDenormalizationResults, ResourceCollectionInterface $resourceCollection, ManagerRegistry $ormManagerRegistry)
    {
        $objectManagerMap = array_reduce($linkedDenormalizationResults, function ($objectManagerMap, DenormalizationResult $denormalizationResult) use ($resourceCollection, $ormManagerRegistry) {
            $resource = $resourceCollection->getResourceForEntity($denormalizationResult->getResult());
            $manager = $ormManagerRegistry->getManagerForClass(get_class($denormalizationResult->getResult()));
            $managerHash = spl_object_hash($manager);
            if (!array_key_exists($managerHash, $objectManagerMap)) {
                $objectManagerMap[$managerHash] = [
                    'manager' => $manager,
                    'objects' => []
                ];
            }
            $result = $denormalizationResult->getResult();
            if($this->isPeerReplicable($resource->getEntityClass())){
                $utilsObj = new Utils();
                $utilsObj->setTenantObject($this, $resource, $denormalizationResult->getFrom(), $result, true);
            }

            if (($this->isConditionalOwnable($resource->getEntityClass()) && $result->getIsPrivate()) || $this->isOwnable($resource->getEntityClass())) {
                $tokenStorage = $this->container->get('security.token_storage');
                if ($token = $tokenStorage->getToken()) {
                    $owners = $token->getUser()->getTeams()->toArray();
                }
                if (method_exists($result, 'getOwners') && $result->getOwners() === null) {
                    foreach ($owners as $owner) {
                        $result->addOwner($owner);
                    }
                }
            }
            $objectManagerMap[$managerHash]['objects'][] = ['result' => $result, 'from' => $denormalizationResult->getFrom()];
            return $objectManagerMap;
        }, []);
        return $objectManagerMap;
    }
    /**
     * @param array $objectManagerMap
     * @return BulkApiResults
     */
    protected function tryFlushObjects($objectManagerMap)
    {
        $persistResults = array_reduce($objectManagerMap, function (BulkApiResults $bulkApiResults, $objectsAndManager) {
            /** @var ObjectManager $manager */
            $manager = $objectsAndManager['manager'];
            try {
                $objectsToPersist = array_map(function ($object) {
                    return $object['result'];
                }, $objectsAndManager['objects']);
                $manager->flush();
                $bulkApiResults->addPersisted($objectsToPersist);
            } catch (\Exception $exception) {
                $originalData = array_map(function ($object) {
                    return $object['from'];
                }, $objectsAndManager['objects']);
                $bulkManagerError = new BulkManagerError($exception, $originalData);
                $bulkApiResults->addFailed($bulkManagerError);
            }
            return $bulkApiResults;
        }, new BulkApiResults());
        return $persistResults;
    }
}
