<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Controller;

use DOMDocument;
use Ramsey\Uuid\Uuid;
use Doctrine\ORM\EntityManager;
use Symfony\Component\Finder\Finder;
use Cybersponse\UtilsBundle\ARCUtils;
use Dunglas\ApiBundle\JsonLd\Response;
use Cybersponse\FilesBundle\Entity\File;
use Doctrine\Common\Inflector\Inflector;
use Cybersponse\CoreBundle\Entity\Widget;
use Cybersponse\FilesBundle\Entity\PgFile;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Cybersponse\CoreBundle\Controller\AuthorizationResourceController;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;

class WidgetController extends AuthorizationResourceController
{
    const TMP_PATH = '/tmp/widget/';
    const WIDGET_PREVIEW_PATH = '/opt/cyops-ui/widgets_preview/';
    const WIDGET_PATH = '/opt/cyops-ui/widgets/';
    const ASSET_PATH= '/widgetAssets';

    /** @var FileSystem */
    protected $fileSystem;

    
    public function installWidgetAction(Request $request)
    {
        $requestBody = ARCUtils::extractRequestBody($request, null);
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_CREATE);

        if (!$requestBody->name) {
            return new Response('Missing widget name', Response::HTTP_BAD_REQUEST);
        }

        if (!$requestBody->version) {
            return new Response('Missing widget version', Response::HTTP_BAD_REQUEST);
        }
        $client = new \GuzzleHttp\Client();
        try {
            $widgetName = $requestBody->name."-".$requestBody->version;
            $widgetRepoLink = sprintf('https://%s/widgets/%s/%s.tgz', getenv("REPOSERVER"), $widgetName, $widgetName);
            $responseObject = $client->request('GET', $widgetRepoLink, ['verify' => false]);
            $fileData = $responseObject->getBody()->getContents();
        } catch (\Exception $e) {
            throw new HttpException(Response::HTTP_NOT_FOUND, $e->getMessage(), $e);
        }

        $widgetFileName = $requestBody->name.'-'.$requestBody->version;
        $tgzFileName = sprintf('%s.tgz', $widgetFileName);

        $this->unzipWidgetFile($fileData, $tgzFileName, $widgetFileName);
        $replace = $requestBody->replace? $requestBody->replace:false;
        return $this->createWidget($request, $widgetFileName, $operation="install", null, $replace);
    }

    public function uploadWidgetAction(Request $request)
    {
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_CREATE);

        if (!$request->files->has('file')) {
            throw new BadRequestHttpException('No file specified.');
        }

        /** @var UploadedFile $upload */
        $upload = $request->files->get('file');
        $replace = $request->request->get('replace', false);
        if ($replace === "false") {
            $replace =false;
        }
        $fileName = $upload->getClientOriginalName();
        $fileExt = $upload->getClientOriginalExtension();
        $widgetFileName = explode(".".$fileExt, $fileName)[0];

        $openFile = $upload->openFile();
        $readFile = $openFile->fread($openFile->getSize());
        
        $this->unzipWidgetFile($readFile, $fileName, $widgetFileName);
        
        return $this->createWidget($request, $widgetFileName, $operation = "upload", null, $replace);
    }

    public function revertWidgetAction(Request $request, $id)
    {
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);
        /** @var EntityManager $entityManager */
       
        $entityManager = $this->container->get('doctrine')->getManagerForClass(Widget::class);
        $widgetRepository = $entityManager->getRepository(Widget::class);
        $widgetsFileSystem = new Filesystem();
        /** @var Widget $widget */
        $widget = $widgetRepository->find($id);
        if ($widget) {
            $widgetsPath =self::WIDGET_PATH.$widget->getName();
            if ($widgetsFileSystem->exists($widgetsPath)) {
                try {
                    $widgetsHtml = sprintf('%s/html/view.html', $widgetsPath, $widget->getName());
                    $widgetsController = sprintf('%s/js/view.controller.js', $widgetsPath, $widget->getName());
                    $widgetsEditHtml = sprintf('%s/html/edit.html', $widgetsPath, ucfirst($widget->getName()));
                    $widgetsEditController = sprintf('%s/js/edit.controller.js', $widgetsPath, ucfirst($widget->getName()));
                    $widgetsMetadata = sprintf('%s/info.json', $widgetsPath, $widget->getName());
                    $data = [];
                    $data["viewHtml"] = file_get_contents($widgetsHtml);
                    $data["viewController"] = file_get_contents($widgetsController);
                    $data["editHtml"] =  file_get_contents($widgetsEditHtml);
                    $data["editController"] = file_get_contents($widgetsEditController);
                    $data["metaInfo"] = json_decode(file_get_contents($widgetsMetadata));
                    $data["name"] =$data["metaInfo"]->name ;
                    $data["subTitle"] =$data["metaInfo"]->subTitle ;
                    $data["title"] =$data["metaInfo"]->title ;
                    $data["version"] =$data["metaInfo"]->version ;
                    $data['draft'] = false;
                    unset($data["metaInfo"]->name);
                    unset($data["metaInfo"]->subTitle);
                    unset($data["metaInfo"]->title);
                    unset($data["metaInfo"]->version);
                    $data["metadata"] =[];
                    foreach ($data["metaInfo"]->metadata as $key => $value) {
                        $data["metadata"][$key] = $value;
                    } 
                    $files = $this->addWidgetAssets($widgetsPath.'/widgetAssets/');
                    $data['files']=$files;
                    $data['widgetOperation']='revert';
                    $request->initialize(
                        $request->query->all(),
                        $request->request->all(),
                        $request->attributes->all(),
                        $request->cookies->all(),
                        $request->files->all(),
                        $request->server->all(),
                        json_encode($data)
                    );
                    return parent::putAction($request, $id);
                } catch (\Exception $e) {
                    throw new HttpException(Response::HTTP_BAD_REQUEST, $e->getMessage(), $e);
                }
            }
        }
        return new Response($this->container->getParameter("resource_not_found"), Response::HTTP_NOT_FOUND);
    }

    public function downloadWidgetAction(Request $request, $id)
    {
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
        /** @var EntityManager $entityManager */
       
        $entityManager = $this->container->get('doctrine')->getManagerForClass(Widget::class);
        $widgetRepository = $entityManager->getRepository(Widget::class);
        
        /** @var Widget $widget */
        $widget = $widgetRepository->find($id);

        if ($widget) {
            try {
                $widgetFileName = $widget->getName().'-'.$widget->getVersion();
                $this->fileSystem = new Filesystem();
                $util = new ARCUtils();
                $util->buildWidgetFiles($widget, self::TMP_PATH.$widget->getName(), $entityManager);
                if ($this->fileSystem->exists(self::TMP_PATH.$widget->getName())) {
                    $widgetTgz = sprintf("%s/%s.tar", self::TMP_PATH, $widgetFileName);
                    $widgetTgzGz =sprintf("%s/%s.tar.gz", self::TMP_PATH, $widgetFileName);
                    if ($this->fileSystem->exists($widgetTgz)) {
                        $this->fileSystem->remove($widgetTgz);
                    }
                    if ($this->fileSystem->exists($widgetTgzGz)) {
                        $this->fileSystem->remove($widgetTgzGz);
                    }
                    $phar = new \PharData($widgetTgz);
                    $phar->buildFromDirectory(sprintf("%s%s", self::TMP_PATH, $widget->getName()));
                    $phar->compress(\Phar::GZ);
                    $tgzFileData = file_get_contents($widgetTgzGz);
                    $tgzFileSize = filesize($widgetTgzGz);
                }
            } catch (\Exception $e) {
                error_log($e->getMessage());
            } finally {
                $this->fileSystem = new Filesystem();
                $this->removeTmpFiles($widget->getName());
            }

            $response = new Response();
            $response->headers->set('Content-Type', 'application/gzip');
            $response->headers->set('Content-Disposition', 'attachment; filename="' . $widgetFileName . '.tgz' . '"');
            $response->headers->set('filename', $widgetFileName . ".tgz");
            $response->headers->set('Content-Length', $tgzFileSize);
            $response->setContent($tgzFileData);

            return $response;
        }
        return new Response($this->container->getParameter("resource_not_found"), Response::HTTP_NOT_FOUND);
    }

    public function listWidgetAction(Request $request)
    {
        return parent::cgetAction($request,[PermissionsDictionaryBuilder::PERM_CRUD_READ,PermissionsDictionaryBuilder::PERM_CRUD_EXECUTE],['ignore_relationship_authorization' => true,'ignore_field_authorization' => true]);   
    }


    public function previewWidgetAction(Request $request, $id)
    {
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
        /** @var EntityManager $entityManager */
       
        $entityManager = $this->container->get('doctrine')->getManagerForClass(Widget::class);
        $widgetRepository = $entityManager->getRepository(Widget::class);
        
        /** @var Widget $widget */
        $widget = $widgetRepository->find($id);

        if ($widget) {
            try {
                $widgetFileName = $widget->getName().'-'.$widget->getVersion();
                $this->fileSystem = new Filesystem();
                $util = new ARCUtils();
                $util->buildWidgetFiles($widget, self::WIDGET_PREVIEW_PATH.$widget->getName(), $entityManager);
            } catch (\Exception $e) {
                error_log($e->getMessage());
            } finally {
                $this->fileSystem = new Filesystem();
                $this->removeTmpFiles($widget->getName());
            }

            return new Response("Widget preview files generated.", Response::HTTP_OK);
        }
        return new Response($this->container->getParameter("resource_not_found"), Response::HTTP_NOT_FOUND);
    }
    public function widgetCloneAction(Request $request)
    {
        $requestBody = ARCUtils::extractRequestBody($request, null);

        if (!$requestBody->widgetCloneId || !$requestBody->widgetTitle) {
            throw new BadRequestHttpException('Widget Clone ID and Widget Name are mandatory.');
        }

        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_CREATE);
        /** @var EntityManager $entityManager */
       
        $entityManager = $this->container->get('doctrine')->getManagerForClass(Widget::class);
        $widgetRepository = $entityManager->getRepository(Widget::class);
        
        /** @var Widget $widget */
        $widget = $widgetRepository->find($requestBody->widgetCloneId);
        $widgetExisting = $widgetRepository->findOneBy(array("name" => $requestBody->widgetTitle));
        if ($widgetExisting) {
            return new Response($this->container->getParameter("resource_exists_already"), Response::HTTP_CONFLICT);
        }
        if ($widget) {
            return $this->createWidget($request, $requestBody, $operation = "clone", $widget, true);
        }

        return new Response($this->container->getParameter("resource_not_found"), Response::HTTP_NOT_FOUND);
    }

    protected function createWidget($request, $widgetRequestBody, $operation = "upload", $widgetRecord=null, $replace=false)
    {
        if ($operation == 'clone') {
            $name = Inflector::camelize($widgetRequestBody->widgetTitle);
            $viewCtrlPattern = '/'.$widgetRecord->getName().'Ctrl/';
            $viewCtrlReplaceStr = $name.'Ctrl';
            $editCtrlPattern = '/edit'.ucfirst($widgetRecord->getName()).'Ctrl/';
            $editCtrlReplaceStr = 'edit'.ucfirst($name).'Ctrl';

            $requestBody = array(
                'name' => $name,
                'title' => $widgetRequestBody->widgetTitle,
                'subTitle' => $widgetRecord->getSubTitle(),
                'version' => $widgetRecord->getVersion(),
                'metadata' => $widgetRecord->getMetadata(),
                'viewHtml' => $widgetRecord->getViewHtml(),
                'editHtml' => $widgetRecord->getEditHtml(),
                'viewController' => preg_replace($viewCtrlPattern, $viewCtrlReplaceStr, $widgetRecord->getViewController()),
                'editController' => preg_replace($editCtrlPattern, $editCtrlReplaceStr, $widgetRecord->getEditController())
            );
            $files = $this->addWidgetAssetsFromDB($widgetRecord->getFiles()->toArray());
        } else {
            $requestBody = $this->buildWidgetPayload($widgetRequestBody);
            $files = $this->addWidgetAssets(self::TMP_PATH.$widgetRequestBody.'/widgetAssets/');
        }

        if ($operation == 'install') {
            $requestBody['draft'] = false;
            $requestBody['installed'] = true;
            if (!isset($requestBody['metadata']['publisher'])) {
                $requestBody['metadata']['publisher'] = "Community";
            }
        } elseif ($operation == 'clone') {
            $requestBody['draft'] = true;
            $requestBody['installed'] = false;
            $timestamp = (new \DateTime('NOW', new \DateTimeZone('UTC')))->getTimestamp();
            $requestBody['published_date']=$timestamp;
        } elseif ($operation == 'upload') {
            $requestBody['draft'] = true;
            $requestBody['installed'] = false;
            $timestamp = (new \DateTime('NOW', new \DateTimeZone('UTC')))->getTimestamp();
            $requestBody['published_date']=$timestamp;
            $requestBody['metadata']['certified'] = "No";
            $requestBody['metadata']['publisher'] = "Community";
        }
        $requestBody['files']=$files;
        $requestBody['widgetOperation']=$operation;
        $request->initialize(
            $request->query->all(),
            $request->request->all(),
            $request->attributes->all(),
            $request->cookies->all(),
            $request->files->all(),
            $request->server->all(),
            json_encode($requestBody)
        );

        if ($operation != 'clone') {
            $this->removeTmpFiles($widgetRequestBody);
        }
        
        /** @var EntityManager $entityManager */
        $entityManager = $this->container->get('doctrine')->getManagerForClass(Widget::class);
        $widgetRepository = $entityManager->getRepository(Widget::class);
        $widget = $widgetRepository->findOneBy(['name' => $requestBody['name']]);

        if ($widget && $replace) {
            return parent::putAction($request, $widget->getUuid());
        } elseif ($widget) {
            $this->removeAssets($requestBody['files']);
            return new Response('Widget '.$requestBody['name']. ' already exists. select the replace option if you like to replace the widget', Response::HTTP_BAD_REQUEST);
        }
        return parent::cpostAction($request);
    }

    public function addWidgetAssets($path)
    {
        /** @var EntityManager $entityManager */
        $pgEntityManager = $this->container->get('doctrine')->getManagerForClass(PgFile::class);
        /** @var EntityManager $entityManager */
        $fileEntityManager = $this->container->get('doctrine')->getManagerForClass(PgFile::class);
         
        $widgetsFileSystem = new Filesystem();
        $filePaths= [];
        $fileIRI =[];
        if ($widgetsFileSystem->exists($path)) {
            try {
                $finder = new Finder();
                $files= $finder->files()->in($path);
                foreach ($files as $file) {
                    array_push($filePaths, $file->getRelativePathname());
                }
                foreach ($filePaths as $key => $value) {
                    $pgFile = new PgFile();
                    $strm = fopen($path.$value, 'rb');
                    $pgFile->setFile(base64_encode(stream_get_contents($strm)));
                    $pgEntityManager->persist($pgFile);
                    $pgEntityManager->flush();
                    $fileBody = new File();
                    $fileBody->setFilename(end(explode('/', $value)));
                    $fileBody->setMimeType(mime_content_type($path.$value));
                    $fileBody->setSize(filesize($path.$value));
                    $timestamp = (new \DateTime('NOW', new \DateTimeZone('UTC')));
                    $fileBody->setUploadDate($timestamp);
                    $dirPath = self::ASSET_PATH;
                    if (dirname($value) != '.') {
                        $dirPath = $dirPath.'/'.dirname($value);
                    }
                    $fileBody->setMetadata(['path'=>$dirPath]);
                    $fileBody->setFile("/api/3/pgfile/" . $pgFile->getUuid());
                    $fileBody->setUuid(Uuid::uuid4()->toString());
                    $fileBody->setId($fileBody->getUuid());
                    foreach ($this->getUser()->getTeams() as $owner) {
                        $fileBody->addOwner($owner);
                    }
                    $fileEntityManager->persist($fileBody);
                    $fileEntityManager->flush();
                    array_push($fileIRI, "/api/3/files/".$fileBody->getUuid());
                }
            } catch (\Exception $e) {
                $this->removeAssets($fileIRI);
                throw new HttpException(Response::HTTP_BAD_REQUEST, $e->getMessage(), $e);
            }
            return $fileIRI;
        }
        return $fileIRI;
    }

    public function addWidgetAssetsFromDB($fileRecords)
    {
        /** @var EntityManager $entityManager */
        $pgEntityManager = $this->container->get('doctrine')->getManagerForClass(PgFile::class);
        /** @var EntityManager $entityManager */
        $fileEntityManager = $this->container->get('doctrine')->getManagerForClass(File::class);
        $fileIRI = [];
        foreach ($fileRecords as $key => $value) {
            $pgFile = $pgEntityManager->find(PgFile::class, end(explode('/', $value->getFile())));
            $pgFileClone = new PgFile();
            $pgFileClone->setFile(stream_get_contents($pgFile->getFile()));
            $pgEntityManager->persist($pgFileClone);
            $pgEntityManager->flush();
            $fileBody = new File();
            $fileBody->setFilename($value->getFilename());
            $fileBody->setMimeType($value->getMimeType());
            $fileBody->setSize($value->getSize());
            $fileBody->setUuid(Uuid::uuid4()->toString());
            $fileBody->setId($value->getUuid());
            $timestamp = (new \DateTime('NOW', new \DateTimeZone('UTC')));
            $fileBody->setUploadDate($timestamp);
            $fileBody->setMetadata($value->getMetadata());
            $fileBody->setFile("/api/3/pgfile/" . $pgFile->getUuid());
            foreach ($this->getUser()->getTeams() as $owner) {
                $fileBody->addOwner($owner);
            }
            $fileEntityManager->persist($value);
            $fileEntityManager->flush();
            array_push($fileIRI, "/api/3/files/".$value->getUuid());
        }
        return $fileIRI;
    }

    protected function unzipWidgetFile($tgzFileData, $tgzFileName, $fileName="")
    {
        try {
            $this->fileSystem = new Filesystem();
            if ($this->fileSystem->exists(self::TMP_PATH.$fileName)) {
                $this->removeTmpFiles($fileName);
            }
            $this->fileSystem->dumpFile(sprintf('%s%s', self::TMP_PATH, $tgzFileName), $tgzFileData);
            $phar = new \PharData(sprintf('%s%s', self::TMP_PATH, $tgzFileName));
            $phar->decompress();
            $phar->extractTo(self::TMP_PATH);
        } catch (\Exception $e) {
            $this->removeTmpFiles($fileName);
            throw new HttpException(Response::HTTP_BAD_REQUEST, $e->getMessage(), $e);
        }
    }

    protected function buildWidgetPayload($name)
    {
        $data = array();
        try {
            $fPath = sprintf('%s%s/info.json', self::TMP_PATH, $name);
            $this->fileSystem = new Filesystem();
            if (!$this->fileSystem->exists($fPath)) {
                $this->removeTmpFiles($name);
                throw new HttpException(Response::HTTP_BAD_REQUEST, "Widget TGZ file name mismatch with metadata or metadata file missing. please upload the correct file.");
            }
            $fOpen = fopen($fPath, "r");
            $meta = fread($fOpen, filesize($fPath));
            foreach (json_decode($meta, true) as $k => $v) {
                $data[$k] = $v;
            }
        } catch (\Exception $e) {
            throw new HttpException(Response::HTTP_BAD_REQUEST, $e->getMessage(), $e);
        }
       
        $widgetFiles = array(
            'viewHtml' => $name.'/html/view.html',
            'viewController' => $name.'/js/view.controller.js',
            'editHtml' => $name.'/html/edit.html',
            'editController' => $name.'/js/edit.controller.js'
        );
        try {
            foreach ($widgetFiles as $key => $value) {
                $fPath = sprintf('%s/%s', self::TMP_PATH, $value);
                $fOpen = fopen($fPath, "r");

                if ($key == "info") {
                    $meta = fread($fOpen, filesize($fPath));
                    foreach (json_decode($meta) as $k => $v) {
                        $data[$k] = $v;
                    }
                } else {
                    $data[$key] = fread($fOpen, filesize($fPath));
                }
            }
        } catch (\Exception $e) {
            $this->removeTmpFiles($name);
            throw new HttpException(Response::HTTP_BAD_REQUEST, $e->getMessage(), $e);
        }
        return $data;
    }

    protected function widgetValidation($data, $pathName=[])
    {
        try {
            $domHtml = new DOMDocument;
            $domHtml->loadHTML($data['viewHtml']);
        } catch (\Exception $e) {
            return array('error'=>true, 'error_msg'=>$this->container->getParameter('html_syntax_error'));
        }

        try {
            $domEditHtml = new DOMDocument;
            $domEditHtml->loadHTML($data['editHtml']);
        } catch (\Exception $e) {
            return array('error'=>true, 'error_msg'=>$this->container->getParameter('edithtml_syntax_error'));
        }
    }

    protected function removeTmpFiles($name)
    {
        $filePaths = [sprintf('%s/%s.tar.gz', self::TMP_PATH, $name),sprintf('%s/%s.tgz', self::TMP_PATH, $name),sprintf('%s/%s.tar', self::TMP_PATH, $name),sprintf('%s/%s', self::TMP_PATH, $name),sprintf('%s/._%s', self::TMP_PATH, $name)];
        foreach ($filePaths as $key => $value) {
            if ($this->fileSystem->exists($value)) {
                $this->fileSystem->remove($value);
            }
        }
    }

    protected function removeAssets($fileIris=[])
    {
        $fileEntityManager = $this->container->get('doctrine.orm.entity_manager');
        $fileRepo = $fileEntityManager->getRepository(File::class);
        foreach ($fileIris as $key => $iri) {
            $file = $fileRepo->find(end(explode('/', $iri)));
            $fileProvider = $this->get('cybersponse.files.provider.files');
            $fileProvider->removeFiles($file);
        }
    }

    public function deleteWidgetsAction(Request $request)
    {
        $requestResource = $this->getResource($request);
        $this->denyAccessUnlessGranted($requestResource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE);
        $resource = ARCUtils::extractRequestBody($request, null);
        $partialUpdatedObject =[];
        $partialUpdatedObject["success"]=[];
        $partialUpdatedObject["failure"]=[];

        /** @var EntityManager $entityManager */
       
        $entityManager = $this->container->get('doctrine')->getManagerForClass(Widget::class);
        $widgetRepository = $entityManager->getRepository(Widget::class);
        foreach ($resource->ids as $id) {
            /** @var Widget $widget */
            $widget = $widgetRepository->find($id);
            if ($widget) {
                $widgetsPath = sprintf("/opt/cyops-ui/widgets/%s", $widget->getName());

                try {
                    parent::deleteAction($request, $id);
                    array_push($partialUpdatedObject["success"], $widget->getName());
                } catch (\Exception $e) {
                    $err[$widget->getName()] = $e->getMessage();
                    array_push($partialUpdatedObject["failure"], $err);
                    continue;
                }
            }
        }
        if (count($partialUpdatedObject["failure"])>0) {
            return new Response($partialUpdatedObject, Response::HTTP_MULTI_STATUS);
        }
        return new Response($partialUpdatedObject, Response::HTTP_OK);
    }
}
