<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Doctrine\Orm;


use Cybersponse\CoreBundle\Entity\BaseEntity;
use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Mapping as ORM;
use Faker\Provider\Uuid;

class BaseEntityListener
{
    /** @var ManagerRegistry */
    protected $managerRegistry;

    /**
     * @ORM\PrePersist
     */
    public function prePersist(LifecycleEventArgs $args)
    {
        $object = $args->getObject();

        if ($this->extendsBaseEntity($object)) {
            /** @var BaseEntity $object */
            if (is_null($object->getUuid())) {
                $manager = $this->managerRegistry->getManagerForClass(get_class($object));

                if ($manager instanceof EntityManager) {
                    $object->setUuid(self::uuidv4(openssl_random_pseudo_bytes(16)));
                }
            }
        }
    }

    /**
     * @see http://stackoverflow.com/a/15875555
     *
     * @param $data
     * @return string
     */
    private static function uuidv4($data)
    {
        assert(strlen($data) == 16);

        $data[6] = chr(ord($data[6]) & 0x0f | 0x40); // set version to 0100
        $data[8] = chr(ord($data[8]) & 0x3f | 0x80); // set bits 6-7 to 10

        return vsprintf('%s%s-%s-%s-%s-%s%s%s', str_split(bin2hex($data), 4));
    }

    /**
     * @param ManagerRegistry $managerRegistry
     */
    public function setManagerRegistry($managerRegistry)
    {
        $this->managerRegistry = $managerRegistry;
    }

    /**
     * @param $object
     * @return bool
     */
    protected function extendsBaseEntity($object)
    {
        return $object instanceof BaseEntity;
    }
}
