<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Doctrine\Orm;


use Cybersponse\CoreBundle\Traits\Indexable;
use Cybersponse\CoreBundle\Traits\PeerReplicable;
use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\Common\Util\ClassUtils;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Id\SequenceGenerator;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Yaml\Yaml;

class IndexableListener
{

    /** @var ContainerInterface */
    protected $container;

    /**
     * @param ContainerInterface $container
     */
    public function setContainer($container)
    {
        $this->container = $container;
    }

    /** @var ManagerRegistry */
    protected $managerRegistry;

    /** @var SequenceGenerator */
    protected $sequenceGenerator;

    /**
     * The allocation size of the sequence.
     *
     * @var int
     */
    private $_allocationSize = 1;

    /**
     * @var int
     */
    private $_nextValue = 0;


    private $_maxValue = null;

    public function generate(EntityManager $em, $sequence_name)
    {
        if ($this->_maxValue === null || $this->_nextValue == $this->_maxValue) {
            // Allocate new values
            $conn = $em->getConnection();
            $sql  = $conn->getDatabasePlatform()->getSequenceNextValSQL($sequence_name);

            $this->_nextValue = (int) $conn->fetchColumn($sql);
            $this->_maxValue  = $this->_nextValue + $this->_allocationSize;
        }

        return $this->_nextValue++;
    }

    /**
     * @ORM\PrePersist
     */
    public function prePersist(LifecycleEventArgs $args)
    {
        $object = $args->getObject();

        if ($this->isIndexable($object)) {
            /** @var Indexable $object */
            $manager = $this->managerRegistry->getManagerForClass(get_class($object));
            if ($manager instanceof EntityManager) {
                if ($this->container->hasParameter('id_per_module') && $this->container->getParameter('id_per_module')) {
                    $className = ClassUtils::getRealClass(get_class($object));
                    $tableName = $manager->getClassMetadata($className)->getTableName();
                    $seq_name = $tableName . '_id_seq';
                    $id = $this->generate($manager, $seq_name);
                } else {
                    $id = $this->sequenceGenerator->generate($manager, $object);
                }

                $object->setId($id);
                if ($this->isPeerReplicable($object) && is_null($object->getTenantRecordId())) {
                    $object->setTenantRecordId($id);
                }
            }
        }
    }

    /**
     * @param ManagerRegistry $managerRegistry
     */
    public function setManagerRegistry($managerRegistry)
    {
        $this->managerRegistry = $managerRegistry;
    }

    /**
     * @param SequenceGenerator $sequenceGenerator
     */
    public function setSequenceGenerator($sequenceGenerator)
    {
        $this->sequenceGenerator = $sequenceGenerator;
    }

    /**
     * @param $object
     * @return bool
     */
    protected function isIndexable($object)
    {
        return in_array(Indexable::class, class_uses(get_class($object)));
    }

    /**
     * @param $object
     * @return bool
     */
    public function isPeerReplicable($object)
    {
        return in_array(PeerReplicable::class, class_uses(get_class($object)));
    }
}
