<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle;


use Symfony\Component\HttpKernel\Exception\HttpException;

class EncryptDecrypt
{
    /** @var string */
    private $privateKey;

    const METHOD = 'aes-256-cbc';

    /**
     * @param str $privateKey
     */
    public function setPrivateKey($privateKey)
    {
        $this->privateKey = $privateKey;
    }

    public function encrypt($plaintext, $publiKey)
    {
        $this->_validateKey($publiKey);

        $publiKey = substr(hash('sha256', $publiKey . $this->privateKey), 0, 32);

        /*$ivSize = mcrypt_get_iv_size(MCRYPT_RIJNDAEL_128, MCRYPT_MODE_CBC);
        $iv = mcrypt_create_iv($ivSize, MCRYPT_RAND);
        $encryptedValue = mcrypt_encrypt(MCRYPT_RIJNDAEL_128, $publiKey, $plaintext, MCRYPT_MODE_CBC, $iv);*/

        //https://paragonie.com/blog/2015/05/if-you-re-typing-word-mcrypt-into-your-code-you-re-doing-it-wrong
        $ivSize = openssl_cipher_iv_length(self::METHOD);
        $iv = openssl_random_pseudo_bytes($ivSize);

        $encryptedValue = openssl_encrypt(
            $plaintext,
            self::METHOD,
            $publiKey,
            OPENSSL_RAW_DATA,
            $iv
        );

        $encryptedValue = $iv . $encryptedValue;
        $hashMac = hash_hmac('sha256', $encryptedValue, $publiKey);
        return utf8_encode($hashMac . $encryptedValue);
    }

    private function _validateKey($key)
    {
        if (strlen($key) < 32) {
            throw new HttpException(500, "Invalid encryption key $key, key must be at least 32 bytes long.");
        }
    }

    public function decrypt($encryptedKey, $publiKey)
    {
        $encryptedKey = utf8_decode($encryptedKey);
        $this->_validateKey($publiKey);
        $publiKey = substr(hash('sha256', $publiKey . $this->privateKey), 0, 32);

        $macSize = 64;
        $hashMac = substr($encryptedKey, 0, $macSize);
        $encryptedKey = substr($encryptedKey, $macSize);
        $compareHmac = hash_hmac('sha256', $encryptedKey, $publiKey);

        if ($hashMac !== $compareHmac) {
            throw new HttpException(500, "Something went wrong while decrypting the value.");
        }

        $ivsize = openssl_cipher_iv_length(self::METHOD);
        $iv = mb_substr($encryptedKey, 0, $ivsize, '8bit');
        $ciphertext = mb_substr($encryptedKey, $ivsize, null, '8bit');

        $plain = openssl_decrypt(
            $ciphertext,
            self::METHOD,
            $publiKey,
            OPENSSL_RAW_DATA,
            $iv
        );

        return rtrim($plain, "\0");
    }

    public function genRandString($length = 0)
    {
        $charset = 'HOuZG8Lzpe4UwdI0jkMEblRFVn9cyoNCxA7taKD1Tfmqgsv25J63hiQBSPXrWY';
        $str = '';
        $count = strlen($charset);
        while ($length-- > 0) {
            $str .= $charset[mt_rand(0, $count - 1)];
        }
        return $str;
    }
}