<?php

namespace Cybersponse\CoreBundle\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * ExportJob
 *
 * @Doctrine\ORM\Mapping\Table(name="export_jobs", indexes={@Doctrine\ORM\Mapping\Index(name="export_jobs_search_idx", columns={"modifydate"}), @Doctrine\ORM\Mapping\Index(name="export_jobs_id_search_idx", columns={"id"})})
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="export_jobs")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="security")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ id }}")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="Export Job",plural="Export Jobs")
 */
class ExportJob extends BaseEntity
{
    use \Cybersponse\CoreBundle\Traits\Trackable;
    use \Cybersponse\CoreBundle\Traits\Indexable;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`file`", type="string", length=10485761, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="files")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="File")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonText="", buttonIcon="", buttonClass="btn btn-default btn-sm")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="file")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     */
    protected $file;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`status`", type="string", length=1024, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Status")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     */
    protected $status;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`errormessage`", type="string", length=8192, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Error Message")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="textarea")
     */
    protected $errorMessage = '';
    
    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`fileName`", type="string", length=1024, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="File Name")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     */
    protected $fileName = '';

    /**
     * @var integer 
     * 
     * @Doctrine\ORM\Mapping\Column(name="`progress_percent`", type="integer", nullable=false, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Progress Percent")"
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="integer")
     *
     */
    protected $progressPercent = 0;

    /**
     * @var string
     *
     * @ORM\ManyToOne(targetEntity="ExportTemplate", cascade={"persist"}, inversedBy="exportJobs")
     * @ORM\JoinColumn(name="template", referencedColumnName="uuid")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="export_templates")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Template")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="lookup")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     */
    private $template;


    /**
     * Set file
     *
     * @param string  $file
     *
     * @return ExportJob
     */
    public function setFile($file)
    {
        $this->file = $file;

        return $this;
    }

    /**
     * Get file
     *
     * @return string 
     */
    public function getFile()
    {
        return $this->file;
    }
    

    /**
     * Get the value of status
     *
     * @return string
     */ 
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * Set the value of status
     *
     * @param string $status
     *
     * @return ExportJob
     */ 
    public function setStatus(string $status)
    {
        $this->status = $status;

        return $this;
    }

    /**
     * Get the value of progressPercent
     *
     * @return integer
     */ 
    public function getProgressPercent()
    {
        return $this->progressPercent;
    }

    /**
     * Set the value of progressPercent
     *
     * @param integer $progressPercent
     *
     * @return ExportJob
     */ 
    public function setProgressPercent($progressPercent)
    {
        $this->progressPercent = $progressPercent;

        return $this;
    }

    /**
     * Get the value of template
     *
     * @return  \Cybersponse\CoreBundle\Entity\ExportTemplate
     */ 
    public function getTemplate()
    {
        return $this->template;
    }

    /**
     * Set the value of template
     *
     * @param  \Cybersponse\CoreBundle\Entity\ExportTemplate  $template
     *
     * @return  self
     */ 
    public function setTemplate(\Cybersponse\CoreBundle\Entity\ExportTemplate $template = null)
    {
        $this->template = $template;

        return $this;
    }

    /**
     * Get the value of fileName
     *
     * @return  string
     */ 
    public function getFileName()
    {
        return $this->fileName;
    }

    /**
     * Set the value of fileName
     *
     * @param  string  $fileName
     *
     * @return  self
     */ 
    public function setFileName(string $fileName)
    {
        $this->fileName = $fileName;

        return $this;
    }

    /**
     * Get the value of errorMessage
     *
     * @return  string
     */ 
    public function getErrorMessage()
    {
        return $this->errorMessage;
    }

    /**
     * Set the value of errorMessage
     *
     * @param  string  $errorMessage
     *
     * @return  self
     */ 
    public function setErrorMessage(string $errorMessage)
    {
        $this->errorMessage = $errorMessage;

        return $this;
    }
}