<?php

namespace Cybersponse\CoreBundle\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * ImportJob
 *
 * @Doctrine\ORM\Mapping\Table(name="export_templates", indexes={@Doctrine\ORM\Mapping\Index(name="export_templates_search_idx", columns={"modifydate"}), @Doctrine\ORM\Mapping\Index(name="export_templates_id_search_idx", columns={"id"})})
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="export_templates")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="security")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ name }}")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="Export Template",plural="Export Templates")
 */
class ExportTemplate extends BaseEntity
{
    use \Cybersponse\CoreBundle\Traits\Trackable;
    use \Cybersponse\CoreBundle\Traits\Indexable;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`name`", type="string", length=1024, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Template Name")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     */
    protected $name;

    /**
     * @var \DateTime
     *
     * @Doctrine\ORM\Mapping\Column(name="lastexportdate", type="datetime", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="datetime")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Last Export Date")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     */
    protected $lastExportDate;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @ORM\OneToMany(targetEntity="ExportJob", mappedBy="template", cascade={"persist", "remove"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="export_jobs")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Export Jobs")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="oneToMany")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Exportable\Exportable()
     *
     */
    protected $exportJobs;

    /**
     * @var string Import options which allow users to customize the import
     * @Doctrine\ORM\Mapping\Column(name="`options`", type="jsonb", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Options")"
     *
     */
    protected $options = [];


    /**
     * Get the value of options
     *
     * @return mixed
     */ 
    public function getOptions()
    {
        return $this->options;
    }

    /**
     * Set the value of options
     *
     * @param  $options
     *
     * @return ImportJob
     */ 
    public function setOptions($options)
    {
        $this->options = $options;

        return $this;
    }

    /**
     * Get the value of lastExportDate
     *
     * @return  \DateTime
     */ 
    public function getLastExportDate()
    {
        return $this->lastExportDate;
    }

    /**
     * Set the value of lastExportDate
     *
     * @param  \DateTime  $lastExportDate
     *
     * @return  self
     */ 
    public function setLastExportDate(\DateTime $lastExportDate = null)
    {
        $this->lastExportDate = $lastExportDate;

        return $this;
    }


    /**
     * Get the value of name
     *
     * @return  string
     */ 
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set the value of name
     *
     * @param  string  $name
     *
     * @return  self
     */ 
    public function setName(string $name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Add exportJobs
     *
     * @param ExportJob $exportJobs
     * @return ExportTemplate
     */
    public function addExportJob(ExportJob $exportJob)
    {
        $this->exportJobs[] = $exportJob;
        $exportJob->setTemplate($this);
        return $this;
    }

    /**
     * Remove exportJobs
     *
     * @param ExportJob $exportJob
     */
    public function removeExportJob(ExportJob $exportJob)
    {
        $this->exportJobs->removeElement($exportJob);
        $exportJob->setTemplate(null);
    }

    /**
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getExportJobs()
    {
        return $this->exportJobs;
    }
}