<?php

namespace Cybersponse\CoreBundle\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * ImportJob
 *
 * @Doctrine\ORM\Mapping\Table(name="import_jobs", indexes={@Doctrine\ORM\Mapping\Index(name="import_jobs_search_idx", columns={"modifydate"}), @Doctrine\ORM\Mapping\Index(name="import_jobs_id_search_idx", columns={"id"})})
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="import_jobs")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="security")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ id }}")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="Import Job",plural="Import Jobs")
 */
class ImportJob extends BaseEntity
{
    use \Cybersponse\CoreBundle\Traits\Trackable;
    use \Cybersponse\CoreBundle\Traits\Indexable;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`file`", type="string", length=10485761, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="files")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="File")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonText="", buttonIcon="", buttonClass="btn btn-default btn-sm")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="file")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     */
    protected $file;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`status`", type="string", length=1024, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Status")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     */
    protected $status;
    
    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`errormessage`", type="string", length=10485761, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Error Message")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="textarea")
     */
    protected $errorMessage = '';

    /**
     * @var integer 
     * 
     * @Doctrine\ORM\Mapping\Column(name="`progress_percent`", type="integer", nullable=false, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Progress Percent")"
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="integer")
     *
     */
    protected $progressPercent = 0;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`currently_importing`", type="string", length=1024, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Currently Importing")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     */
    protected $currentlyImporting;

    /**
     * @var string Import options which allow users to customize the import
     * @Doctrine\ORM\Mapping\Column(name="`options`", type="jsonb", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Options")"
     *
     */
    protected $options = [];


    /**
     * Set file
     *
     * @param string  $file
     *
     * @return ImportJob
     */
    public function setFile($file)
    {
        $this->file = $file;

        return $this;
    }

    /**
     * Get file
     *
     * @return string 
     */
    public function getFile()
    {
        return $this->file;
    }
    

    /**
     * Get the value of status
     *
     * @return string
     */ 
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * Set the value of status
     *
     * @param string $status
     *
     * @return ImportJob
     */ 
    public function setStatus(string $status)
    {
        $this->status = $status;

        return $this;
    }

    /**
     * Get the value of options
     *
     * @return array
     */ 
    public function getOptions()
    {
        return $this->options;
    }

    /**
     * Set the value of options
     *
     * @param $options
     *
     * @return ImportJob
     */ 
    public function setOptions($options)
    {
        $this->options = $options;

        return $this;
    }

    /**
     * Get the value of progressPercent
     *
     * @return  integer
     */ 
    public function getProgressPercent()
    {
        return $this->progressPercent;
    }

    /**
     * Set the value of progressPercent
     *
     * @param  integer  $progressPercent
     *
     * @return  ImportJob
     */ 
    public function setProgressPercent($progressPercent)
    {
        $this->progressPercent = $progressPercent;

        return $this;
    }

    /**
     * Get the value of errorMessage
     *
     * @return  string
     */ 
    public function getErrorMessage()
    {
        return $this->errorMessage;
    }

    /**
     * Set the value of errorMessage
     *
     * @param  string  $errorMessage
     *
     * @return  self
     */ 
    public function setErrorMessage(string $errorMessage)
    {
        $this->errorMessage = $errorMessage;

        return $this;
    }

    /**
     * Get the value of currentlyImporting
     *
     * @return  string
     */ 
    public function getCurrentlyImporting()
    {
        return $this->currentlyImporting;
    }

    /**
     * Set the value of currentlyImporting
     *
     * @param  string  $currentlyImporting
     *
     * @return  self
     */ 
    public function setCurrentlyImporting(string $currentlyImporting)
    {
        $this->currentlyImporting = $currentlyImporting;

        return $this;
    }
}