<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Entity;

use Cybersponse\CoreBundle\Traits\Indexable;
use Doctrine\ORM\Mapping as ORM;

/**
 * Picklist
 *
 * @Doctrine\ORM\Mapping\Table(name="picklist_names")
 * @Doctrine\ORM\Mapping\Entity
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="picklist_names")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="application")
 */
class PicklistNames extends BaseEntity
{
    use Indexable;

    /**
     * @var string
     *
     * @ORM\Column(name="name", type="string", nullable=false, unique=true)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     */
    protected $name;

    /**
     * @ORM\Column(type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="System")"
     */
    protected $system = false;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @ORM\OneToMany(targetEntity="Picklist", mappedBy="listName", cascade={"persist", "remove"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="picklists")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Picklist")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="oneToMany")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Exportable\Exportable()
     *
     */
    protected $picklists;

    /**
     * Set listname
     *
     * @param string $name
     * @return Picklist
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get listname
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set System
     *
     * @param string $system
     * @return Picklist
     */
    public function setSystem($system)
    {
        $this->system = $system;

        return $this;
    }

    /**
     * Get System
     *
     * @return string
     */
    public function getSystem()
    {
        return $this->system;
    }

    /**
     * Add picklists
     *
     * @param Picklist $picklists
     * @return PicklistNames
     */
    public function addPicklist(Picklist $picklists)
    {
        $this->picklists[] = $picklists;
        $picklists->setListName($this);
        return $this;
    }

    /**
     * Remove picklists
     *
     * @param Picklist $picklist
     */
    public function removePicklist(Picklist $picklist)
    {
        $this->picklists->removeElement($picklist);
        $picklist->setListName(null);
    }

    /**
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getPicklists()
    {
        return $this->picklists;
    }
}
