<?php

namespace Cybersponse\CoreBundle\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * Widget
 *
 * @Doctrine\ORM\Mapping\Table(name="widgets", uniqueConstraints={@Doctrine\ORM\Mapping\UniqueConstraint(name="widgets_unique", columns={"name", "version"})}, indexes={@Doctrine\ORM\Mapping\Index(name="widgets_search_idx", columns={"modifydate"}), @Doctrine\ORM\Mapping\Index(name="widgets_id_search_idx", columns={"id"})})
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="widgets")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="widgets")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ title }}")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="Widget",plural="Widgets")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\UniqueConstraint\UniqueConstraint(uniqueConstraint={{"widgets_unique"={"columns"={"name","version"}}}})
 */
class Widget extends \Cybersponse\CoreBundle\Entity\BaseEntity
{
    use \Cybersponse\AuthorizationBundle\Traits\ConditionalOwnable;
    use \Cybersponse\CoreBundle\Traits\Trackable;
    use \Cybersponse\CoreBundle\Traits\Indexable;

    /**
     * @var boolean
     *
     * @Doctrine\ORM\Mapping\Column(name="`active`", type="boolean", length=10485761, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Active")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     */
    protected $active;

    /**
     * @var \DateTime
     *
     * @Doctrine\ORM\Mapping\Column(name="`publisheddate`", type="datetime", length=10485761, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Published Date")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="datetime")
     */
    protected $publishedDate;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`subtitle`", type="text", length=10485761, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Sub Title")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     */
    protected $subTitle;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`viewcontroller`", type="text", length=10485761, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="View Controller")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="textarea")
     */
    protected $viewController;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`editcontroller`", type="text", length=10485761, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Edit Controller")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="textarea")
     */
    protected $editController;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`edithtml`", type="text", length=10485761, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Edit HTML")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="textarea")
     */
    protected $editHtml;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`viewhtml`", type="text", length=10485761, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="View HTML")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="textarea")
     */
    protected $viewHtml;

    /**
     * @var boolean
     *
     * @Doctrine\ORM\Mapping\Column(name="`installed`", type="boolean", length=10485761, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Installed")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     */
    protected $installed;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`name`", type="text", length=10485761, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Name")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     */
    protected $name;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`title`", type="text", length=10485761, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Title")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     */
    protected $title;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`version`", type="text", length=10485761, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Version")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     */
    protected $version;

    /**
     * @var mixed
     *
     * @Doctrine\ORM\Mapping\Column(name="`metadata`", type="jsonb", length=10485761, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Validation\IsJsonType("object")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Metadata")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="object")
     */
    protected $metadata;

    /**
     * @var boolean
     *
     * @Doctrine\ORM\Mapping\Column(name="`draft`", type="boolean", length=10485761, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Draft")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false, buttonClass="btn btn-default btn-sm", buttonIcon="", buttonText="")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     */
    protected $draft;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="Cybersponse\FilesBundle\Entity\File", cascade={"persist","remove","detach"}, orphanRemoval=true, fetch="EXTRA_LAZY")
     * @Doctrine\ORM\Mapping\JoinTable(name="widget_file",
     *   joinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="widget_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   },
     *   inverseJoinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="file_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   }
     * )
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="files")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="files")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\PeerReplicable\PeerReplicable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="manyToMany")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     */
    protected $files;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->files = new \Doctrine\Common\Collections\ArrayCollection();
    }

    /**
     * Set active
     *
     * @param boolean  $active
     *
     * @return Widget
     */
    public function setActive($active)
    {
        $this->active = $active;

        return $this;
    }

    /**
     * Get active
     *
     * @return boolean 
     */
    public function getActive()
    {
        return $this->active;
    }

    /**
     * Set publishedDate
     *
     * @param \DateTime  $publishedDate
     *
     * @return Widget
     */
    public function setPublishedDate($publishedDate)
    {
        $this->publishedDate = $publishedDate;

        return $this;
    }

    /**
     * Get publishedDate
     *
     * @return \DateTime 
     */
    public function getPublishedDate()
    {
        return $this->publishedDate;
    }

    /**
     * Set subTitle
     *
     * @param string  $subTitle
     *
     * @return Widget
     */
    public function setSubTitle($subTitle)
    {
        $this->subTitle = $subTitle;

        return $this;
    }

    /**
     * Get subTitle
     *
     * @return string 
     */
    public function getSubTitle()
    {
        return $this->subTitle;
    }

    /**
     * Set viewController
     *
     * @param string  $viewController
     *
     * @return Widget
     */
    public function setViewController($viewController)
    {
        $this->viewController = $viewController;

        return $this;
    }

    /**
     * Get viewController
     *
     * @return string 
     */
    public function getViewController()
    {
        return $this->viewController;
    }

    /**
     * Set editController
     *
     * @param string  $editController
     *
     * @return Widget
     */
    public function setEditController($editController)
    {
        $this->editController = $editController;

        return $this;
    }

    /**
     * Get editController
     *
     * @return string 
     */
    public function getEditController()
    {
        return $this->editController;
    }

    /**
     * Set editHtml
     *
     * @param string  $editHtml
     *
     * @return Widget
     */
    public function setEditHtml($editHtml)
    {
        $this->editHtml = $editHtml;

        return $this;
    }

    /**
     * Get editHtml
     *
     * @return string 
     */
    public function getEditHtml()
    {
        return $this->editHtml;
    }

    /**
     * Set viewHtml
     *
     * @param string  $viewHtml
     *
     * @return Widget
     */
    public function setViewHtml($viewHtml)
    {
        $this->viewHtml = $viewHtml;

        return $this;
    }

    /**
     * Get viewHtml
     *
     * @return string 
     */
    public function getViewHtml()
    {
        return $this->viewHtml;
    }

    /**
     * Set installed
     *
     * @param boolean  $installed
     *
     * @return Widget
     */
    public function setInstalled($installed)
    {
        $this->installed = $installed;

        return $this;
    }

    /**
     * Get installed
     *
     * @return boolean 
     */
    public function getInstalled()
    {
        return $this->installed;
    }

    /**
     * Set name
     *
     * @param string  $name
     *
     * @return Widget
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string 
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set title
     *
     * @param string  $title
     *
     * @return Widget
     */
    public function setTitle($title)
    {
        $this->title = $title;

        return $this;
    }

    /**
     * Get title
     *
     * @return string 
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * Set version
     *
     * @param string  $version
     *
     * @return Widget
     */
    public function setVersion($version)
    {
        $this->version = $version;

        return $this;
    }

    /**
     * Get version
     *
     * @return string 
     */
    public function getVersion()
    {
        return $this->version;
    }

    /**
     * Set metadata
     *
     * @param mixed  $metadata
     *
     * @return Widget
     */
    public function setMetadata($metadata)
    {
        $this->metadata = $metadata;

        return $this;
    }

    /**
     * Get metadata
     *
     * @return mixed 
     */
    public function getMetadata()
    {
        return $this->metadata;
    }

    /**
     * Set draft
     *
     * @param boolean  $draft
     *
     * @return Widget
     */
    public function setDraft($draft)
    {
        $this->draft = $draft;

        return $this;
    }

    /**
     * Get draft
     *
     * @return boolean 
     */
    public function getDraft()
    {
        return $this->draft;
    }

    /**
     * Add files
     *
     * @param \Cybersponse\FilesBundle\Entity\File  $file
     *
     * @return Widget
     */
    public function addFile(\Cybersponse\FilesBundle\Entity\File $file)
    {
        if (!$this->files->contains($file)) {
            $this->files[] = $file;
        }
        return $this;
    }

    /**
     * Remove files
     *
     * @param \Cybersponse\FilesBundle\Entity\File  $file
     */
    public function removeFile(\Cybersponse\FilesBundle\Entity\File $file)
    {
        $this->files->removeElement($file);
    }

    /**
     * Get files
     *
     * @return \Doctrine\Common\Collections\Collection 
     */
    public function getFiles()
    {
        return $this->files;
    }
}

