<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Listener;


use Doctrine\DBAL\DBALException;
use Symfony\Component\HttpKernel\Event\GetResponseForExceptionEvent;
use Symfony\Component\HttpKernel\EventListener\ExceptionListener;
use Symfony\Component\HttpKernel\Exception\ConflictHttpException;

class ForeignKeyViolationExceptionListener extends ExceptionListener
{
    const FOREIGN_KEY_VIOLATION = 23503;
    const FOREIGN_KEY_VIOLATION_REGEX_DELETE_REF = "/Key \((.*?)\)=\((.*)\) is still referenced from table \"(.*)\"\./";
    const FOREIGN_KEY_VIOLATION_REGEX_INVALID_FOREIGN_KEY = "/Key \((.*?)\)=\((.*)\) is not present in table \"(.*)\"\./";

    public function onKernelException(GetResponseForExceptionEvent $event)
    {
        $exception = $event->getException();

        if ($exception instanceof DBALException) {
            $previous = $exception->getPrevious();
            if ($previous->getCode() == self::FOREIGN_KEY_VIOLATION) {
                $confictMessage = 'Foreign key violation';

                preg_match(self::FOREIGN_KEY_VIOLATION_REGEX_DELETE_REF, $previous->getMessage(), $matches);
                if (count($matches) == 4) {
                    $conflictMessage = sprintf("%s on field '%s' with data '%s' conflicts with entity '%s'.", $confictMessage, $matches[1], $matches[2], $matches[3]);
                } else {
                    preg_match(self::FOREIGN_KEY_VIOLATION_REGEX_INVALID_FOREIGN_KEY, $previous->getMessage(), $matches);
                    if (count($matches) == 4) {
                        $conflictMessage = sprintf("%s on field '%s' with data '%s' is not a valid entry of type '%s'.", $confictMessage, $matches[1], $matches[2], $matches[3]);
                    } else {
                        $conflictMessage = $previous->getMessage();
                    }
                }

                $event->setException(new ConflictHttpException($conflictMessage, $exception));
                parent::onKernelException($event);
            }
        }
    }
}
