CRUD API
====

Supported CRUD Features
----
### Model Operations
* [Create Individual Model](#create-individual-model) `POST /api/{version}/{collection}`
* [Retrieve Collection of Models](#retrieve-collection-of-models) `GET /api/{version}/{collection}`
 * Pagination
 * Filtering
 * Searching
* [Retrieve Single Model](#retrieve-single-model) `GET /api/{version}/{collection}/{uuid}`
* [Update Single Model](#update-single-model) `PUT /api/{version}/{collection}/{uuid}`
* [Delete Single Model](#delete-single-model) `DELETE /api/{version}/{collection}/{uuid}`

### Relationship Operations
* [Get Relationship](#get-relationship) `GET /api/{version}/{collection}/{uuid}/{field}`
* [Append Existing Object to Relationship](#append-existing-object-to-relationship) `POST /api/{version}/{collection}/{uuid}/{field}`
* [Append New Object to Relationship\*](#append-new-object-to-relationship) `POST /api/{version}/{collection}/{uuid}/{field}`
* [Delete Relationship](#delete-relationship) `DELETE /api/{version}/{collection}/{uuid}/{field}/{fieldUuid}`

\* not currently supported
# Model Operations

## Create Individual Model
**Request**
```
METHOD: POST
URL: /api/{version}/{collection}
BODY:
{
  "field1": "value1",
  "field2": "value2"
}
```
#### Example
**Request**
```
METHOD: POST
URL: /api/3/assets
BODY:
{
  "ip": "8.8.8.8",
  "hostname": "google-public-dns-a.google.com"
}
```
**Response**
```
STATUS: 201
BODY:
{
  "@id": "/api/3/assets/01199609-d60f-356b-a762-129a6e1b353b",
  "@type": "Asset",
  "ip": "8.8.8.8",
  "hostname": "google-public-dns-a.google.com"
}
```

## Retrieve Collection of Models
The responses adhere to the [Hydra Spec](https://www.hydra-cg.com/) for pagination. Records that match the specified query (see Filtering for more info) are returned as objects within the "hydra:member" array.

**Request**
```
METHOD: GET
URL: /api/{version}/{collection}
```
**Response**
```
STATUS: 200
BODY:
{
  "@context": "/api/{version}/contexts/{collection type}",
  "@id": "/api/{version}/{collection}",
  "@type": "hydra:PagedCollection",
  /* Contains some Hydra pagination properties */
  "hydra:member": [
    { "@id": "/api/{version}/{collection}/{uuid}", ... },
    { "@id": "/api/{version}/{collection}/{uuid}", ... },
    { "@id": "/api/{version}/{collection}/{uuid}", ... }
  ]
}
```
#### Example
**Request**
```
METHOD: GET
URL: /api/3/assets
```
**Response**
```
STATUS: 200
BODY:
{
  "@context": "/api/3/contexts/Asset",
  "@id": "/api/3/assets",
  "@type": "hydra:PagedCollection",
  "hydra:totalItems": 2,
  "hydra:itemsPerPage": 30,
  "hydra:firstPage": "/api/3/assets",
  "hydra:lastPage": "/api/3/assets",
  "hydra:member": [
    {
      "@id": "/api/3/assets/01199609-d60f-356b-a762-129a6e1b353b",
      "@type": "Asset",
      "ip": "8.8.8.8",
      "hostname": "google-public-dns-a.google.com"
    },
    {
      "@id": "/api/3/assets/017d3845-e204-367c-b74d-6bb37a8239b7",
      "@type": "Asset",
      "ip": "8.8.4.4",
      "hostname": "google-public-dns-b.google.com"
    }
  ]
}
```

## Retrieve Single Model
**Request**
```
METHOD: GET
URL: /api/{version}/{collection}/{uuid}
```
**Response**
```
STATUS: 200
BODY:
{
  "@id": "/api/{version}/{collection}/{uuid}",
  "@type": "{collection type}",
  "field1": "value1",
  "field2": "value2"
}
```
#### Example
**Request**
```
METHOD: GET
URL: /api/3/assets/01199609-d60f-356b-a762-129a6e1b353b
```
**Response**
```
STATUS: 200
BODY:
{
  "@id": "/api/3/assets/01199609-d60f-356b-a762-129a6e1b353b",
  "@type": "Asset",
  "ip": "8.8.8.8",
  "hostname": "google-public-dns-a.google.com"
}
```

## Update Single Model
**Request**
```
METHOD: PUT
URL: /api/{version}/{collection}/{uuid}
BODY:
{
  "field1": "value1",
  "field2": "value2"
}
```
#### Example
**Request**
```
METHOD: GET
URL: /api/3/assets/01199609-d60f-356b-a762-129a6e1b353b
BODY:
{
  "ip": "8.8.8.8"
}
```
**Response**
```
STATUS: 200
BODY:
{
  "@id": "/api/3/assets/01199609-d60f-356b-a762-129a6e1b353b",
  "@type": "Asset",
  "ip": "8.8.8.8",
  "hostname": "google-public-dns-a.google.com"
}
```

## Delete Single Model
**Request**
```
METHOD: DELETE
URL: /api/{version}/{collection}/{uuid}
```
#### Example
**Request**
```
METHOD: DELETE
URL: /api/3/assets/01199609-d60f-356b-a762-129a6e1b353b
```
**Response**
```
STATUS: 204
REASON: No Content
```

# Relationship Operations

For these examples, we will expect that an Incident has already been created in the system. All these operations will affect the `assets` field of an Incident.

## Get Relationship
**Request**
```
METHOD: GET
URL: /api/3/incidents/bbdc13f8-015f-4615-8c63-4cebb6ec991b/assets
```
**Response**
```
{
  "@context": "/api/3/contexts/Asset",
  "@id": "/api/3/incidents/bbdc13f8-015f-4615-8c63-4cebb6ec991b/assets",
  "@type": "hydra:PagedCollection",
  "hydra:totalItems": 1,
  "hydra:itemsPerPage": 30,
  "hydra:firstPage": "/api/3/incidents/bbdc13f8-015f-4615-8c63-4cebb6ec991b/assets",
  "hydra:lastPage": "/api/3/incidents/bbdc13f8-015f-4615-8c63-4cebb6ec991b/assets",
  "hydra:member": [
    {
      "@id": "/api/3/assets/2ac5eafd-e7dd-465e-8ddd-6b9aeb7125c0",
      "@type": "Asset",
      "macAddress": "as:se:t2",
      "id": 639
    }
  ]
}
```

## Append Existing Object to Relationship
**Request**
```
METHOD: POST
URL: /api/3/incidents/bbdc13f8-015f-4615-8c63-4cebb6ec991b/assets
BODY:
{
  "@id": "/api/3/assets/e0a3521f-67de-473f-a9a0-7f2d44626567"
}
```
**Response**
```
{
  "@id": "/api/3/assets/e0a3521f-67de-473f-a9a0-7f2d44626567",
  "@type": "Asset",
  "macAddress": "as:se:t1",
  "id": 638
}
```

## Append New Object to Relationship

This is not currently supported.

**Request**
```
METHOD: POST
URL: /api/3/incidents/bbdc13f8-015f-4615-8c63-4cebb6ec991b/assets
BODY:
{
  "uuid": "5434a6e1-e9cd-4ecf-ba6b-f8ac82c3effc",
  "macAddress": "as:se:t3"
}
```
**Response**
```
{
  "@id": "/api/3/assets/5434a6e1-e9cd-4ecf-ba6b-f8ac82c3effc",
  "@type": "Asset",
  "macAddress": "as:se:t3",
  "id": 639
}
```

## Delete Relationship
**Request**
```
METHOD: DELETE
URL: /api/3/incidents/bbdc13f8-015f-4615-8c63-4cebb6ec991b/assets/e0a3521f-67de-473f-a9a0-7f2d44626567
```

**Response**
```
STATUS: 204
REASON: No Content
```
