<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Serializer;


use Cybersponse\CoreBundle\Api\Bulk\BulkApiResults;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;
use Symfony\Component\Serializer\Normalizer\SerializerAwareNormalizer;

class BulkApiResponseNormalizer extends SerializerAwareNormalizer implements NormalizerInterface
{
    public function supportsNormalization($data, $format = null)
    {
        return 'json-ld' === $format && $data instanceof BulkApiResults;
    }

    /**
     * @inheritdoc
     * @param BulkApiResults $object
     */
    public function normalize($object, $format = null, array $context = [])
    {
        $result = [];

        if (count($object->getPersisted()) > 0) {
            $normalizedPersistedObjects = [];
            foreach ($object->getPersisted() as $persistedObject) {
                $normalizedPersistedObjects[] = $this->serializer->normalize($persistedObject, 'json-ld', [
                    'ignore_authorization' => true, 'ignore_relationship_authorization' => false
                ]);
            }
            $result[BulkApiResults::PERSISTED_COLLECTION_NAME] = $normalizedPersistedObjects;
        }

        if (count($object->getFailed()) > 0) {
            $normalizedFailedObjects = [];
            foreach ($object->getFailed() as $bulkManagerError) {
                $normalizedFailedObjects[] = $this->serializer->normalize($bulkManagerError, 'json-ld', [
                    'ignore_authorization' => true, 'ignore_relationship_authorization' => false
                ]);
            }
            $result[BulkApiResults::FAILED_COLLECTION_NAME] = $normalizedFailedObjects;
        }

        return $result;
    }
}
