<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Serializer;


use Cybersponse\CoreBundle\Api\Bulk\DenormalizationResult;
use Cybersponse\CoreBundle\Api\Bulk\DenormalizationResultCollection;
use Doctrine\Common\Persistence\ManagerRegistry;
use Dunglas\ApiBundle\Api\ResourceCollectionInterface;
use Dunglas\ApiBundle\Api\ResourceResolverTrait;
use Dunglas\ApiBundle\Exception\InvalidArgumentException;
use Symfony\Component\Serializer\Normalizer\DenormalizerInterface;
use Symfony\Component\Serializer\SerializerAwareInterface;
use Symfony\Component\Serializer\SerializerInterface;

class TypeAgnosticCollectionNormalizer implements SerializerAwareInterface, DenormalizerInterface
{
    const TYPE_AGNOSTIC_TYPE = 'type-agnostic';

    use ResourceResolverTrait;

    /** @var ManagerRegistry */
    protected $ormManagerRegistry;

    /**
     * @var DenormalizerInterface
     */
    protected $serializer;

    public function __construct(
        ResourceCollectionInterface $resourceCollection
    ) {
        $this->resourceCollection = $resourceCollection;
    }

    /**
     * @inheritdoc
     *
     * @throws InvalidArgumentException
     */
    public function denormalize($data, $class, $format = null, array $context = array())
    {
        /** @var DenormalizationResult[] $denormalizationResults */
        $denormalizationResults = [];

        foreach ($data as $normalizedObject) {
            if (!array_key_exists('@type', $normalizedObject)) {
                throw new InvalidArgumentException('The "@type" must be specified for all models.');
            }

            $resourceShortName = $normalizedObject['@type'];
            $resource = $this->resourceCollection->getResourceForShortName($resourceShortName);

            $denormalizedObject = $this->serializer->denormalize(
                $normalizedObject,
                $resource->getEntityClass(),
                'json-ld',
                $resource->getDenormalizationContext() + $context
            );

            $denormalizationResults[] = new DenormalizationResult($normalizedObject, $denormalizedObject);
        }

        return $denormalizationResults;
    }

    /**
     * @inheritdoc
     */
    public function supportsDenormalization($data, $type, $format = null)
    {
        return self::TYPE_AGNOSTIC_TYPE === $type && 'json-ld' === $format && is_array($data) && (empty($data) || array_key_exists(0, $data));
    }

    /**
     * @param $object
     */
    protected function persist($object)
    {
        $manager = $this->ormManagerRegistry->getManagerForClass(get_class($object));
        $manager->persist($object);
    }

    /**
     * @inheritdoc
     */
    public function setSerializer(SerializerInterface $serializer)
    {
        $this->serializer = $serializer;
    }
}

