<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Serializer;


use Cybersponse\CoreBundle\Api\Bulk\DenormalizationResult;
use Dunglas\ApiBundle\Api\ResourceCollectionInterface;
use Symfony\Component\Serializer\Serializer;

// TODO: Make this a real serializer?
class TypeAgnosticLinkedDataSerializer
{
    /** @var Serializer */
    protected $serializer;

    /** @var ResourceCollectionInterface */
    protected $resourceCollection;

    /**
     * TypeAgnosticLinkedDataDenormalizer constructor.
     * @param Serializer $serializer
     * @param ResourceCollectionInterface $resourceCollection
     */
    public function __construct(Serializer $serializer, ResourceCollectionInterface $resourceCollection)
    {
        $this->serializer = $serializer;
        $this->resourceCollection = $resourceCollection;
    }


    /**
     * @param mixed $data
     * @return DenormalizationResult[]
     */
    public function deserializeUnlinked($data)
    {
        /** @var DenormalizationResult[] $unlinkedDenormalizationResults */
        $unlinkedDenormalizationResults = $this->serializer->deserialize(
            $data,
            TypeAgnosticCollectionNormalizer::TYPE_AGNOSTIC_TYPE,
            'json-ld',
            ['parse_to_many' => false, 'parse_to_one' => false]
        );
        return $unlinkedDenormalizationResults;
    }

    /**
     * @param mixed $data
     * @return DenormalizationResult[]
     */
    public function denormalizeUnlinked($data)
    {
        return $this->serializer->denormalize(
            $data,
            TypeAgnosticCollectionNormalizer::TYPE_AGNOSTIC_TYPE,
            'json-ld',
            ['parse_to_one' => false, 'parse_to_many' => false, 'ignore_field_authorization' => true]
        );
    }

    /**
     * @param DenormalizationResult[] $data
     * @return DenormalizationResult[]
     */
    public function denormalizeLinked($data)
    {
        $linkedDenormalizationResults = [];

        foreach ($data as $denormalizationResult) {
            $resource = $this->resourceCollection->getResourceForEntity($denormalizationResult->getResult());
            $linkedObject = $this->serializer->denormalize(
                $denormalizationResult->getFrom(),
                $resource->getEntityClass(),
                'json-ld',
                $resource->getDenormalizationContext() + ['object_to_populate' => $denormalizationResult->getResult(), 'ignore_field_authorization' => true]
            );
            $linkedDenormalizationResults[] = new DenormalizationResult($denormalizationResult->getFrom(), $linkedObject);
        }

        return $linkedDenormalizationResults;
    }
}
