<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Service\Config;

use Cybersponse\CoreBundle\Entity\Proxy;
use Cybersponse\SettingsBundle\Entity\SystemSettings;
use GuzzleHttp\Psr7\Request as GuzzleRequest;

/**
 * Class AppSettingsConfig
 * @package Cybersponse\CoreBundle\Service\Config
 */
class AppSettingsConfig extends PortingConfig
{
    /**
     * 
     * @param array $data List of settings
     * @return array
     */
    public function getImportOptions(array $data, $tokenType)
    {
        $options = [];
        foreach ($data as $setting) {
            $individualOptions = [];
            $individualOptions['name'] =  $setting['type'];
            $individualOptions['include'] = true;

            $options[] = $individualOptions;
        }
        return $options;
    }

    /**
     * Imports the config data using the specified options
     *
     * @param array $data
     * @param array $options
     * @return mixed
     */
    public function import(array $data, array $options, $tokenType)
    {
        $results = [];
        $authConfigUri = sprintf('%s/api/auth/config', $this->container->getParameter('crudhub.uri'));
        foreach ($options as $option) {
            if (!$option['include']) {
                continue;
            }
            $setting = $option['name'];
            $value = null;
            foreach ($data as $item) {
                if ($item['type'] == $setting) {
                    $value = $item['value'];
                }
            }
            if (!$value) {
                continue;
            }
            if ($setting == 'systemSettings') {
                $repository = $this->loadRepository(SystemSettings::class);
                /** @var SystemSettings $existingSettings */
                $existingSettings = $repository->findAll()[0];
                $fullURI = sprintf('%s/api/3/system_settings/%s', $this->container->getParameter('crudhub.uri'), $existingSettings->getUuid());
                $results[] = !!$this->sendRequest($fullURI, 'put', $value, $tokenType);
            } else if ($setting == 'sso') {
                try {
                    $tokenStorage = $this->container->get('security.token_storage');
                    $token = $tokenStorage->getToken();
                    if ($token) {
                        $jsonData = $this->sendRequest(sprintf('%s/api/saml/config', $this->container->getParameter('crudhub.uri')), 'post', $value, $tokenType);
                    } else {
                        $jsonData = $this->sendGatewayRequest(sprintf('%s/saml/config', $this->container->getParameter('gateway.uri')), 'post', $value);
                    }
                    $result = json_decode($jsonData, true);
                    $results[] = true;
                } catch (\Exception $exception) {
                    $results[] = true;
                }
            } else if ($setting == 'syslog') {
                try {
                    $tokenStorage = $this->container->get('security.token_storage');
                    $token = $tokenStorage->getToken();
                    if (count($value)) {
                        if ($token) {
                            $jsonData = $this->sendRequest(sprintf('%s/api/gateway/config/syslog', $this->container->getParameter('crudhub.uri')), 'post', $value[0], $tokenType);
                        } else {
                            $jsonData = $this->sendGatewayRequest(sprintf('%s/config/syslog', $this->container->getParameter('gateway.uri')), 'post', $value[0], $tokenType);
                        }
                    }
                    $result = json_decode($jsonData, true);
                    $results[] = true;
                } catch (\Exception $exception) {
                    $results[] = true;
                }
            } else if ($setting == 'LDAP' || $setting == 'TOKEN' || $setting == 'HA') {
                $sectionValues = [];
                foreach ($value as $item) {
                    $sectionValues[$item['key']] = $item['value'];
                }
                $body = [
                    "section" => $setting,
                    "value" => $sectionValues
                ];
                $jsonData = $this->sendRequest($authConfigUri, 'put', $body, $tokenType);
                $result = json_decode($jsonData, true);
                $results[] = $result['response'] == 'Success';
            } else if ($setting == 'proxy') {
                $repository = $this->loadRepository(Proxy::class);
                /** @var Proxy $proxySettings */
                $proxySettings = $repository->findAll()[0];
                if ($proxySettings) {
                    $fullURI = sprintf('%s/api/3/proxy/%s', $this->container->getParameter('crudhub.uri'), $proxySettings->getUuid());
                    $results[] = !!$this->sendRequest($fullURI, 'put', $value, $tokenType);
                } else {
                    $fullURI = sprintf('%s/api/3/proxy', $this->container->getParameter('crudhub.uri'));
                    $results[] = !!$this->sendRequest($fullURI, 'post', $value, $tokenType);
                }
            }
        }
        if (in_array(false, $results)) {
            return false;
        }
        return true;
    }

    /**
     * Exports the config using the specified options
     *
     * @param array $options
     * @return array
     */
    public function export(array $options, $tokenType = 'JWT')
    {
        $settings = [];
        $authConfigUri = sprintf('%s/api/auth/config', $this->container->getParameter('crudhub.uri'));
        foreach ($options as $setting) {
            $value = null;
            if ($setting == 'systemSettings') {
                $repository = $this->loadRepository(SystemSettings::class);
                /** @var SystemSettings $existingSettings */
                $existingSettings = $repository->findAll()[0];
                /** @var ResourceInterface $picklistResource */
                $resource = $this->container->get('resource.systemsettings');
                $context = $resource->getDenormalizationContext();
                $context['export'] = True;
                $context["ignore_field_authorization"] = true;
                $context["ignore_authorization"] = true;
                $normalizedObject = $this->container->get('serializer')->normalize($existingSettings, 'json-ld', $context);
                $value = $normalizedObject;
            } else if ($setting == 'sso') {
                $jsonData = $this->sendGatewayRequest(sprintf('%s/saml/config', $this->container->getParameter('gateway.uri')), 'get', null);
                $data = json_decode($jsonData, true);
                $value = $data;
            } else if ($setting == 'syslog') {
                try {
                    $jsonData = $this->sendGatewayRequest(sprintf('%s/config/syslog', $this->container->getParameter('gateway.uri')), 'get', null);
                    $data = json_decode($jsonData, true);
                    $value = $data;
                } catch (\Exception $exception) {
                    $value = null;
                }
            } else if ($setting == 'LDAP' || $setting == 'TOKEN' || $setting == 'HA') {
                $jsonData = $this->sendRequest(sprintf('%s?section=%s', $authConfigUri, $setting), 'get', null, $tokenType);
                $data = json_decode($jsonData, true);
                $value = $data['hydra:member'];
            } else if ($setting == 'proxy') {
                /** @var ResourceInterface $picklistResource */
                $resource = $this->container->get('resource.proxy');
                $context = $resource->getDenormalizationContext();
                $context['export'] = True;
                $context["ignore_field_authorization"] = true;
                $context["ignore_authorization"] = true;
                $repository = $this->loadRepository(Proxy::class);
                /** @var Proxy $proxySettings */
                $proxySettings = $repository->findAll()[0];
                if (!$proxySettings) {
                    // No proxy settings exist
                    continue;
                }
                $normalizedObject = $this->container->get('serializer')->normalize($proxySettings, 'json-ld', $context);
                // Remove password from export
                $proxies = json_decode($normalizedObject['proxies'], true);
                foreach ($proxies as $ot_key => $fields) {
                    foreach ($fields as $in_key => $value) {
                        if ($in_key == 'password') {
                            unset($proxies[$ot_key][$in_key]);
                        }
                    }
                }
                $normalizedObject['proxies'] = json_encode($proxies);
                $value = $normalizedObject;
            }
            $settings[] = [
                'type' => $setting,
                'value' => $value
            ];
        }

        return $settings;
    }

    private function sendGatewayRequest($uri, $method, $data = null)
    {
        /** @var ProxyProvider $proxyProvider */
        $proxyProvider = $this->container->get('cybersponse.core.provider.proxy');
        /** @var ProxyClient $client */
        $client = $proxyProvider->getGatewayClient();
        $headers = array(
            "Content-type" => "application/json",
            "Authorization" => null
        );
        $request = new GuzzleRequest($method, $uri, $headers, $data ? json_encode($data) : null);
        try {
            $response = $client->send($request, ['verify' => false]);
            $statusCode = $response->getStatusCode();
            if ($statusCode < 400) {
                //$jsonDecodeResponse = json_decode($response->getBody()->getContents(), true);
                return $response->getBody()->getContents();
            } else {
                throw new \Exception($response->getBody()->getContents());
            }
        } catch (\Exception $exception) {
            $errCode = $exception->getCode();
            $message = $exception->getResponse() ? $exception->getResponse()->getBody()->getContents() : $exception->getMessage();
            throw new \Exception(sprintf('%d: %s', $errCode, $message));
        }
    }
}
