<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Service\Config;

use Cybersponse\CoreBundle\Entity\Picklist;
use Cybersponse\CoreBundle\Entity\PicklistNames;

/**
 * Class PicklistNameConfig
 * @package Cybersponse\CoreBundle\Service\Config
 */
class PicklistNameConfig extends PortingConfig
{
    /**
     * 
     * @param array $data List of picklist names
     * @return array
     */
    public function getImportOptions(array $data, $tokenType) {
        $repository = $this->loadRepository(PicklistNames::class);
        $options = [];
        foreach ($data as $picklistName) {
            $existing = $repository->findOneBy(['uuid' => $picklistName['uuid']]);
            $individualOptions = [];
            $individualOptions['uuid'] = $picklistName['uuid'];
            $individualOptions['name'] = $picklistName['name'];
            $individualOptions['count'] = count($picklistName['picklists']);
            $individualOptions['include'] = true;
            $individualOptions['exists'] = $existing ? true : false;

            $options[] = $individualOptions;
        }
        return $options;
    }

    /**
     * Imports the config data using the specified options
     *
     * @param array $data
     * @param array $options
     * @return mixed
     */
    public function import(array $data, array $options, $tokenType) {
        $toAdd = [];
        $toUpdate = [];
        $picklistRepository = $this->loadRepository(Picklist::class);
        foreach ($options as $picklistNameOption) {
            if (!$picklistNameOption['include']) {
                // Do not merge skipped items
                continue;
            }
            foreach ($data as $picklistName) {
                if ($picklistNameOption['uuid'] == $picklistName['uuid']) {
                    $newPicklists = [];
                    foreach ($picklistName['picklists'] as $picklist) {
                        $existingPicklist = $picklistRepository->findOneBy(['uuid' => $picklist['uuid']]);
                        // Use existing uuids if possible
                        if ($existingPicklist) {
                            $picklist['@id'] = '/api/3/picklists/' . $existingPicklist->getUuid();
                            unset($picklist['uuid']);
                        }
                        $newPicklists[] = $picklist;
                    }
                    $picklistName['picklists'] = $newPicklists;
                    if (!$picklistNameOption['exists']) {
                        $toAdd[] = $picklistName;
                        break;
                    }
                    unset($picklistName['uuid']);
                    $picklistName['@id'] = '/api/3/picklist_names/' . $picklistNameOption['uuid'];
                    $toUpdate[] = $picklistName;
                    break;
                }
            }
        }
        $results = [];
        foreach ($toAdd as $item) {
            $fullURI = sprintf('%s/api/3/picklist_names?$relationships=true', $this->container->getParameter('crudhub.uri'));
            $result = $this->sendRequest($fullURI, 'post', $item, $tokenType);
            $results[] = $result;
        }
        if (count($toUpdate) > 0) {
            $fullURI = sprintf('%s/api/3/update/picklist_names?$relationships=true', $this->container->getParameter('crudhub.uri'));
            $body = [
              "data" => $toUpdate
            ];
            $result = $this->sendRequest($fullURI, 'put', $body, $tokenType);
            $results[] = $result;
        }
        
        if (in_array(false, $results)) {
            return false;
        }
        return true;
    }

    /**
     * Exports the config using the specified options
     *
     * @param array $options
     * @return array
     */
    public function export(array $options, $tokenType='JWT') {
        return $this->getResourceFromIRI($options, PicklistNames::class, 'resource.picklistnames', true);
    }
}
