<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Service\Config;

use Cybersponse\AuthorizationBundle\Entity\Role;
use Cybersponse\CoreBundle\Entity\Module;

/**
 * Class RoleConfig
 * @package Cybersponse\CoreBundle\Service\Config
 */
class RoleConfig extends PortingConfig
{
    /**
     * 
     * @param array $data List of roles
     * @return array
     */
    public function getImportOptions(array $data, $tokenType) {
        $repository = $this->loadRepository(Role::class);
        $options = [];
        foreach ($data as $role) {
            $iriParts = explode('/', $role['@id']);
            $uuid = end($iriParts);
            $existing = $repository->findOneBy(['uuid' => $uuid]);
            $individualOptions = [];
            $individualOptions['uuid'] = $uuid;
            $individualOptions['name'] = $role['name'];
            $individualOptions['description'] = $role['description'];
            $individualOptions['include'] = true;
            $individualOptions['exists'] = $existing ? true : false;

            $options[] = $individualOptions;
        }
        return $options;
    }

    /**
     * Imports the config data using the specified options
     *
     * @param array $data
     * @param array $options
     * @return mixed
     */
    public function import(array $data, array $options, $tokenType) {
        $toAdd = [];
        $toUpdate = [];
        $moduleRepository = $this->loadRepository(Module::class);
        foreach ($options as $roleOption) {
            if (!$roleOption['include']) {
                // Do not merge skipped items
                continue;
            }
            foreach ($data as $role) {
                $iriParts = explode('/', $role['@id']);
                $uuid = end($iriParts);
                if ($roleOption['uuid'] == $uuid) {
                    $existingModulePermissions = [];
                    foreach ($role['modulePermissions'] as $modulePermission) {
                        $module = $moduleRepository->findOneBy(['type' => $modulePermission['module']]);
                        // Only include permissions for existing modules
                        if ($module) {
                            $modulePermission['module'] = '/api/3/modules/' . $module->getUuid();
                            unset($modulePermission['@id']);
                            $existingModulePermissions[] = $modulePermission;
                        }
                    }
                    $role['modulePermissions'] = $existingModulePermissions;
                    if (!$roleOption['exists']) {
                        unset($role['@id']);
                        $role['uuid'] = $uuid;
                        $toAdd[] = $role;
                        break;
                    }
                    $toUpdate[] = $role;
                    break;
                }
            }
        }
        $results = [];
        foreach ($toAdd as $item) {
            $fullURI = sprintf('%s/api/3/roles', $this->container->getParameter('crudhub.uri'));
            $result = $this->sendRequest($fullURI, 'post', $item, $tokenType);
            $results[] = $result;
        }
        if (count($toUpdate) > 0) {
            $fullURI = sprintf('%s/api/3/update/roles', $this->container->getParameter('crudhub.uri'));
            $body = [
              "data" => $toUpdate
            ];
            $result = $this->sendRequest($fullURI, 'put', $body, $tokenType);
            $results[] = $result;
        }
        
        if (in_array(false, $results)) {
            return false;
        }
        return true;
    }

    /**
     * Exports the config using the specified options
     *
     * @param array $options
     * @return array
     */
    public function export(array $options, $tokenType='JWT') {
        $roles = $this->getResourceFromIRI($options, Role::class, 'resource.role', true, false);
        foreach ($roles as $index => $role) {
            foreach ($role['modulePermissions'] as $modulePermissionIndex => $modulePermission) {
                if (array_key_exists('module', $modulePermission) && is_array($modulePermission['module'])) {
                    $roles[$index]['modulePermissions'][$modulePermissionIndex]['module'] = $modulePermission['module']['type'];
                }
            }
            
        }
        return $roles;
    }
}
