<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Service;

use Cybersponse\CoreBundle\Provider\CacheableInterface;
use Dunglas\ApiBundle\Api\ResourceCollectionInterface;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Dunglas\ApiBundle\JsonLd\ContextBuilder as DunglasContextBuilder;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\RouterInterface;

/**
 * @
 *
 * Class ContextBuilder
 * @package Cybersponse\CoreBundle\Service
 */
class ContextBuilder extends DunglasContextBuilder
{
    const API_CONTEXT_MAP_REDIS_KEY = 'api:context:map';

    /** @var array */
    protected $contextMap;

    /** @var CacheableInterface */
    protected $cache;

    /** @var ResourceCollectionInterface */
    protected $newResourceCollection;

    public function __construct(
        RouterInterface $router,
        EventDispatcherInterface $eventDispatcher,
        ResourceCollectionInterface $resourceCollection
    ) {
        parent::__construct($router, $eventDispatcher, $resourceCollection);
        $this->newResourceCollection = $resourceCollection;
    }

    public function getContext(ResourceInterface $resource = null)
    {
        if (is_null($resource)) {
            return parent::getContext($resource);
        }

        $contextMap = $this->getContexts();
        $entityName = $resource->getShortName();

        if (!array_key_exists($entityName, $contextMap)) {
            throw new NotFoundHttpException();
        }

        return $contextMap[$entityName];
    }

    /**
     * Returns the context entity map from redis cache
     *
     * @return array
     */
    public function getContexts()
    {
        $serializedContextMap = $this->cache->getFromCache(self::API_CONTEXT_MAP_REDIS_KEY);

        if ($serializedContextMap) {
            $this->contextMap = $serializedContextMap;
        } else {
            /** @var ResourceInterface $resource */
            foreach ($this->newResourceCollection as $resource) {
                $this->contextMap[$resource->getShortName()] = parent::getContext($resource);
            }
            $this->cache->persistToCache(self::API_CONTEXT_MAP_REDIS_KEY, $this->contextMap);
        }

        return $this->contextMap;
    }

    /**
     * @param CacheableInterface $cache
     */
    public function setCache($cache)
    {
        $this->cache = $cache;
    }


}
