<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Service;


use Dunglas\ApiBundle\Api\IriConverter;
use Dunglas\ApiBundle\JsonLd\Serializer\ItemNormalizer;

class DepthItemNormalizer extends ItemNormalizer
{

    /** @var IriConverter */
    private $depthIriConverter;

    public function normalize($object, $format = null, array $context = array())
    {
//        return parent::normalize($object, $format, $context);
        if (!array_key_exists('depth', $context)) {
            $context['depth'] = 0;
        }

        $context['depth']++;

        $returnValue = parent::normalize($object, $format, $context);

        $context['depth']--;

        return $returnValue;
    }

    /**
     * @param IriConverter $depthIriConverter
     */
    public function setDepthIriConverter($depthIriConverter)
    {
        $this->depthIriConverter = $depthIriConverter;
    }

    protected function isCircularReference($object, &$context)
    {
//        return parent::isCircularReference($object, $context);
        return parent::isCircularReference($object, $context) || $context['depth'] > 3;
    }

    protected function handleCircularReference($object)
    {
        $displayName = 'n/a';

        if (method_exists($object, 'getDisplayName')) {
            $displayName = $object->getDisplayName();
        }

        return [
            '@id' => $this->depthIriConverter->getIriFromItem($object),
            'displayName' => $displayName
        ];
    }
}
