<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Service;

use Cybersponse\CoreBundle\Entity\ImportJob;
use Cybersponse\CoreBundle\Service\Config\PortingConfig;

/**
 * Class ImportOptionsGenerator
 * @package Cybersponse\CoreBundle\Service
 */
class ImportOptionsGenerator extends ConfigPorter
{
    /**
     * Generates all options for import given the config object
     *
     * @param array $config
     * @return array
     */
    public function generate(array $fullConfig, ImportJob $importJob, $tokenType) {
        $options = [];
        $entityManager = $this->container->get('doctrine')->getManagerForClass(ImportJob::class);
        $increment = 90 / count($this->configTypes);
        foreach ($this->configTypes as $key => $service) {
            $config = array_key_exists($key, $fullConfig) ? $fullConfig[$key] : [];
            /** @var PortingConfig $portingConfig */
            $portingConfig = $this->container->get($service);
            $importOptions = $config ? $portingConfig->getImportOptions($config, $tokenType) : [];
            if (count($importOptions) > 0) {
                $options[$key] = [
                    "include" => true,
                    "values" => $importOptions
                ];
            }
            $importJob
                ->setProgressPercent($importJob->getProgressPercent() + $increment);
            $entityManager->persist($importJob);
            $entityManager->flush();
        }
        return $options;
    }
    
    /**
     * @param array $config
     * @return array
     */
    private function generateRecordSetOptions(array $recordSets) {
        $options = [];
        foreach ($recordSets as $recordSet) {
            $individualOptions = [];
            $individualOptions['module'] = $recordSet['module'];
            $individualOptions['count'] = $recordSet['count'];
            $individualOptions['include'] = true;

            $options[] = $individualOptions;
        }
        return $options;
    }

}
