<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Tests\Functional\AuthorizationResourceController\ResolvedInversedFieldImplementation;


use Cybersponse\TestBundle\Functional\ApiTestCase;

class PostFieldTest extends ApiTestCase
{
    const MOCK_M_ENTITY_UUID = 'e39b4db2-14d1-4755-8716-a0b11002f746';

    const MOCK_A_ENTITY_UUID = 'e71b7c08-d6eb-4d17-92ac-001c60a550dd';

    protected function setUp()
    {
        parent::setUp();
        $this->setRoles([
            $this->createModulePermission('mock_a_entities', true, true),
            $this->createModulePermission('mock_m_entities', true, true)
        ]);
        $this->requestVerb = 'POST';
    }

    public function getRequestUri()
    {
        return '/api/3/mock_a_entities';
    }

    public function test_GivenValidPermissions_GivenValidRequiredFields_ShouldReturn201()
    {
        $this->setJsonRequestContent([
            'uuid' => self::MOCK_A_ENTITY_UUID
        ]);

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $this->assertEquals(201, $response->getStatusCode());
    }

    public function test_GivenValidPermissions_GivenValidRequiredFields_GivenRelationshipsTrue_ShouldReturnCreatedFields()
    {
        $this->setJsonRequestContent([
            'uuid' => self::MOCK_A_ENTITY_UUID,
            'mockMEntities' => [
                ['uuid' => self::MOCK_M_ENTITY_UUID]
            ]
        ]);

        $this->requestParameters = ['$relationships' => true];

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $deserializedContent = json_decode($response->getContent(), true);

        $this->assertArrayHasKey('mockMEntities', $deserializedContent);
    }

    public function test_GivenValidPermissions_GivenValidRequiredFields_GivenValidItemId_ShouldReturnIriInId()
    {
        $expectedUuid = self::MOCK_A_ENTITY_UUID;
        $this->setJsonRequestContent([
            'uuid' => $expectedUuid
        ]);

        $this->executeSystemUnderTest();

        $response = $this->client->getResponse();
        $deserializedContent = json_decode($response->getContent(), true);
        $expectedIri = sprintf('%s/%s', $this->getRequestUri(), $expectedUuid);
        $this->assertEquals($deserializedContent['@id'], $expectedIri);
    }
}