<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Tests\Functional\Controller;


use Cybersponse\TestBundle\Functional\ApiTestCase;

class AuthorizationResourceController_GetWorkflowTest extends ApiTestCase
{
    protected function setUp()
    {
        parent::setUp();
        $this->requestUri = '/api/3/workflows/5c8d0bcb-1a17-4d0a-a9c8-d5ea6fbfcd30';
    }

    public function test_GivenExportFlag_GivenRelationshipsFlag_ShouldHaveZeroSteps()
    {
        $actualWorkflowObject = $this->getWorkflow();

        $this->assertCount(0, $actualWorkflowObject['steps']);
    }

    public function test_GivenExportFlag_GivenRelationshipsFlag_ShouldHaveStepsAsArrays()
    {
        $this->markTestSkipped('Test fixtures with nested objects causes problems on insert currently.');
        $actualWorkflowObject = $this->getWorkflow();

        $this->assertTrue(is_array($actualWorkflowObject['steps'][0]));
    }

    public function test_GivenExportFlag_GivenRelationshipsFlag_ShouldHaveOneRoute()
    {
        $actualWorkflowObject = $this->getWorkflow();

        $this->assertCount(0, $actualWorkflowObject['routes']);
    }

    public function test_GivenExportFlag_GivenRelationshipsFlag_ShouldHaveUuidField()
    {
        $actualWorkflowObject = $this->getWorkflow();

        $this->assertArrayHasKey('uuid', $actualWorkflowObject);
    }

    public function test_GivenExportFlag_GivenRelationshipsFlag_ShouldNotHaveAtIdField()
    {
        $actualWorkflowObject = $this->getWorkflow();

        $this->assertArrayNotHasKey('@id', $actualWorkflowObject);
    }

    public function test_GivenExportFlag_GivenRelationshipsFlag_ShouldHaveAtTypeField()
    {
        $actualWorkflowObject = $this->getWorkflow();

        $this->assertArrayHasKey('@type', $actualWorkflowObject);
    }

    public function test_GivenExportFlag_GivenRelationshipsFlag_ShouldHaveUuidFieldOnStep()
    {
        $this->markTestSkipped('Test fixtures with nested objects causes problems on insert currently.');
        $actualWorkflowObject = $this->getWorkflow();

        $this->assertArrayHasKey('uuid', $actualWorkflowObject['steps'][0]);
    }

    public function test_GivenExportFlag_GivenRelationshipsFlag_ShouldHaveUuidFieldOnRoute()
    {
        $this->markTestSkipped('Test fixtures with nested objects causes problems on insert currently.');
        $actualWorkflowObject = $this->getWorkflow();

        $this->assertArrayHasKey('uuid', $actualWorkflowObject['routes'][0]);
    }

    public function test_GivenExportFlag_GivenRelationshipsFlag_ShouldHaveStringStepTypeFieldOnStep()
    {
        $this->markTestSkipped('Test fixtures with nested objects causes problems on insert currently.');
        $actualWorkflowObject = $this->getWorkflow();

        $this->assertTrue(is_string($actualWorkflowObject['steps'][0]['stepType']));
    }

    public function test_GivenExportFlag_ShouldHaveNoRoutes()
    {
        $actualWorkflowObject = $this->getWorkflow(true, false);

        $this->assertArrayNotHasKey('routes', $actualWorkflowObject);
    }

    public function test_GivenExportFlag_ShouldHaveNoSteps()
    {
        $actualWorkflowObject = $this->getWorkflow(true, false);

        $this->assertArrayNotHasKey('steps', $actualWorkflowObject);
    }

    public function test_GivenRelationshipFlag_ShouldHaveSteps()
    {
        $actualWorkflowObject = $this->getWorkflow(true, true);

        $this->assertArrayHasKey('steps', $actualWorkflowObject);
    }

    public function test_GivenRelationshipFlag_ShouldHaveRoutes()
    {
        $actualWorkflowObject = $this->getWorkflow(true, true);

        $this->assertArrayHasKey('routes', $actualWorkflowObject);
    }

    public function test_GivenExportFlag_ShouldHaveUuidField()
    {
        $actualWorkflowObject = $this->getWorkflow(true, false);

        $this->assertArrayHasKey('uuid', $actualWorkflowObject);
    }

    public function test_GivenNoFlags_ShouldNotHaveUuidField()
    {
        $actualWorkflowObject = $this->getWorkflow(false, false);

        $this->assertArrayNotHasKey('uuid', $actualWorkflowObject);
    }

    public function test_GivenNoFlags_ShouldHaveAtIdField()
    {
        $actualWorkflowObject = $this->getWorkflow(false, false);

        $this->assertArrayHasKey('@id', $actualWorkflowObject);
    }

    /**
     * @return mixed
     */
    protected function getWorkflow($export = true, $relationships = true)
    {
        $this->setRoles([
            $this->createModulePermission('workflows', false, true)
        ]);

        $queryParameters = [
            '$export' => $export,
            '$relationships' => $relationships
        ];

        //Remove any keys with false as the value
        $filteredQueryParameters = array_filter($queryParameters, function ($value) {
            return $value === true;
        });

        $this->requestParameters = $filteredQueryParameters;

        $this->executeSystemUnderTest();

        $actualWorkflowObject = json_decode($this->client->getResponse()->getContent(), true);
        return $actualWorkflowObject;
    }
}
