<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Traits;


use Cybersponse\CoreBundle\Provider\RuntimeContextProvider;
use Cybersponse\QueryBundle\Constants\AttributeLevelOperators;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

class TraitFlagProvider
{
    const IGNORE_TRAITS_KEY = AttributeLevelOperators::OPERATOR_TOKEN . 'ignoreTraits';

    /** @var boolean[] */
    protected $traitFlags;

    /** @var ContainerInterface */
    protected $container;

    /** @var RuntimeContextProvider */
    protected $runtimeContextProvider;

    /**
     * @param ContainerInterface $container
     */
    public function setContainer($container)
    {
        $this->container = $container;
    }

    /**
     * @param $trait
     * @return bool
     */
    public function ignoreTrait($trait)
    {
        $token = $this->container->get('security.token_storage');
        $rawToken = $token->getToken();

        // Use default if the token is not set
        if (is_null($rawToken)) {
            return $this->runtimeContextProvider->isCli();
        }

        if (!isset($this->traitFlags)) {
            $this->initializeFlags();
        }

        $traitInstance = new \ReflectionClass($trait);
        $traitShortName = $traitInstance->getShortName();

        $traitFlagExists = array_key_exists(strtolower($traitShortName), $this->traitFlags);

        $authorizationChecker = $this->container->get('security.authorization_checker');
        if ($traitFlagExists && !$authorizationChecker->isGranted(['perm:security.update'])) {
            throw new AccessDeniedException(sprintf('Access Denied. Security UPDATE permission required to ignore "%s" trait.', $traitShortName));
        }

        return $traitFlagExists;
    }

    /**
     * @param RuntimeContextProvider $runtimeContextProvider
     */
    public function setRuntimeContextProvider($runtimeContextProvider)
    {
        $this->runtimeContextProvider = $runtimeContextProvider;
    }

    protected function initializeFlags()
    {
        $requestStack = $this->container->get('request_stack');

        $this->traitFlags = [];
        $currentRequest = $requestStack->getCurrentRequest();
        if (!is_null($currentRequest)) {
            $traitsCsv = $currentRequest->query->get(self::IGNORE_TRAITS_KEY, '');
            $traits = explode(',', $traitsCsv);

            foreach ($traits as $trait) {
                $this->traitFlags[strtolower($trait)] = true;
            }
        }
    }
}
