<?php

namespace Cybersponse\CustomerTenantBundle\Controller;

use Symfony\Component\Yaml\Yaml;
use Cybersponse\UtilsBundle\Utils;
use Dunglas\ApiBundle\Event\Events;
use Cybersponse\UtilsBundle\ARCUtils;
use Dunglas\ApiBundle\JsonLd\Response;
use Cybersponse\CoreBundle\Api\ApiEvents;
use Symfony\Component\Serializer\Serializer;
use Symfony\Component\HttpFoundation\Request;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Cybersponse\QueryBundle\Query\ExpressionBuilder;
use Dunglas\ApiBundle\Exception\DeserializationException;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Cybersponse\CoreBundle\Controller\AuthorizationResourceController;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;

class AgentController extends AuthorizationResourceController
{
    const MODULE_TYPE = "agents";
    const ALLOW_REMOTE_MMD = 'allowRemoteMMDModification';
    const REPLICATION_ROUTINGKEY = 'route.crudhub.tenant.request.agentconfig';
    const ARCHIVESOURCE = '/opt/cyops-api/agentconfig/source/';
    const ARCHIVEDESTINATION = '/opt/cyops-api/agentconfig/destination/';
    const ARCHIVEFILE = '/opt/cyops-api/agentconfig/';

    protected function appendAdditionalFilters()
    {
        $queryProvider = $this->container->get('cybersponse.query.provider.query_provider');
        $accessibleQueryFilter =
            ExpressionBuilder::logicAnd(
                ExpressionBuilder::logicOr(
                    ExpressionBuilder::field('role')->notEquals('master')
                ),
                ExpressionBuilder::logicOr(
                    ExpressionBuilder::field('role')->notEquals('self')
                )
            );
        $queryProvider->setAdditionalFilters($accessibleQueryFilter->toArray());
    }

    /**
     * @param Request $request
     * @return Response|static
     */
    public function getAction(Request $request)
    {
        $this->appendAdditionalFilters();
        return parent::cgetAction($request);
    }

    public function postAction(Request $request)
    {
        $utilsObj = new Utils();
        $resource = $this->identifyResourceByShortName(self::MODULE_TYPE);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_CREATE);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_CREATE, new DataEvent($resource, null));

        $blockPassword = Utils::isPasswordBlock($request);
        $decodedRequestData = json_decode($request->getContent(), true);
        $this->prePostCheck($decodedRequestData, $resource, $utilsObj);
        $request_data = $this->prePostAdd($resource, $decodedRequestData, $utilsObj);

        $object = $this->createObject($request_data, $resource, $resource->getDenormalizationContext());

        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_CREATE_VALIDATION, new DataEvent($resource, $object));

        $violations = $this->get('validator')->validate($object, null, $resource->getValidationGroups());
        if (0 === count($violations)) {
            foreach ($this->getUser()->getTeams() as $owner) {
                $object->addOwner($owner);
            }
            $existing_relationship = ARCUtils::setRelationShip($request, false);
            $existing_set_asJson = ARCUtils::setAsJson($request, false);
            // Validation succeed
            $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_CREATE, new DataEvent($resource, $object));
            /** @var Serializer $serializer */
            $serializer = $this->get('serializer');
            $updatedObject = $serializer->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext() + ['block_password' => false]);

            ARCUtils::setRelationShip($request, $existing_relationship);
            ARCUtils::setAsJson($request, $existing_set_asJson);

            $data = $this->prepareChangeData($updatedObject, (object) null, 'Create');
            $data['esoperation'] = 'index';
            $utilsObj->sendData($this, [$data]);

            $agentDiffObj = $utilsObj->prepareReplicationDiffData($updatedObject, null);
            $agentData = $utilsObj->prepareReplicationData($agentDiffObj, 'POST', null, null);

            $utilsObj->sendReplicationData($this, $agentData, self::REPLICATION_ROUTINGKEY);

            return $this->getSuccessResponse($resource, $object, Response::HTTP_CREATED, [], [], [], $blockPassword);
        }
        return $this->getErrorResponse($violations);
    }

    /**
     * @param $decodedRequestData
     * @return bool|Response
     * @throws HttpException
     */
    public function prePostCheck($decodedRequestData, $resource, $utilsObj)
    {

        $agentId = isset($decodedRequestData['agentId']) ? $decodedRequestData['agentId'] : null;
        if ($agentId) {
            $existingAgentObj = $utilsObj->findWithWhere($this, $resource, ['agentId' => $agentId]);
            if ($existingAgentObj) {
                throw new HttpException(Response::HTTP_BAD_REQUEST, "AgentId: '" . $agentId . "' already exists, use different AgentId");
            }
        }
        $agentName = isset($decodedRequestData['name']) ? $decodedRequestData['name'] : null;
        if ($agentName) {
            $existingAgentObj = $utilsObj->findWithWhere($this, $resource, ['name' => $agentName]);
            if ($existingAgentObj) {
                throw new HttpException(Response::HTTP_BAD_REQUEST, "Agent Name: '" . $agentName . "' already exists, use different Agent Name");
            }
        }

        if ($decodedRequestData['isLocal']) {
            throw new HttpException(Response::HTTP_FORBIDDEN, 'UnAuthorized Access');
        }

        if (isset($decodedRequestData['role']) && $decodedRequestData['role'] == 'self') {
            throw new HttpException(Response::HTTP_BAD_REQUEST, 'Self Agent cannot be added');
        }
    }

    public function startsWith($haystack, $needle)
    {
        $length = strlen($needle);
        return (substr($haystack, 0, $length) === $needle);
    }

    public function getAgentPassword($password)
    {
        $command = sprintf("python3.6 /opt/cyops/configs/scripts/manage_passwords.py --decrypt '" . $password . "' 'jQp3(7@jod#j38d1'");
        exec($command, $output, $errorCode);
        if (!$errorCode) {
            $output = $output[0];
            if (!$this->startswith($output, 'Password:')) {
                throw new HttpException(Response::HTTP_BAD_REQUEST, 'DB Password decrypt failed with error %s' % $output);
            } else {
                return substr($output, strlen('Password:'), strlen($output));
            }
        } else {
            throw new HttpException(Response::HTTP_BAD_REQUEST, 'DB Password decrypt failed with error %s' % $errorCode);
        }
    }

    public function prePostAdd($resource, $decodedRequestData, $utilsObj)
    {
        if (!isset($decodedRequestData['configstatus'])) {
            $decodedRequestData['configstatus'] = '/api/3/picklists/0dff7239-38a4-4dc8-9fe6-97932d2bd35f';
        }
        if (!isset($decodedRequestData['configurationHealth'])) {
            $decodedRequestData['configurationHealth'] = '/api/3/picklists/fb340c77-c2c6-471d-8081-77c9302b4bfb';
        }
        if (!isset($decodedRequestData['agentId'])) {
            $decodedRequestData['agentId'] = bin2hex(random_bytes(16));
        }
        if (!isset($decodedRequestData['password'])) {
            $decodedRequestData['password'] = bin2hex(random_bytes(16));
        } else {
            $decodedRequestData['password'] = $this->getAgentPassword($decodedRequestData['password']);
        }
        if (!isset($decodedRequestData['vhost'])) {
            $decodedRequestData['vhost'] = 'vhost_' . $decodedRequestData['agentId'];
        }
        if (!isset($decodedRequestData['username'])) {
            $decodedRequestData['username'] = 'user_' . $decodedRequestData['agentId'];
        }
        if (!isset($decodedRequestData['tenants'])) {
            $decodedRequestData['tenants'] = ['b3a700f7-00be-4ef9-90c6-3c8fe6e1be63'];
        }
        if (!isset($decodedRequestData['ssl'])) {
            $decodedRequestData['ssl'] = true;
        }
        if (!isset($decodedRequestData['agentType'])) {
            $decodedRequestData['agentType'] = '502cb999-d0fe-466f-9d99-1f3300513e14';
        }
        if (!isset($decodedRequestData['role'])) {
            $decodedRequestData['role'] = 'tenant';
        }
        if (!isset($decodedRequestData['active'])) {
            $decodedRequestData['active'] = true;
        }
        if (!isset($decodedRequestData['version'])) {
            $decodedRequestData['version'] = $this->getParameter('product_version');
        }
        $decodedRequestData['isLocal'] = false;

        return $decodedRequestData;
    }

    public function prePutValidate($decodedRequestData)
    {

        if (isset($decodedRequestData['password'])) {
            $decodedRequestData['password'] = $this->getAgentPassword($decodedRequestData['password']);
        }
        return $decodedRequestData;
    }

    public function putAction(Request $request, $id)
    {
        $resource = $this->identifyResourceByShortName(self::MODULE_TYPE);
        $utilsObj = new Utils();
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_UPDATE, new DataEvent($resource, null));
        $objectData = json_decode($request->getContent(), true);
        $blockPassword = Utils::isPasswordBlock($request);

        $objectData = $this->prePutValidate($objectData);

        $object = $this->findOrThrowNotFound($resource, $id, $request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE, $object);
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_UPDATE_DESERIALIZE, new DataEvent($resource, $object));

        $context = $resource->getDenormalizationContext();
        $context['object_to_populate'] = $object;
        /** @var Serializer $serializer */
        $serializer = $this->get('serializer');

        $existing_set_asJson = ARCUtils::setAsJson($request, false);
        $existing_relationship = ARCUtils::setRelationShip($request, false);
        $preUpdatedObject = $serializer->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext() + ['block_password' => false]);

        if ($objectData['upgradeMessage'] && $preUpdatedObject['upgradeMessage']) {
            $objectData['upgradeMessage'] = $preUpdatedObject['upgradeMessage'] . ' <br> ' . $objectData['upgradeMessage'];
        }

        if (array_key_exists("__link", $objectData) || array_key_exists("__unlink", $objectData)) {
            $objectData = ARCUtils::mergeRequest($objectData, $preUpdatedObject);
        }

        $object = $this->createObject($objectData, $resource, $context);
        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_UPDATE_VALIDATION, new DataEvent($resource, $object));

        $violations = $this->get('validator')->validate($object, null, $resource->getValidationGroups());
        if (0 === count($violations)) {
            // Validation succeed
            $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_UPDATE, new DataEvent($resource, $object, $preUpdatedObject));

            $updatedObject = $serializer->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext() + ['block_password' => false]);
            $data = $this->prepareChangeData($updatedObject, $preUpdatedObject, 'Update', $objectData);
            $data['esoperation'] = 'modify';
            $agentDiffObj = $utilsObj->prepareReplicationDiffData($updatedObject, $preUpdatedObject);
            $agentData = $utilsObj->prepareReplicationData($agentDiffObj, 'PUT', null, null);

            if (array_key_exists("cs_notify", $objectData)) {
                // If The call is from Integration determined by <cs_notify> flag in request
                // check for last status and current status if change notify all else skip
                if ($preUpdatedObject['configurationHealth'] != $updatedObject['configurationHealth']) {
                    $utilsObj->sendData($this, [$data]);
                    $utilsObj->sendReplicationData($this, $agentData, self::REPLICATION_ROUTINGKEY);
                }
            } else {
                $utilsObj->sendData($this, [$data]);
                $utilsObj->sendReplicationData($this, $agentData, self::REPLICATION_ROUTINGKEY);
            }

            ARCUtils::setRelationShip($request, $existing_relationship);
            ARCUtils::setAsJson($request, $existing_set_asJson);

            return $this->getSuccessResponse($resource, $object, Response::HTTP_OK, [], [], [], $blockPassword);
        }
        return $this->getErrorResponse($violations);
    }

    /**
     * @param Request $request
     * @return Response
     * @throws DeserializationException
     */


    public function deleteAgent($request, $resource, $object)
    {
        $this->get(self::EVENT_DISPATCHER)->dispatch(ApiEvents::PRE_API_DELETE, new DataEvent($resource, null));
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE, $object);

        /** @var Serializer $serializer */
        $serializer = $this->get('serializer');
        $request->query->set("\$relationships", true);
        $previousObject = $serializer->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext() + ['block_password' => false]);
        if (isset($previousObject['role']) && $previousObject['role'] == 'self') {
            throw new HttpException(Response::HTTP_BAD_REQUEST, "Deletion of self Agent is not allowed");
        }
        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_DELETE, new DataEvent($resource, $object));

        $request->query->set("\$relationships", true);

        $data = $this->prepareChangeData((object) null, $previousObject, 'Delete');
        $utilsObj = new Utils();

        $utilsObj->sendData($this, [$data]);

        $agentDiffObj = $utilsObj->prepareReplicationDiffData((object) null, $previousObject);
        $agentData = $utilsObj->prepareReplicationData($agentDiffObj, 'DELETE', null, null);
        $utilsObj->sendReplicationData($this, $agentData, self::REPLICATION_ROUTINGKEY);
    }


    public function deleteAction(Request $request, $id)
    {
        $resource = $this->identifyResourceByShortName(self::MODULE_TYPE);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE);
        $object = $this->findOrThrowNotFound($resource, $id, $request);
        $this->deleteAgent($request, $resource, $object);
        return new Response(null, Response::HTTP_NO_CONTENT, []);
    }

    public function bulkDeleteAction(Request $request)
    {
        $resource = $this->identifyResourceByShortName(self::MODULE_TYPE);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_DELETE);
        $request_body = json_decode($request->getContent());
        $ids = $request_body->ids;
        foreach ($ids as $id) {
            $object = $this->findOrThrowNotFound($resource, $id, $request);
            $this->deleteAgent($request, $resource, $object);
        }
        return new Response(null, Response::HTTP_NO_CONTENT, []);
    }

    /**
     * @param Request $request
     * @return Response
     */
    public function configureAction(Request $request)
    {
        $resource = $this->identifyResourceByShortName(self::MODULE_TYPE);
        $request_body = json_decode($request->getContent(), true);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_CREATE);
        $agentId = isset($request_body['agentId']) ? $request_body['agentId'] : null;

        $explodeAgentObj = explode('/', $agentId);
        $request->query->set("\$relationships", true);
        $object = $this->findOrThrowNotFound($resource, end($explodeAgentObj), $request);

        /** @var Serializer $serializer */
        $serializer = $this->get('serializer');
        $normalizedAgentData = $serializer->normalize($object, self::NORMALIZER_METHOD, $resource->getNormalizationContext() + ['block_password' => false]);
        $utilsObj = new Utils();
        $normalizedAgentData['configstatus'] = '/api/3/picklists/0dff7239-38a4-4dc8-9fe6-97932d2bd35f';
        $agentDiffObj = $utilsObj->prepareReplicationDiffData($normalizedAgentData);
        $agentData = $utilsObj->prepareReplicationData($agentDiffObj, 'POST', null, null);
        $utilsObj->sendReplicationData($this, $agentData, self::REPLICATION_ROUTINGKEY);

        /*Updating Config to InProgress*/
        $request_data = $normalizedAgentData;
        $context = $resource->getDenormalizationContext();
        $context['object_to_populate'] = $object;

        $object = $this->createObject($request_data, $resource, $context);
        $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_UPDATE_VALIDATION, new DataEvent($resource, $object));

        $violations = $this->get('validator')->validate($object, null, $resource->getValidationGroups());
        if (0 === count($violations)) {
            $this->get(self::EVENT_DISPATCHER)->dispatch(Events::PRE_UPDATE, new DataEvent($resource, $object, $normalizedAgentData));
            return new Response(
                null,
                Response::HTTP_NO_CONTENT
            );
        }
        return $this->getErrorResponse($violations);
    }
}
