<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\FilesBundle\Controller;

use Ramsey\Uuid\Uuid;
use Dunglas\ApiBundle\JsonLd\Response;
use Cybersponse\FilesBundle\Entity\PgFile;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Serializer\Serializer;
use Symfony\Component\HttpFoundation\Request;
use Cybersponse\FilesBundle\Entity\AbstractFile;
use Cybersponse\FilesBundle\Provider\FilesProvider;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Cybersponse\AuthorizationBundle\Ownership\OwnershipProvider;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Cybersponse\CoreBundle\Controller\AuthorizationResourceController;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;

class FilesController extends AuthorizationResourceController
{
    const WIDTH = 65;
    const HEIGHT = 65;
    /**
     * @return FilesProvider
     */
    private function fileProvider()
    {
        return $this->get('cybersponse.files.provider.files');
    }

    /** @var OwnershipProvider */
    protected $ownershipProvider;
    /**
     * @param OwnershipProvider $ownershipProvider
     */
    public function setOwnershipProvider($ownershipProvider)
    {
        $this->ownershipProvider = $ownershipProvider;
    }
    /** @var TeamsProvider */
    protected $teamProvider;
    /**
     * @param TeamsProvider $teamProvider
     */

    protected $fileProvider;

    public function setFileProvider()
    {
        $this->fileProvider = $this->fileProvider();
    }
    public function setTeamProvider($teamProvider)
    {
        $this->teamProvider = $teamProvider;
    }


    public function uploadFile($request)
    {
        $this->setFileProvider();
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_CREATE);

        if (!$request->files->has('file')) {
            // throw new BadRequestHttpException('No file specified.');
            $request_body = json_decode($request->getContent(), true);
            if ($request_body['content'] && $request_body['filename']) {
                $tempPath = "/tmp/widget/".$request_body['filename'];
                $tempFileSystem = new Filesystem();
                $tempFileSystem->dumpFile($tempPath, $request_body['content']);
                /** @var UploadedFile $upload */
                $uploadedFile = new uploadedFile($tempPath, $request_body['filename']);
                $upload = $request->files->get('file', $uploadedFile);
                $request_body['metadata'] ? $request->request->set('metadata', json_encode($request_body['metadata'])):$request->request->set('metadata', json_encode([]));
            }
        } else {
            /** @var UploadedFile $upload */
            $upload = $request->files->get('file');
            if (!$upload->isValid()) {
                throw new BadRequestHttpException($upload->getErrorMessage());
            }
        }
        $fileBody = [];
        $fileBody['filename'] = $upload->getClientOriginalName();
        $fileBody['mimeType'] = $upload->getClientMimeType();
        $fileBody['size'] = $upload->getSize();
        $timestamp = (new \DateTime('NOW', new \DateTimeZone('UTC')))->getTimestamp();
        $fileBody['uploadDate'] = $timestamp;
        $fileBody['metadata'] = json_decode($request->request->get('metadata', "{}"), true);

        $pgFile = new PgFile();
        $container = $this->container->get('doctrine');
        $EntityManager = $container->getManagerForClass(PgFile::class);
        /** @var AbstractFile $file */
        $strm = fopen($upload->getPathname(), 'rb');
        $pgFile->setFile(base64_encode(stream_get_contents($strm)));
        $EntityManager->persist($pgFile);
        $EntityManager->flush();
        if (substr_count($fileBody['mimeType'], 'image/')) {
            list($wid, $ht) = getimagesize($upload->getPathname());
            $fileBody['thumbnail'] = "data:image/jpeg;base64,".base64_encode($this->getImageThumbnail($fileBody['mimeType'], $pgFile->getFile(), $wid, $ht));
        }
        $fileBody['file'] = "/api/3/pgfile/" . $pgFile->getUuid();
        $fileBody['uuid'] = Uuid::uuid4()->toString();
        $fileBody['id'] = $fileBody['uuid'];
        return $fileBody;
    }

    public function updateAction(Request $request, $id)
    {
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_UPDATE);
        if (!$request->files->has('file')) {
            $request_body = json_decode($request->getContent(), true);
            $file =  parent::findOrThrowNotFound($resource, $id);
            $strm = $request_body['content'];
            $request_body['size'] = strlen($request_body['content']);
        } else {
            /** @var UploadedFile $upload */
            $upload = $request->files->get('file');
            if (!$upload->isValid()) {
                throw new BadRequestHttpException($upload->getErrorMessage());
            }
            $fstrm = fopen($upload->getPathname(), 'rb');
            $strm = stream_get_contents($fstrm);
        }

        $entityManager = $this->container->get('doctrine')->getManagerForClass(PgFile::class);
        $pgFile = $entityManager->find(PgFile::class, end(explode('/', $file->getFile())));
        $pgFile->setFile(base64_encode($strm));
        $entityManager->persist($pgFile);
        $entityManager->flush();
        
        $request->initialize(
            $request->query->all(),
            $request->request->all(),
            $request->attributes->all(),
            $request->cookies->all(),
            $request->files->all(),
            $request->server->all(),
            json_encode($request_body)
        );
        return $this->putAction($request, $id);
    }
    public function uploadAction(Request $request)
    {
        $fileBody = $this->uploadFile($request);
        return $this->cpostAction($request, $fileBody);
    }

    public function downloadAction(Request $request, $id)
    {
        $resource = $this->getResource($request);
        $accept = $request->headers->get('Accept', 'application/json');
        $request->attributes->set('id', $id);
        /** @var AbstractFile $file */
        $file =  parent::findOrThrowNotFound($resource, $id);

        if (strpos($accept, 'application/json') !== false) {
            $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
            return $this->getSuccessResponse($resource, $file);
        } else {
            if ($file->getSize() > 0) {
                $container = $this->container->get('doctrine');
                $EntityManager = $container->getManagerForClass(PgFile::class);
                $EntityRepository = $EntityManager->getRepository(PgFile::class);
                $explodeId = explode('/', $file->getFile());
                $fid = end($explodeId);
                $FileRecord = $EntityRepository->findOneBy(['uuid' => $fid]);
                $response = new Response();
                $response->headers->set('Content-Type', $file->getMimeType());
                $response->headers->set('Content-Disposition', 'attachment; filename="' . $file->getFilename() . '"');
                $response->headers->set('filename', $file->getFilename());
                $response->headers->set('Content-Length', $file->getSize());
                $response->setContent(base64_decode(stream_get_contents($FileRecord->getFile())));

                return $response;
            } else {
                return new Response(null, Response::HTTP_NO_CONTENT);
            }
        }
    }

    /**
     * @param Request $request
     * @param $content
     */
    protected function reinitializeRequest(Request &$request, $content)
    {
        $request->initialize(
            $request->query->all(),
            $request->request->all(),
            $request->attributes->all(),
            $request->cookies->all(),
            $request->files->all(),
            $request->server->all(),
            $content
        );
    }
    protected function getImageThumbnail($mimeType, $imageString, $wid, $ht)
    {
        $r = $wid/$ht;
        {
            if (self::WIDTH/self::HEIGHT > $r) {
                $new_width = self::HEIGHT*$r;
                $new_height = self::HEIGHT;
            } else {
                $new_height = self::WIDTH/$r;
                $new_width = self::WIDTH;
            }
        }
        $source = imagecreatefromstring(base64_decode($imageString));
        $dst = imagecreatetruecolor($new_width, $new_height);
        imagecopyresampled($dst, $source, 0, 0, 0, 0, $new_width, $new_height, $wid, $ht);
        ob_start();
        imagejpeg($dst, null, 100);
        return ob_get_clean();
    }
    /**
     * @param $object object to normalize
     * @return string[] Array of teamIris
     */
    protected function getTeamIris($object)
    {
        if ($object != null) {
            $owners = $object->getOwners();
        }
        $teamIds = array();
        if ($owners != null) {
            foreach ($owners->toArray() as $team) {
                array_push($teamIds, $team->getUuid());
            }
        }
        $teamBaseIri = $this->fileProvider->iriConverter->getIriFromResource($this->fileProvider->teamResource);
        return array_map(function ($teamId) use ($teamBaseIri) {
            return sprintf('%s/%s', $teamBaseIri, $teamId);
        }, $teamIds);
    }
}
