<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */
namespace Cybersponse\FilesBundle\Provider;

use Cybersponse\AuthorizationBundle\Ownership\OwnershipProvider;
use Doctrine\ORM\EntityManager;
use Cybersponse\FilesBundle\Entity\PgFile;

class FilesProvider
{
    /** @var IriConverterInterface */
    public $iriConverter;
    /** @var ResourceInterface */
    public $teamResource;
    /** @var EntityManager */
    protected $entityManager;
    /** @var OwnershipProvider */
    protected $ownershipProvider;
    /** @var TeamsProvider */
    protected $teamProvider;
    /** @var ItemNormalizer */
    protected $itemNormalizer;
    /** @var ResourceCollectionInterface */
    protected $resourceCollection;
    /** @var ContainerInterface */
    protected $container;

    /**
     * @param ContainerInterface $container
     */
    public function setContainer($container)
    {
        $this->container = $container;
    }

    /**
     * @param EntityManager $entityManager
     */
    public function setViewTemplateEntityManager($entityManager)
    {
        $this->entityManager = $entityManager;
    }

    /**
     * @inheritdoc
     * @return null
     */
    public function saveTemplate($template)
    {
        $this->entityManager->persist($template);
        $this->entityManager->flush();
    }

    /**
     *
     * @param $type string
     * @param array $templateBody
     * @param String $id
     * @return GenericTemplate $genericTemplateObj
     *
     */
    public function saveFiles($fileObj)
    {
        $this->entityManager->persist($fileObj);
        $this->entityManager->flush();
        return $fileObj;
    }

    public function removePgFile($fileObj) {
        $metadataEntityManager = $this->container->get('doctrine.orm.entity_manager');
        $pgFileRepo = $metadataEntityManager->getRepository(PgFile::class);
        if ($fileObj->getFile()) {
            $pgFileId = end(explode('/', $fileObj->getFile()));
            $pgFile = $pgFileRepo->findOneBy(array("uuid" => $pgFileId));
            $this->entityManager->remove($pgFile);
            $this->entityManager->flush();
        }
    }

    public function removeFiles($fileObj)
    {
        $this->removePgFile($fileObj);
        $this->entityManager->remove($fileObj);
        $this->entityManager->flush();
        return $fileObj;
    }

    /**
     * @param OwnershipProvider $ownershipProvider
     */
    public function setOwnershipProvider(OwnershipProvider $ownershipProvider)
    {
        $this->ownershipProvider = $ownershipProvider;
    }

    /**
     * @param ItemNormalizer $itemNormalizer
     */
    public function setItemNormalizer($itemNormalizer)
    {
        $this->itemNormalizer = $itemNormalizer;
    }

    /**
     * @param TeamsProvider $teamProvider
     */
    public function setTeamProvider($teamProvider)
    {
        $this->teamProvider = $teamProvider;
    }

    /**
     * @param IriConverterInterface $iriConverter
     */
    public function setIriConverter($iriConverter)
    {
        $this->iriConverter = $iriConverter;
    }

    /**
     * @param ResourceCollectionInterface $resourceCollection
     */
    public function setResourceCollection($resourceCollection)
    {
        $this->resourceCollection = $resourceCollection;
    }

    /**
     * @param ResourceInterface $teamResource
     */
    public function setTeamResource($teamResource)
    {
        $this->teamResource = $teamResource;
    }
}
