<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */
namespace Cybersponse\FilesBundle\Serializer;
use Symfony\Component\HttpFoundation\File\File as httpFile;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;
use Symfony\Component\Serializer\Normalizer\DenormalizerInterface;
class FileNormalizer implements NormalizerInterface, DenormalizerInterface
{
    const FORMAT_JSON_LD = 'json-ld';
    const FORMAT_JSON = 'json';
    /** @var IriConverterInterface */
    protected $iriConverter;
    /**
     * {@inheritdoc}
     */
    public function supportsNormalization($data, $format = null)
    {
        return (self::FORMAT_JSON_LD === $format || self::FORMAT_JSON === $format) && is_string($data)  && strpos($data,'api/3/pgfile');
    }
    /**
     * {@inheritdoc}
     */
    public function supportsDenormalization($data, $type, $format = null)
    {
        return false;
    }
    /**
     * @param httpFile $object
     * {@inheritdoc}
     */
    public function normalize($object, $format = null, array $context = [])
    {
        $data = [];
        if (array_key_exists('include_file_data', $context) && $context['include_file_data'] === true) {
            $file = $this->iriConverter->getItemFromIri($object, true);
            if(is_string($file->getFile()))
            {
                $data['data'] = $file->getFile();
            } elseif(get_resource_type($file->getFile()) == "stream")
            {
                $data['data'] = stream_get_contents($file->getFile());
            }
        }
        return $data;
    }
    /**
     * @param File $data
     * {@inheritdoc}
     */
    public function denormalize($data, $class, $format = null, array $context = [])
    {
        throw new \Exception('Uploading files are not supported via this method.');
    }


    /**
     * @param IriConverterInterface $iriConverter
     */
    public function setIriConverter($iriConverter)
    {
        $this->iriConverter = $iriConverter;
    }

}
