<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\FilesBundle\Tests\Functional;


use Doctrine\ORM\EntityManager;
use Doctrine\Common\Persistence\ManagerRegistry;
use Cybersponse\TestBundle\Functional\ApiTestCase;
use Symfony\Component\HttpFoundation\File\UploadedFile;

class FilesControllerTestCase extends ApiTestCase
{
    protected $fileContents = 'asdf';
    protected $filename = 'test_filename.txt';
    protected $modelType;
    protected $moduleType = null;
    protected $allowCreate = false;
    protected $allowRead = false;
    protected $allowUpdate = false;
    protected $allowDelete = false;
    protected $className;
    protected $classShortName;
    protected $databasesBuildOnce = false;

    protected function setUp()
    {
        $this->rebuildDatabases = false;
        if (!$this->moduleType) {
            $this->moduleType = $this->modelType;
        }
        $this->requestUri = '/api/3/' . $this->modelType;
        if (!$this->databasesBuildOnce) {
            $this->rebuildDatabases = true;
            $this->databasesBuildOnce = true;
        }
        parent::setUp();

        $container = $this->getContainer();

        /** @var ManagerRegistry $registry */
        $registry = $container->get('doctrine_mongodb');

        /** @var EntityManager $documentManager */
        $documentManager = $registry->getManager('default');
        $schemaManager = $documentManager->getSchemaManager();

        //Files are not dropped properly with the `dropCollections` method
        $fileDatabase = $documentManager->getDocumentDatabase($this->className);
        $fileDatabase->dropCollection($this->classShortName . '.files');
        $fileDatabase->dropCollection($this->classShortName . '.chunks');
        $schemaManager->createCollections();

        if ($this->filename) {
            file_put_contents($this->filename, $this->fileContents);
        }
    }

    protected function executeSystemUnderTest()
    {
        $this->setRoles([
            $this->createModulePermission($this->moduleType, $this->allowCreate, $this->allowRead, $this->allowUpdate, $this->allowDelete)
        ]);

        $uploadedFile = new UploadedFile(
            $this->filename,
            $this->filename
        );

        $this->setRequestFiles([
            'file' => $uploadedFile
        ]);

        parent::executeSystemUnderTest();
    }

    protected function tearDown()
    {
        if ($this->filename) {
            unlink($this->filename);
        }
    }
}
