<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

/**
 * Class ActivateStagingMetadataCommand
 */
class ActivateStagingMetadataCommand extends ContainerAwareCommand
{
    protected function configure()
    {
        $this
            ->setName('cybersponse:metadata:activate')
            ->setDescription('Activates staging model metadata by copying it to model metadata.');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);

        $container = $this->getContainer();
        $metadataEntityManager = $container->get('doctrine.orm.entity_manager');

        $commandsToExecute = [
            "CREATE TABLE IF NOT EXISTS attrib_model_metadata_copy (LIKE attrib_model_metadata INCLUDING ALL)",
            "CREATE TABLE IF NOT EXISTS model_meta_copy (LIKE model_metadatas INCLUDING ALL)",

            "DELETE from attrib_model_metadata_copy",
            "DELETE from model_meta_copy",

            "insert into model_meta_copy select * from model_metadatas",
            "insert into attrib_model_metadata_copy select * from attrib_model_metadata",

            "DELETE from attrib_model_metadata",
            "insert into model_metadatas select * from staging_model_metadatas ON CONFLICT (uuid) DO UPDATE set uniqueconstraint=EXCLUDED.uniqueconstraint, displayname=EXCLUDED.displayname, tablename=EXCLUDED.tablename, ownable=EXCLUDED.ownable, taggable=EXCLUDED.taggable, trackable=EXCLUDED.trackable, peerreplicable=EXCLUDED.peerreplicable, defaultsort=EXCLUDED.defaultsort, indexable=EXCLUDED.indexable, writable=EXCLUDED.writable, descriptions=EXCLUDED.descriptions",
            "insert into attrib_model_metadata select * from attribute_metadatas",
            "delete from model_metadatas where uuid not in (select uuid from staging_model_metadatas)",
            "delete from attrib_model_metadata where sattrib is null",
            "delete from attribute_metadatas where sattrib is null"
        ];

        foreach ($commandsToExecute as $commandToExecute) {
            $metadataEntityDatabase = $metadataEntityManager->getConnection()->prepare($commandToExecute);
            try{
                $metadataEntityDatabase->execute();
            }catch(\Exception $e) {
                throw new \Exception('Failure: ' . $e);
            }
        }

        $io->success('Metadata has been successfully activated from StagingModelMetadata to ModelMetadata.');
    }
}
