<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Command;

use Monolog\Logger;
use Cybersponse\UtilsBundle\ARCUtils;
use Cybersponse\CoreBundle\Entity\Widget;
use Cybersponse\FilesBundle\Entity\PgFile;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;

/**
 * Class BuildWidgetFilesCommand
 */
class BuildWidgetFilesCommand extends ContainerAwareCommand
{
    const WIDGET_PATH = '/opt/cyops-ui/widgets';

    /** @var Logger */
    protected $monolog;

    protected function configure()
    {
        $this
            ->setName('cybersponse:widgets:build')
            ->setDescription('Load all widgets from the widget module, build the widget cache file, and copy to exo-ui.')
            ->setDefinition([
              new InputOption(
                  'widgetUuId',
                  'w',
                  InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY,
                  'Arguments for building the widget with uuId.'
              ),
              new InputOption(
                  'delete',
                  'd',
                  InputOption::VALUE_OPTIONAL | InputOption::VALUE_IS_ARRAY,
                  'Arguments for the widget delete.'
              ),
              new InputOption(
                  'rebuildAll',
                  'r',
                  InputOption::VALUE_OPTIONAL | InputOption::VALUE_IS_ARRAY,
                  'Arguments for rebuilding all widgets.'
              )
          ]);
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $output->writeln([
        '===========================================================',
        'Starting Widgets build',
        '===========================================================',
        '',
    ]);
        $container = $this->getContainer();
        $widgetsUuid = $input->getOption('widgetUuId');
        $deleteWidget= $input->getOption('delete');
        $this->resourceCollection = $container->get('api.resource_collection');
        $widgetManager = $this->getContainer()->get('doctrine.orm.entity_manager');

        /** @var EntityRepository $widgetRepository */
        $widgetRepository = $widgetManager->getRepository(Widget::class);

        /** @var widgets[] $widgets */
        $widgets = $widgetRepository->findAll();
        $arcUtil = new ARCUtils();
        foreach ($widgets as $widget) {
            if (in_array($widget->getUuid(), $widgetsUuid)|| in_array($widget->getId(), $widgetsUuid)) {
                $output->writeln([
                '===========================================================',
                'Generating files for '.$widget->getName().'-'.$widget->getVersion().' Widget',
                '===========================================================',
                '',
              ]);
                $arcUtil->buildWidgetFiles($widget, self::WIDGET_PATH, $container->get('doctrine')->getManagerForClass(PgFile::class));
            } elseif (in_array($widget->getUuid(), $deleteWidget)|| in_array($widget->getId(), $deleteWidget)) {
                $arcUtil->removeWidgetFiles($widget);
            } elseif (count($input->getOption('rebuildAll'))) {
                $output->writeln([
                  '===========================================================',
                  'Rebuilding '.$widget->getName().'-'.$widget->getVersion().' Widget',
                  '===========================================================',
                  '',
                ]);
                $arcUtil->buildWidgetFiles($widget, self::WIDGET_PATH,$container->get('doctrine')->getManagerForClass(PgFile::class));
            }
        }
        $output->writeln([
          '===========================================================',
          'Widgets build finished',
          '===========================================================',
          '',
      ]);
    }
}
