<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Command;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Cybersponse\MetadataBundle\Entity\ClusterCommand;
use GuzzleHttp\Psr7\Request as GuzzleRequest;

/**
 * Class ClusterPublishWaitCommand
 */
class ClusterPublishWaitCommand extends ContainerAwareCommand
{

    protected function configure()
    {
        $this
            ->setName('cybersponse:cluster:publish:wait')
            ->setDescription('Waiting for Model File Generation on the other nodes.');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);
        $errorIo = $io;
        $sequenceId = substr(str_shuffle("0123456789abcdefghijklmnopqrstuvwxyz"), 0, 10);
        $dasUri = $this->getContainer()->getParameter('das.uri');
        $proxyProvider = $this->getContainer()->get('cybersponse.core.provider.proxy');
        $dasClient = $proxyProvider->getDasClient();
        $objectData = array(
            'commandId' => 'cluster_publish',
            'command' => 'sudo -u nginx php /opt/cyops-api/app/console cybersponse:cluster:publish --env=prod --no-interaction --nodeId=$nodeId$ --sequenceId=' . $sequenceId,
            'includeFiredrillNodes' => false
        );
        $headers = array(
            "Content-type" => "application/json"
        );
        $dasRequests = new GuzzleRequest('post', $dasUri . '/clustercommand/', $headers, json_encode($objectData));
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $resCode = $dasResponse->getStatusCode();
            if ($resCode != 200){
                return -1;
            }
        } catch (\Exception $error) {
            $errorIo->error($error->getMessage());
            return -1;
        }

        $dasRequests = new GuzzleRequest('get', $dasUri . '/cluster/?self=false', $headers);

        $nodeIds = [];
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $dasResponseData = $dasResponse->getBody()->getContents();
            $jsonDecodeResponse = json_decode($dasResponseData);
            $nodes = $jsonDecodeResponse->{"nodes"};
            foreach ($nodes as $node){
                if (strpos(strtolower($node->{"comment"}), "firedrill in progress") === FALSE){
                    array_push($nodeIds, $node->{"nodeId"});
                }
            }

        } catch (\Exception $error) {
            $errorIo->error($error->getMessage());
            return -1;
        }

        $container = $this->getContainer();
        $entityManager = $container->get('doctrine.orm.entity_manager');
        foreach ($nodeIds as &$nodeId) {
            $clusterCommand = new ClusterCommand();
            $clusterCommand->setNodeId($nodeId);
            $clusterCommand->setCommandId("cybersponse:cluster:publish");
            $clusterCommand->setSequenceId($sequenceId);
            $clusterCommand->setStatus('todo');
            $entityManager->persist($clusterCommand);
        }
        $entityManager->flush();

        $clusterCommandRepository = $entityManager->getRepository(ClusterCommand::class);
        # wait for 10 mins for other cluster nodes to complete
        $counter = 60;
        $waitForStatus = count($clusterCommandRepository->findAll([
            'sequenceId' => $sequenceId,
            'status' => 'todo']));
        while ($waitForStatus > 1 && $counter > 0) {
            sleep(10);
            $counter = $counter - 1;
            $waitForStatus = count($clusterCommandRepository->findAll([
                'sequenceId' => $sequenceId,
                'status' => 'todo']));
            }
        $io->success('Finished with publish across cluster!');
    }
}
