<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Command;

use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Cybersponse\MetadataBundle\Entity\ModelMetadata;
use Doctrine\ORM\EntityRepository;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputOption;
use PhpAmqpLib\Connection\AMQPStreamConnection;
use PhpAmqpLib\Message\AMQPMessage;
use Symfony\Component\Serializer\Encoder\JsonEncoder;
use Symfony\Component\Serializer\Normalizer\ObjectNormalizer;
use Symfony\Component\Serializer\Serializer;

class FalconCreateCommand extends ContainerAwareCommand
{
    const PUBLISH_CREATE = 'publish_create';
    const PUBLISH_MODIFY = 'publish_modify';
    const PUBLISH_DELETE = 'publish_delete';
    const POPULATE_INDEX = 'populate_index';
    private $callback_queue;
    private $corelation;
    private $response;

    protected function configure()
    {
        $this
            ->setName('cybersponse:elastic:create')
            ->setDescription('Publish new mapping to elasticsearch')
            ->setHelp("This command allows you to create mapping in elasticsearch based on model metadata by producing a message and sending it to falcon.")
            ->setDefinition([
                new InputOption(
                    'index',
                    'i',
                    InputOption::VALUE_OPTIONAL | InputOption::VALUE_IS_ARRAY,
                    'Arguments for indexing modules in ES.'
                ),
                new InputOption(
                    'sync',
                    's',
                    InputOption::VALUE_OPTIONAL,
                    'Arguments for synchronous indexing of elasticsearch'
                ),
                new InputOption(
                    'timeout',
                    't',
                    InputOption::VALUE_OPTIONAL,
                    'Arguments for timeout in case of sync'
                ),
                new InputOption(
                    'update',
                    'u',
                    InputOption::VALUE_OPTIONAL | InputOption::VALUE_IS_ARRAY,
                    'Arguments for updating existing index in ES.'
                )
            ]);
    }

    public function remove_element($array, $value)
    {
        foreach ($value as $val) {
            if (($key = array_search($val, $array)) !== false) {
                unset($array[$key]);
            }
        }
        return $array;
    }

    protected function identify_module_to_populate($input)
    {
        $container = $this->getContainer();
        $modelMetadatasWithKeys = $this->getAllMetadata(ModelMetadata::class);

        $encoder = new JsonEncoder();
        $normalizer = new ObjectNormalizer();

        $normalizer->setCircularReferenceHandler(function ($modelMetadatasWithKeys) {
            return $modelMetadatasWithKeys->getUuid();
        });

        $serializer = new Serializer([$normalizer], [$encoder]);

        $normalizedMetadata = $serializer->serialize($modelMetadatasWithKeys, 'json');
        $normalizedMetadata = json_decode($normalizedMetadata, true);

        $object['modelToCreate'] = [];
        $indexNames = $input->getOption('index');
        $module_exclude_list = $container->getParameter('module.exclude.list');
        $module_exclude_list = $this->remove_element($module_exclude_list, ["tenants", "routers","agents"]);

        foreach ($normalizedMetadata as $key => $value) {
            if (count($indexNames)) {
                $indexName = json_decode($indexNames[0], true);
                for ($i = 0; $i < count($indexName['type']); $i++) {
                    if ($indexName['type'][$i] == $value['type'] && !in_array($value['type'], $module_exclude_list)) {
                        array_push($object['modelToCreate'], $value);
                    }
                }
            } else {
                if (!in_array($value['type'], $module_exclude_list)) {
                    array_push($object['modelToCreate'], $value);
                }
            }
        }
        return $object['modelToCreate'];
    }

    protected function identify_module_to_update($input)
    {
        $indexNames=[];
        if ($input->getOption('update')[0]) {
            $indexNames = json_decode($input->getOption('update')[0], true);
        }
        return $indexNames;
    }

    protected function publish_message($input, $object, $publishType=self::PUBLISH_CREATE)
    {
        $mqCredObj = parse_ini_file('/opt/cyops/configs/rabbitmq/rabbitmq_users.conf', false, INI_SCANNER_NORMAL);
        $rabbitMqUsername = $mqCredObj['cyops.rabbitmq.user'] ? $mqCredObj['cyops.rabbitmq.user'] : 'guest';
        $rabbitMqPassword = $mqCredObj['cyops.rabbitmq.password'] ? $mqCredObj['cyops.rabbitmq.password'] : 'guest';
        $rabbitMqHost = $mqCredObj['cyops.rabbitmq.host'] ? $mqCredObj['cyops.rabbitmq.host'] : 'localhost';
        $rabbitMqPort = $mqCredObj['cyops.rabbitmq.port'] ? $mqCredObj['cyops.rabbitmq.port'] : '5672';
        $rabbitMqVHost = $mqCredObj['cyops.rabbitmq.vhost'] ? $mqCredObj['cyops.rabbitmq.vhost'] : '/';

        $connection = new AMQPStreamConnection($rabbitMqHost, $rabbitMqPort, $rabbitMqUsername, $rabbitMqPassword, $rabbitMqVHost);
        $channel = $connection->channel();
        $channel->exchange_declare('cyops.crudhub.datanotify', 'topic', false, true, false);

        if ($input->getOption('sync')) {
            if ($input->getOption('timeout')) {
                $timeout = (int)$input->getOption('timeout');
            } else {
                $timeout = 7200;
            }

            list($this->callback_queue, , ) = $channel->queue_declare("", false, false, true, false);
            $channel->basic_consume($this->callback_queue, '', false, false, false, false, array($this,'onResponse'));
            $this->response = null;
            $this->corelation = uniqid();
            $msg = new AMQPMessage(json_encode(array('esoperation' => $publishType, 'payload' => $object)), array(
                'correlation_id' => $this->corelation,
                'reply_to' => $this->callback_queue,
                'content_type' => 'application/json'
            ));
            $channel->basic_publish($msg, 'cyops.crudhub.datanotify', 'cyops.crudhub.systemnotify');
            while (!$this->response) {
                try {
                    $channel->wait(null, false, $timeout);
                } catch (\PhpAmqpLib\Exception\AMQPTimeoutException $e) {
                    echo "TimeOut Occurred please run following command post install: sudo -u nginx php /opt/cyops-api/app/console cybersponse:elastic:create --env='Prod' ";
                    $channel->close();
                    $connection->close();
                    exit;
                }
            }
        } else {
            $msg = new AMQPMessage(json_encode(array('esoperation' => $publishType, 'payload' => $object)), array('content_type' => 'application/json'));
            $channel->basic_publish($msg, 'cyops.crudhub.datanotify', 'cyops.crudhub.systemnotify');
        }
        $channel->close();
        $connection->close();
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $output->writeln([
            '===========================================================',
            'Passing Mapping To Falcon',
            '===========================================================',
            '',
        ]);
        $object['modelToUpdate']=[];
        $object['modelToCreate']=[];    
        $indexOption = $input->getOption('index');
        $updateOption =$input->getOption('update');
        if ((!$indexOption && !$updateOption) || count($indexOption)) {
            $object['modelToCreate'] = $this->identify_module_to_populate($input);
        }

        if ($input->getOption('update')) {
            $object['modelToUpdate'] = $this->identify_module_to_update($input);
        }
        $meta = $input->getOption('index');
        $metaObject = json_decode($meta[0], true);
        if (count($object['modelToCreate'])) {
            foreach ($object['modelToCreate'] as $key => $value) {
                $objectToSend = [];
                if (isset($metaObject['meta']) && isset($metaObject['meta'][$value['type']]['modified']))
                    {
                        $value['meta'] = $metaObject['meta'][$value['type']];
                    }
                $objectToSend['modelToCreate'] = [$value];
                $this->publish_message($input, $objectToSend);
            }
        }
        if ($object['modelToUpdate'] && count($object['modelToUpdate'])) {
            $this->publish_message($input, $object['modelToUpdate'], self::PUBLISH_MODIFY);
        }

        $output->writeln([
            '===========================================================',
            'Mapping Passed To Falcon',
            '===========================================================',
            '',
        ]);
    }

    /**
     * @param $class
     * @return array
     */
    private function getAllMetadata($class)
    {
        $metadataDocumentManager = $this->getContainer()->get('doctrine.orm.entity_manager');

        /** @var EntityRepository $modelMetadataRepository */
        $modelMetadataRepository = $metadataDocumentManager->getRepository($class);

        /** @var ModelMetadata[] $modelMetadatas */
        $modelMetadatas = $modelMetadataRepository->findAll();
        $modelMetadatasWithKeys = [];

        foreach ($modelMetadatas as $modelMetadata) {
            $modelMetadatasWithKeys[$modelMetadata->getType()] = $modelMetadata;
        }
        return $modelMetadatasWithKeys;
    }

    public function onResponse($rep)
    {
        if ($rep->get('correlation_id') == $this->corelation) {
            $this->response = true;
        }
    }
}
