<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Command;

use Cybersponse\CoreBundle\Api\ModelAwareResource;
use Cybersponse\CoreBundle\Entity\Module;
use Cybersponse\MetadataBundle\Mapping\ModelMetadataFactory;
use Doctrine\ORM\EntityManager;
use Dunglas\ApiBundle\Api\ResourceCollectionInterface;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

/**
 * Class GenerateModulesCommand
 */
class GenerateModulesCommand extends ContainerAwareCommand
{
    /** @var ResourceCollectionInterface */
    protected $resourceCollection;

    protected function configure()
    {
        $this
            ->setName('cybersponse:metadata:generate:modules')
            ->setDescription('Process current resources in api and generate modules.');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);

        $container = $this->getContainer();

        $progress = $io->createProgressBar();
        $progress->setFormat(' %current%/%max% [%bar%] Working on %message%...');
        $progress->setBarWidth(50);

        /** @var ResourceCollectionInterface $resourceCollection */
        $resourceCollection = $container->get('api.resource_collection');

        /** @var EntityManager $entityManager */
        $entityManager = $container->get('doctrine')->getManagerForClass(Module::class);
        $moduleRepository = $entityManager->getRepository(Module::class);
        $queryBuilder = $moduleRepository->createQueryBuilder(Module::class);

        /** @var ModelMetadataFactory $modelMetadataFactory */
        $modelMetadataFactory = $container->get('cybersponse.metadata.model_metadata_factory');

        $progress->start(count($resourceCollection));

        $allModuleTypes = [];

        /** @var ModelAwareResource $resource */
        foreach ($resourceCollection as $resource) {
            $classShortName = $resource->getShortName();
            $progress->setMessage($classShortName);
            if (!$resource instanceof ModelAwareResource) {
                throw new \Exception(sprintf('Resource: %s is not instance of ModelAwareResource.', $resource->getShortName()));
            }
            $moduleType = $resource->getModuleType();
            $module = $moduleRepository->findOneBy(['type' => $moduleType]);
            $allModuleTypes[] = $moduleType;

            $modelMetadata = $modelMetadataFactory->getModelMetadataByShortName($classShortName);
            if (is_null($modelMetadata)) {
                continue;
            }
            $descriptions = $modelMetadata->getDescriptions();
            $name = $moduleType;
            if (array_key_exists('plural', $descriptions)) {
                $name = $descriptions['plural'];
            }

            if (is_null($module)) {
                $module = new Module();
                $module->setType($moduleType);
            }

            $module
                ->setName($name)
                ->setDescription(sprintf('Allows interactions with the %s Module', $name))
                ->setSystem(false);
            $entityManager->persist($module);
            $entityManager->flush();

            $progress->advance();
        }

        $query = $queryBuilder
            ->delete(Module::class, 'm')
            ->where($queryBuilder->expr()->notIn('m.type', $allModuleTypes))
            ->andWhere('m.system = :system')
            ->setParameter('system', false)
            ->getQuery();

        $query->execute();
        $entityManager->flush();

        $progress->finish();
        $io->success(sprintf('Generated all modules successfully'));
    }
}
