<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Command;

use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Filesystem\Filesystem;

class LockApiCommand extends ContainerAwareCommand
{

    protected function configure()
    {
        $this
            ->setName('cybersponse:publish:lock')
            ->setDescription('Log 503 to the current node.')
            ->setHelp("This command allows you to lock request by setting 503.json.");
        $this->filesystem = new Filesystem();
    }


    protected function execute(InputInterface $input, OutputInterface $output)
    {

        $output->writeln([
            '===========================================================',
            'Locking the API request',
            '===========================================================',
            '',
        ]);
        $container = $this->getContainer();
        $rootDir = $container->getParameter('kernel.root_dir');
        $logDir = sprintf('%s/../web', $rootDir);
        $systemUpdate = new SystemUpdate();
        $systemUpdate
            ->setMessage('Locking For System Update')
            ->setProgressPercent(100);

        $this->log($input, $systemUpdate);
        $this->filesystem->dumpFile(sprintf('%s/503.json', $logDir), json_encode($systemUpdate));

        $output->writeln([
            '===========================================================',
            'API Request Locked',
            '===========================================================',
            '',
        ]);
    }

    /**
     * @param InputInterface $input
     * @param SystemUpdate $systemUpdate
     */
    protected function log(InputInterface $input, SystemUpdate $systemUpdate)
    {
        $logDir = $this->getLogDir();
        $this->filesystem->dumpFile(sprintf('%s/503.json', $logDir), json_encode($systemUpdate));
    }

    private function getLogDir()
    {
        $container = $this->getContainer();

        $rootDir = $container->getParameter('kernel.root_dir');
        return sprintf('%s/../web', $rootDir);
    }
}