<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Command;

use Exception;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputOption;

class MsspSelfEntryCommand extends ContainerAwareCommand
{
    protected function configure()
    {
        $this
            ->setName('cybersponse:mssp:entry')
            ->setDescription('Publish new mapping to elasticsearch')
            ->setHelp("This command allows you to create mapping in elasticsearch based on model metadata by producing a message and sending it to falcon.")
            ->setDefinition([
                new InputOption(
                    'hardwareKey',
                    'hkey',
                    InputOption::VALUE_OPTIONAL,
                    'Arguments for getting hardware key.'
                ),
                new InputOption(
                    'tenancy_enabled',
                    't_enabled',
                    InputOption::VALUE_OPTIONAL,
                    'Argument to specify whether Multi Tenancy is enabled.'
                ),
                new InputOption(
                    'user_id',
                    'usr_id',
                    InputOption::VALUE_OPTIONAL,
                    'Arguments for getting the DAS user_id.'
                ),
                new InputOption(
                    'role',
                    'rle',
                    InputOption::VALUE_OPTIONAL,
                    'Arguments for getting mssp role.'
                )
            ]);
    }

    protected function updateSelfId($entityManager, $uniqId, $out)
    {
        $out->writeln([
            '=========================================================================',
            'Updating Self TenantId and AgentId Of Tenant and Agent Module',
            '=========================================================================',
            '',
        ]);

        $sql = "update TENANTS set tenantId='" . $uniqId . "' where role='self' and tenantid='745d976a-6ae8-4837-a364-64c251ef9e27'";
        $entityManager->getConnection()->prepare($sql)->execute();

        $sql = "update AGENTS set agentId='" . $uniqId . "' where role='self' and agentid='745d976a-6ae8-4837-a364-64c251ef9e27'";
        $entityManager->getConnection()->prepare($sql)->execute();

        $out->writeln([
            '=========================================================================',
            'Updated Self TenantId and AgentId Of Tenant and Agent Module',
            '=========================================================================',
            '',
        ]);
    }

    protected function createDefaultRemoteUser($entityManager, $user_id, $username, $out)
    {
        $sql = "SELECT setval('actors_id_seq', 5, FALSE)";
        $entityManager->getConnection()->prepare($sql)->execute();

        $out->writeln([
            '=========================================================================',
            'Creating Default Remote User',
            '=========================================================================',
            '',
        ]);

        $sql = "INSERT INTO ACTORS (id, uuid, createuser, modifyuser, user_id, record_type, firstname, lastname, title, email, phonework) VALUES ('4', '441c0092-8db1-4299-ba1a-6d704465465a', '441c0092-8db1-4299-ba1a-6d704465465a', '441c0092-8db1-4299-ba1a-6d704465465a', '" . $user_id . "', 'person', '" . $username . "', 'User', 'Admin', 'admin@example.com', '646-275-9691')";
        $tenantActor = $entityManager->getConnection()->prepare($sql);
        
        try {
            $tenantActor->execute();
            $out->writeln([
                '=========================================================================',
                'Default Remote User Created',
                '=========================================================================',
                '',
            ]);
        } catch (\Exception $e) {
            $out->writeln(['Remote User Creation Failed']);
            exit;
        }
    }

    protected function defaultUserRole($entityManager, $out)
    {
        $out->writeln([
            '=========================================================================',
            'Assigning Remote User Default Roles',
            '=========================================================================',
            '',
        ]);

        $sql = "INSERT INTO ACTOR_ROLE (actor_uuid, role_uuid) VALUES ('441c0092-8db1-4299-ba1a-6d704465465a', '17562d60-b841-4ed7-912b-1ebaf063c8c9'), ('441c0092-8db1-4299-ba1a-6d704465465a', '21d7a7c6-9d68-438f-b732-e8950c5f745b')";
        $actorRole = $entityManager->getConnection()->prepare($sql);
        try {
            $actorRole->execute();
            $out->writeln([
                '=========================================================================',
                'Assigned Remote User Default Roles',
                '=========================================================================',
                '',
            ]);
        } catch (\Exception $e) {
            $out->writeln(['Remote User Roles Creation Failed']);
            exit;
        }
    }

    protected function defaultUserTeam($entityManager, $out)
    {
        $out->writeln([
            '=========================================================================',
            'Assigning Default Remote User Teams',
            '=========================================================================',
            '',
        ]);

        $sql = "INSERT INTO TEAM_ACTOR (actor_uuid, team_uuid) VALUES ('441c0092-8db1-4299-ba1a-6d704465465a', '6e569c09-3bd4-40f1-98b0-cc994464c3c5')";
        $actorTeam = $entityManager->getConnection()->prepare($sql);

        try {
            $actorTeam->execute();
            $out->writeln([
                '=========================================================================',
                'Assigned Default Remote User Teams',
                '=========================================================================',
                '',
            ]);
        } catch (\Exception $e) {
            $out->writeln(['Assigning Team to Remote user Failed']);
            exit;
        }
    }

    protected function enableModuleReplication($entityManager, $role, $out)
    {
        $out->writeln([
            '=========================================================================',
            'Enabling replication for modules',
            '=========================================================================',
            '',
        ]);
        
        if ($role == "master") {
            $sql = "Update staging_model_metadatas set peerreplicable=true where type not in ('people', 'appliances', 'agents', 'tenants', 'routers')";
            $entityManager->getConnection()->prepare($sql)->execute();
        } else {
            $sql = "Update staging_model_metadatas set peerreplicable=true where type in ('alerts', 'incidents', 'emails', 'indicators')";
            $entityManager->getConnection()->prepare($sql)->execute();
        }
        $out->writeln([
            '=========================================================================',
            'Enabled replication for modules',
            '=========================================================================',
            '',
        ]);
    }

    protected function AddRemoteWorkflowStep($entityManager, $out)
    {
        $out->writeln([
            '=========================================================================',
            'Adding Remote Execution Playbook Workflow Step',
            '=========================================================================',
            '',
        ]);

        $sql = "INSERT INTO public.workflow_step_types (uuid, collection_uuid, parent_uuid, name, displayname, widget, arguments, icon, background, index, visible, description, id) VALUES ('ab3b2e02-5e77-4ed6-8ebd-580f390063a5', 'f3561053-417f-4591-8fa4-ca9da8e1da8c', 'ee73e569-2188-43fe-a7f0-1964ba82a4de', 'RemotePlaybookReference', 'Reference Remote Playbook', NULL, '{\"script\": \"/wf/workflow/tasks/remote_workflow_reference\"}', 'fa fa-sitemap', '#fa5b46', 425, true, NULL, 200)";

        $workflowStep = $entityManager->getConnection()->prepare($sql);

        try {
            $workflowStep->execute();
            $out->writeln([
                '=========================================================================',
                'Remote Reference Playbook Step Insertion Success',
                '=========================================================================',
                '',
            ]);
        } catch (\Exception $e) {
            $out->writeln(['Remote Reference Playbook Step Insertion Failed']);
            exit;
        }
    }

    protected function execute(InputInterface $input, OutputInterface $out)
    {
        $container = $this->getContainer();
        $entityManager = $container->get('doctrine.orm.entity_manager');

        $role = $input->getOption('role');
        $tenancy_enabled = $input->getOption('tenancy_enabled');
        if ($tenancy_enabled == null) {
            $tenancy_enabled = false;
        }
        $user_id = $input->getOption('user_id') ? $input->getOption('user_id') : 'c59e0943-12e1-491f-a041-1071c93311ef';

        if ($role == 'master') {
            $username = 'Tenant';
        } else {
            $username = 'Master';
        }

        $uniqId = $input->getOption('hardwareKey');
        $this->updateSelfId($entityManager, $uniqId, $out);
        
        if (!$tenancy_enabled) {
            exit;
        }
        
        $this->createDefaultRemoteUser($entityManager, $user_id, $username, $out);

        $this->defaultUserRole($entityManager, $out);

        $this->defaultUserTeam($entityManager, $out);

        $this->enableModuleReplication($entityManager, $role, $out);

        $this->AddRemoteWorkflowStep($entityManager, $out);

        $out->writeln([
            '=========================================================================',
            'Publishing for MSSP Changes',
            '=========================================================================',
            '',
        ]);
        $rootDir = $container->getParameter('kernel.root_dir');
        $command =sprintf('sudo -u nginx php %s/console cybersponse:system:update -laf --env=prod', $rootDir);
        exec($command, $output, $errorCode);

        if ($errorCode) {
            $out->writeln(['Publish For MSSP Changes Failed']);
        } else {
            $out->writeln([
                '=========================================================================',
                'Publish For MSSP Changes Successful',
                '=========================================================================',
                '',
            ]);
        }
    }
}
