<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use PhpAmqpLib\Message\AMQPMessage;
use PhpAmqpLib\Connection\AMQPStreamConnection;

/**
 * Class ActivateStagingMetadataCommand
 */
class RevertStagingMetadataCommand extends ContainerAwareCommand
{

    const COMPONENT = 'component';
    const ENTITY_TYPE = 'entityType';
    const DELTA_DATA = 'data';
    const TRANSACTION_DATE = 'transactionDate';
    const ACTION = 'action';
    const METHOD = 'method';

    protected function configure()
    {
        $this
            ->setName('cybersponse:metadata:revert')
            ->setDescription('Reverts staging model metadata by copying it from model metadata.');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);

        $container = $this->getContainer();
        $metadataEntityManager = $container->get('doctrine.orm.entity_manager');

        $commandsToExecute = [
            "TRUNCATE TABLE attribute_metadatas, staging_model_metadatas RESTART IDENTITY",
            "insert into staging_model_metadatas select * from model_metadatas",
            "insert into attribute_metadatas select * from attrib_model_metadata"
        ];

        foreach ($commandsToExecute as $commandToExecute) {
            $metadataEntityDatabase = $metadataEntityManager->getConnection()->prepare($commandToExecute);
            try{
                $metadataEntityDatabase->execute();
            }catch(\Exception $e) {
                throw new \Exception('Failure: ' . $e);
            }
        }
        $this->publishRevertMessage();
        $io->success('Metadata has been successfully reverted from StagingModelMetadata to ModelMetadata.');
    }

    protected function prepareChangeData()
    {
        return [
            self::DELTA_DATA => [],
            self::COMPONENT => 'crudhub',
            self::ENTITY_TYPE => 'StagingModelMetadata',
            self::METHOD => 'POST',
            self::TRANSACTION_DATE => (int) ($_SERVER['REQUEST_TIME_FLOAT'] * 1000),
            self::ACTION => 'REVERT',
        ];
    }

    protected function publishRevertMessage()
    {
        $body = $this->prepareChangeData();
        $mqCredObj = parse_ini_file('/opt/cyops/configs/rabbitmq/rabbitmq_users.conf', false, INI_SCANNER_NORMAL);
        $rabbitMqUsername = $mqCredObj['cyops.rabbitmq.user'] ? $mqCredObj['cyops.rabbitmq.user'] : 'guest';
        $rabbitMqPassword = $mqCredObj['cyops.rabbitmq.password'] ? $mqCredObj['cyops.rabbitmq.password'] : 'guest';
        $rabbitMqHost = $mqCredObj['cyops.rabbitmq.host'] ? $mqCredObj['cyops.rabbitmq.host'] : 'localhost';
        $rabbitMqPort = $mqCredObj['cyops.rabbitmq.port'] ? $mqCredObj['cyops.rabbitmq.port'] : '5672';
        $rabbitMqVHost = $mqCredObj['cyops.rabbitmq.vhost'] ? $mqCredObj['cyops.rabbitmq.vhost'] : '/';

        $connection = new AMQPStreamConnection($rabbitMqHost, $rabbitMqPort, $rabbitMqUsername, $rabbitMqPassword, $rabbitMqVHost);
        $channel = $connection->channel();
        $channel->exchange_declare('cyops.crudhub.datanotify', 'topic', false, true, false);

        $msg = new AMQPMessage(json_encode($body), array('content_type' => 'application/json'));
        $channel->basic_publish($msg, 'cyops.crudhub.datanotify', 'route.crudhub.instructions.remoterequest.stagingmmdupdate');
        $channel->close();
        $connection->close();
    }
}
