<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Command;

use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Filesystem\Filesystem;
use GuzzleHttp\Psr7\Request as GuzzleRequest;

class UtilApiCommand extends ContainerAwareCommand
{
    protected function configure()
    {
        $this
            ->setName('cybersponse:util')
            ->setDescription('API Util command.')
            ->setHelp("This command allows you to access various sub-command operation like lock/unlock cluster etc..")
            ->setDefinition([
                new InputOption(
                    'lock',
                    'l',
                    InputOption::VALUE_OPTIONAL | InputOption::VALUE_IS_ARRAY,
                    'Argument for locking self node in HA cluster.'
                ),
                new InputOption(
                    'unlock',
                    'u',
                    InputOption::VALUE_OPTIONAL | InputOption::VALUE_IS_ARRAY,
                    'Argument for unlock self node in HA cluster.'
                ),
                new InputOption(
                    'redis-flush',
                    'r',
                    InputOption::VALUE_OPTIONAL | InputOption::VALUE_IS_ARRAY,
                    'Argument for clearing API redis db.'
                )
            ]);
        $this->filesystem = new Filesystem();
    }


    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $output->writeln([
            '===========================================================',
            'Started Executing Util Command',
            '===========================================================',
            '',
        ]);
        $container = $this->getContainer();
        
        if (count($input->getOption('redis-flush'))) {
            $caches = [];
            array_push($caches,$container->get('cybersponse.authorization.permissions.permissions_list_cache'));
            array_push($caches,$container->get('cybersponse.authorization.permissions.permissions_dictionary_cache'));
            array_push($caches,$container->get('cybersponse.authorization.ownership.accessible_team_ids_cache'));
            foreach ($caches as $cache) {
                $cache->clearCache();
            }
        } elseif (count($input->getOption('unlock'))) {
            $output->writeln([
                '===========================================================',
                'Executing Un-Lock API Command',
                '===========================================================',
                '',
            ]);
            $rootDir = $container->getParameter('kernel.root_dir');
            $logDir = sprintf('%s/../web', $rootDir);
            
            $this->filesystem->remove(sprintf('%s/503.json', $logDir));
           
        } elseif (count($input->getOption('lock'))) {
            $output->writeln([
                '===========================================================',
                'Executing Lock API Command',
                '===========================================================',
                '',
            ]);
            $rootDir = $container->getParameter('kernel.root_dir');
            $logDir = sprintf('%s/../web', $rootDir);
            $systemUpdate = new SystemUpdate();
            $systemUpdate
            ->setMessage('Locking For System Update')
            ->setProgressPercent(100);

            $this->log($input, $systemUpdate);
            $this->filesystem->dumpFile(sprintf('%s/503.json', $logDir), json_encode($systemUpdate));
        }
        
        $output->writeln([
            '===========================================================',
            'Finished Executing Util Command',
            '===========================================================',
            '',
        ]);
    }

    /**
     * @param InputInterface $input
     * @param SystemUpdate $systemUpdate
     */
    protected function log(InputInterface $input, SystemUpdate $systemUpdate)
    {
        $logDir = $this->getLogDir();
        $this->filesystem->dumpFile(sprintf('%s/503.json', $logDir), json_encode($systemUpdate));
    }

    private function getLogDir()
    {
        $container = $this->getContainer();

        $rootDir = $container->getParameter('kernel.root_dir');
        return sprintf('%s/../web', $rootDir);
    }

}
