<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Controller;

use Cybersponse\UtilsBundle\Utils;
use Doctrine\ORM\EntityRepository;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Console\Input\ArrayInput;
use Symfony\Component\Console\Output\NullOutput;
use Symfony\Component\HttpFoundation\JsonResponse;
use Cybersponse\MetadataBundle\Command\SystemUpdate;
use Cybersponse\MetadataBundle\Entity\ModelMetadata;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Cybersponse\MetadataBundle\Entity\StagingModelMetadata;
use Symfony\Component\Validator\ConstraintViolationListInterface;
use Cybersponse\MetadataBundle\Command\RevertStagingMetadataCommand;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Cybersponse\MetadataBundle\Validation\ModelMetadataCollectionConstraint;

class PublishMetadataController extends Controller
{
    const COMPONENT = 'component';
    const ENTITY_TYPE = 'entityType';
    const HTTP_METHOD = 'operation';
    const REMOTE_ADDR = 'source';
    const USER_UUID = 'userId';
    const USER = 'user';
    const TRANSACTION_DATE = 'transactionDate';

    /**
     * @return string
     */
    private function extractUserName()
    {
        if (method_exists($this->getUser(), "getfirstname")) {
            if (method_exists($this->getUser(), "getlastname")) {
                $userName = $this->getUser()->getfirstname() . ' ' . $this->getUser()->getlastname();
            } else {
                $userName = $this->getUser()->getfirstname();
            }
        } else {
            $userName = $this->getUser()->getname();
        }
        return $userName;
    }

    /**
     * @return array
     */
    protected function prepareChangeData()
    {
        $userName = $this->extractUserName();
        return [
            self::COMPONENT => 'crudhub',
            self::ENTITY_TYPE => 'ModelMetadata',
            self::HTTP_METHOD => 'Update',
            self::REMOTE_ADDR => $_SERVER['REMOTE_ADDR'],
            self::TRANSACTION_DATE => (int)($_SERVER['REQUEST_TIME_FLOAT']*1000),
            self::USER => $userName,
            self::USER_UUID => $this->getUser()->getUuid(),

        ];
    }


    public function publishAction()
    {
        $authorizationChecker = $this->container->get('security.authorization_checker');

        if (!$authorizationChecker->isGranted(['perm:application.update'])) {
            throw new AccessDeniedException();
        }

        $modelMetadatasWithKeys = $this->getAllMetadata(StagingModelMetadata::class);

        $violations = $this->get('validator')->validate($modelMetadatasWithKeys, new ModelMetadataCollectionConstraint());

        if (count($violations) > 0) {
            return $this->getErrorResponse($violations);
        }

        $rootDir = $this->container->getParameter('kernel.root_dir');
        $logFile = sprintf('%s/../web/503.json', $rootDir);

        $systemUpdate = new SystemUpdate();
        $systemUpdate->setMessage('Publish queued.');
        $systemUpdate->setProgressPercent(0);
        $filesystem = $this->container->get('filesystem');
        $filesystem->dumpFile($logFile, json_encode($systemUpdate));
        $rootDir = $this->container->getParameter('kernel.root_dir');
        $command = sprintf('/usr/bin/php %s/console cybersponse:system:update -vvv --env=%s -la', $rootDir, $this->container->getParameter('kernel.environment'));
        chdir($this->container->getParameter('kernel.root_dir') . '/../');
        exec(sprintf('bash -c "exec nohup setsid %s > /var/log/cyops/cyops-api/last_system_publish.log 2>&1 &"', $command));

        $body = $this->prepareChangeData();
        $utilsObj = new Utils();
        $utilsObj->sendData($this, [$body]);

        return new JsonResponse([
            '@type' => 'Publish',
            'status' => 'started'
        ]);
    }

    public function revertAction()
    {
        $input = new ArrayInput([]);
        $output = new NullOutput();

        /** @var RevertStagingMetadataCommand $command */
        $command = $this->container->get('cybersponse.metadata.command.revert_metadata');
        $resultCode = $command->run($input, $output);

        $status = 'reverted';

        if ($resultCode !== 0) {
            $status = 'failed';
        }

        return new JsonResponse([
            '@type' => 'Publish',
            'status' => $status
        ]);
    }

    /**
     * @inheritdoc
     */
    protected function getErrorResponse(ConstraintViolationListInterface $violations)
    {
        $clientErrorCode = Response::HTTP_BAD_REQUEST;
        return new JsonResponse($this->get('serializer')->normalize($violations, 'hydra-error'), $clientErrorCode);
    }


    public function getLastPublishStatusAction()
    {
        $file = '/var/log/cyops/cyops-api/last_system_publish.log';
        $searchfor = array("An exception occurred", "ERROR:");
        $errors = array();

        if (file_exists($file)) {
            // Get the file contents, assuming the file to be readable (and exist)
            $contents = file_get_contents($file);
            $fileModificationTime = filectime($file);

            foreach ($searchfor as $searchString) {
                // escape special characters in the query
                $pattern = preg_quote($searchString, '/');
                // finalise the regular expression, matching the whole line
                $pattern = "/^.*$pattern.*\$/m";
                // search, and store all matching occurences in $matches
                if (preg_match_all($pattern, $contents, $matches)) {
                    $errors = array_merge($errors, array_unique($matches[0]));
                }
            }
            if (count($errors) == 0) {
                return new JsonResponse([
                    '@type' => 'Publish',
                    'status' => 'Success',
                    'last_publish_time' => $fileModificationTime
                ]);
            } else {
                $error = implode("", $errors);
                $error = preg_replace('#\\x1b[[][^A-Za-z]*[A-Za-z]#', '', $error);
                return new JsonResponse([
                    '@type' => 'Publish',
                    'status' => 'Fail',
                    'last_publish_time' => $fileModificationTime,
                    'errors' => $error,
                    'display_error_text' => $this->getDisplayText($errors),
                    'display_error' => $this->getMeaningfulDisplayError($errors)
                ], Response::HTTP_BAD_REQUEST);
            }
        } else {
            //if file not present it means no publish happened yet
            return new JsonResponse([
                '@type' => 'Publish',
                'status' => 'Not available.',
                'message' => 'No traces found for last system update.'
            ], Response::HTTP_NOT_FOUND);
        }
    }

    private function getDisplayText($errors)
    {
        $errors = implode(" ", $errors);
        $pos = strpos($errors, 'CREATE UNIQUE INDEX');
        if ($pos) {
            return  'Unique constraint could not be created=>' . $this->getUniqueConstraint($errors) . ': as duplicate records exist. Please ensure all duplicate records are removed before applying this unique constraint.' ;
        }
    }

    private function getMeaningfulDisplayError($errors)
    {
        $displayError = "";
        $space_separated = implode(" ", $errors);
        $moduleName = $this->getModuleName($space_separated);
        $errorStatement = $this->getErrorStatement($space_separated);
        if ($errorStatement != '' && $moduleName != '') {
            $displayError = $moduleName . " : " . $errorStatement;
        }
        return $displayError;
    }

    private function getUniqueConstraint($error)
    {
        //Removing all words staring with special characters
        $pos = strpos($error, 'CREATE UNIQUE INDEX');
        $pos2 = strpos($error, "':  [39;49m");
        if ($pos === false) {
            return '';
        }
        $pos = $pos + strlen('CREATE UNIQUE INDEX');
        return substr($error, $pos, $pos2 - $pos);
    }

    private function getModuleName($error)
    {
        //Removing all words staring with special characters
        $pos = strpos($error, 'ALTER TABLE');
        if ($pos === false) {
            return '';
        }
        $pos = $pos + strlen('ALTER TABLE');
        return substr($error, $pos, (strpos($error, 'ALTER', $pos) - $pos));
    }

    private function getErrorStatement($error)
    {
        $pos = strpos($error, 'Datatype mismatch');
        if ($pos === false) {
            return '';
        }
        $message = substr($error, $pos);
        $pos = strpos($message, 'ERROR:');
        $message = substr($message, $pos + strlen('ERROR:'));
        return str_replace('column', 'field', $message);
    }

    /**
     * @param string $class
     * @return array
     */
    private function getAllMetadata($class)
    {
        $metadataEntityManager = $this->container->get('doctrine.orm.entity_manager');

        /** @var EntityRepository $modelMetadataRepository */
        $modelMetadataRepository = $metadataEntityManager->getRepository($class);

        /** @var ModelMetadata[] $modelMetadatas */
        $modelMetadatas = $modelMetadataRepository->findAll();
        $modelMetadatasWithKeys = [];

        foreach ($modelMetadatas as $modelMetadata) {
            $modelMetadatasWithKeys[$modelMetadata->getType()] = $modelMetadata;
        }
        return $modelMetadatasWithKeys;
    }
}
