<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\DependencyInjection;

use Cybersponse\MetadataBundle\Loader\ModelClassMetadataFactory;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Reference;

class MetadataCompilerPass implements CompilerPassInterface
{
    /**
     * @inheritdoc
     */
    public function process(ContainerBuilder $container)
    {
        //Set class metadata factory
        $definition = $container->getDefinition('api.mapping.class_metadata_factory');
        $definition->setClass(ModelClassMetadataFactory::class);

        // Set Generators
        $this->handleClassAnnotationGenerators($container);
        $this->handleFieldAnnotationGenerators($container);
        $this->handleTraitGenerators($container);

        //Set Custom Dunglas Loaders
        $loaderChain = $container->getDefinition('api.mapping.loaders.chain');

        $loaders = $loaderChain->getArgument(0);

        foreach ($container->findTaggedServiceIds('cybersponse.annotation.loader') as $id => $attributes) {
            $loaders[] = new Reference($id);
        }

        $loaderChain->setArguments([$loaders]);
    }

    private function handleClassAnnotationGenerators(ContainerBuilder $container)
    {
        $generators = $this->getServiceReferences($container, 'cybersponse.metadata.generator.class_annotation');

        $entityGenerator = $container->getDefinition('cybersponse.entity_generator');
        $entityGenerator->addMethodCall('setClassAnnotationGenerators', [$generators]);

        $metadataAnnotationLoader = $container->getDefinition('cybersponse.annotation.metadata_loader');
        $metadataAnnotationLoader->addMethodCall('setClassAnnotationGenerators', [$generators]);
    }

    private function handleFieldAnnotationGenerators(ContainerBuilder $container)
    {
        $generators = $this->getServiceReferences($container, 'cybersponse.metadata.generator.field_annotation');

        $entityGenerator = $container->getDefinition('cybersponse.entity_generator');
        $entityGenerator->addMethodCall('setFieldAnnotationGenerators', [$generators]);

        $metadataAnnotationLoader = $container->getDefinition('cybersponse.annotation.metadata_loader');
        $metadataAnnotationLoader->addMethodCall('setFieldAnnotationGenerators', [$generators]);
    }

    private function handleTraitGenerators(ContainerBuilder $container)
    {
        $generators = $this->getServiceReferences($container, 'cybersponse.metadata.generator.trait');

        $entityGenerator = $container->getDefinition('cybersponse.entity_generator');
        $entityGenerator->addMethodCall('setTraitGenerators', [$generators]);

        $metadataAnnotationLoader = $container->getDefinition('cybersponse.annotation.metadata_loader');
        $metadataAnnotationLoader->addMethodCall('setTraitGenerators', [$generators]);
    }

    /**
     * @param ContainerBuilder $container
     * @param string $serviceTag
     * @return Reference[]
     */
    private function getServiceReferences(ContainerBuilder $container, $serviceTag)
    {
        $serviceReferences = [];

        foreach ($container->findTaggedServiceIds($serviceTag) as $id => $attributes) {
            $serviceReferences[] = new Reference($id);
        }

        return $serviceReferences;
    }
}
