<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\DependencyInjection;


use Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType;
use Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType;
use ReflectionClass;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Definition;

class ModuleAwareResourcePass implements CompilerPassInterface
{

    /**
     * You can modify the container here before it is dumped to PHP code.
     *
     * @param ContainerBuilder $container
     *
     * @api
     */
    public function process(ContainerBuilder $container)
    {
        $moduleAwareResources = $container->findTaggedServiceIds('api.module_aware_resource');

        foreach (array_keys($moduleAwareResources) as $resourceServiceName) {
            $resourceReference = $container->getDefinition($resourceServiceName);
            $resourceDefinition = new Definition();

            $resourceDefinition
                ->setClass($container->getParameter('cybersponse.core.model_aware_resource.class'))
                ->setArguments($resourceReference->getArguments())
                ->setMethodCalls($resourceReference->getMethodCalls())
                ->addTag('api.resource');

            // Read annotation for class to get model metadata type (if applicable)
            $annotationReader = $container->get('annotation_reader');
            $reflectionClass = new ReflectionClass($resourceDefinition->getArgument(0));

            /** @var ModelType $classAnnotation */
            $classAnnotation = $annotationReader->getClassAnnotation($reflectionClass, ModelType::class);

            if ($classAnnotation) {
                $resourceDefinition->addMethodCall('setModelType', [$classAnnotation->modelType]);
            }

            /** @var ModuleType $classAnnotation */
            $classAnnotation = $annotationReader->getClassAnnotation($reflectionClass, ModuleType::class);

            if ($classAnnotation) {
                $resourceDefinition->addMethodCall('setModuleType', [$classAnnotation->moduleType]);
            }

            $container->setDefinition($resourceServiceName, $resourceDefinition);
        }
    }
}
