<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Entity;


use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\ORM\Mapping\UniqueConstraint;

/**
 * Class AttributeMetadata
 *
 * This class defines fields to describe the attributes for a model in the system
 *
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="attribute_metadatas")
 * @Cybersponse\MetadataBundle\Validation\ConditionSetConstraint(fields={
 *      "ownsRelationship"={
 *          "notNull"={
 *             @Cybersponse\MetadataBundle\Conditional\Query(
 *                 result=true,
 *                 logic="AND",
 *                 filters={
 *                     {"field"="collection", "operator"="eq", "value"=true}
 *                 }
 *             )
 *          }
 *      },
 *      "visibility"={
 *          "isTrue"={
 *             @Cybersponse\MetadataBundle\Conditional\Query(
 *                 result=true,
 *                 logic="AND",
 *                 filters={
 *                     {"field"="validation[required]", "operator"="eq", "value"=true}
 *                 }
 *             )
 *          }
 *      }
 * })
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="application")
 * @ORM\Table(name="attribute_metadatas", uniqueConstraints={@UniqueConstraint(name="staging_name_unique", columns={"name", "sattrib"})})
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Operations\Operations(update=false, delete=false)
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="attribute_metadata", plural="attribute_metadatas")
 */
class AttributeMetadata extends Metadatas
{

    /**
     * The type of form to use to render this field as well as validate the data input. We strongly recommend this never
     * be changed after the field is set or you may risk publish failure as there may not be a valid migration path for
     * the data type.
     *
     * @var string Metadata type, json primitive for attributes or internal types for models
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\Regex(
     *     pattern="/^[a-z]+[a-z0-9_]*$/",
     *     message="Type must contain only lowercase characters, underscores and start with a letter, given: {{ value }}"
     * )
     * @Cybersponse\MetadataBundle\Validation\Keyword()
     * @Doctrine\ORM\Mapping\Column(name="`type`", type="string", length=500, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Type")
     */
    protected $type = 'string';

    /**
     * The name of the field on the API level. This must be alphanumeric and start with a lowercase letter. No spaces,
     * underscores, or special characters are allowed. We strongly recommend that this not be changed after creation as
     * there is no migration path from the old name to the new name, resulting in data loss.
     *
     * @var string The name of the attribute
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\Regex(
     *     pattern="/^[a-z]+[a-zA-Z0-9]*$/",
     *     message="Name must contain only alphanumeric characters and start with a lowercase letter, given: {{ value }}"
     * )
     * @Cybersponse\MetadataBundle\Validation\Keyword()
     * @Doctrine\ORM\Mapping\Column(name="`name`", type="string", length=500, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Name")
     */
    protected $name;

    /**
     * @var integer
     *
     * @Doctrine\ORM\Mapping\Column(name="`length`", type="integer", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Length")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="integer")
     */
    protected $length = 0;

    /**
     * @var integer
     *
     * @Doctrine\ORM\Mapping\Column(name="`orderindex`", type="integer", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Order Index")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="integer")
     */
    protected $orderIndex = 0;

    /**
     * The name of the field on the API level. This must be alphanumeric and start with a lowercase letter. No spaces,
     * underscores, or special characters are allowed. We strongly recommend that this not be changed after creation as
     * there is no migration path from the old name to the new name, resulting in data loss.
     *
     * @var string The type of form to use to render this attribute
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\Regex(
     *     pattern="/^[a-z]+[a-zA-Z0-9]*$/",
     *     message="Name must contain only alphanumeric characters and start with a lowercase letter, given: {{ value }}"
     * )
     * @Cybersponse\MetadataBundle\Validation\Keyword()
     * @Doctrine\ORM\Mapping\Column(name="`formtype`", type="string", length=500, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Form Type")
     */
    protected $formType = 'text';

    /**
     * The module for the source of the data, not defined explicitly, to which the field points.
     *
     * @var object The source of the data, not defined explicitly
     * @Doctrine\ORM\Mapping\Column(name="`datasource`", type="json_array", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Data Source")
     */
    protected $dataSource;

    /**
     * If the field is searchable, it may be searched in the grid view. If it is not searchable, it will not be
     * searchable in the grid view.
     * @var boolean Determines if this attribute should be searchable
     *
     * @ORM\Column(name="`searchable`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Searchable")"
     */
    protected $searchable = false;

    /**
     * If the field is peer replicable, it will be replicable to remote master. If its not peer replicable it values will not
     * be passed to master
     *
     * @var boolean Determines if this attribute should be peer replicable
     * @ORM\Column(name="`peerreplicable`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Peer Replicable")"
     */
    protected $peerReplicable = true;

    /**
     * If the field is system, you won't be able to update/delete from UI.
     *
     * @var boolean Determines if this attribute should be system field
     * @ORM\Column(name="`system`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="System")"
     */
    protected $system = false;

    /**
     * Enables encrypting of field values before storing in the database for enhanced security. UI will continue to show non-encrypted values.
     *
     * @var boolean Determines if this attribute should be encrypted
     * @ORM\Column(name="`encrypted`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Encrypted")"
     */
    protected $encrypted = false;

    /**
     * Enables html escape of field values before storing in the database for enhanced security. UI will continue to show the values in non executable or clickable format.
     *
     * @var boolean Determines if this attribute should be htmlEscaped
     * @ORM\Column(name="`htmlescape`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="HTMLEscape")"
     */
    protected $htmlEscape = false;

    /**
     * Cascade the delete of the parent record to all associated records (irrespective of delete permission) of this type.
     *
     * @var boolean Determines if this attribute should be orphanRemoval
     * @ORM\Column(name="`orphanremoval`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="OrphanRemoval")"
     */
    protected $orphanRemoval = false;

    /**
     * If set, this field will show up in a grid by default unless otherwise defined in the View Template. Note that the
     * order of the default grid columns is defined by the order of the fields in the Field Editor list.
     *
     * @var boolean Determines if this attribute should show up in a grid by default
     * @ORM\Column(name="`gridcolumn`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Grid Column")"
     */
    protected $gridColumn = false;

    /**
     * @var boolean Determines if this attribute is a collection
     * @ORM\Column(name="`collection`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Collection")"
     */
    protected $collection = false;

    /**
     * The field name on the referenced module, if it does have a related module.
     *
     * @var string Field name on the referenced type, if is collection
     * @ORM\Column(name="`inversefield`", type="string", length=200, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Inverse Field")"
     */
    protected $inversedField;

    /**
     * @var string Whether this model owns the relationships, if is collection
     *
     * @Doctrine\ORM\Mapping\Column(name="`ownsrelationship`", type="string", length=6, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Owns Relationship")
     */
    protected $ownsRelationship = null;

    /**
     * If set to Not Required, this never requires data to be entered into the field to save a module record. If
     * Required, a record cannot be saved with this field being null. If set to Required by Condition, this will make
     * the field required in specific circumstances as defined by the conditions. The True / False condition for
     * Required by sets the default state of the field until the conditions are met, for instance, if True, will not
     * display until the conditions are met.
     *
     * @var array
     * @Doctrine\ORM\Mapping\Column(name="`validation`", type="json_array", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Validation")
     */
    protected $validation = ['required' => false];

    /**
     * If 'allow' is set to true, the frontend will provide a button on the grid that will allow bulk updates on
     * the field. Optional parameters include 'buttonText', 'buttonClass', and 'buttonIcon'.
     *
     * @var array
     * @Doctrine\ORM\Mapping\Column(name="`bulkaction`", type="json_array", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Bulk Action")
     */
    protected $bulkAction = ['allow' => false];

    /**
     * If option is set for peoples module & module type is lookup, users are filterd on the basis of these filters.
     * Optional parameters include 'modulePermissions', 'modulePermissionsType', 'isOwnable'.
     *
     *  @var array The filter on source data, not defined explicitly
     * @Doctrine\ORM\Mapping\Column(name="`datasourcefilters`", type="json_array", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Data Source Filters")
     */
    protected $dataSourceFilters;


    /**
     * If set to a value, the UI will show the value by default in the form field.
     *
     * @var string
     * @Doctrine\ORM\Mapping\Column(name="`defaultvalue`", type="text", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Default Value")
     */
    protected $defaultValue = '';

    /**
     * If set to a value, the UI will display a tooltip containing the value
     *
     * @var string
     * @Doctrine\ORM\Mapping\Column(name="`tooltip`", type="text", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Tool Tip")
     */
    protected $tooltip = '';

    /**
     * If set to hidden, the field will only be accessible on the API level and never shown in the UI. If set to
     * Visible, the field will be eligible for display in the UI if added to the View Template. If set to Visible by
     * Condition, the field will not be visible / not visible until the specific condition has been met. The True /
     * False condition for Required by sets the default state of the field until the conditions are met, for instance,
     * if True, will not display until the conditions are met.
     *
     * @var array|boolean Determines if this attribute should be shown under certain conditions
     * @Doctrine\ORM\Mapping\Column(name="`visibility`", type="text", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Visibility")
     */
    protected $visibility = true;

    /**
     * @var boolean Determines if this attribute should be returned during READ operations
     * @ORM\Column(name="`readable`", type="boolean", options={"default" = true})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Readable")"
     */
    protected $readable = true;

    /**
     * If the field is writeable, it may be modified after creation of a module record. If it is not writeable, the
     * initial value cannot be changed after the record is created.
     *
     * @var boolean Determines if this attribute should be modified during UPDATE operations
     * @ORM\Column(name="`writeable`", type="boolean", options={"default" = true})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Writeable")"
     */
    protected $writeable = true;

    /**
     * @var boolean Determines if this attribute is a unique identifier for this class
     * @ORM\Column(name="`identifier`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Identifier")"
     */
    protected $identifier = false;

    /**
     * @var boolean Determines if the attribute is unique
     * @TODO Needs annotation generated for validation?
     * @ORM\Column(name="`unique`", type="boolean", options={"default" = true})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Unique")"
     */
    protected $unique = false;

    /**
     * @var \Cybersponse\MetadataBundle\Entity\StagingModelMetadata
     *
     * @Doctrine\ORM\Mapping\ManyToOne(targetEntity="Cybersponse\MetadataBundle\Entity\StagingModelMetadata", inversedBy="attributes", cascade={"persist"})
     * @Doctrine\ORM\Mapping\JoinColumns({
     *   @Doctrine\ORM\Mapping\JoinColumn(name="sattrib", referencedColumnName="uuid", onDelete="SET NULL")
     * })
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="staging_model_metadatas")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Staging Attributes")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="lookup")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     */
    protected $sattrib;

    /**
     * @return StagingModelMetadata
     */
    public function getSattrib()
    {
        return $this->sattrib;
    }

    /**
     * @param StagingModelMetadata $sattrib
     */
    public function setSattrib($sattrib)
    {
        $this->sattrib = $sattrib;
    }

    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param string $type
     */
    public function setType($type)
    {
        $this->type = $type;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @return int
     */
    public function getLength()
    {
        return $this->length;
    }

    /**
     * @param int $length
     */
    public function setLength($length)
    {
        $this->length = $length;
    }

    /**
     * @return string
     */
    public function getFormType()
    {
        return $this->formType;
    }

    /**
     * @param string $formType
     */
    public function setFormType($formType)
    {
        $this->formType = $formType;
    }

    /**
     * @return object
     */
    public function getDataSource()
    {
        return $this->dataSource;
    }

    /**
     * @param object $dataSource
     */
    public function setDataSource($dataSource)
    {
        $this->dataSource = $dataSource;
    }

    /**
     * @return boolean
     */
    public function isSearchable()
    {
        return $this->searchable;
    }

    /**
     * @param boolean $searchable
     */
    public function setSearchable($searchable)
    {
        $this->searchable = $searchable;
    }

    /**
     * @return boolean
     */
    public function isSystem()
    {
        return $this->system;
    }


    /**
     * @param $system
     */
    public function setSystem($system)
    {
        $this->system = $system;
    }

    /**
     * @return boolean
     */
    public function isEncrypted()
    {
        return $this->encrypted;
    }

    /**
     * @param boolean $encrypted
     */
    public function setEncrypted($encrypted)
    {
        $this->encrypted = $encrypted;
    }

    /**
     * @return boolean
     */
    public function isGridColumn()
    {
        return $this->gridColumn;
    }

    /**
     * @param boolean $gridColumn
     */
    public function setGridColumn($gridColumn)
    {
        $this->gridColumn = $gridColumn;
    }

    /**
     * @return boolean
     */
    public function isCollection()
    {
        return $this->collection;
    }

    /**
     * @param boolean $collection
     */
    public function setCollection($collection)
    {
        $this->collection = $collection;
    }

    /**
     * @return string
     */
    public function getInversedField()
    {
        return $this->inversedField;
    }

    /**
     * @param string $inversedField
     */
    public function setInversedField($inversedField)
    {
        $this->inversedField = $inversedField;
    }

    /**
     * @return boolean
     */
    public function isOwnsRelationship()
    {
        return json_decode($this->ownsRelationship, true);
    }

    /**
     * @param boolean $ownsRelationship
     */
    public function setOwnsRelationship($ownsRelationship)
    {
        $this->ownsRelationship = json_encode($ownsRelationship);
    }

    /**
     * @return array
     */
    public function getValidation()
    {
        return $this->validation;
    }

    /**
     * @param array $validation
     */
    public function setValidation($validation)
    {
        $this->validation = $validation;
    }

    /**
     * @return array
     */
    public function getBulkAction()
    {
        return $this->bulkAction;
    }

    /**
     * @param array $bulkAction
     */
    public function setBulkAction($bulkAction)
    {
        $this->bulkAction = $bulkAction;
    }

    /**
     * @return string
     */
    public function getDefaultValue()
    {
        return $this->defaultValue;
    }

    /**
     * @return object
     */
    public function getDataSourceFilters()
    {
        return $this->dataSourceFilters;
    }

    /**
     * @param object $dataSourceFilters
     */
    public function setDataSourceFilters($dataSourceFilters)
    {
        $this->dataSourceFilters = json_decode(json_encode($dataSourceFilters), true);
    }

    /**
     * @param string $defaultValue
     */
    public function setDefaultValue($defaultValue)
    {
        $this->defaultValue = $defaultValue;
    }

    /**
     * @return string
     */
    public function getTooltip()
    {
        return $this->tooltip;
    }

    /**
     * @param string $tooltip
     */
    public function setTooltip($tooltip)
    {
        $this->tooltip = $tooltip;
    }

    /**
     * @return array|bool
     */
    public function getVisibility()
    {
        return json_decode($this->visibility, true);
    }

    /**
     * @param array|bool $visibility
     */
    public function setVisibility($visibility)
    {
        $this->visibility = json_encode($visibility);
    }

    /**
     * @return boolean
     */
    public function isReadable()
    {
        return $this->readable;
    }

    /**
     * @param boolean $readable
     */
    public function setReadable($readable)
    {
        $this->readable = $readable;
    }

    /**
     * @return boolean
     */
    public function isWriteable()
    {
        return $this->writeable;
    }

    /**
     * @param boolean $writeable
     */
    public function setWriteable($writeable)
    {
        $this->writeable = $writeable;
    }

    /**
     * @return boolean
     */
    public function isIdentifier()
    {
        return $this->identifier;
    }

    /**
     * @param boolean $identifier
     */
    public function setIdentifier($identifier)
    {
        $this->identifier = $identifier;
    }

    /**
     * @return boolean
     */
    public function isUnique()
    {
        return $this->unique;
    }

    /**
     * @param boolean $unique
     */
    public function setUnique($unique)
    {
        $this->unique = $unique;
    }

    /**
     * @return boolean
     */
    public function isPeerReplicable()
    {
        return $this->peerReplicable;
    }

    /**
     * @param $peerReplicable
     */
    public function setPeerReplicable($peerReplicable)
    {
        $this->peerReplicable = $peerReplicable;
    }

    /**
     * @return int
     */
    public function getOrderIndex()
    {
        return $this->orderIndex;
    }

    /**
     * @param int $orderIndex
     */
    public function setOrderIndex($orderIndex)
    {
        $this->orderIndex = $orderIndex;
    }

    /**
     * Get determines if this attribute should be htmlEscaped
     *
     * @return  boolean
     */ 
    public function isHtmlEscape()
    {
        return $this->htmlEscape;
    }

    /**
     * Set determines if this attribute should be htmlEscaped
     *
     * @param  boolean  $htmlEscape  Determines if this attribute should be htmlEscaped
     *
     * @return  self
     */ 
    public function setHtmlEscape($htmlEscape)
    {
        $this->htmlEscape = $htmlEscape;
    }

    /**
     * Get determines if this attribute should be orphanRemoval
     *
     * @return  boolean
     */
    public function getOrphanRemoval()
    {
        return $this->orphanRemoval;
    }

    /**
     * Set determines if this attribute should be orphanRemoval
     *
     * @param  boolean  $orphanRemoval  Determines if this attribute should be orphanRemoval
     *
     * @return  self
     */
    public function setOrphanRemoval($orphanRemoval)
    {
        $this->orphanRemoval = $orphanRemoval;
    }
}
