<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Entity;


use Doctrine\ORM\Mapping as ORM;
use Cybersponse\CoreBundle\Entity\BaseEntity;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

/**
 * @ORM\MappedSuperclass
 * @ORM\HasLifecycleCallbacks
 */
abstract class Metadatas extends BaseEntity
{

    /**
     * @var string
     * This template specifies the fields that will be displayed when a record from this module is referenced in the application. Fields of a module can be specified in the display template as {{ field_name }}.
     *
     * @Doctrine\ORM\Mapping\Column(name="`displayname`", type="string", length=255, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Display Name")
     */
    protected $displayName;

    /**
     * @return string
     */
    public function getDisplayName()
    {
        return $this->displayName;
    }

    /**
     * @var string
     * The desired name to describe this item.
     *
     * @Symfony\Component\Validator\Constraints\Collection(
     *     fields = {
     *         "singular" = @Symfony\Component\Validator\Constraints\NotBlank()
     *     },
     *     allowExtraFields = true
     * )
     * @Doctrine\ORM\Mapping\Column(name="`descriptions`", type="json_array", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Descriptions")
     */
    protected $descriptions = ['singular' => null];

    /**
     * @param string $displayName
     */
    public function setDisplayName($displayName)
    {
        $this->displayName = $displayName;
    }

    /**
     * @return array
     */
    public function getDescriptions()
    {
        return $this->descriptions;
    }

    /**
     * @param array $descriptions
     */
    public function setDescriptions($descriptions)
    {
        #Fix for 15401 ( TODO : Ideally we should add Field level constraint for updating the fields )
        #dont update plural and singular values if present
        if (!array_key_exists('plural', $descriptions))
            $this->descriptions = $descriptions;
        else {
            #At the time of fixture load during fresh install
            if ($this->descriptions === null ||
                !array_key_exists('plural', $this->descriptions) ||
                $this->descriptions['plural'] === '' ||
                $this->descriptions['plural'] === null) {
                $this->descriptions = $descriptions;
            }
        }
    }

    /**
     * @Symfony\Component\Validator\Constraints\Callback
     */
    public function isPluralValid(ExecutionContextInterface $context)
    {
        if (array_key_exists('plural', $this->descriptions)) {
            $validator = $context->getValidator();
            $constraintViolations = $validator->validate($this->descriptions['plural'], [new NotBlank()]);
            foreach ($constraintViolations as $constraintViolation) {
                //Wrap violation with new context
                $context
                    ->buildViolation($constraintViolation->getMessage())
                    ->setCause($constraintViolation->getCause())
                    ->atPath('descriptions.plural')
                    ->setCode($constraintViolation->getCode())
                    ->setInvalidValue($constraintViolation->getInvalidValue())
                    ->setParameters($constraintViolation->getParameters())
                    ->setPlural($constraintViolation->getPlural())
                    ->addViolation();
            }
        }

        if (array_key_exists('singular', $this->descriptions)) {
            if ($this->isKeyword($this->descriptions['singular'])) {
                $value = $this->descriptions['singular'];
                $context
                    ->buildViolation("Invalid name- singular : " . "\"$value\"" . "conflicts with reserved keywords.")
                    ->setCause("Reserved PHP keyword")
                    ->setCode(22)
                    ->setPlural(22)
                    ->addViolation();
            }
        }
    }

    private function isKeyword($singular)
    {
        $keywords = array('__halt_compiler', 'abstract', 'and', 'array', 'as', 'break', 'callable', 'case', 'catch', 'clone', 'const', 'continue', 'declare', 'default', 'die', 'do', 'echo', 'else', 'elseif', 'empty', 'enddeclare', 'endfor', 'endforeach', 'endif', 'endswitch', 'endwhile', 'eval', 'exit', 'extends', 'final', 'for', 'foreach', 'function', 'global', 'goto', 'if', 'implements', 'include', 'include_once', 'instanceof', 'insteadof', 'interface', 'isset', 'list', 'namespace', 'new', 'or', 'print', 'private', 'protected', 'public', 'require', 'require_once', 'return', 'static', 'switch', 'throw', 'trait', 'try', 'unset', 'use', 'var', 'while', 'xor', 'class', 'files', 'file', 'staging_model_metadatas', 'staging_model_metadata', 'attribute_metadatas', 'attribute_metadata', 'model_metadatas', 'model_metadata', 'attrib_model_metadata', 'attrib_model_metadatas', 'cyops', 'system_settings', 'system_setting', 'user_settings', 'user_setting', 'generic_template', 'generic_templates', 'queries', 'system_view_template', 'system_view_templates', 'user_view_template', 'user_view_templates', 'emailTemplate', 'emailTemplates', 'proxy', 'secret', 'workflow', 'workflows', 'team', 'role', 'pgfile','email_templates','email_template', 'widgets', 'widget');
        if (in_array(strtolower($singular), $keywords)) {
            if (method_exists($this, 'getType')) {
                $type = $this->getType();
                if ($type !== 'picklists') {
                    return true;
                }
            }
        }
        return false;
    }
}
