<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Entity;


use Doctrine\ORM\Mapping as ORM;
use Doctrine\ORM\Mapping\UniqueConstraint;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Exportable\Exportable;

/**
 * Class ModelMetadata
 *
 * This class defines fields to describe models in the system
 *
 * @ORM\Table(name="model_metadatas", uniqueConstraints={@UniqueConstraint(name="field_model_uniques", columns={"type"})})
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Operations\Operations(create=false, update=false, delete=false)
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="model_metadatas")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="application")
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Doctrine\ORM\EntityRepository")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="model_metadata",plural="model_metadatas")
 */
class ModelMetadata extends Metadatas
{

    /**
     * The unique name of the module, must start with a letter, be all lowercase, and underscores are allowed.
     * This also becomes the API endpoint reference. We recommend that this describe the module content, for
     * instance, 'assets' or 'indicators'. We strongly recommend that you never change the module type once
     * set or you risk data loss and publish failure.
     *
     * @var string Metadata type, json primitive for attributes or iri/internal types for models
     * @Doctrine\ORM\Mapping\Column(name="`type`", type="string", length=500, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Form Type")
     * @Symfony\Component\Validator\Constraints\Regex(
     *     pattern="/^[a-z]+[a-z0-9_]*$/",
     *     message="Type must contain only lowercase characters, underscores and start with a letter, given: {{ value }}"
     * )
     * @Cybersponse\MetadataBundle\Validation\Keyword()
     */
    protected $type;

    /**
     * @var string The type of the parent
     * @Doctrine\ORM\Mapping\Column(name="`parenttype`", type="string", length=500, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Parent Type")
     * @Symfony\Component\Validator\Constraints\Regex(
     *     pattern="/^[a-z]+[a-z0-9_]*$/",
     *     message="Parent type must contain only lowercase characters, underscores and start with a letter, given: {{ value }}"
     * )
     * @Cybersponse\MetadataBundle\Validation\Keyword()
     */
    protected $parentType;

    /**
     * The actual table name for the module, must be lowercase. We recommend it to be unique and match the Type. We
     * strongly recommend you do not change the table name or you risk data loss as the referenced table will change,
     * though the old table will remain intact in the database.
     *
     * @var string The table name for ORM
     * @Cybersponse\MetadataBundle\Validation\Keyword()
     * @Doctrine\ORM\Mapping\Column(name="`tablename`", type="string", length=500, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Table Name")
     */
    protected $tableName;

    /**
     * Records are owned by a team or teams when you select Ownable. If you do not set this, then records within this module will be visible to all teams and users.
     *
     * @var boolean Determines if this class should have the ownable fields
     * @ORM\Column(name="`ownable`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Ownable")"
     */
    protected $ownable = true;

    /**
     * Trackable models enable the tracking of modify User and date as well as create User and date from a system level.
     *
     * @var boolean Determines if this class should have the trackable fields
     * @ORM\Column(name="`trackable`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Trackable")"
     */
    protected $trackable = true;

    /**
     * Taggable models enable the tracking of modify User and date as well as create User and date from a system level.
     *
     * @var boolean Determines if this class should have the taggable fields
     * @ORM\Column(name="`taggable`", type="boolean", options={"default" = true})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Taggable")"
     */
    protected $taggable = true;

    /**
     * PeerReplicable models enable the replication of data on their Peers.
     *
     * @var boolean Determines if this class should have the peerReplicable fields
     * @ORM\Column(name="`peerreplicable`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Peer Replicable")"
     */
    protected $peerReplicable = false;

    /**
     * @var string Default Sort configuration, to sort list by default based on chosen field.
     * @Doctrine\ORM\Mapping\Column(name="`defaultsort`", type="json_array", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Default Sort")"
     *
     */
    protected $defaultSort = [];

    /**
     * @var string Composite Unique Key for a given model.
     * @Doctrine\ORM\Mapping\Column(name="`uniqueconstraint`", type="json_array", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Unique Constraint")"
     *
     */
    protected $uniqueConstraint = [];

    /**
     * @var boolean Determines if this class should have the indexable fields
     * @ORM\Column(name="`indexable`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Indexable")"
     */
    protected $indexable = true;

    /**
     * @var boolean Determines if this class should have the writable fields
     * @ORM\Column(name="`writable`", type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Writeable")"
     */
    protected $writable = true;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @Doctrine\ORM\Mapping\OneToMany(targetEntity="Cybersponse\MetadataBundle\Entity\AttribModelMetadata", mappedBy="sattrib", cascade={"persist", "remove"})
     * @ORM\OrderBy({"orderIndex" = "ASC"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="attrib_model_metadata")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Attributes")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="oneToMany")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Exportable()
     */
    protected $attributes;

    public function __construct()
    {
        $this->attributes = new \Doctrine\Common\Collections\ArrayCollection();
    }


    /**
     * Add attributes
     *
     * @param \Cybersponse\MetadataBundle\Entity\AttribModelMetadata $attribute
     * @return ModelMetadata
     */
    public function addAttribute(\Cybersponse\MetadataBundle\Entity\AttribModelMetadata $attributes)
    {
        if (!$this->attributes->contains($attributes)) {
            $this->attributes[] = $attributes;
            $attributes->setSattrib($this);
        }
        return $this;
    }


    /**
     * Remove attributes
     *
     * @param \Cybersponse\MetadataBundle\Entity\AttribModelMetadata $attribute
     */
    public function removeAttribute(\Cybersponse\MetadataBundle\Entity\AttribModelMetadata $attribute)
    {
        if ($this->attributes->contains($attribute)) {
            $this->attributes->removeElement($attribute);
            $attribute->setSattrib(null);
        }
    }

    /**
     * Get events
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getAttributes()
    {
        return $this->attributes;
    }

    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param string $type
     */
    public function setType($type)
    {
        $this->type = $type;
    }

    /**
     * @return string
     */
    public function getParentType()
    {
        return $this->parentType;
    }

    /**
     * @param string $parentType
     */
    public function setParentType($parentType)
    {
        $this->parentType = $parentType;
    }

    /**
     * @return string
     */
    public function getTableName()
    {
        return $this->tableName;
    }

    /**
     * @param string $tableName
     */
    public function setTableName($tableName)
    {
        $this->tableName = $tableName;
    }

    /**
     * @return boolean
     */
    public function isOwnable()
    {
        return $this->ownable;
    }

    /**
     * @param boolean $ownable
     */
    public function setOwnable($ownable)
    {
        $this->ownable = $ownable;
    }

    /**
     * @return boolean
     */
    public function isTaggable()
    {
        return $this->taggable;
    }
     /**
     * @param boolean $taggable
     */
    public function setTaggable($taggable)
    {
        $this->taggable = $taggable;
    }

    /**
     * @return boolean
     */
    public function isTrackable()
    {
        return $this->trackable;
    }

    /**
     * @param boolean $trackable
     */
    public function setTrackable($trackable)
    {
        $this->trackable = $trackable;
    }

    /**
     * @return boolean
     */
    public function isPeerReplicable()
    {
        return $this->peerReplicable;
    }

    /**
     * @param boolean $peerReplicable
     */
    public function setPeerReplicable($peerReplicable)
    {
        $this->peerReplicable = $peerReplicable;
    }

    /**
     * @return array
     */
    public function getDefaultSort()
    {
        return $this->defaultSort;
    }

    /**
     * @param array $defaultSort
     */
    public function setDefaultSort($defaultSort)
    {
        $this->defaultSort = $defaultSort;
    }

    /**
     * @return array
     */
    public function getUniqueConstraint()
    {
        return $this->uniqueConstraint;
    }

    /**
     * @param array $uniqueConstraint
     */
    public function setUniqueConstraint($uniqueConstraint)
    {
        $this->uniqueConstraint = $uniqueConstraint;
    }


    /**
     * @return boolean
     */
    public function isIndexable()
    {
        return $this->indexable;
    }

    /**
     * @param boolean $indexable
     */
    public function setIndexable($indexable)
    {
        $this->indexable = $indexable;
    }


    /**
     * @return boolean
     */
    public function isWritable()
    {
        return $this->writable;
    }

    /**
     * @param boolean $writable
     */
    public function setWritable($writable)
    {
        $this->writable = $writable;
    }
}
