<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Generation\Annotations\Classes\DefaultSort;


use Cybersponse\MetadataBundle\Entity\ModelMetadata;
use Cybersponse\MetadataBundle\Generation\Annotations\Classes\ClassAnnotationGeneratorInterface;
use Doctrine\ORM\Mapping\ClassMetadataInfo;

class DefaultSortAnnotationGenerator implements ClassAnnotationGeneratorInterface
{
    /**
     * @return string
     */
    public function getAnnotationClassName()
    {
        return DefaultSort::class;
    }

    /**
     * @param ModelMetadata $metadata
     * @inheritdoc
     */
    public function supportsGenerationForMetadata($metadata)
    {
        return !empty($metadata->getDefaultSort());
    }

    /**
     * @param ModelMetadata $modelMetadata
     * @param ClassMetadataInfo $metadata
     * @return bool|string In the format '@Full\Namespace\Annotation(etc)' or false
     */
    public function generateClassAnnotationFromModelMetadata($modelMetadata, $metadata)
    {
        $defaultSort = $modelMetadata->getDefaultSort();
        $sorts = [];

        foreach ($defaultSort as $sort) {
            $sorts[] = sprintf('{"field"="%s", "direction"="%s"}', $sort['field'], $sort['direction']);
        }
        return sprintf('@%s(defaultSort={%s})', DefaultSort::class, implode(', ', $sorts));
    }

    /**
     * @inheritdoc
     * @param DefaultSort $annotation
     */
    public function loadClassMetadata($metadata, $annotation)
    {
        $metadata->setDefaultSort($annotation->defaultSort);
    }
}
