<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions;

use Cybersponse\MetadataBundle\Entity\ModelMetadata;
use Cybersponse\MetadataBundle\Generation\Annotations\Classes\ClassAnnotationGeneratorInterface;
use Doctrine\ORM\Mapping\ClassMetadataInfo;

class DescriptionsAnnotationGenerator implements ClassAnnotationGeneratorInterface
{
    /**
     * @return string
     */
    public function getAnnotationClassName()
    {
        return Descriptions::class;
    }

    /**
     * @param ModelMetadata $metadata
     * @inheritdoc
     */
    public function supportsGenerationForMetadata($metadata)
    {
        return !empty($metadata->getDescriptions());
    }

    /**
     * @param ModelMetadata $modelMetadata
     * @param ClassMetadataInfo $metadata
     * @return bool|string In the format '@Full\Namespace\Annotation(etc)' or false
     */
    public function generateClassAnnotationFromModelMetadata($modelMetadata, $metadata)
    {
        $singular = '';
        $plural = '';
        $descriptions = $modelMetadata->getDescriptions();

        if (array_key_exists('singular', $descriptions)) {
            $singular = $descriptions['singular'];
        }
        if (array_key_exists('plural', $descriptions)) {
            $plural = $descriptions['plural'];
        }

        return sprintf('@%s(singular="%s",plural="%s")', Descriptions::class, $singular, $plural);
    }

    /**
     * @inheritdoc
     * @param Descriptions $annotation
     */
    public function loadClassMetadata($metadata, $annotation)
    {
        $metadata->setDescriptions($annotation);
    }
}
