<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Generation\Annotations\Classes\UniqueConstraint;


use Cybersponse\MetadataBundle\Entity\ModelMetadata;
use Cybersponse\MetadataBundle\Generation\Annotations\Classes\ClassAnnotationGeneratorInterface;
use Doctrine\ORM\Mapping\ClassMetadataInfo;

class UniqueConstraintAnnotationGenerator implements ClassAnnotationGeneratorInterface
{
    /**
     * @return string
     */
    public function getAnnotationClassName()
    {
        return UniqueConstraint::class;
    }

    /**
     * @param ModelMetadata $metadata
     * @inheritdoc
     */
    public function supportsGenerationForMetadata($metadata)
    {
        return !empty($metadata->getUniqueConstraint());
    }

    /**
     * @param ModelMetadata $modelMetadata
     * @param ClassMetadataInfo $metadata
     * @return bool|string In the format '@Full\Namespace\Annotation(etc)' or false
     */
    public function generateClassAnnotationFromModelMetadata($modelMetadata, $metadata)
    {
        $uniqueConstraint = $modelMetadata->getUniqueConstraint();
        $unqConstraint = [];

        if (count($modelMetadata->getUniqueConstraint()))
        {
            foreach ($uniqueConstraint as $key => $value) 
            {
                foreach ($value as $innerKey => $innerval){
                    $implodedColumns = '';
                    foreach($innerval['columns'] as $columns) {
                        $implodedColumns .= $implodedColumns ? '","' .$columns :  $columns;
                    }
                    $unqConstraint[] = sprintf('{"%s"={"columns"={"%s"}}}', $innerKey, $implodedColumns);
                }
            }
            return sprintf('@%s(uniqueConstraint={%s})', UniqueConstraint::class, implode(', ', $unqConstraint)); 
        }
    }

    /**
     * @inheritdoc
     * @param DefaultSort $annotation
     */
    public function loadClassMetadata($metadata, $annotation)
    {
        $metadata->setUniqueConstraint($annotation->uniqueConstraint);
    }
}
