<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction;

use Cybersponse\MetadataBundle\Entity\AttributeMetadata;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\FieldAnnotationGeneratorInterface;

class BulkActionAnnotationGenerator implements FieldAnnotationGeneratorInterface
{
    /**
     * @inheritdoc
     */
    public function getAnnotationClassName()
    {
        return BulkAction::class;
    }

    /**
     * @param BulkAction $annotation
     * @inheritdoc
     */
    public function loadAttributeMetadata($metadata, $annotation)
    {
        $metadata->setBulkAction([
            'allow' => $annotation->allow,
            'buttonText' => $annotation->buttonText,
            'buttonIcon' => $annotation->buttonIcon,
            'buttonClass' => $annotation->buttonClass
        ]);
    }

    /**
     * @param AttributeMetadata $attributeMetadata
     * @return bool
     */
    public function supportsGenerationForMetadata($attributeMetadata)
    {
        return !empty($attributeMetadata->getBulkAction());
    }

    /**
     * @param AttributeMetadata $attributeMetadata
     * @inheritdoc
     */
    public function generateFieldAnnotationFromAttributeMetadata($attributeMetadata, $metadata)
    {
        $annotations = [];
        foreach ($attributeMetadata->getBulkAction() as $key => $value) {
            if ($key == 'allow') {
                $annotations[] = sprintf('%s=%s', $key, $value ? 'true' : 'false');
            } else {
                $annotations[] = sprintf('%s="%s"', $key, $value);
            }
        }
        return '@' . BulkAction::class . '(' . implode($annotations, ', ') . ')';
    }
}
