<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource;


use Cybersponse\MetadataBundle\Entity\AttributeMetadata;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\FieldAnnotationGeneratorInterface;

class DataSourceAnnotationGenerator implements FieldAnnotationGeneratorInterface
{
    /**
     * @inheritdoc
     */
    public function getAnnotationClassName()
    {
        return DataSource::class;
    }

    /**
     * @param DataSource $annotation
     * @inheritdoc
     */
    public function loadAttributeMetadata($metadata, $annotation)
    {
        $metadata->setDataSource(['model' => $annotation->model, 'query' => (array)$annotation->query]);
    }

    /**
     * @param AttributeMetadata $attributeMetadata
     * @return bool
     */
    public function supportsGenerationForMetadata($attributeMetadata)
    {
        return !empty($attributeMetadata->getDataSource()) || $attributeMetadata->isCollection();
    }

    /**
     * @inheritdoc
     *
     * Ex. (module="picklists", query={"listName__name"="AlertType", "$orderby"="orderIndex"})
     */
    public function generateFieldAnnotationFromAttributeMetadata($attributeMetadata, $metadata)
    {
        $dataSource = $attributeMetadata->getDataSource();

        $queryObject = null;
        $queryObjectString = '';

        if (array_key_exists('query', $dataSource)) {
            $queryObject = $dataSource['query'];

            if (isset($queryObject['logic'])) {
                $queryObjectString = sprintf('logic="%s"', $queryObject['logic']);
            }

            $filters = [];
            $sorts = [];
            if (isset($queryObject['filters'])) {
                foreach ($queryObject['filters'] as $filter) {
                    $filters[] = sprintf(
                        '{"field"="%s", "operator"="%s", "value"="%s"}',
                        $filter['field'],
                        $filter['operator'],
                        $filter['value']
                    );
                }
                $queryObjectString = sprintf('%s, filters={%s}', $queryObjectString, implode(', ', $filters));
            }

            if (isset($queryObject['sort'])) {
                foreach ($queryObject['sort'] as $sort) {
                    $sorts[] = sprintf('{"field"="%s", "direction"="%s"}', $sort['field'], $sort['direction']);
                }
                $queryObjectString = sprintf('%s, sort={%s}', $queryObjectString, implode(', ', $sorts));
            }
        }

        $dataSourceString = sprintf('model="%s"', $dataSource['model']);

        if (!empty($queryObjectString)) {
            $dataSourceString = sprintf(
                "%s, query=\n<spaces> * <spaces>@%s(%s)\n<spaces> * ",
                $dataSourceString,
                Query::class,
                $queryObjectString
            );
        }

        return sprintf('@%s(%s)', DataSource::class, $dataSourceString);
    }
}
