<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSourceFilters;

use Cybersponse\MetadataBundle\Entity\AttributeMetadata;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\FieldAnnotationGeneratorInterface;

class DataSourceFiltersAnnotationGenerator implements FieldAnnotationGeneratorInterface
{
    /**
     * @inheritdoc
     */
    public function getAnnotationClassName()
    {
        return DataSourceFilters::class;
    }

    /**
     * @param DataSourceFilters $annotation
     * @inheritdoc
     */
    public function loadAttributeMetadata($metadata, $annotation)
    {
        $metadata->setDataSourceFilters(json_decode(json_encode([
            'isOwnable' => $annotation->isOwnable,
            'modulePermissions' => $annotation->modulePermissions,
            'modulePermissionsType' => (array)$annotation->modulePermissionsType
        ])), true);
    }

    /**
     * @param AttributeMetadata $attributeMetadata
     * @return bool
     */
    public function supportsGenerationForMetadata($attributeMetadata)
    {
        return !empty($attributeMetadata->getDataSourceFilters());
    }

    public function getStringifiedObject($dataSource, $nesting = false)
    {
        $dataQueryString = '';
        $count = 0;
        $dataSourceLength = count($dataSource);

        foreach ($dataSource as $key => $value) {

            if (gettype($value) == 'string') {
                if ($nesting) {
                    if ($count == $dataSourceLength - 1) {
                        $tempObj = sprintf('"%s"="%s"', $key, $value);
                    } else {
                        $tempObj = sprintf('"%s"="%s", ', $key, $value);
                    }
                } else {
                    if ($count == $dataSourceLength - 1) {
                        $tempObj = sprintf('%s="%s"', $key, $value);
                    } else {
                        $tempObj = sprintf('%s="%s", ', $key, $value);
                    }
                }
                $dataQueryString = $dataQueryString . $tempObj;
            } elseif (gettype($value) == 'boolean') {
                if ($nesting) {
                    if ($count == $dataSourceLength - 1) {
                        $tempObj = sprintf('%s=%s', $key, $value ? 'true' : 'false');
                    } else {
                        $tempObj = sprintf('%s=%s, ', $key, $value ? 'true' : 'false');
                    }
                } else {
                    if ($count == $dataSourceLength - 1) {
                        $tempObj = sprintf('%s=%s', $key, $value ? "true" : "false");
                    } else {
                        $tempObj = sprintf('%s=%s, ', $key, $value ? "true" : "false");
                    }
                }
                $dataQueryString = $dataQueryString . $tempObj;
            } elseif (gettype($value) == 'array') {
                $keyValue = sprintf('%s', $key);
                $dataQuerynestedString = sprintf("=\n     *   @%s", modulePermissionsType::class);
                $dataQueryString = $dataQueryString . $keyValue . $dataQuerynestedString . '(';
                $innerObj = $this->getStringifiedObject($value, true);
                $dataQueryString = $dataQueryString . $innerObj . '), ';
            }
            $count = $count + 1;
        }
        return $dataQueryString;

    }

    /**
     * @param AttributeMetadata $attributeMetadata
     * @inheritdoc
     */
    public function generateFieldAnnotationFromAttributeMetadata($attributeMetadata, $metadata)
    {
        $dataSource = $attributeMetadata->getDataSourceFilters();
        $dataQueryString = $this->getStringifiedObject($dataSource);
        return '@' . DataSourceFilters::class . sprintf('(%s)', $dataQueryString);
    }
}
