<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Generation\Annotations\Fields\Length;

use Cybersponse\MetadataBundle\Entity\AttributeMetadata;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\FieldAnnotationGeneratorInterface;
use Cybersponse\MetadataBundle\Generation\Metadata\ModelAttributeMetadata;
use Doctrine\ORM\Mapping\ClassMetadataInfo;
use Symfony\Component\Validator\Constraints\Length;

class LengthAnnotationGenerator implements FieldAnnotationGeneratorInterface
{

    /**
     * @return string
     */
    public function getAnnotationClassName()
    {
        return Length::class;
    }

    /**
     * @param AttributeMetadata $attributeMetadata
     * @param ClassMetadataInfo $metadata
     * @return bool|string In the format '@Full\Namespace\Annotation(etc)' or false
     */
    public function generateFieldAnnotationFromAttributeMetadata($attributeMetadata, $metadata)
    {
        $maxLength = $attributeMetadata->getValidation()['maxlength'];
        $fieldName = $attributeMetadata->getName();
        return sprintf('@%s(max=%s, maxMessage = "%s cannot be longer than {{ limit }} characters")', Length::class, $maxLength, strtoupper($fieldName));
    }

    /**
     * @param ModelAttributeMetadata $metadata
     * @param $annotation
     * @return mixed
     */
    public function loadAttributeMetadata($metadata, $annotation)
    {
        $validation = $metadata->getValidation();
        $validation['maxlength'] = true;
        $metadata->setValidation($validation);
    }

    /**
     * @param AttributeMetadata $attributeMetadata
     * @return bool
     */
    public function supportsGenerationForMetadata($attributeMetadata)
    {
        $validation = (array)$attributeMetadata->getValidation();
        $formType = $attributeMetadata->getFormType();

        return $formType=='text' && array_key_exists('maxlength', $validation) && $validation['maxlength'] > 0 && $validation['maxlength'] < 10485761;
    }
}
