<?php	
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Generation\Annotations\Fields\Required;	

use Cybersponse\MetadataBundle\Entity\AttributeMetadata;	
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\FieldAnnotationGeneratorInterface;	
use Doctrine\ORM\Mapping\ClassMetadataInfo;	
use Symfony\Component\Validator\Constraints\NotBlank;	
use Symfony\Component\Validator\Constraints\NotNull;	
use Symfony\Component\Validator\Constraints\Regex;	

class RequiredAnnotationGenerator implements FieldAnnotationGeneratorInterface	
{	
    /**	
     * @inheritdoc	
     */	
    public function getAnnotationClassName()	
    {	
        return NotBlank::class;	
    }	

    /**	
     * @param NotBlank $annotation	
     * @inheritdoc	
     */	
    public function loadAttributeMetadata($metadata, $annotation)	
    {	
        $validation = $metadata->getValidation();	
        $validation['required'] = true;	
        $metadata->setValidation($validation);	
    }	

    /**	
     * @param AttributeMetadata $attributeMetadata	
     * @return bool	
     */	
    public function supportsGenerationForMetadata($attributeMetadata)	
    {	
        $validation = (array)$attributeMetadata->getValidation();	

        return array_key_exists('required', $validation)	
            && filter_var($validation['required'], FILTER_VALIDATE_BOOLEAN);	
    }	

    /**	
     * @param AttributeMetadata $attributeMetadata	
     * @param ClassMetadataInfo $metadata	
     * @return bool|string In the format '@Full\Namespace\Annotation(etc)' or false	
     */	
    public function generateFieldAnnotationFromAttributeMetadata($attributeMetadata, $metadata)	
    {	
        if ($attributeMetadata->getType() == 'string')
            return sprintf("@%s()\n <spaces>* @%s()\n <spaces>* @%s(pattern=\"/^ *$/\",\n <spaces>* match=false,\n <spaces>* message=\"Required value should not contain only space.\")", NotBlank::class, NotNull::class, Regex::class);	
        return sprintf("@%s()\n <spaces>* @%s()", NotBlank::class, NotNull::class);	

    }	
}	
