<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Generation\Annotations\Fields\Url;

use Cybersponse\MetadataBundle\Entity\AttributeMetadata;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\FieldAnnotationGeneratorInterface;
use Cybersponse\MetadataBundle\Generation\Metadata\ModelAttributeMetadata;
use Doctrine\ORM\Mapping\ClassMetadataInfo;
use Symfony\Component\Validator\Constraints\Url;

class UrlAnnotationGenerator implements FieldAnnotationGeneratorInterface
{

    /**
     * @return string
     */
    public function getAnnotationClassName()
    {
        return Url::class;
    }

    /**
     * @param AttributeMetadata $attributeMetadata
     * @param ClassMetadataInfo $metadata
     * @return bool|string In the format '@Full\Namespace\Annotation(etc)' or false
     */
    public function generateFieldAnnotationFromAttributeMetadata($attributeMetadata, $metadata)
    {
        return sprintf('@%s(protocols={"http", "https", "ftp"})', Url::class);
    }

    /**
     * @param ModelAttributeMetadata $metadata
     * @param $annotation
     * @return mixed
     */
    public function loadAttributeMetadata($metadata, $annotation)
    {
        $validation = $metadata->getValidation();
        $validation['url'] = true;
        $metadata->setValidation($validation);
    }

    /**
     * @param AttributeMetadata $attributeMetadata
     * @return bool
     */
    public function supportsGenerationForMetadata($attributeMetadata)
    {
        $validation = (array)$attributeMetadata->getValidation();

        return array_key_exists('url', $validation)
            && filter_var($validation['url'], FILTER_VALIDATE_BOOLEAN);
    }
}
