<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Generation\Metadata;


use Dunglas\ApiBundle\Mapping\AttributeMetadata;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 * Class ModelAttributeMetadata
 *
 * All new fields must be added to the __sleep() function
 *
 * @package Cybersponse\MetadataBundle\Generation\Metadata
 */
class ModelAttributeMetadata extends AttributeMetadata
{

    /**
     * @var string
     * @Groups({"metadata"})
     */
    public $formType;

    /**
     * @var array
     * @Groups({"metadata"})
     */
    public $dataSource;

    /**
     * @var boolean
     * @Groups({"metadata"})
     */
    public $searchable = false;

    /**
     * @var boolean
     * @Groups({"metadata"})
     */
    public $encrypted = false;

    /**
     * @var boolean
     * @Groups({"metadata"})
     */
    public $htmlEscape = false;

    /**
     * @var boolean
     * @Groups({"metadata"})
     */
    public $peerReplicable = true;

    /**
     * @var boolean
     * @Groups({"metadata"})
     */
    public $gridColumn = false;

    /**
     * @var string
     * @Groups({"metadata"})
     */
    public $title;

    /**
     * @var mixed
     * @Groups({"metadata"})
     */
    public $validation;

    /**
     * @var mixed
     * @Groups({"metadata"})
     */
    public $visibility;

    /**
     * @var array
     * @Groups({"metadata"})
     */
    public $bulkAction;

    /**
     * @var string
     * @Groups({"metadata"})
     */
    public $defaultValue;

    /**
     * @var string
     * @Groups({"metadata"})
     */
    public $tooltip;

    /**
     * @var boolean
     * @Groups({"metadata"})
     */
    public $unique = false;

    /**
     * Used to determine if this attribute should be serialized as an object or an iri during an export
     *
     * @var boolean
     * @Groups({"metadata"})
     */
    public $exportable = false;

    /**
     * @var array
     * @Groups({"metadata"})
     */
    public $dataSourceFilters;


    function __construct(AttributeMetadata $attributeMetadata)
    {
        parent::__construct($attributeMetadata->getName(), $attributeMetadata->isIdentifier());

        $this->setTypes((array)$attributeMetadata->getTypes());
        $this->setDescription($attributeMetadata->getDescription());
        $this->setReadable(true);
        $this->setWritable(true);
        $this->setRequired($attributeMetadata->isRequired());
        $this->setNormalizationLink($attributeMetadata->isNormalizationLink());
        $this->setDenormalizationLink($attributeMetadata->isDenormalizationLink());
        $this->setIri($attributeMetadata->getIri());

        $this->setValidation([]);
        $this->setVisibility(true);
        $this->setBulkAction([]);
    }

    /**
     * @return string
     */
    public function getFormType()
    {
        return $this->formType;
    }

    /**
     * @param string $formType
     */
    public function setFormType($formType)
    {
        $this->formType = $formType;
    }

    /**
     * @return array
     */
    public function getDataSource()
    {
        return $this->dataSource;
    }

    /**
     * @param array $dataSource
     */
    public function setDataSource($dataSource)
    {
        $this->dataSource = $dataSource;
    }

    /**
     * @return boolean
     */
    public function isSearchable()
    {
        return $this->searchable;
    }

    /**
     * @param boolean $searchable
     */
    public function setSearchable($searchable)
    {
        $this->searchable = $searchable;
    }

    /**
     * @return boolean
     */
    public function isEncrypted()
    {
        return $this->encrypted;
    }

    /**
     * @param boolean $encrypted
     */
    public function setEncrypted($encrypted)
    {
        $this->encrypted = $encrypted;
    }

    /**
     * @return boolean
     */
    public function isHtmlEscape()
    {
        return $this->htmlEscape;
    }

    /**
     * @param boolean $htmlEscape
     */
    public function setHtmlEscape($htmlEscape)
    {
        $this->htmlEscape = $htmlEscape;
    }

    /**
     * @return boolean
     */
    public function isGridColumn()
    {
        return $this->gridColumn;
    }

    /**
     * @param boolean $gridColumn
     */
    public function setGridColumn($gridColumn)
    {
        $this->gridColumn = $gridColumn;
    }

    /**
     * @return boolean
     */
    public function isPeerReplicable()
    {
        return $this->peerReplicable;
    }

    /**
     * @param $peerReplicable
     */
    public function setPeerReplicable($peerReplicable)
    {
        $this->peerReplicable = $peerReplicable;
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = $title;
    }

    /**
     * @return mixed
     */
    public function getValidation()
    {
        return $this->validation;
    }

    /**
     * @param mixed $validation
     */
    public function setValidation($validation)
    {
        $this->validation = $validation;
    }

    /**
     * @return mixed
     */
    public function getVisibility()
    {
        return $this->visibility;
    }

    /**
     * @param mixed $visibility
     */
    public function setVisibility($visibility)
    {
        $this->visibility = $visibility;
    }

    /**
     * @return array
     */
    public function getBulkAction()
    {
        return $this->bulkAction;
    }

    /**
     * @param array $bulkAction
     */
    public function setBulkAction($bulkAction)
    {
        $this->bulkAction = $bulkAction;
    }

    /**
     * @return string
     */
    public function getDefaultValue()
    {
        return $this->defaultValue;
    }

    /**
     * @param string $defaultValue
     */
    public function setDefaultValue($defaultValue)
    {
        $this->defaultValue = $defaultValue;
    }

    /**
     * @return string
     */
    public function getTooltip()
    {
        return $this->tooltip;
    }

    /**
     * @param string $tooltip
     */
    public function setTooltip($tooltip)
    {
        $this->tooltip = $tooltip;
    }

    /**
     * @return boolean
     */
    public function isExportable()
    {
        return $this->exportable;
    }

    /**
     * @param boolean $exportable
     */
    public function setExportable($exportable)
    {
        $this->exportable = $exportable;
    }

    /**
     * @return boolean
     */
    public function isUnique()
    {
        return $this->unique;
    }

    /**
     * @param boolean $unique
     */
    public function setUnique($unique)
    {
        $this->unique = $unique;
    }

    /**
     * @return object
     */
    public function getDataSourceFilters()
    {
        return $this->dataSourceFilters;
    }

    /**
     * @param object $dataSourceFilters
     */
    public function setDataSourceFilters($dataSourceFilters)
    {
        $this->dataSourceFilters = json_decode(json_encode($dataSourceFilters), true);
    }

    /**
     * @inheritdoc
     */
    public function __sleep()
    {
        $properties = parent::__sleep();

        return array_merge([
            'formType',
            'dataSource',
            'dataSourceFilters',
            'searchable',
            'encrypted',
            'htmlEscape',
            'gridColumn',
            'title',
            'validation',
            'visibility',
            'exportable',
            'unique'
        ], $properties);
    }
}
