<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Generation\Metadata;


use Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions;
use Dunglas\ApiBundle\Mapping\ClassMetadata;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 * Class ModelClassMetadata
 *
 * All new fields must be added to the __sleep() function
 *
 * @package Cybersponse\MetadataBundle\Generation\Metadata
 */
class ModelClassMetadata extends ClassMetadata
{

    /**
     * @var string
     * @Groups({"metadata"})
     */
    public $type;

    /**
     * @var string
     * @Groups({"metadata"})
     */
    public $module;

    /**
     * @var string
     * @Groups({"metadata"})
     */
    public $displayName;

    /**
     * @var Descriptions
     * @Groups({"metadata"})
     */
    public $descriptions;

    /**
     * @var array
     * @Groups({"metadata"})
     */
    public $defaultSort;
    
    /**
     * @var array
     * @Groups({"metadata"})
     */
    public $uniqueConstraint;
    
    /**
     * @var boolean
     * @Groups({"metadata"})
     */
    public $ownable;

    /**
     * @var boolean
     * @Groups({"metadata"})
     */
    public $trackable;

    /**
     * @var boolean
     * @Groups({"metadata"})
     */
    public $taggable;

    /**
     * @var boolean
     * @Groups({"metadata"})
     */
    public $peerReplicable;

    /**
     * @todo this could turn from a boolean (where 'id' is default) to a list of indexes
     * @var boolean
     * @Groups({"metadata"})
     */
    public $indexable;

    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param string $type
     */
    public function setType($type)
    {
        $this->type = $type;
    }

    /**
     * @return string
     */
    public function getModule()
    {
        return $this->module;
    }

    /**
     * @param string $module
     */
    public function setModule($module)
    {
        $this->module = $module;
    }

    /**
     * @param string $displayName
     */
    public function setDisplayName($displayName)
    {
        $this->displayName = $displayName;
    }

    /**
     * @param Descriptions $descriptions
     */
    public function setDescriptions($descriptions)
    {
        $this->descriptions = $descriptions;
    }

    /**
     * @param array $defaultSort
     */
    public function setDefaultSort($defaultSort)
    {
        $this->defaultSort = $defaultSort;
    }
    
    /**
     * @param array $uniqueConstraint
     */
    public function setUniqueConstraint($uniqueConstraint)
    {
        $this->uniqueConstraint = $uniqueConstraint;
    }

    /**
     * @param boolean $ownable
     */
    public function setOwnable($ownable)
    {
        $this->ownable = $ownable;
    }

    /**
     * @param boolean $trackable
     */
    public function setTrackable($trackable)
    {
        $this->trackable = $trackable;
    }

    /**
     * @param boolean $taggable
     */
    public function setTaggable($taggable)
    {
        $this->taggable = $taggable;
    }

    /**
     * @param boolean $peerReplicable
     */
    public function setPeerReplicable($peerReplicable)
    {
        $this->peerReplicable = $peerReplicable;
    }

    /**
     * @return ModelAttributeMetadata[]
     */
    public function getAttributes()
    {
        return parent::getAttributes();
    }

    /**
     * @param boolean $indexable
     */
    public function setIndexable($indexable)
    {
        $this->indexable = $indexable;
    }

    /**
     * @inheritdoc
     */
    public function __sleep()
    {
        $properties = parent::__sleep();

        return array_merge([
            'type',
            'module',
            'displayName',
            'descriptions',
            'defaultSort',
            'uniqueConstraint',
            'ownable',
            'trackable',
            'taggable',
            'peerReplicable',
            'indexable'
        ], $properties);
    }
}
