<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Mapping;


use Cybersponse\MetadataBundle\Entity\ModelMetadata;
use Doctrine\Common\Persistence\ObjectRepository;

/**
 * Class ModelMetadataFactory
 *
 * @package Cybersponse\MetadataBundle\Mapping
 */
class ModelMetadataFactory
{
    /** @var ModelMetadata[] */
    protected $loadedModelMetadata;

    /** @var ObjectRepository */
    protected $modelMetadataRepository;

    /** @var ObjectRepository */
    protected $attribModelMetadataRepository;

    /**
     * @param ObjectRepository $attribModelMetadataRepository
     */
    public function setAttribModelMetadataRepository($attribModelMetadataRepository)
    {
        $this->attribModelMetadataRepository = $attribModelMetadataRepository;
    }

    /**
     * @param ObjectRepository $modelMetadataRepository
     */
    public function setModelMetadataRepository($modelMetadataRepository)
    {
        $this->modelMetadataRepository = $modelMetadataRepository;
    }

    /**
     * Returns all the custom model metadata in the system with class short names as keys
     *
     * @return array
     */
    public function getAllModelMetadata()
    {
        return $this->loadedModelMetadata = $this->loadModelMetadata();
    }

    /**
     * @param $shortName
     *
     * @return null|ModelMetadata
     */
    public function getModelMetadataByShortName($shortName)
    {
        $modelMetadata = $this->getAllModelMetadata();

        if (array_key_exists($shortName, $modelMetadata)) {
            return $modelMetadata[$shortName];
        }

        return null;
    }

    public function getModulesForPicklist($picklistName)
    {
        $moduleNames = [];
        /** @var ModelMetadata[] $modelMetadatas */
        $attributes = $this->attribModelMetadataRepository->createQueryBuilder('o')->andwhere("JSON_GET_TEXT(o.dataSource, 'query') like :picklistname ")->setParameter('picklistname', '%\"value\":"' . $picklistName . '"%')->getQuery()->getResult();
        foreach($attributes as $attribute) {
            $moduleNames[] = $attribute->getSattrib()->getType();
        }
        return $moduleNames;
    }

    private function loadModelMetadata()
    {
        $modelMetadataMap = [];

        /** @var ModelMetadata[] $modelMetadatas */
        $modelMetadatas = $this->modelMetadataRepository->findAll();

        foreach ($modelMetadatas as $modelMetadata) {
            $classShortName = self::getClassShortNameForModelMetadata($modelMetadata);
            $modelMetadataMap[$classShortName] = $modelMetadata;
        }

        return $modelMetadataMap;
    }

    /**
     * @param ModelMetadata $modelMetadata
     * @return string
     */
    public static function getClassShortNameForModelMetadata($modelMetadata)
    {
        $singular = $modelMetadata->getDescriptions()['singular'];
        $escapedSingular = preg_replace('/[^a-z0-9\s]+/i', '', $singular);

        return \Doctrine\Common\Util\Inflector::classify($escapedSingular);
    }
}
