<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Tests\Unit\Conditional\Query\Logic;


use Cybersponse\QueryBundle\Constants\AttributeLevelOperators;
use Cybersponse\MetadataBundle\Conditional\QueryMatcher;

class AndLogicTest extends LogicTestCase
{
    protected function setUp()
    {
        parent::setUp();
        $this->logic = QueryMatcher::LOGIC_AND;
    }

    // region No Filters
    public function test_GivenEmptyFilters_ShouldReturnFalse()
    {
        $this->objectToValidate = (object)[];
        $this->filters = [];

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }
    // endregion
    // region Single Filter
    public function test_GivenSingleValidFilter_ShouldReturnTrue()
    {
        $expectedField = 'foo';
        $expectedValue = 5;
        $this->objectToValidate = (object)[$expectedField => $expectedValue];
        $this->filters = [
            [
                'field' => $expectedField,
                'operator' => AttributeLevelOperators::EQUALS,
                'value' => $expectedValue
            ]
        ];

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenSingleInvalidFilter_ShouldReturnFalse()
    {
        $expectedField = 'foo';
        $expectedValue = 5;
        $this->objectToValidate = (object)[$expectedField => $expectedValue];
        $this->filters = [
            [
                'field' => $expectedField,
                'operator' => AttributeLevelOperators::EQUALS,
                'value' => $expectedValue + 1
            ]
        ];

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }
    // endregion
    // region Multiple Filters
    public function test_GivenMultipleValidFilters_ShouldReturnTrue()
    {
        $expectedField = 'foo';
        $expectedValue = 5;
        $this->objectToValidate = (object)[$expectedField => $expectedValue];
        $this->filters = [
            [
                'field' => $expectedField,
                'operator' => AttributeLevelOperators::EQUALS,
                'value' => $expectedValue
            ],
            [
                'field' => $expectedField,
                'operator' => AttributeLevelOperators::EQUALS,
                'value' => $expectedValue
            ]
        ];

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenSingleValidFilter_GivenSingleInvalidFilter_ShouldReturnFalse()
    {
        $expectedField = 'foo';
        $expectedValue = 5;
        $this->objectToValidate = (object)[$expectedField => $expectedValue];
        $this->filters = [
            [
                'field' => $expectedField,
                'operator' => AttributeLevelOperators::EQUALS,
                'value' => $expectedValue
            ],
            [
                'field' => $expectedField,
                'operator' => AttributeLevelOperators::EQUALS,
                'value' => $expectedValue + 1
            ]
        ];

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenSingleInvalidFilter_GivenSingleValidFilter_ShouldReturnFalse()
    {
        $expectedField = 'foo';
        $expectedValue = 5;
        $this->objectToValidate = (object)[$expectedField => $expectedValue];
        $this->filters = [
            [
                'field' => $expectedField,
                'operator' => AttributeLevelOperators::EQUALS,
                'value' => $expectedValue + 1
            ],
            [
                'field' => $expectedField,
                'operator' => AttributeLevelOperators::EQUALS,
                'value' => $expectedValue
            ]
        ];

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }
    // endregion
}