<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Tests\Unit\Conditional\Query\NestedQueries;


use Cybersponse\MetadataBundle\Conditional\QueryMatcher;

class AndLogicTest extends NestedQueriesTestCase
{
    protected function setUp()
    {
        parent::setUp();
        $this->logic = QueryMatcher::LOGIC_AND;
    }

    public function test_GivenMatchingNestedQuery_ShouldReturnTrue()
    {
        $this->addMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenNonMatchingNestedQuery_ShouldReturnFalse()
    {
        $this->addNonMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenMatchingNestedQuery_GivenNonMatchingNestedQuery_ShouldReturnFalse()
    {
        $this->addMatchingNestedQuery();
        $this->addNonMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenNonMatchingNestedQuery_GivenMatchingNestedQuery_ShouldReturnFalse()
    {
        $this->addNonMatchingNestedQuery();
        $this->addMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenMatchingNested_GivenMatchingFilter_ShouldReturnTrue()
    {
        $this->addMatchingNestedQuery();
        $this->addMatchingFilter();

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenMatchingFilter_GivenMatchingNested_ShouldReturnTrue()
    {
        $this->addMatchingFilter();
        $this->addMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenMatchingNested_GivenNonMatchingFilter_ShouldReturnFalse()
    {
        $this->addMatchingNestedQuery();
        $this->addNonMatchingFilter();

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenNonMatchingFilter_GivenMatchingNested_ShouldReturnFalse()
    {
        $this->addNonMatchingFilter();
        $this->addMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenNonMatchingNested_GivenMatchingFilter_ShouldReturnFalse()
    {
        $this->addNonMatchingNestedQuery();
        $this->addMatchingFilter();

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenMatchingFilter_GivenNonMatchingNested_ShouldReturnFalse()
    {
        $this->addMatchingFilter();
        $this->addNonMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenNonMatchingNested_GivenNonMatchingFilter_ShouldReturnFalse()
    {
        $this->addNonMatchingNestedQuery();
        $this->addNonMatchingFilter();

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenNonMatchingFilter_GivenNonMatchingNested_ShouldReturnFalse()
    {
        $this->addNonMatchingFilter();
        $this->addNonMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }
}