<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Tests\Unit\Conditional\Query\NestedQueries;


use Cybersponse\QueryBundle\Constants\AttributeLevelOperators;
use Cybersponse\MetadataBundle\Conditional\QueryInterface;
use Cybersponse\MetadataBundle\Conditional\QueryMatcher;
use Cybersponse\MetadataBundle\Tests\Unit\Conditional\Query\QueryMatcherTestCase;

class NestedQueriesTestCase extends QueryMatcherTestCase
{
    const EXPECTED_FIELD = 'foo';
    const EXPECTED_VALUE = 10;
    const MATCHING_FILTER = [
        'field' => self::EXPECTED_FIELD,
        'operator' => AttributeLevelOperators::EQUALS,
        'value' => self::EXPECTED_VALUE
    ];
    const NON_MATCHING_FILTER = [
        'field' => self::EXPECTED_FIELD,
        'operator' => AttributeLevelOperators::EQUALS,
        'value' => self::EXPECTED_VALUE + 15
    ];

    /** @var string */
    protected $logic;

    /** @var array */
    protected $filters;

    protected function setUp()
    {
        parent::setUp();
        $this->filters = [];
    }

    protected function executeSystemUnderTest()
    {
        $this->objectToValidate = (object)[self::EXPECTED_FIELD => self::EXPECTED_VALUE];
        $this->query = $this->getMockBuilder(QueryInterface::class)->getMock();
        $this->query->method('getLogic')->willReturn($this->logic);
        $this->query->method('getFilters')->willReturn($this->filters);

        return parent::executeSystemUnderTest();
    }

    protected function tearDown()
    {
        parent::tearDown();
        unset($this->logic);
        unset($this->filters);
    }

    protected function addMatchingNestedQuery()
    {
        $matchingQuery = $this->getMockBuilder(QueryInterface::class)->getMock();
        $matchingQuery->method('getLogic')->willReturn(QueryMatcher::LOGIC_AND);
        $matchingQuery->method('getFilters')->willReturn([self::MATCHING_FILTER]);
        $this->filters[] = $matchingQuery;
    }

    protected function addNonMatchingNestedQuery()
    {
        $nonMatchingQuery = $this->getMockBuilder(QueryInterface::class)->getMock();
        $nonMatchingQuery->method('getLogic')->willReturn(QueryMatcher::LOGIC_AND);
        $nonMatchingQuery->method('getFilters')->willReturn([self::NON_MATCHING_FILTER]);
        $this->filters[] = $nonMatchingQuery;
    }

    protected function addMatchingFilter()
    {
        $this->filters[] = self::MATCHING_FILTER;
    }

    protected function addNonMatchingFilter()
    {
        $this->filters[] = self::NON_MATCHING_FILTER;
    }
}