<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Tests\Unit\Conditional\Query\NestedQueries;


use Cybersponse\MetadataBundle\Conditional\QueryInterface;
use Cybersponse\MetadataBundle\Conditional\QueryMatcher;

class OrLogicTest extends NestedQueriesTestCase
{
    protected function setUp()
    {
        parent::setUp();
        $this->logic = QueryMatcher::LOGIC_OR;
    }

    public function test_GivenMatchingSuperDuperDuperNestedQuery_ShouldReturnTrue()
    {
        // trust me. this is absolutely vital.
        $superDuperNestedMatchingQuery = $this->getMockBuilder(QueryInterface::class)->getMock();
        $superDuperNestedMatchingQuery->method('getLogic')->willReturn(QueryMatcher::LOGIC_AND);
        $superDuperNestedMatchingQuery->method('getFilters')->willReturn([self::MATCHING_FILTER]);
        $superDuperNestedMatchingQuery->expects($this->once())->method('getFilters');
        $superNestedMatchingQuery = $this->getMockBuilder(QueryInterface::class)->getMock();
        $superNestedMatchingQuery->method('getLogic')->willReturn(QueryMatcher::LOGIC_AND);
        $superNestedMatchingQuery->method('getFilters')->willReturn([$superDuperNestedMatchingQuery]);
        $superNestedMatchingQuery->expects($this->once())->method('getFilters');
        $nestedMatchingQuery = $this->getMockBuilder(QueryInterface::class)->getMock();
        $nestedMatchingQuery->method('getLogic')->willReturn(QueryMatcher::LOGIC_AND);
        $nestedMatchingQuery->method('getFilters')->willReturn([$superNestedMatchingQuery]);
        $nestedMatchingQuery->expects($this->once())->method('getFilters');
        $this->filters[] = $nestedMatchingQuery;

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenMatchingNestedQuery_ShouldReturnTrue()
    {
        $this->addMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenNonMatchingNestedQuery_ShouldReturnFalse()
    {
        $this->addNonMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenMatchingNestedQuery_GivenNonMatchingNestedQuery_ShouldReturnTrue()
    {
        $this->addMatchingNestedQuery();
        $this->addNonMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenNonMatchingNestedQuery_GivenMatchingNestedQuery_ShouldReturnTrue()
    {
        $this->addNonMatchingNestedQuery();
        $this->addMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenMatchingNested_GivenMatchingFilter_ShouldReturnTrue()
    {
        $this->addMatchingNestedQuery();
        $this->addMatchingFilter();

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenMatchingFilter_GivenMatchingNested_ShouldReturnTrue()
    {
        $this->addMatchingFilter();
        $this->addMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenMatchingNested_GivenNonMatchingFilter_ShouldReturnTrue()
    {
        $this->addMatchingNestedQuery();
        $this->addNonMatchingFilter();

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenNonMatchingFilter_GivenMatchingNested_ShouldReturnTrue()
    {
        $this->addNonMatchingFilter();
        $this->addMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenNonMatchingNested_GivenMatchingFilter_ShouldReturnTrue()
    {
        $this->addNonMatchingNestedQuery();
        $this->addMatchingFilter();

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenMatchingFilter_GivenNonMatchingNested_ShouldReturnTrue()
    {
        $this->addMatchingFilter();
        $this->addNonMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenNonMatchingNested_GivenNonMatchingFilter_ShouldReturnFalse()
    {
        $this->addNonMatchingNestedQuery();
        $this->addNonMatchingFilter();

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenNonMatchingFilter_GivenNonMatchingNested_ShouldReturnFalse()
    {
        $this->addNonMatchingFilter();
        $this->addNonMatchingNestedQuery();

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }
}