<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Tests\Unit\Conditional\Query\Operators;

use Cybersponse\QueryBundle\Constants\AttributeLevelOperators;

class FloatOperatorsTest extends OperatorsTestCase
{
    protected function setUp()
    {
        parent::setUp();
        $this->actualValue = 1.8;
    }

    // region Equal
    public function test_GivenEqualOperator_GivenMatchingValue_ShouldReturnTrue()
    {
        $this->operator = AttributeLevelOperators::EQUALS;
        $this->filterValue = $this->actualValue;

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenEqualOperator_GivenNonMatchingValue_ShouldReturnFalse()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::EQUALS;
        $this->filterValue = $this->actualValue + 1.5;

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }
    // endregion
    // region Not Equal
    public function test_GivenNotEqualOperator_GivenMatchingValue_ShouldReturnFalse()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::NOT_EQUALS;
        $this->filterValue = $this->actualValue;

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenNotEqualOperator_GivenNonMatchingValue_ShouldReturnTrue()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::NOT_EQUALS;
        $this->filterValue = $this->actualValue + 1.5;

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }
    // endregion
    // region Greater Than
    public function test_GivenGreaterThanOperator_GivenMatchingValue_ShouldReturnFalse()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::GREATER_THAN;
        $this->filterValue = $this->actualValue;

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenGreaterThanOperator_GivenSmallerValue_ShouldReturnTrue()
    {
        $this->operator = AttributeLevelOperators::GREATER_THAN;
        $this->filterValue = $this->actualValue - 1.5;

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenGreaterThanOperator_GivenLargerValue_ShouldReturnFalse()
    {
        $this->operator = AttributeLevelOperators::GREATER_THAN;
        $this->filterValue = $this->actualValue + 1.5;

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }
    // endregion
    // region Greater Than Equals
    public function test_GivenGreaterThanEqualsOperator_GivenMatchingValue_ShouldReturnTrue()
    {
        $this->operator = AttributeLevelOperators::GREATER_THAN_EQUALS;
        $this->filterValue = $this->actualValue;

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenGreaterThanEqualsOperator_GivenSmallerValue_ShouldReturnTrue()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::GREATER_THAN_EQUALS;
        $this->filterValue = $this->actualValue - 1.5;

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenGreaterThanEqualsOperator_GivenLargerValue_ShouldReturnFalse()
    {
        $this->operator = AttributeLevelOperators::GREATER_THAN_EQUALS;
        $this->filterValue = $this->actualValue + 1.5;

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }
    // endregion
    // region Less Than
    public function test_GivenLessThanOperator_GivenMatchingValue_ShouldReturnFalse()
    {
        $this->operator = AttributeLevelOperators::LESS_THAN;
        $this->filterValue = $this->actualValue;

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenLessThanOperator_GivenSmallerValue_ShouldReturnFalse()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::LESS_THAN;
        $this->filterValue = $this->actualValue - 1.5;

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenLessThanOperator_GivenLargerValue_ShouldReturnTrue()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::LESS_THAN;
        $this->filterValue = $this->actualValue + 1.5;

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }
    // endregion
    // region Less Than Equals
    public function test_GivenLessThanEqualsOperator_GivenMatchingValue_ShouldReturnTrue()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::LESS_THAN_EQUALS;
        $this->filterValue = $this->actualValue;

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenLessThanEqualsOperator_GivenSmallerValue_ShouldReturnFalse()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::LESS_THAN_EQUALS;
        $this->filterValue = $this->actualValue - 1.5;

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenLessThanEqualsOperator_GivenLargerValue_ShouldReturnTrue()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::LESS_THAN_EQUALS;
        $this->filterValue = $this->actualValue + 1.5;

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }
    // endregion
    // region Is Null
    public function test_GivenIsNullOperator_GivenTrueFilterValue_GivenNonNullActualValue_ShouldReturnFalse()
    {
        $this->operator = AttributeLevelOperators::IS_NULL;
        $this->filterValue = true;

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenIsNullOperator_GivenFalseFilterValue_GivenNonNullActualValue_ShouldReturnTrue()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::IS_NULL;
        $this->filterValue = false;

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }
    // endregion
}