<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Tests\Unit\Conditional\Query\Operators;

use Cybersponse\AuthorizationBundle\Entity\Team;
use Cybersponse\QueryBundle\Constants\AttributeLevelOperators;

class ObjectOperatorsTest extends OperatorsTestCase
{
    const EXPECTED_UUID = 'lololololololol';
    const EXPECTED_IRI = '/api/3/teams/' . self::EXPECTED_UUID;

    protected function setUp()
    {
        parent::setUp();
        $this->actualValue = new Team();
        $this->actualValue->setUuid(self::EXPECTED_UUID);

        $this->mockIriConverter->method('getIriFromItem')->willReturn(self::EXPECTED_IRI);
    }

    // region Equal
    public function test_GivenEqualOperator_GivenMatchingValue_ShouldReturnTrue()
    {
        $this->operator = AttributeLevelOperators::EQUALS;
        $this->filterValue = self::EXPECTED_IRI;

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenEqualOperator_GivenNonMatchingValue_ShouldReturnFalse()
    {
        $this->operator = AttributeLevelOperators::EQUALS;
        $this->filterValue = self::EXPECTED_IRI . 'JUST KIDDING';

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }
    // endregion
    // region Not Equal
    public function test_GivenNotEqualOperator_GivenMatchingValue_ShouldReturnFalse()
    {
        $this->operator = AttributeLevelOperators::NOT_EQUALS;
        $this->filterValue = self::EXPECTED_IRI;

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenNotEqualOperator_GivenNonMatchingValue_ShouldReturnTrue()
    {
        $this->operator = AttributeLevelOperators::NOT_EQUALS;
        $this->filterValue = self::EXPECTED_IRI . 'JUST KIDDING';

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }
    // endregion
    // region Is Null
    public function test_GivenIsNullOperator_GivenTrueFilterValue_GivenNonNullActualValue_ShouldReturnFalse()
    {
        $this->operator = AttributeLevelOperators::IS_NULL;
        $this->filterValue = true;

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenIsNullOperator_GivenFalseFilterValue_GivenNonNullActualValue_ShouldReturnTrue()
    {
        $this->operator = AttributeLevelOperators::IS_NULL;
        $this->filterValue = false;

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }
    // endregion
}