<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Tests\Unit\Conditional\Query\Operators;


use Cybersponse\QueryBundle\Constants\AttributeLevelOperators;
use Cybersponse\MetadataBundle\Conditional\QueryInterface;
use Cybersponse\MetadataBundle\Conditional\QueryMatcher;
use Cybersponse\MetadataBundle\Tests\Unit\Conditional\Query\QueryMatcherTestCase;

class OperatorsTestCase extends QueryMatcherTestCase
{
    /** @var string */
    protected $operator;

    /** @var mixed */
    protected $actualValue;

    /** @var mixed */
    protected $filterValue;

    protected function executeSystemUnderTest()
    {
        $expectedField = 'foo';
        $this->objectToValidate = (object)[$expectedField => $this->actualValue];

        $this->query = $this->getMockBuilder(QueryInterface::class)->getMock();
        $this->query->method('getLogic')->willReturn(QueryMatcher::LOGIC_AND);
        $this->query->method('getFilters')->willReturn([
            [
                'field' => $expectedField,
                'operator' => $this->operator,
                'value' => $this->filterValue
            ]
        ]);

        return parent::executeSystemUnderTest();
    }

    protected function tearDown()
    {
        parent::tearDown();
        unset($this->actualValue);
        unset($this->filterValue);
        unset($this->operator);
    }

    // region In
    public function test_GivenInOperator_GivenArrayWithActualValueInIt_ShouldReturnTrue()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::IN;
        $this->filterValue = [$this->actualValue];

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }

    public function test_GivenInOperator_GivenArrayWithoutActualValueInIt_ShouldReturnFalse()
    {
        $this->operator = AttributeLevelOperators::IN;
        $this->filterValue = [];

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }
    // endregion
    // region Not In
    public function test_GivenNotInOperator_GivenArrayWithActualValueInIt_ShouldReturnFalse()
    {
        $this->operator = AttributeLevelOperators::NIN;
        $this->filterValue = [$this->actualValue];

        $matches = $this->executeSystemUnderTest();

        $this->assertFalse($matches);
    }

    public function test_GivenNotInOperator_GivenArrayWithoutActualValueInIt_ShouldReturnTrue()
    {
        $this->operator = \Cybersponse\QueryBundle\Constants\AttributeLevelOperators::NIN;
        $this->filterValue = [];

        $matches = $this->executeSystemUnderTest();

        $this->assertTrue($matches);
    }
    // endregion
}