<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Tests\Unit\Validation\Constraint\Keyword;


use Cybersponse\MetadataBundle\Tests\Unit\Validation\MockConstraint;
use Cybersponse\MetadataBundle\Tests\Unit\Validation\ValidatorTestCase;
use Cybersponse\MetadataBundle\Validation\Keyword;
use Cybersponse\MetadataBundle\Validation\KeywordConstraintValidator;
use Symfony\Component\Validator\ExecutionContextInterface;

class KeywordTestCase extends ValidatorTestCase
{
    const RESERVED_KEYWORD = 'keyword';

    protected $reservedKeywords = [
        self::RESERVED_KEYWORD
    ];

    /** @var string */
    protected $keyword;

    protected function setUp()
    {
        $this->constraint = new Keyword();
        $this->validator = new KeywordConstraintValidator();
        $this->validator->setReservedKeywords($this->reservedKeywords);
        parent::setUp();
    }

    protected function tearDown()
    {
        unset($this->keyword);
        $this->keyword = null;
        parent::tearDown();
    }

    protected function executeSystemUnderTest()
    {
        $this->value = $this->keyword;

        parent::executeSystemUnderTest();
    }

    public function test_GivenNonKeywordConstraint_ShouldReturnZeroViolations()
    {
        $this->constraint = new MockConstraint();
        $this->keyword = null;
        $this->violationCount = 0;

        $this->executeSystemUnderTest();
    }

    public function test_GivenNonExecutionContext_ShouldReturnZeroViolations()
    {
        $this->mockExecutionContext = $this->getMockBuilder(ExecutionContextInterface::class)->getMock();
        $this->keyword = null;
        $this->violationCount = 0;

        $this->executeSystemUnderTest();
    }
}