<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Tests\Unit\Validation\Constraint\ModelMetadataCollection;


use Cybersponse\MetadataBundle\Document\AttributeMetadata;
use Cybersponse\MetadataBundle\Document\ModelMetadata;

class ValidMetadatasTest extends ModelMetadataCollectionTestCase
{
    public function test_GivenOnlyType_ShouldReturnZeroViolations()
    {
        $modelMetadata = new ModelMetadata();

        $this->modelMetadataList = [$modelMetadata];;

        $this->executeSystemUnderTest();
    }

    public function test_GivenNonRelationshipAttribute_ShouldReturnZeroViolations()
    {
        $attribute = new AttributeMetadata();
        $attributes = [$attribute];

        $modelMetadata = new ModelMetadata();
        $modelMetadata->setAttributes($attributes);

        $this->modelMetadataList = [$modelMetadata];

        $this->executeSystemUnderTest();
    }

    public function test_GivenRelationshipAttribute_GivenExistingCoreType_ShouldReturnZeroViolations()
    {
        $attribute = new AttributeMetadata();
        $attribute->setType(static::CORE_TYPE);
        $attributes = [$attribute];

        $modelMetadata = new ModelMetadata();
        $modelMetadata->setAttributes($attributes);

        $this->modelMetadataList = [$modelMetadata];

        $this->executeSystemUnderTest();
    }

    public function test_GivenRelationshipAttribute_GivenNonExistingCoreType_ShouldReturnOneViolation()
    {
        $attribute = new AttributeMetadata();
        $attribute->setType('non_existing');
        $attributes = [$attribute];

        $modelMetadata = new ModelMetadata();
        $modelMetadata->setAttributes($attributes);

        $this->violationCount = 1;

        $this->modelMetadataList = [$modelMetadata];

        $this->executeSystemUnderTest();
    }

    public function test_GivenRelationshipAttribute_GivenExistingCustomType_ShouldReturnZeroViolations()
    {
        $attribute = new AttributeMetadata();
        $attribute->setType(static::CUSTOM_TYPE);
        $attributes = [$attribute];

        $modelMetadata = new ModelMetadata();
        $modelMetadata->setType(static::CUSTOM_TYPE_2);
        $modelMetadata->setAttributes($attributes);

        $modelMetadata2 = new ModelMetadata();
        $modelMetadata2->setType(static::CUSTOM_TYPE);

        $this->modelMetadataList = [$modelMetadata, $modelMetadata2];

        $this->executeSystemUnderTest();
    }

    public function test_GivenRelationshipAttribute_GivenExistingCustomType_GivenNonExistingInversedField_ShouldReturnOneViolation()
    {
        $attribute = new AttributeMetadata();
        $attribute->setType(static::CUSTOM_TYPE);
        $attribute->setInversedField('non-existing');
        $attributes = [$attribute];

        $modelMetadata = new ModelMetadata();
        $modelMetadata->setType(static::CUSTOM_TYPE_2);
        $modelMetadata->setAttributes($attributes);

        $modelMetadata2 = new ModelMetadata();
        $modelMetadata2->setType(static::CUSTOM_TYPE);

        $this->violationCount = 1;

        $this->modelMetadataList = [$modelMetadata, $modelMetadata2];

        $this->executeSystemUnderTest();
    }

    public function test_GivenExistingCoreType_ShouldReturnOneViolation()
    {
        $modelMetadata = new ModelMetadata();
        $modelMetadata->setType(static::CORE_TYPE);

        $this->violationCount = 1;

        $this->modelMetadataList = [$modelMetadata];

        $this->executeSystemUnderTest();
    }

    public function test_GivenRelationshipAttribute_GivenExistingCustomType_GivenExistingInversedField_ShouldReturnZeroViolations()
    {
        $attribute = new AttributeMetadata();
        $attribute->setType(static::CUSTOM_TYPE);
        $attribute->setInversedField('existingField');

        $modelMetadata = new ModelMetadata();
        $modelMetadata->setType(static::CUSTOM_TYPE_2);
        $modelMetadata->setAttributes([$attribute]);

        $modelMetadata2 = new ModelMetadata();
        $modelMetadata2->setType(static::CUSTOM_TYPE);

        $attribute2 = new AttributeMetadata();
        $attribute2->setName('existingField');
        $modelMetadata->setAttributes([$attribute2]);

        $this->modelMetadataList = [$modelMetadata, $modelMetadata2];

        $this->executeSystemUnderTest();
    }
}