<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Tests\Unit\Validation\Constraint\UniqueCollection;


use Cybersponse\MetadataBundle\Tests\Unit\Validation\MockConstraint;
use Cybersponse\MetadataBundle\Tests\Unit\Validation\ValidatorTestCase;
use Cybersponse\MetadataBundle\Validation\UniqueCollectionItem;
use Cybersponse\MetadataBundle\Validation\UniqueCollectionItemValidator;
use Symfony\Component\Validator\ExecutionContextInterface;

class UniqueCollectionTestCase extends ValidatorTestCase
{
    /** @var \stdClass[] */
    protected $objectList;

    protected $uniqueFieldName = 'test';

    protected function setUp()
    {
        $this->constraint = new UniqueCollectionItem();
        $this->validator = new UniqueCollectionItemValidator();
        parent::setUp();
    }

    protected function tearDown()
    {
        unset($this->objectList);
        $this->objectList = [];
        parent::tearDown();
    }

    protected function executeSystemUnderTest()
    {
        if ($this->constraint instanceof UniqueCollectionItem) {
            $this->constraint->fields = [$this->uniqueFieldName];
        }
        $this->value = $this->objectList;

        parent::executeSystemUnderTest();
    }

    public function test_GivenNonUniqueCollectionItemConstraint_ShouldReturnZeroViolations()
    {
        $this->constraint = new MockConstraint();
        $this->objectList = [];
        $this->violationCount = 0;

        $this->executeSystemUnderTest();
    }

    public function test_GivenNonExecutionContext_ShouldReturnZeroViolations()
    {
        $this->mockExecutionContext = $this->getMockBuilder(ExecutionContextInterface::class)->getMock();
        $this->objectList = [];
        $this->violationCount = 0;

        $this->executeSystemUnderTest();
    }
}

class MockObject
{
    /** @var string */
    public $test;

    /**
     * MockObject constructor.
     * @param string $test
     */
    public function __construct($test)
    {
        $this->test = $test;
    }
}