<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Tests\Unit\Validation;


use Symfony\Component\PropertyAccess\PropertyAccessor;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Context\ExecutionContextInterface;
use Symfony\Component\Validator\Violation\ConstraintViolationBuilderInterface;

class ValidatorTestCase extends \PHPUnit_Framework_TestCase
{
    /** @var ExecutionContextInterface|\PHPUnit_Framework_MockObject_MockObject */
    protected $mockExecutionContext;

    /** @var ConstraintViolationBuilderInterface|\PHPUnit_Framework_MockObject_MockObject */
    protected $constraintViolationBuilderMock;

    /** @var int */
    protected $violationCount = 0;

    /** @var ConstraintValidator */
    protected $validator;

    /** @var mixed */
    protected $value;

    /** @var Constraint */
    protected $constraint;

    /** @var PropertyAccessor */
    protected $propertyAccessor;

    protected function setUp()
    {
        parent::setUp();

        $this->propertyAccessor = new PropertyAccessor();
        $this->mockExecutionContext = $this->getMockBuilder(ExecutionContextInterface::class)->getMock();
        $this->constraintViolationBuilderMock = $this->getMockBuilder(ConstraintViolationBuilderInterface::class)->getMock();

        $this->constraintViolationBuilderMock->method('atPath')
            ->willReturn($this->constraintViolationBuilderMock);

        $this->constraintViolationBuilderMock->method('setInvalidValue')
            ->willReturn($this->constraintViolationBuilderMock);

        $this->constraintViolationBuilderMock->method('setParameter')
            ->willReturn($this->constraintViolationBuilderMock);

        $this->constraintViolationBuilderMock->method('addViolation');

        if (method_exists($this->validator, 'setPropertyAccessor')) {
            $this->validator->setPropertyAccessor($this->propertyAccessor);
        }
    }

    protected function executeSystemUnderTest()
    {
        $this->mockExecutionContext->expects($this->exactly($this->violationCount))
            ->method('buildViolation')
            ->willReturn($this->constraintViolationBuilderMock);

        $this->validator->initialize($this->mockExecutionContext);
        $this->validator->validate($this->value, $this->constraint);
    }

    protected function tearDown()
    {
        parent::tearDown();
        unset($this->value);
        unset($this->validator);
        unset($this->constraint);
    }
}

class MockConstraint extends Constraint
{

}