<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

/**
 * Created by PhpStorm.
 * User: deepak
 * Date: 27/10/17
 * Time: 12:22 PM
 */

namespace Cybersponse\MetadataBundle\Validation;

use Doctrine\ORM\EntityManager;
use Symfony\Component\Validator\Constraint;
use Cybersponse\MetadataBundle\Entity\ModelMetadata;
use Symfony\Component\Validator\ConstraintValidator;

class CompatibilityConstraintValidator extends ConstraintValidator
{
    protected $documentManager;

    public function __construct(EntityManager $documentManager)
    {
        $this->documentManager = $documentManager;
    }

    private $compatibility = array("integer" => array("string", "float"));

    private function isTypecastingAllowed($old_value, $new_value)
    {
        $allowedConversion = [];
        if (array_key_exists($old_value, $this->compatibility))
            $allowedConversion = $this->compatibility[$old_value];
        $isAllowed = in_array($new_value, $allowedConversion);
        return $isAllowed;
    }

    /**
     * Checks if the passed value is valid.
     *
     * @param mixed $object The value that should be validated
     * @param Keyword $constraint The constraint for the validation
     */
    public function validate($value, Constraint $constraint)
    {

        $modelType = $value->getType();
        $attributeList = [];
        $modelType = $value->getType();
        foreach ($value->getAttributes()->getValues() as $valueAttribute) {
            if (in_array(\strtolower($valueAttribute->getName()), $attributeList)) {
                $this->context->buildViolation($constraint->duplicateFieldViolation)
                    ->atPath('name')
                    ->setInvalidValue($valueAttribute->getName())
                    ->setParameter('{{ value }}', $valueAttribute->getName())
                    ->setInvalidValue($modelType)
                    ->setParameter('{{ model }}', $modelType)
                    ->addViolation();
                break;
            } else {
                array_push($attributeList, \strtolower($valueAttribute->getName()));
            }
        }

        $genericTemplateRepository = $this->documentManager->getRepository(ModelMetadata::class);
        $qb = $genericTemplateRepository->createQueryBuilder('o');
        if ($value->getUuid() != null) {
            $qb->where('o.uuid = :uuid');
            $qb->setParameter('uuid', $value->getUuid());
            $modelMetadataObject = $qb->getQuery()->execute();
            if($modelMetadataObject){
                $entityDocument = $modelMetadataObject[0];
            }else{
                return true;
            }
            foreach ($entityDocument->getAttributes()->getValues() as $attribute) {
                foreach ($value->getAttributes()->getValues() as $valueAttribute) {
                    if ($attribute->getUuid() == $valueAttribute->getUuid()) { // compare only same attributes
                        if ($attribute->getType() != $valueAttribute->getType() && !$this->isTypecastingAllowed($attribute->getType(), $valueAttribute->getType()) && $attribute->getType()) {
                            $this->buildViolation($valueAttribute, $constraint, $entityDocument, $attribute);
                            break;
                        } else {
                            //special case for datetime to integer conversion where both data types are 'integers' but formtypes are 'datetime'
                            if ($attribute->getFormType() == 'datetime' && $valueAttribute->getFormType() != $attribute->getFormType()) {
                                $this->buildViolation($valueAttribute, $constraint, $entityDocument, $attribute);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * @param string $value
     * @param Constraint $constraint
     * @param $entityDocument
     * @param $attribute
     */
    public function buildViolation($value, Constraint $constraint, $entityDocument, $attribute)
    {
        $this->context->buildViolation($constraint::MESSAGE)
            ->setParameter('{{ new_type }}', $this->formatValue($value->getType()))
            ->setParameter('{{ name }}', $this->formatValue($value->getName()))
            ->setParameter('{{ modelType }}', $this->formatValue($entityDocument->getType()))
            ->setParameter('{{ old_type }}', $this->formatValue($attribute->getType()))
            ->setParameter('{{ old_form_type }}', $this->formatValue($attribute->getFormType()))
            ->setParameter('{{ new_form_type }}', $this->formatValue($value->getFormType()))
            ->addViolation();
    }
}
