<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Validation;


use Cybersponse\MetadataBundle\Conditional\ConditionSetEvaluator;
use Symfony\Component\PropertyAccess\PropertyAccessorInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\Constraints\IsTrue;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\Constraints\NotNull;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\ConstraintViolation;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

/**
 * Class ConditionSetConstraintValidator
 *
 * @package Cybersponse\MetadataBundle\Validation
 */
class ConditionSetConstraintValidator extends ConstraintValidator
{
    /** @var ConditionSetEvaluator */
    protected $conditionSetEvaluator;

    /** @var PropertyAccessorInterface */
    protected $propertyAccessor;

    /**
     * Checks if the passed value is valid.
     *
     * @param mixed $object The value that should be validated
     * @param ConditionSetConstraint $constraint The constraint for the validation
     */
    public function validate($object, Constraint $constraint)
    {
        $resultSet = $this->conditionSetEvaluator->evaluate($object, $constraint);

        /** @var ExecutionContextInterface $context */
        $context = $this->context;
        $validator = $context->getValidator();

        foreach ($resultSet as $fieldName => $attributes) {
            foreach ($attributes as $attributeName => $attributeValue) {

                $requiredConstraints = [];

                switch ($attributeName) {
                    case 'required':
                        if ($attributeValue === true) {
                            $requiredConstraints = [new NotNull(), new NotBlank()];
                        }
                        break;
                    case 'notNull':
                        if ($attributeValue === true) {
                            $requiredConstraints = [new NotNull()];
                        }
                        break;
                    case 'isTrue':
                        if ($attributeValue === true) {
                            $requiredConstraints = [new IsTrue()];
                        }
                        break;
                }

                if (!empty($requiredConstraints)) {
                    $value = $this->propertyAccessor->getValue($object, $fieldName);

                    /** @var ConstraintViolation[] $constraintViolations */
                    $constraintViolations = $validator->validate($value, $requiredConstraints);
                    foreach ($constraintViolations as $constraintViolation) {
                        /** @var ConstraintViolation[] $existingViolations */
                        $existingViolations = $context->getViolations();

                        foreach ($existingViolations as $existingViolation) {
                            if ($existingViolation->getPropertyPath() == $fieldName
                                && $existingViolation->getMessage() == $constraintViolation->getMessage()
                                && $existingViolation->getInvalidValue() == $constraintViolation->getInvalidValue()) {
                                // Violation already exists, skip.
                                continue 2;
                            }
                        }

                        //Wrap violation with new context
                        $context
                            ->buildViolation($constraintViolation->getMessage())
                            ->setCause($constraintViolation->getCause())
                            ->atPath($fieldName)
                            ->setCode($constraintViolation->getCode())
                            ->setInvalidValue($constraintViolation->getInvalidValue())
                            ->setParameters($constraintViolation->getParameters())
                            ->setPlural($constraintViolation->getPlural())
                            ->addViolation();
                    }
                }
            }
        }
    }

    /**
     * @param ConditionSetEvaluator $conditionSetEvaluator
     */
    public function setConditionSetEvaluator($conditionSetEvaluator)
    {
        $this->conditionSetEvaluator = $conditionSetEvaluator;
    }

    /**
     * @param PropertyAccessorInterface $propertyAccessor
     */
    public function setPropertyAccessor($propertyAccessor)
    {
        $this->propertyAccessor = $propertyAccessor;
    }
}
