<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Validation;


use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

/**
 * Class IsJsonTypeConstraintValidator
 *
 * @package Cybersponse\MetadataBundle\Validation
 */
class IsJsonTypeConstraintValidator extends ConstraintValidator
{
    /**
     * @var array
     *
     * @todo add more support for other json primitives
     */
    protected $jsonPrimitives = [
        'array',
        'object'
    ];

    /**
     * Checks if the passed value is valid.
     *
     * @param mixed $object The value that should be validated
     * @param Keyword $constraint The constraint for the validation
     */
    public function validate($value, Constraint $constraint)
    {
        if (!$constraint instanceof IsJsonType) {
            return;
        }

        if (!$this->context instanceof ExecutionContextInterface) {
            return;
        }

        // Do not validate null values
        if (is_null($value)) {
            return;
        }

        $type = $constraint->type;

        if (!in_array($type, $this->jsonPrimitives)) {
            $this->context->buildViolation($constraint::INVALID_JSON_TYPE_MESSAGE)
                ->setParameter('{{ value }}', $this->formatValue($type))
                ->addViolation();
            return;
        }

        $methodName = 'validate' . ucfirst($type);

        if (!$this->{$methodName}($value)) {
            $this->context->buildViolation($constraint::INVALID_TYPE_MESSAGE)
                ->setParameter('{{ value }}', $this->formatValue($value))
                ->setParameter('{{ type }}', $this->formatValue($type))
                ->addViolation();
            return;
        }
    }

    /**
     * @param $value
     *
     * @return bool
     */
    protected function validateArray($value)
    {
        if (is_array($value)) {
            if (empty($value)) {
                return true;
            }
            if (array_key_exists(0, $value)) {
                return true;
            }
        }

        return false;
    }

    /**
     * @param $value
     *
     * @return bool
     */
    protected function validateObject($value)
    {
        if (is_array($value)) {
            if (empty($value)) {
                return true;
            }
            if (!array_key_exists(0, $value)) {
                return true;
            }
        }

        return false;
    }
}
