<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Validation;


use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

/**
 * Class KeywordConstraintValidator
 *
 * @package Cybersponse\MetadataBundle\Validation
 */
class KeywordConstraintValidator extends ConstraintValidator
{
    /** @var array */
    protected $reservedKeywords;

    /**
     * Checks if the passed value is valid.
     *
     * @param mixed $object The value that should be validated
     * @param Keyword $constraint The constraint for the validation
     */
    public function validate($value, Constraint $constraint)
    {

        if (gettype($value) == "object") {
            // Force UI to Change and send request per attribute.
            foreach($value as $item) {
                if (in_array(strtolower($item->getName()), $this->reservedKeywords)) {
                    $this->context->buildViolation($constraint::MESSAGE)
                    ->setParameter('{{ value }}', $this->formatValue($item->getName()))
                    ->addViolation();
                    break;
                }
            }
            return;
        }

        if (!$constraint instanceof Keyword) {
            return;
        }

        if (!$this->context instanceof ExecutionContextInterface) {
            return;
        }

        // Do not validate null values
        if (is_null($value)) {
            return;
        }

        if (!is_string($value)) {
            $this->context->buildViolation($constraint::NON_STRING_MESSAGE)
                ->setParameter('{{ value }}', $this->formatValue($value))
                ->addViolation();
            return;
        }

        if (in_array(strtolower($value), $this->reservedKeywords)) {
            $this->context->buildViolation($constraint::MESSAGE)
                ->setParameter('{{ value }}', $this->formatValue($value))
                ->addViolation();
        }
    }

    /**
     * @param array $reservedKeywords
     */
    public function setReservedKeywords($reservedKeywords)
    {
        $this->reservedKeywords = $reservedKeywords;
    }
}
