<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\MetadataBundle\Validation;


use Symfony\Component\PropertyAccess\PropertyAccessorInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

/**
 * Class UniqueCollectionItemValidator
 *
 * Validates a list of objects with the given keys, and ensures that their values are unique.
 *
 * @package Cybersponse\MetadataBundle\Validation
 */
class UniqueCollectionItemValidator extends ConstraintValidator
{
    /** @var PropertyAccessorInterface */
    protected $propertyAccessor;

    /**
     * @inheritdoc
     */
    public function validate($value, Constraint $constraint)
    {
        if (!$constraint instanceof UniqueCollectionItem) {
            return;
        }

        if (!$this->context instanceof ExecutionContextInterface) {
            return;
        }

        if (empty($value) || !is_array($value)) {
            return;
        }

        //Build field list with keys and empty arrays to start
        $uniqueFieldList = array_map(function () {
            return [];
        }, array_flip($constraint->fields));

        foreach ($uniqueFieldList as $uniqueFieldName => &$uniqueValues) {
            foreach ($value as $arrayItem) {
                if (!is_object($arrayItem)) {
                    continue;
                }

                $itemValue = $this->propertyAccessor->getValue($arrayItem, $uniqueFieldName);

                if (in_array($itemValue, $uniqueValues)) {
                    $this->context->buildViolation($constraint->message)
                        ->atPath($uniqueFieldName)
                        ->setInvalidValue($itemValue)
                        ->addViolation();
                } else {
                    $uniqueValues[] = $itemValue;
                }
            }
        }
    }

    /**
     * @param PropertyAccessorInterface $propertyAccessor
     */
    public function setPropertyAccessor($propertyAccessor)
    {
        $this->propertyAccessor = $propertyAccessor;
    }
}
