<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

/**
 * Created by PhpStorm.
 * User: deepak
 * Date: 27/10/17
 * Time: 12:22 PM
 */

namespace Cybersponse\MetadataBundle\Validation;

use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

class UniqueConstraintValidator extends ConstraintValidator
{

    const PEER_REPLICABLE_MODULES = ['tenant'];

    const TRACKABLE_MODULE = ['createDate', 'modifyUser', 'modifyDate', 'createUser'];

    /**
     * Checks if the passed value is valid.
     *
     * @param mixed $object The value that should be validated
     * @param Keyword $constraint The constraint for the validation
     */
    public function validate($value, Constraint $constraint)
    {
        if ($value->getUuid() != null) {
            $uniqeConstraint = $value->getUniqueConstraint();
            if(count($uniqeConstraint)){
                $uniqeConstraint = $uniqeConstraint[0];
                $constraintName = array_keys($uniqeConstraint)[0];
                $columns = $uniqeConstraint[$constraintName]['columns'];
            }
           if ($columns){
               foreach($columns as $column) {
                    if ((in_array($column, self::PEER_REPLICABLE_MODULES) && !$value->isPeerReplicable()) || (in_array($column, self::TRACKABLE_MODULE) && !$value->isTrackable())){
                        $this->buildViolation($column, $constraint);
                        continue;
                    } else if ((in_array($column, self::PEER_REPLICABLE_MODULES) && $value->isPeerReplicable()) || (in_array($column, self::TRACKABLE_MODULE) && $value->isTrackable())) {
                        continue;
                    }
                    $matched = false;
                    foreach ($value->getAttributes()->getValues() as $valueAttribute) {
                        if($valueAttribute->getName() == $column) {
                            $matched = true;
                            break;
                        }
                    }
                    if(!$matched){
                        $this->buildViolation($column, $constraint);
                    }
                }
            }
        }
    }

    /**
     * @param string $value
     * @param Constraint $constraint
     * @param $entityDocument
     * @param $attribute
     */
    public function buildViolation($columnName, Constraint $constraint)
    {
        $this->context->buildViolation($constraint::MESSAGE)
            ->setParameter('{{ field_name }}', $this->formatValue($columnName))
            ->addViolation();
    }
}
