<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ProxyBundle\Controller;

use GuzzleHttp\Client;
use Symfony\Component\Yaml\Yaml;
use Symfony\Component\Serializer\Serializer;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\HttpFoundation\Request;
use GuzzleHttp\Exception\BadResponseException;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Cybersponse\ProxyBundle\GuzzleHttp\ProxyClient;
use Cybersponse\ProxyBundle\Provider\SamlActorProvider;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;

class PublicActionController extends Controller
{
    const PROXY_UUID = 'c90e8c1b-cu11-as33-3sk-717199a2e182';
    /** @var ProxyClient */
    protected $proxyClient;

    /** @var DasClient */
    protected $dasClient;

    /** @var $integrationsClient */
    protected $integrationClient;

    /**
     * @param $integrationsClient
     */
    public function setIntegrationClient($integrationClient)
    {
        $this->integrationClient = $integrationClient;
    }

    /**
     * @return integrationsClient
     */
    public function getIntegrationClient()
    {
        return $this->integrationClient;
    }

    /**
     * @param $dasClient
     */
    public function setDasClient($dasClient)
    {
        $this->dasClient = $dasClient;
    }

    /**
     * @return DasClient
     */
    public function getDasClient()
    {
        return $this->dasClient;
    }

    /**
     * @param ProxyClient $proxyClient
     */
    public function setProxyClient($proxyClient)
    {
        $this->proxyClient = $proxyClient;
    }

    /**
     * @return ProxyClient
     */
    public function getProxyClient()
    {
        return $this->proxyClient;
    }

    public function sendMail($templateId, $dataToReplace, $mailDetails, $integrationClient, $defaultMessage)
    {
        $resource = $this->get('api.resource_collection')->getResourceForShortName('EmailTemplate');
        $item = $this->get('api.data_provider')->getItem($resource, $templateId, true);
        if ($item == null) {
            $email_options = Yaml::parse(file_get_contents('/opt/cyops/configs/cyops-api/mailtemplate.yml'));
            $item = $email_options[$defaultMessage];
        } else {
            $context['ignore_authorization'] = true;
            $context['ignore_field_authorization'] = true;
            $item = $this->get('serializer')->normalize($item, 'json-ld', $context);
        }

        $headers = array(
            "Content-type" => "application/json"
        );
        if (strpos($item['content'], 'https://{{@Server_fqhn}}')) {
            $item['content'] = str_replace('https://{{@Server_fqhn}}', '{{@Server_fqhn}}', $item['content']);
        }
        if (strpos($item['content'], 'https://{{@cyops_hostname}}')) {
            $item['content'] = str_replace('https://{{@cyops_hostname}}', '{{@cyops_hostname}}', $item['content']);
        }
        foreach ($dataToReplace as $key => $value) {
            $item['content'] = str_replace($key, $value, $item['content']);
        }
        $smtpObject = '{
                            "connector": "smtp",
                            "operation": "send_richtext_email",
                            "params": {
                                "ccc": "",
                                "bcc": "",
                                "content_type":"text/html"
                            }
                        }';
        $jsonDecodedSmtp = json_decode($smtpObject, true);
        $jsonDecodedSmtp['params']['subject'] = $item['subject'];
        $jsonDecodedSmtp['params']['content'] = $item['content'];
        $jsonDecodedSmtp['params']['to'] = $mailDetails;
        $smtpUri = $this->container->getParameter('integrations.uri');
        $smtpUri = $smtpUri . '/execute/?format=json';
        $smtpRequest = new GuzzleRequest('post', $smtpUri, $headers, json_encode($jsonDecodedSmtp));
        try {
            $integrationClient->send($smtpRequest);
        } catch (\Exception $exception) {
            error_log($exception->getMessage());
            $errVar = json_decode($exception->getResponse()->getBody()->read(2048), true);
            error_log($errVar);
            return $errVar;
        }
        return true;
    }

    public function routeForgotPasswordAction(Request $request)
    {
        $this->setProxyClient($this->get('cybersponse.proxy.client.workflow'));
        $this->setDasClient($this->get('cybersponse.proxy.client.das'));
        $this->setIntegrationClient($this->get('cybersponse.proxy.client.integrations'));
        $objectData = json_decode($request->getContent(), true);
        $token = $objectData['captchResponse'];
        $success = $this->verifyReCaptcha($token);
        if (!$success) {
            return Response::create('Invalid Captcha, please verify', Response::HTTP_FORBIDDEN);
        }
        $loginId = $objectData['loginId'];
        $userId = $this->getUserId($loginId);

        if (empty($userId)) {
            return Response::create("", Response::HTTP_OK);
        }

        $email = $this->getEmail($userId);
        $data = array(
            'uuid' => $userId,
            'newUser' => false,
            'email' => $email
        );

        $dasClient = $this->getDasClient();
        $headers = array(
            "Content-type" => "application/json"
        );

        $dasUri = $this->container->getParameter('das.uri');
        $keyFile = file_get_contents('/opt/cyops-api/app/keys/daspublic.key', true);
        $idKey = base64_encode($keyFile);
        $data['idKey'] = $idKey;
        $dasRequests = new GuzzleRequest('post', $dasUri . '/token?uuid=' . $data['uuid'], $headers, json_encode($data));
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $dasResponseData = $dasResponse->getBody()->getContents();
            $jsonDecodeResponse = json_decode($dasResponseData, true);
            $resCode = $dasResponse->getStatusCode();
        } catch (\Exception $error) {
            error_log($error->getMessage());
            $errVar = json_decode($error->getResponse()->getBody()->read(2048), true);
            $errMsg['message'] = $errVar['Error'];
            $errCode = $error->getCode();
            return JsonResponse::create($errMsg, $errCode);
        }

        if ($resCode < 300) {
            $responseCode = 400;
            $integrationClient = $this->getIntegrationClient();
            $dataToReplace = [];
            $dataToReplace['{{vars.token}}'] = $jsonDecodeResponse['token'];
            $dataToReplace['{{vars.request.baseUri}}'] = $request->getScheme() . "://" . $request->getHttpHost();
            $dataToReplace['{{@Server_fqhn}}'] = $request->getScheme() . "://" . $request->getHttpHost();
            $dataToReplace['{{@cyops_hostname}}'] = $request->getScheme() . "://" . $request->getHttpHost();
            $defaultMessage = 'forgotpassword';
            $mailResponse = $this->sendMail('b9265a6e-7c0d-420a-a8e9-e77f01baf7cb', $dataToReplace, $data['email'], $integrationClient, $defaultMessage);
            if (!(is_array($mailResponse)) && $mailResponse === true) {
                $mailResponse = "Reset token sent to user";
                $responseCode = 200;
            }
            return JsonResponse::create($mailResponse, $responseCode);
        }
    }

    /**
     * @param $userId
     * @return string
     */
    private function getEmail($userId)
    {
        /** @var SamlActorProvider $samlProvider */
        $actorProvider = $this->get('cybersponse.saml_proxy.provider.samlactor');
        $actor = $actorProvider->getActor($userId);
        $email = $actor->getEmail();
        return $email;
    }


    /**
     * @param $token
     * @return bool
     */
    private function verifyReCaptcha($token)
    {
        $googleAPI = $this->container->getParameter('googleAPI');
        $googleSecrete = $this->container->getParameter('secretKey');

        $post_data =array(
            'form_params' => [
            'secret' => $googleSecrete,
            'response' => $token]);

        $client = new \GuzzleHttp\Client();
        $responseObject = $client->request('POST', $googleAPI, $post_data);
        if ($responseObject->getStatusCode() >= 200 && $responseObject->getStatusCode() < 300) {
            $result = json_decode($responseObject->getBody());
            return $result->success;
        } else {
            return false;
        }
    }

    /**
     * @param $routeIdentifier
     * @return array
     */
    private function findHandlerMapForRoute($routeIdentifier)
    {
        $handlerMap = $this->container->getParameter('cybersponse.proxy.handler.map');

        if (!array_key_exists($routeIdentifier, $handlerMap)) {
            throw new NotFoundHttpException();
        }
        return $handlerMap;
    }

    /**
     * @param $routeIdentifier
     * @return ProxyClient
     */
    public function findHandlerForRoute($routeIdentifier)
    {
        $handler = $this->findHandlerMapForRoute($routeIdentifier);

        return $this->container->get($handler[$routeIdentifier]['handlerName']);
    }

    /**
     * @param String $loginId
     * @return String
     */
    private function getUserId($loginId)
    {
        $client = $this->findHandlerForRoute('auth');
        $body = ["json" => ["loginid" => $loginId]];
        $userRequestUri = sprintf("%s/%s", $client->getTargetUri(), 'users');
        $response = null;
        try {
            $response = $client->request('POST', $userRequestUri, $body);
            $respBody = json_decode($response->getBody()->getContents(), true);
            return $respBody['uuid'];
        } catch (BadResponseException $exception) {
            return "";
        }
    }
}
