<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ProxyBundle\DependencyInjection;

use Symfony\Component\Config\Definition\Builder\TreeBuilder;
use Symfony\Component\Config\Definition\ConfigurationInterface;

/**
 * This is the class that validates and merges configuration from your app/config files
 *
 * To learn more see {@link http://symfony.com/doc/current/cookbook/bundles/extension.html#cookbook-bundles-extension-config-class}
 */
class Configuration implements ConfigurationInterface
{
    const QUERY_STRING = 'query_string';
    const URL = 'url';
    const BODY = 'body';

    /**
     * {@inheritdoc}
     */
    public function getConfigTreeBuilder()
    {
        $treeBuilder = new TreeBuilder();

        $rootNode = $treeBuilder->root('cybersponse_proxy');

        $rootNode
            ->children()
            ->arrayNode('handlers')
            ->useAttributeAsKey('name')
            ->isRequired()
            ->requiresAtLeastOneElement()
            ->prototype('array')
            ->children()
            ->scalarNode('routePrefix')->defaultValue('/')->end()
            ->scalarNode('targetUri')->isRequired()->end()
            ->arrayNode('authorization')
            ->requiresAtLeastOneElement()
            ->useAttributeAsKey('permission')
            ->prototype('array')
            ->children()
            ->scalarNode('action')->isRequired()->end()
            ->scalarNode('route')->isRequired()->end()
            ->scalarNode('operator')->defaultValue('OR')->end()
            ->booleanNode('ignoreAuthorization')->defaultFalse()->end()
            ->arrayNode('uuidLocation')
            ->children()
            ->arrayNode(self::QUERY_STRING)
            ->children()
            ->scalarNode('key')->isRequired()->end()
            ->end()
            ->end()
            ->arrayNode(self::URL)
            ->children()
            ->scalarNode('matchNumber')->isRequired()->end()
            ->end()
            ->end()
            ->arrayNode(self::BODY)
            ->children()
            ->scalarNode('path')->isRequired()->end()
            ->end()
            ->end()
            ->end()
            ->end()
            ->arrayNode('permissions')->defaultValue([])
            ->prototype('variable')->end()
            ->end()
            ->arrayNode('additionalVoters')->defaultValue([])
            ->prototype('variable')->end()
            ->end()
            ->arrayNode('params')->defaultValue([])
            ->prototype('variable')->end()
            ->end()
            ->end()
            ->end()
            ->end()
            ->arrayNode('authentication')
            ->children()
            ->scalarNode('type')->isRequired()->end()
            ->arrayNode('arguments')->isRequired()->prototype('variable')->end()
            ->end()
            ->end()
            ->end()
            ->end()
            ->end()
            ->end();

        // Here you should define the parameters that are allowed to
        // configure your bundle. See the documentation linked above for
        // more information on that topic.

        return $treeBuilder;
    }
}
