<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ProxyBundle\DependencyInjection;


use Cybersponse\ProxyBundle\GuzzleHttp\HmacClient;
use Cybersponse\ProxyBundle\GuzzleHttp\ProxyClient;
use GuzzleHttp\Client;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Definition;
use Symfony\Component\DependencyInjection\Reference;

class ProxyClientCompilerPass implements CompilerPassInterface
{

    /**
     * You can modify the container here before it is dumped to PHP code.
     *
     * @param ContainerBuilder $container
     *
     * @api
     */
    public function process(ContainerBuilder $container)
    {
        // todo get services for supported handlers by tag
        $supportedHandlers = [
            'hmac' => HmacClient::class
        ];

        $handlersConfig = $container->getParameter('cybersponse.proxy.handlers');

        $handlerMap = [];

        foreach ($handlersConfig as $handlerName => $handlerConfig) {
            $handlerType = $handlerConfig['authentication']['type'];
            $handlerClass = Client::class;
            $handlerName = "cybersponse.proxy.client.{$handlerName}";

            if (array_key_exists($handlerType, $supportedHandlers)) {
                $handlerClass = $supportedHandlers[$handlerType];
            }

            $clientDefinition = new Definition($handlerClass);

            if (is_subclass_of($handlerClass, ProxyClient::class)) {
                $clientDefinition->addMethodCall('setRoutePrefix', [$handlerConfig['routePrefix']]);
                $clientDefinition->addMethodCall('setTargetUri', [$handlerConfig['targetUri']]);
                $clientDefinition->addMethodCall('setMonolog', [new Reference('logger')]);
                $clientDefinition->addMethodCall('setDebug', [$container->getParameter('kernel.debug')]);
                $clientDefinition->addMethodCall('setArguments', [$handlerConfig['authentication']['arguments']]);
                $handlerMap[$handlerConfig['routePrefix']] = [
                    'permissions' => $handlerConfig['authorization'],
                    'handlerName' => $handlerName
                ];
            }

            $container->setDefinition($handlerName, $clientDefinition);
        }

        $container->setParameter('cybersponse.proxy.handler.map', $handlerMap);
    }
}
