<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ProxyBundle\GuzzleHttp;


use Cybersponse\ProxyBundle\Provider\FingerprintProvider;
use Monolog\Logger;
use Psr\Http\Message\RequestInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class HmacClient extends ProxyClient
{
    /** @var string */
    protected $privateKey;

    /** @var string */
    protected $publicKey;

    /** @var Logger */
    protected $monolog;

    /** @var boolean */
    protected $debug;

    /**
     * @param string $privateKey
     */
    public function setPrivateKey($privateKey)
    {
        $this->privateKey = $privateKey;
    }

    /**
     * @param string $publicKey
     */
    public function setPublicKey($publicKey)
    {
        $this->publicKey = $publicKey;
    }

    /**
     * @param Logger $monolog
     */
    public function setMonolog($monolog)
    {
        $this->monolog = $monolog;
    }

    /**
     * @param boolean $debug
     */
    public function setDebug($debug)
    {
        $this->debug = $debug;
    }

    /**
     * @return boolean
     */
    public function isDebug()
    {
        return $this->debug;
    }

    public function sendAsync(RequestInterface $request, array $options = [])
    {
        if (!array_key_exists('verify', $options)) {
            $options['verify'] = !$this->isDebug();
        }

        $this->monolog->addInfo(sprintf(
            "HmacClient: Making request to %s...",
            $request->getUri()
        ));
        $options = $this->hmacify($request, $options);
        return parent::sendAsync($request, $options);
    }

    /**
     * @param RequestInterface $request
     * @param array $options
     * @return array
     */
    private function hmacify(RequestInterface $request, array $options)
    {
        $uri = (string)$request->getUri();
        $content = $request->getBody()->getContents();
        $method = $request->getMethod();

        $algorithm = 'sha256';
        $currentTimestamp = (new \DateTime(null, new \DateTimeZone('UTC')))->format('Y-m-d H:i:s');

        if (empty($content)) {
            $content = $this->publicKey;
        }

        $authorization = FingerprintProvider::createFingerprint(
            $this->publicKey,
            $this->privateKey,
            $algorithm,
            $method,
            $currentTimestamp,
            $uri,
            $content
        );

        $headers = $request->getHeaders();

        if ($headers['authorization']) {
          $options['headers']['forwarded-authorization'] = $headers['authorization'];
        }

        $options['headers']['Authorization'] = 'CS ' . $authorization;
        if (isset($_SERVER['REMOTE_ADDR'])) {
            $options['headers']['X-Forwarded-For'] = $_SERVER['REMOTE_ADDR'];
        }
        return $options;
    }

    /**
     * @param array $arguments
     * @return mixed
     *
     * @throws \Exception
     */
    public function setArguments($arguments)
    {
        $this->publicKey = @file_get_contents($arguments['publicKeyLocation']);
        if ($this->publicKey === false) {
            throw new \Exception(sprintf('The proxy system for "%s" is not set up. Please contact an Administrator.', $this->routePrefix));
        }

        $this->privateKey = @file_get_contents($arguments['privateKeyLocation']);
        if ($this->privateKey === false) {
            throw new \Exception(sprintf('The proxy system for "%s" is not set up. Please contact an Administrator.', $this->routePrefix));
        }
    }
}
