<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ProxyBundle\Tests\Unit\GuzzleHttp;


use Cybersponse\ProxyBundle\GuzzleHttp\HmacClient;
use GuzzleHttp\Psr7\Request;

class HmacClientTest extends \PHPUnit_Framework_TestCase
{
    /** @var HmacClient|\PHPUnit_Framework_MockObject_MockObject */
    private $mockHmacClient;

    public function setUp()
    {
        $this->mockHmacClient = $this->getMockBuilder(HmacClient::class)
            ->disableOriginalConstructor()
            ->setMethods([])
            ->getMock();
    }

    public function test_Hmacify_GivenHeaders_WillRetainHeaders()
    {
        $request = new Request('GET', 'http://localhost');

        $options = [
            'headers' => [
                'MyHeader' => 'MyValue'
            ]
        ];

        $headers = $this->callHmacifyWithRequestAndOptions($request, $options)['headers'];

        $this->assertEquals('MyValue', $headers['MyHeader']);
    }

    public function test_Hmacify_GivenHeaders_WillReturnAuthorizationKey()
    {
        $request = new Request('GET', 'http://localhost');

        $headers = $this->callHmacifyWithRequestAndOptions($request, [])['headers'];

        $this->assertArrayHasKey('Authorization', $headers);
    }

    public function test_Hmacify_GivenPublicKey_DecodedAuthHeaderWillContainPublicKeyOnIndexTwo()
    {
        $request = new Request('GET', 'http://localhost');

        $expectedPublicKey = 'expectedPublicKey';

        $headers = $this->callHmacifyWithRequestAndOptions($request, [], $expectedPublicKey)['headers'];

        $authorization = $headers['Authorization'];

        $decodedAuth = base64_decode(ltrim($authorization, 'CS '));

        $authParts = explode(';', $decodedAuth);

        $actualPublicKey = $authParts[2];

        $this->assertEquals($expectedPublicKey, $actualPublicKey);
    }

    /**
     * @return \ReflectionClass
     */
    private function getReflectionClassForHttpClient()
    {
        return new \ReflectionClass(HmacClient::class);
    }

    /**
     * @param $request
     * @param $options
     * @return array
     */
    private function callHmacifyWithRequestAndOptions($request, $options, $publicKey = null)
    {
        $reflectionClass = $this->getReflectionClassForHttpClient();

        if (!is_null($publicKey)) {
            $reflectionClass->getMethod('setPublicKey')->invoke($this->mockHmacClient, $publicKey);
        }

        $reflectionMethod = $reflectionClass->getMethod('hmacify');

        $reflectionMethod->setAccessible(true);
        $options = $reflectionMethod->invoke($this->mockHmacClient, $request, $options);

        return $options;
    }
}
