<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\QueryBundle\Controller;


use Cybersponse\CoreBundle\Controller\AuthorizationResourceController;
use Cybersponse\QueryBundle\Provider\QueryProvider;
use Cybersponse\QueryBundle\Query\ExpressionBuilder;
use Dunglas\ApiBundle\Api\IriConverterInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class QueryResourceController extends AuthorizationResourceController
{

    const ASSIGNEE_KEY = 'assignee';
    /**
     * @param Request $request
     * @return \Dunglas\ApiBundle\JsonLd\Response
     */
    public function cgetAction(Request $request)
    {
        $this->appendAdditionalFilters();
        return parent::cgetAction($request);
    }

    /**
     * @TODO Data drive this so we can enforce this across any DataProvider queries
     */
    protected function appendAdditionalFilters()
    {
        /** @var TokenStorageInterface $tokenStorage */
        $tokenStorage = $this->container->get('security.token_storage');
        /** @var IriConverterInterface $iriConverter */
        $iriConverter = $this->container->get('api.iri_converter');
        /** @var QueryProvider $queryProvider */
        $queryProvider = $this->container->get('cybersponse.query.provider.query_provider');

        if ($token = $tokenStorage->getToken()) {
            if ($user = $token->getUser()) {
                $currentUserIri = $iriConverter->getIriFromItem($user);

                $accessibleQueryFilter =
                    ExpressionBuilder::logicOr(
                        ExpressionBuilder::field(self::ASSIGNEE_KEY)->equals(''),
                        ExpressionBuilder::field(self::ASSIGNEE_KEY)->isNull(true),
                        ExpressionBuilder::field(self::ASSIGNEE_KEY)->equals($currentUserIri)
                    );

                $queryProvider->setAdditionalFilters($accessibleQueryFilter->toArray());
            }
        }
    }
}
