<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\QueryBundle\Controller;


use Dunglas\ApiBundle\Event\Events;
use Dunglas\ApiBundle\JsonLd\Response;
use Cybersponse\CoreBundle\Api\ApiEvents;
use Symfony\Component\HttpFoundation\Request;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Cybersponse\CoreBundle\Controller\AuthorizationResourceController;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;

class UserQueryResourceController extends AuthorizationResourceController
{
    protected function denyAccessUnlessGranted($resource, $actions, $object = null)
    {
        // Skip authorization. The DB queries are limited to the user's subset of queries, so authorization is enforced during data access.
    }

    /**
     * @inheritdoc
     */
    public function cpostAction(Request $request)
    {
        $request_body = json_decode($request->getContent());
        $resource = $this->getResource($request);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_READ);
        $this->get('event_dispatcher')->dispatch(ApiEvents::PRE_API_CREATE, new DataEvent($resource, null));

        try {
            $object = $this->get('serializer')->deserialize(
                json_encode($request_body),
                $resource->getEntityClass(),
                'json-ld',
                array_merge(
                    [
                        'ignore_authorization' => true,
                        'ignore_field_authorization' => true
                    ],
                    $resource->getDenormalizationContext()
                )
            );
        } catch (ExceptionInterface $e) {
            throw new DeserializationException($e->getMessage(), $e->getCode(), $e);
        } catch (SerializerExceptionInterface $e) {
            throw new DeserializationException($e->getMessage(), $e->getCode(), $e);
        }
        $this->get('event_dispatcher')->dispatch(Events::PRE_CREATE_VALIDATION, new DataEvent($resource, $object));
        $violations = $this->get('validator')->validate($object, null, $resource->getValidationGroups());
        if (0 === count($violations)) {
            // Validation succeed
            $this->get('event_dispatcher')->dispatch(Events::PRE_CREATE, new DataEvent($resource, $object));
            /** @var Serializer $serializer */
            $serializer = $this->get('serializer');
            $serializer->normalize($object, 'json-ld', $resource->getNormalizationContext() + ['ignore_authorization' => true]);
            return $this->getSuccessResponse($resource, $object, Response::HTTP_CREATED);
        }
        return $this->getErrorResponse($violations);
    }

}
