<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\QueryBundle\Entity;


use Cybersponse\MetadataBundle\Entity\ModelMetadata;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize;
use Doctrine\ORM\Mapping\UniqueConstraint;

use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Table(name="queries", uniqueConstraints={@UniqueConstraint(name="query_unique", columns={"name", "models", "assignee"})})
 * @ORM\Entity
 * @ORM\InheritanceType("SINGLE_TABLE")
 * @ORM\DiscriminatorMap({"UserQuery"="UserQuery", "SystemQuery"="SystemQuery"})
 * @ORM\DiscriminatorColumn(name="type", type="string", length=20)
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="queries")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="application")
 */
abstract class Query
{
    const KEYWORD_AGGREGATES = 'aggregates';
    const KEYWORD_SORT = 'sort';
    const KEYWORD_LOGIC = 'logic';
    const KEYWORD_FILTERS = 'filters';

    /**
     * @ORM\Id
     * @ORM\Column(name="uuid", type="string", length=150, nullable=false)
     */
    protected $uuid;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`name`", type="string", length=255, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Name")
     */
    protected $name;

    /**
     * @var \Cybersponse\MetadataBundle\Entity\ModelMetadata
     *
     * @Doctrine\ORM\Mapping\ManyToOne(targetEntity="Cybersponse\MetadataBundle\Entity\ModelMetadata", cascade={"persist"})
     * @Doctrine\ORM\Mapping\JoinColumns({
     *   @Doctrine\ORM\Mapping\JoinColumn(name="models", referencedColumnName="uuid", onDelete="SET NULL")
     * })
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="model_metadatas")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Models")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="picklist")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     */
    protected $models;

    /**
     * @var integer
     *
     * @Doctrine\ORM\Mapping\Column(name="`resultcacheseconds`", type="integer", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Result Cache Seconds")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="integer")
     */
    protected $resultCacheSeconds=0;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`query`", type="json_array", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Query")
     */
    protected $query;

    /**
     * @var string
     *
     * @ORM\Column(name="`assignee`", type="string", nullable=true, unique=false)
     */
    protected $assignee='';

    /**
     * @return string
     */
    public function getAssignee()
    {
        return $this->assignee;
    }

    /**
     * @param string $assignee
     */
    public function setAssignee($assignee)
    {
        $this->assignee = $assignee;
    }

    /**
     * @return string
     */
    public function getUuid()
    {
        return $this->uuid;
    }

    /**
     * @param string $uuid
     */
    public function setUuid($uuid)
    {
        $this->uuid = $uuid;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @return \Cybersponse\MetadataBundle\Entity\ModelMetadata[]
     */
    public function getModels()
    {
        return $this->models;
    }

    /**
     * @param \Cybersponse\MetadataBundle\Entity\ModelMetadata[] $models
     */
    public function setModels($models)
    {
        $this->models = $models;
    }

    /**
     * @return int
     */
    public function getResultCacheSeconds()
    {
        return $this->resultCacheSeconds;
    }

    /**
     * @param int $resultCacheSeconds
     */
    public function setResultCacheSeconds($resultCacheSeconds)
    {
        $this->resultCacheSeconds = $resultCacheSeconds;
    }

    /**
     * @return array
     */
    public function getQuery()
    {
        return $this->query;
    }

    /**
     * @param array $query
     */
    public function setQuery($query)
    {
        $this->query = $query;
    }
}
