<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\QueryBundle\Filter;


use Cybersponse\QueryBundle\Entity\Query;
use Cybersponse\QueryBundle\Constants\AggregateOperators;
use Cybersponse\QueryBundle\Constants\AttributeLevelOperators;
use Cybersponse\QueryBundle\Constants\ModelLevelOperators;

class QueryObjectFactory
{
    /**
     * @param string $queryString
     * @return array
     */
    public static function fromQueryString($queryString)
    {
        $mixedFilters = FilterArrayFactory::fromQueryString($queryString);
        return self::fromMixedFilters($mixedFilters);
    }

    /**
     * @param array $queryParts
     * @return array
     */
    public static function fromQueryParts($queryParts)
    {
        $mixedFilters = FilterArrayFactory::fromQueryParts($queryParts);
        return self::fromMixedFilters($mixedFilters);
    }

    /**
     * @param array $mixedFilters
     * @return array
     */
    public static function fromMixedFilters($mixedFilters)
    {
        $attributeFilters = [];
        $orderings = [];
        $aggregateFilters = [];

        foreach ($mixedFilters as $filter) {
            $operator = $filter['operator'];
            if (array_key_exists($operator, AttributeLevelOperators::ALL_OPERATORS)) {
                $attributeFilters[] = $filter;
            } else if ($operator === ModelLevelOperators::ORDER_BY) {
                $orderings[] = [
                    'field' => $filter['field'],
                    'direction' => $filter['value']
                ];
            } else if (array_key_exists($operator, AggregateOperators::ALL_OPERATORS)) {
                $aggregateFilters[] = $filter;
            }
        }

        return self::buildQueryObject($attributeFilters, $orderings, $aggregateFilters);
    }

    /**
     * @param array $attributeFilters
     * @param array $orderings
     * @param array $aggregateFilters
     * @return array
     */
    public static function buildQueryObject($attributeFilters, $orderings = [], $aggregateFilters = [])
    {
        $query = [
            Query::KEYWORD_LOGIC => 'AND',
            Query::KEYWORD_FILTERS => $attributeFilters
        ];

        if ($orderings) {
            $query[Query::KEYWORD_SORT] = $orderings;
        }

        if ($aggregateFilters) {
            $query[Query::KEYWORD_AGGREGATES] = $aggregateFilters;
        }

        return $query;
    }
}
