<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\QueryBundle\ORM;


use Cybersponse\QueryBundle\Provider\QueryProvider;
use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\ORM\QueryBuilder;
use Dunglas\ApiBundle\Api\IriConverterInterface;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Dunglas\ApiBundle\Doctrine\Orm\Filter\AbstractFilter;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\PropertyAccess\PropertyAccessorInterface;

class VenomQueryFilter extends AbstractFilter
{
    /** @var IriConverterInterface */
    private $iriConverter;

    /** @var PropertyAccessorInterface */
    private $propertyAccessor;

    /** @var FilterQueryBuilder */
    private $filterQueryBuilder;

    /** @var QueryProvider */
    protected $queryProvider;

    /**
     * @param ManagerRegistry $managerRegistry
     * @param IriConverterInterface $iriConverter
     * @param PropertyAccessorInterface $propertyAccessor
     * @param null|array $properties Null to allow filtering on all properties with the exact strategy
     *                                              or a map of property name with strategy.
     */
    public function __construct(
        ManagerRegistry $managerRegistry,
        IriConverterInterface $iriConverter,
        PropertyAccessorInterface $propertyAccessor,
        array $properties = null
    ) {
        parent::__construct($managerRegistry, $properties);

        $this->iriConverter = $iriConverter;
        $this->propertyAccessor = $propertyAccessor;
    }

    /**
     * @param QueryProvider $queryProvider
     */
    public function setQueryProvider($queryProvider)
    {
        $this->queryProvider = $queryProvider;
    }

    /**
     * @param FilterQueryBuilder $filterQueryBuilder
     */
    public function setFilterQueryBuilder(FilterQueryBuilder $filterQueryBuilder)
    {
        $this->filterQueryBuilder = $filterQueryBuilder;
    }

    /**
     * {@inheritdoc}
     */
    public function getDescription(ResourceInterface $resource)
    {
        return [];
    }

    /**
     * {@inheritdoc}
     */
    public function apply(ResourceInterface $resource, QueryBuilder $queryBuilder, Request $request)
    {
        $query = $this->queryProvider->getQueryObject();
        if (!(isset($query['logic']) && isset($query['filters']))) {
            $query['logic'] = "AND";
            $query['filters'] = [];
        }
        $this->filterQueryBuilder->setQueryBuilder($queryBuilder);
        $this->filterQueryBuilder->fromQueryObject($query);
    }
}
