<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\QueryBundle\ORM;


use Cybersponse\MetadataBundle\Generation\Metadata\ModelAttributeMetadata;
use Cybersponse\QueryBundle\Constants\AttributeLevelOperators;
use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\ORM\QueryBuilder;
use Dunglas\ApiBundle\Api\IriConverterInterface;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Dunglas\ApiBundle\Doctrine\Orm\Filter\AbstractFilter;
use Dunglas\ApiBundle\Mapping\ClassMetadataFactory;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\PropertyAccess\PropertyAccessorInterface;


class VenomSearchFilter extends AbstractFilter
{
    /** @var ClassMetadataFactory */
    protected $classMetadataFactory;
    /** @var IriConverterInterface */
    private $iriConverter;

    /** @var PropertyAccessorInterface */
    private $propertyAccessor;

    /**
     * @param ManagerRegistry $managerRegistry
     * @param IriConverterInterface $iriConverter
     * @param PropertyAccessorInterface $propertyAccessor
     * @param null|array $properties Null to allow filtering on all properties with the exact strategy
     *                                              or a map of property name with strategy.
     */
    public function __construct(
        ManagerRegistry $managerRegistry,
        IriConverterInterface $iriConverter,
        PropertyAccessorInterface $propertyAccessor,
        array $properties = null
    ) {
        parent::__construct($managerRegistry, $properties);

        $this->iriConverter = $iriConverter;
        $this->propertyAccessor = $propertyAccessor;
    }

    public function getDescription(ResourceInterface $resource)
    {
        return [];
    }

    /**
     * TODO: Move this logic to the QueryObjectFactory
     * {@inheritdoc}
     */
    public function apply(ResourceInterface $resource, QueryBuilder $queryBuilder, Request $request)
    {
        $searchTerm = $this->extractProperties($request);
        if (empty($searchTerm)) {
            return;
        }

        $searchableFields = $this->getSearchableFields($resource);
        if (empty($searchableFields)) {
            //Invalidate the query to ensure no results
            $queryBuilder->andWhere($queryBuilder->expr()->eq('333', '443'));
        }

        $searchTerm = '%' . strtolower($searchTerm) . '%';
        $searchExpressions = [];

        $doctrineClassMetadata = $queryBuilder->getEntityManager()->getClassMetadata($resource->getEntityClass());

        foreach ($searchableFields as $count => $field) {
            $searchParam = ':searchParam' . $count;
            $queryBuilder->setParameter($searchParam, $searchTerm);

            if ($doctrineClassMetadata->hasAssociation($field)) {
                continue;
            }

            $searchExpressions[] = $queryBuilder->expr()->like('TEXT_LOWER(o.' . $field . ')', $searchParam);
        }

        $searchExpression = call_user_func_array([$queryBuilder->expr(), 'orX'], $searchExpressions);
        $queryBuilder->andWhere($searchExpression);
    }

    /**
     * @param ClassMetadataFactory $classMetadataFactory
     */
    public function setClassMetadataFactory($classMetadataFactory)
    {
        $this->classMetadataFactory = $classMetadataFactory;
    }

    /**
     * {@inheritdoc}
     */
    protected function extractProperties(Request $request)
    {
        $searchKey = sprintf('%s%s', AttributeLevelOperators::OPERATOR_TOKEN, AttributeLevelOperators::SEARCH);
        return $request->query->get($searchKey, null);
    }

    /**
     * @param ResourceInterface $resource
     * @return array
     */
    protected function getSearchableFields(ResourceInterface $resource)
    {
        $classMetadata = $this->classMetadataFactory->getMetadataFor(
            $resource->getEntityClass(),
            $resource->getNormalizationGroups(),
            $resource->getDenormalizationGroups(),
            $resource->getValidationGroups()
        );

        $searchableFields = [];

        /** @var ModelAttributeMetadata $attributeMetadata */
        foreach ($classMetadata->getAttributes() as $attributeMetadata) {
            $isScalar = true;
            if (isset($attributeMetadata->getTypes()[0])) {
                $type = $attributeMetadata->getTypes()[0];
                $isScalar = $type->getClass() === null;
            }

            if ($attributeMetadata->isSearchable() && $isScalar) {
                $searchableFields[] = $attributeMetadata->getName();
            }
        }
        return $searchableFields;
    }
}
