<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\QueryBundle\Provider;


use Dunglas\ApiBundle\Api\ResourceInterface;
use Dunglas\ApiBundle\Model\DataProviderChain;
use Dunglas\ApiBundle\Model\DataProviderInterface;
use Symfony\Component\HttpFoundation\Request;

class AggregateDataProviderChain extends DataProviderChain implements AggregateDataProviderInterface
{
    /**
     * @var DataProviderInterface[]
     */
    protected $dataProviders;

    /**
     * @param DataProviderInterface[] $dataProviders
     */
    public function __construct(array $dataProviders)
    {
        $this->dataProviders = $dataProviders;
        parent::__construct($dataProviders);
    }

    /**
     * @inheritdoc
     */
    public function getAggregate(ResourceInterface $resource, Request $request)
    {
        foreach ($this->dataProviders as $dataProvider) {
            if ($dataProvider instanceof AggregateDataProviderInterface) {
                if ($dataProvider->supports($resource)) {
                    if ($result = $dataProvider->getAggregate($resource, $request)) {
                        return $result;
                    }
                }
            }
        }
    }

    /**
     * {@inheritdoc}
     */
    public function getItemWithWhere(ResourceInterface $resource, $filters, $fetchData = false)
    {
        foreach ($this->dataProviders as $dataProvider) {
            if ($dataProvider instanceof AggregateDataProviderInterface) {
                if ($dataProvider->supports($resource) && $result = $dataProvider->getItemWithWhere($resource, $filters, $fetchData)) {
                    return $result;
                }
            }
        }
    }
}
