<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\QueryBundle\Provider;


use Cybersponse\QueryBundle\Filter\QueryObjectFactory;
use Cybersponse\QueryBundle\Query\ExpressionBuilder;
use Symfony\Component\HttpFoundation\RequestStack;

class QueryProvider
{
    /** @var RequestStack */
    protected $requestStack;

    /** @var array */
    protected $initialQuery;

    /** @var array */
    protected $additionalFilters;

    /** @var array */
    protected $query;

    /**
     * QueryProvider constructor.
     * @param RequestStack $requestStack
     */
    public function __construct(RequestStack $requestStack)
    {
        $this->requestStack = $requestStack;
    }

    /**
     * @param array $initialQuery
     */
    public function setInitialQuery($initialQuery)
    {
        $this->initialQuery = $initialQuery;
    }

    /**
     * @param array $additionalFilters
     */
    public function setAdditionalFilters($additionalFilters)
    {
        $this->additionalFilters = $additionalFilters;
    }

    /**
     * @return array
     */
    public function getQueryObject()
    {
        if (isset($this->query)) {
            return $this->query;
        }

        if (isset($this->initialQuery)) {
            $this->query = $this->appendAdditionalFilters($this->initialQuery);
            return $this->query;
        }

        $request = $this->requestStack->getCurrentRequest();
        $queryString = $request->getQueryString();

        $this->query = $this->appendAdditionalFilters(QueryObjectFactory::fromQueryString($queryString));
        return $this->query;
    }

    /**
     * @param array $baseQuery
     * @return array
     */
    protected function appendAdditionalFilters($baseQuery)
    {
        if (!$this->additionalFilters) {
            return $baseQuery;
        }

        $newBaseQuery = $baseQuery; // clones array
        $newBaseQuery['logic'] = 'AND';
        $newBaseQuery['filters'] = [
            $baseQuery,
            $this->additionalFilters
        ];

        return $newBaseQuery;
    }
}
