<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\QueryBundle\Query;


use Cybersponse\QueryBundle\Constants\AttributeLevelOperators;

class FieldExpressionBuilder
{
    /** @var string */
    protected $fieldName;

    /**
     * @param string $fieldName
     */
    public function __construct($fieldName)
    {
        $this->fieldName = $fieldName;
    }

    /**
     * @param mixed $operator
     * @param mixed $value
     * @return FieldExpression
     */
    protected function buildFieldExpression($operator, $value)
    {
        return new FieldExpression($this->fieldName, $operator, $value);
    }

    /**
     * @param mixed $value
     * @return FieldExpression
     */
    public function equals($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::EQUALS, $value);
    }

    /**
     * @param mixed $value
     * @return FieldExpression
     */
    public function notEquals($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::NOT_EQUALS, $value);
    }

    /**
     * @param mixed $value
     * @return FieldExpression
     */
    public function lessThan($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::LESS_THAN, $value);
    }

    /**
     * @param mixed $value
     * @return FieldExpression
     */
    public function lessThanOrEqualTo($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::LESS_THAN_EQUALS, $value);
    }

    /**
     * @param mixed $value
     * @return FieldExpression
     */
    public function greaterThan($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::GREATER_THAN, $value);
    }

    /**
     * @param mixed $value
     * @return FieldExpression
     */
    public function greaterThanOrEqualTo($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::GREATER_THAN_EQUALS, $value);
    }

    /**
     * @param mixed $value
     * @return FieldExpression
     */
    public function exists($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::EXISTS, $value);
    }

    /**
     * @param mixed $value
     * @return FieldExpression
     */
    public function like($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::LIKE, $value);
    }

    /**
     * @param mixed $value
     * @return FieldExpression
     */
    public function contains($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::CONTAINS, $value);
    }

    /**
     * @param mixed $value
     * @return FieldExpression
     */
    public function in($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::IN, $value);
    }

    /**
     * @param mixed $value
     * @return FieldExpression
     */
    public function notIn($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::NIN, $value);
    }

    /**
     * @param boolean $value
     * @return FieldExpression
     */
    public function isNull($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::IS_NULL, $value);
    }

    /**
     * @param string $value
     * @return FieldExpression
     */
    public function search($value)
    {
        return $this->buildFieldExpression(AttributeLevelOperators::SEARCH, $value);
    }
}
