Query Bundle
======

## Contents
* [API Routes](#api-routes)
  * [Apply Ad Hoc Query](#apply-ad-hoc-query)
  * [Apply Persisted Query](#apply-persisted-query)
* [Query Objects](#query-objects)
  * [Filters](#filters)
  * [Logic](#logic)
  * [Nested Logic](#nested-logic)
  * [Sort](#sort)
  * [Aggregation](#aggregation)
  * [Associations](#associations)
  * [Database Feature Support](#database-feature-support)

# API Routes
## Apply Ad Hoc Query
```
POST /api/query/{collection}
BODY:
{ QUERY OBJECT }
```

* The `collection` parameter matches the collection route in the typical CRUD API (/api/3/{collection})

##### Example
```
POST /api/query/incidents
BODY:
{
  "logic": "AND",
  "filters": [
    {
      "field": "status.itemValue",
      "operator": "eq",
      "value": "Open"
    }
  ]
}
```

## Apply Persisted Query
```
GET /api/query/{collection}/{queryId}
```

* The `collection` parameter matches the collection route in the typical CRUD API (/api/3/{collection})
* The `queryId` parameter matches a Query Object UUID stored in `/api/3/query_objects`

##### Example
```
GET /api/query/incidents/2e77a714-f0c1-45ca-bd49-b71efbd9328c
```

# Query Objects

## Filters

Coming soon...

## Logic
### AND
A collection of filters can be applied in conjunction using `"logic": "AND"`
##### Example
Query all records where `record.assignedToUser !== null && record.status === "Open"`
```JSON
{
  "logic": "AND",
  "filters": [
    {
      "field": "assignedToUser",
      "operator": "isnull",
      "value": false
    },
    {
      "field": "status",
      "operator": "eq",
      "value": "Open"
    }
  ]
}
```

### OR
A collection of filters can be applied in disjunction using `"logic": "OR"`
##### Example
Query all records where `record.status === "Open" || record.status === "Pending"`
```JSON
{
  "logic": "OR",
  "filters": [
    {
      "field": "status",
      "operator": "eq",
      "value": "Open"
    },
    {
      "field": "status",
      "operator": "eq",
      "value": "Pending"
    }
  ]
}
```

## Nested Logic
Nesting logic filters allows you to build queries with nested logic like `conditionX && (conditionY || conditionZ)`
##### Example
Query all records where `record.assignedToUser !== null && (record.status === "Open" || record.status === "Pending")`
```JSON
{
  "logic": "AND",
  "filters": [
    {
      "field": "assignedToUser",
      "operator": "isnull",
      "value": false
    },
    {
      "logic": "OR",
      "filters": [
        {
          "field": "status",
          "operator": "eq",
          "value": "Open"
        },
        {
          "field": "status",
          "operator": "eq",
          "value": "Pending"
        }
      ]
    }
  ]
}
```

## Sort
Records can be sorted by field/direction using the `"sort": []` list. This list
contains objects with `"field"` and `"direction"` keys. The `"field"` key
identifies which field (or association) to sort by. The `"direction"` key
identifies whether to sort ascending (`"ASC"`) or descending (`"DESC"`). Order
is maintained when applying these sorts.
```JSON
{
  "logic": "AND",
  "filters": [],
  "sort": [
    {
      "field": "createDate",
      "direction": "DESC"
    }
  ]
}
```

## Aggregation
Records can be aggregated using the `"aggregates": []` list. This list contains
objects with `"operator"`, `"field"`, and `"alias"` keys. The `"operator"` key
identifies which aggregate operator to apply to the specified field. The
`"field"` key identifies the field to apply it to. The `"alias"` key identifies
what the returned field looks like.
### Supported Operators
* `fields`
* `select`
* `count`
* `countdistinct`
* `groupby`
* `distinct`
* `sum`
* `max`
* `min`
* `avg`

##### Example
Count all records in each status grouping.
```JSON
{
  "logic": "AND",
  "filters": [],
  "aggregates": [
    {
      "operator": "groupby",
      "field": "status",
      "alias": "status"
    },
    {
      "operator": "countdistinct",
      "field": "*",
      "alias": "total"
    }
  ]
}
```

## Associations
Wherever a `"field"` key is specified in a query object, you can use
dot-notation to query against an associated entity using the same operators that
are available for the root record.
##### Example
Query all records assigned to a user named Jeff.
```JSON
{
  "logic": "AND",
  "filters": [
    {
      "field": "assignedToUser.firstname",
      "operator": "eq",
      "value": "Jeff"
    }
  ]
}
```

## Database Feature Support
| Feature | PostgreSQL | MongoDB |
| --- | --- | --- |
| Filters | ✓ | ✓ |
| Logic | ✓ | ✓ |
| Nested Logic | ✓ | ✓ |
| Sort | ✓ | ✓ |
| Aggregation | ✓ | ✗ |
| Associations | ✓ | `eq` operator only |
