<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\QueryBundle\Tests\Unit\Filter\FilterArrayFactory;


use Cybersponse\QueryBundle\Filter\FilterArrayFactory;

class FromQueryStringMethodTest extends \PHPUnit_Framework_TestCase
{
    /** @var string */
    protected $queryString;

    protected function executeSystemUnderTest()
    {
        return FilterArrayFactory::fromQueryString($this->queryString);
    }

    protected function tearDown()
    {
        unset($this->queryParts);
        parent::tearDown();
    }

    /**
     * QueryString: ?field=value
     *
     * Filter:
     * [
     *   {
     *     field: 'field',
     *     operator: 'eq',
     *     value: 'value'
     *   }
     * ]
     */
    public function test_FromRawMethod_GivenSimpleQueryString_ShouldReturnFilterWithOperatorKey()
    {
        $this->queryString = 'field=value';
        $filters = $this->executeSystemUnderTest();
        $this->assertArrayHasKey('operator', $filters[0]);
    }

    /**
     * QueryString: ?field=value
     *
     * Filter:
     * [
     *   {
     *     field: 'field',
     *     operator: 'eq',
     *     value: 'value'
     *   }
     * ]
     */
    public function test_FromRawMethod_GivenSimpleQueryString_ShouldReturnFilterWithFieldKey()
    {
        $this->queryString = 'field=value';
        $filters = $this->executeSystemUnderTest();
        $this->assertArrayHasKey('field', $filters[0]);
    }

    /**
     * QueryString: ?field1=value1&field2=value2
     *
     * Filter:
     * [
     *   {
     *     field: 'field1',
     *     operator: 'eq',
     *     value: 'value1'
     *   },
     *   {
     *     field: 'field2',
     *     operator: 'eq',
     *     value: 'value2'
     *   }
     * ]
     */
    public function test_FromRawMethod_GivenTwoUniqueFields_ShouldReturnTwoFilters()
    {
        $this->queryString = 'field1=value1&field2=value2';
        $filters = $this->executeSystemUnderTest();
        $this->assertCount(2, $filters);
    }
}
