<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */
namespace Cybersponse\SearchBundle\Controller;

use Cybersponse\AuthorizationBundle\Permissions\PermissionResolver;
use Doctrine\Common\Inflector\Inflector;
use Dunglas\ApiBundle\JsonLd\Response;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Cybersponse\MetadataBundle\Entity\StagingModelMetadata;
use Cybersponse\UtilsBundle\Utils;
use Doctrine\Common\Persistence\ManagerRegistry;
use Dunglas\ApiBundle\Api\ResourceCollectionInterface;
use Dunglas\ApiBundle\Api\ResourceInterface;


class SearchController extends Controller
{
    public function searchAction(Request $request)
    {
        $query = json_decode($request->getContent());

        $offset = isset($query->offset) ? intval($query->offset) : 0;
        $size = isset($query->size) ? $offset + intval($query->size) : 30;
        $recordModifiedFromDt = isset($query->modifyDateGte) ? $query->modifyDateGte : 0;
        $recordModifiedToDt = isset($query->modifyDateLte) ? $query->modifyDateLte : 0;

        // Index List Send by UI
        $type = isset($query->index) ? $query->index : [];
        $index = $this->filterAccessibleTypes($type);
        $search_term = isset($query->q) ? $query->q : [];

        if (count($index) === 0) {
            throw new HttpException(Response::HTTP_BAD_REQUEST, 'You must indicate which index to search on.\
            If index were specified, you do not have sufficient permissions to search against them.');
        }
        if (count($search_term) === 0) {
            throw new HttpException(Response::HTTP_BAD_REQUEST, 'You must provide search term with minimal three character.');
        }

        $filter_term = $this->getAccessibleTeamIris();
        $sort = isset($query->sort) ? $query->sort : '_score';

        $payload = [
            'offset' => $offset,
            'size' => $size,
            'index' => $index,
            'filter_term' => $filter_term,
            'search_term' => $search_term,
            'sort' => $sort,
            'recordModifiedFromDt' => $recordModifiedFromDt,
            'recordModifiedToDt' => $recordModifiedToDt
        ];

        $data = [
            'payload' => $payload,
            'esoperation' => 'search'
        ];
        $utilsObj = new Utils();
        $member = $utilsObj->sendSearchAction($this, [$data], 'cyops.crudhub.essearch');
        $response = [
            'totalItems' => $member->search_result->total->value ? $member->search_result->total->value : 0,
            'results' => $member->search_result->hits ? $member->search_result->hits : [],
            'context_code' => $member->context_code ? $member->context_code : 400
        ];
        return new JsonResponse($response);
    }

    public function searchSimilarAction(Request $request)
    {
        $query = json_decode($request->getContent());
        $module_type = $query->module_type;
        if (!$module_type) {
            throw new HttpException(Response::HTTP_BAD_REQUEST, "Module Type is Missing in request");
        }

        $offset = isset($query->offset) ? intval($query->offset) : 0;
        $size = isset($query->size) ? $offset + intval($query->size) : 30;

        $id = $query->id;
        $fields = $query->fields;


        $sort = isset($query->sort) ? $query->sort : '_score';
        $filter = $query->filter;


        $payload = [
            'id' => $id,
            'offset' => $offset,
            'size' => $size,
            'index' => $query->index,
            'sort' => $sort,
            'from_date' => $query->fromDate,
            'fields' => $fields,
            'filter' => $filter
        ];

        $query_type = $query->query_type;
        if ($query_type === 'prediction') {
            $aggs = $query->aggs;
            $payload['aggs'] = $aggs;

            $output_fields = [];

            foreach($fields as $item) {
                array_push($output_fields, $item->name);
            }
            foreach($aggs as $item) {
                array_push($output_fields, $item);
            }



            $payload['output_fields'] = $output_fields;
        }
        else {
            $teams = $this->getAccessibleTeamIris();
            $payload['teams'] = $teams;
            $payload['aggs'] = $aggs;
            $output_fields = $query->output_fields;
            $payload['output_fields'] = $output_fields;

        }

        $data = [
            'esoperation' => 'similar',
            'payload' => $payload
        ];

        $utilsObj = new Utils();
        $response = $utilsObj->sendSearchAction($this, [$data], 'cyops.crudhub.essearch');
        $iris = [];

        if (isset($response->status_code) && $response->status_code >= 400) {
            throw new HttpException(Response::HTTP_BAD_REQUEST, $response->error);
        }
        
        foreach($response->result as $item) {
            array_push($iris, $item->{'_id'});
        }
        if ($iris) {
            $filter = ['uuid' => $iris];
            $moduleresource = $this->getResourceFromShortName($module_type);
            $pg_data = $utilsObj->findWithWhere($this, $moduleresource, $filter);
            $serializer = $this->container->get('serializer');
            $data = $serializer->normalize(
                $pg_data,
                'json-ld',
                $moduleresource->getNormalizationContext() + ['required_attributes' => $output_fields]
            );
            $output = [];
            foreach($response->result as $result) {
                foreach($data['hydra:member'] as $item) {
                    if ($result->{'_id'} == end(explode('/', $item['@id']) )) {
                        $item['__score'] = $result->{'_score'};
                        array_push($output, $item);
                    break;
                    }
                }
            }

            $response->{'result'} = $output;
        }
        return new JsonResponse($response);
    }


    public function searchStatusAction(Request $request)
    {
        // ToDo Added for Support of Specific Index Status in Future
        $payload = [];
        $data = [
            'payload' => $payload,
            'esoperation' => 'status'
        ];
        $utilsObj = new Utils();
        $member = $utilsObj->sendSearchAction($this, [$data], 'cyops.crudhub.status');
        $response = [
            'totalItems' => $member
        ];
        return new JsonResponse($response);
    }

    /**
     * @param string[] $types
     * @return string[]
     */
    protected function filterAccessibleTypes($types)
    {
        /** @var PermissionResolver $permissionResolver */
        $permissionResolver = $this->container->get('cybersponse.authorization.permissions.permission_resolver');
        return array_filter($types, function ($type) use ($permissionResolver) {
            $requiredPermission = sprintf('%s.read', $type);
            return $permissionResolver->resolvePermission($requiredPermission);
        });
    }

    /**
     * @return array
     */
    protected function getAccessibleTeamIris()
    {
        $accessibleTeamIdsProvider = $this->container->get('cybersponse.authorization.ownership.accessible_team_ids_provider');
        $iriConverter = $this->container->get('api.iri_converter');
        $teamResource = $this->container->get('resource.team');
        $teamBaseIri = $iriConverter->getIriFromResource($teamResource);
        $accessibleTeamIds = $accessibleTeamIdsProvider->getDataForCurrentUser();
        $accessibleTeamIris = array_map(function ($teamId) use ($teamBaseIri) {
            return sprintf('%s/%s', $teamBaseIri, $teamId);
        }, $accessibleTeamIds);
        return json_decode(json_encode($accessibleTeamIris), true);
    }

    /**
     * @param string $type
     * @return StagingModelMetadata|null
     */
    protected function getClassNameByType($type)
    {
        /** @var ManagerRegistry $managerRegistry */
        $managerRegistry = $this->container->get('doctrine');
        $queryManager = $managerRegistry->getManagerForClass(StagingModelMetadata::class);
        $queryRepository = $queryManager->getRepository(StagingModelMetadata::class);
        return $queryRepository->findOneBy(array('type' => $type));
    }

    /**
     * @param $resourceShortName
     * @return ResourceInterface|null
     */
    protected function getResourceFromShortName($resourceShortName)
    {
        $resourceClassName = Inflector::singularize(Inflector::classify($resourceShortName));
        /** @var ResourceCollectionInterface $resourceCollection */
        $resourceCollection = $this->container->get('api.resource_collection');
        $resource = $resourceCollection->getResourceForShortName($resourceClassName);

        if ($resource === null) {
            $model = $this->getClassNameByType($resourceShortName);
            $singular = $model->getDescriptions()['singular'];
            $escapedSingular = preg_replace('/[^a-z0-9\s]+/i', '', $singular);
            $resourceName = \Doctrine\Common\Util\Inflector::classify($escapedSingular);
            $resource = $resourceCollection->getResourceForShortName($resourceName);
        }
        return $resource;
    }
}
