<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\SettingsBundle\Provider;


use Cybersponse\ActorsBundle\Entity\Actor;
use Cybersponse\CoreBundle\Provider\ActorDataProvider;
use Cybersponse\SettingsBundle\Entity\SystemSettings;
use Cybersponse\SettingsBundle\Entity\UserSettings;
use Cybersponse\SettingsBundle\Repository\SystemSettingsRepository;
use Cybersponse\SettingsBundle\Repository\UserSettingsRepository;
use Doctrine\ORM\EntityManager;
use Dunglas\ApiBundle\Api\IriConverterInterface;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;

/**
 * Data provider for the Settings document.
 *
 */
class SettingsProvider extends ActorDataProvider
{

    /** @var EntityManager */
    protected $entityManager;

    /** @var IriConverterInterface */
    protected $iriConverter;

    /**
     * @inheritdoc
     * @return array
     */
    public function getDataForActor(Actor $actor)
    {
        $userIri = $this->iriConverter->getIriFromItem($actor);

        /** @var UserSettingsRepository $userSettingsRepo */
        $userSettingsRepo = $this->entityManager->getRepository(UserSettings::class);
        $userSettings = $userSettingsRepo->getUserSettings($userIri);

        /** @var SystemSettingsRepository $systemSettingsRepo */
        $systemSettingsRepo = $this->entityManager->getRepository(SystemSettings::class);
        $systemSettings = $systemSettingsRepo->getSystemSettings();

        if (is_null($userSettings)) {
            $userSettings = new UserSettings();
            $userSettings->setOwner($userIri);
            $userSettings->setPublicValues([]);
        }

        return [$userSettings, $systemSettings];
    }

    /**
     * @inheritdoc
     * @return UserSettings
     */
    public function getDataForToken(TokenInterface $token)
    {
        return parent::getDataForToken($token);
    }

    /**
     * @inheritdoc
     * @return array
     */
    public function getDataForCurrentUser()
    {
        return parent::getDataForCurrentUser();
    }

    /**
     * @param UserSettings $userSettings
     * @param array $newValues
     */
    public function saveUserSettings(UserSettings $userSettings, $newValues = [])
    {
        $userSettings->setPublicValues($newValues);

        $this->entityManager->persist($userSettings);
        $this->entityManager->flush();
    }

    /**
     * @param EntityManager $entityManager
     */
    public function setEntityManager($entityManager)
    {
        $this->entityManager = $entityManager;
    }

    /**
     * @param IriConverterInterface $iriConverter
     */
    public function setIriConverter($iriConverter)
    {
        $this->iriConverter = $iriConverter;
    }

    /**
     * @param UserSettings $userSettings
     * @param SystemSettings $systemSettings
     *
     * @return array
     */
    public function getMergedSettings(UserSettings $userSettings, SystemSettings $systemSettings)
    {
        $mergedValues = array_replace_recursive(
            is_null($systemSettings) || is_null($systemSettings->getPublicValues()) ? [] : $systemSettings->getPublicValues(),
            is_null($userSettings) || is_null($userSettings->getPublicValues()) ? [] : $userSettings->getPublicValues(),
            is_null($systemSettings) || is_null($systemSettings->getPrivateValues()) ? [] : $systemSettings->getPrivateValues()
        );

        return $mergedValues;
    }
}
