# Cybersponse Settings Bundle

This bundle is part of the [Cybersponse Symfony2 Framework](http://cybersponse.com/)
and licensed under the [Cybersponse License](LICENSE).

This bundle provides simple settings management with hierarchies.

### Requirements

* Symfony 2.6.x
* [Api-Platform 1.0](https://api-platform.com/)
* See also the `require` section of [composer.json](composer.json)

### Endpoints Exposed:

``` bash
cybersponse_settings_get    GET         /api/3/settings{path}
cybersponse_settings_set    POST|PUT    /api/3/settings{path}
api_system_settings_cget    GET         /api/3/system_settings
api_system_settings_cpost   POST        /api/3/system_settings
api_system_settings_get     GET         /api/3/system_settings/{id}
api_system_settings_put     PUT         /api/3/system_settings/{id}
api_system_settings_delete  DELETE      /api/3/system_settings/{id}
```

### Get All User Settings
`GET /api/3/settings`

### Set User Setting
`POST /api/3/settings`
``` json
{
    "user": {
        "view": {
            "theme": "light"
        }
    }
}
```

### Get Setting By Name
`GET /api/3/settings/user/view/theme`

### Update Setting By Name
The path given in the URL should always be a reference to an existing setting tree. Updates to a not existing setting tree will not generate the tree for you. It's basically trying to make a file in a directory that does not exist.

`POST /api/3/settings/user/view/theme`
``` json
"light"
```

### Update Setting By Name (2)
Because `timezone_type` is a private system setting, we can't actually modify it directly. The setting `timezone` itself can be modified just fine.

`POST /api/3/settings/user/global/timezone`
``` json
{
    "timezone_type": 2,
    "timezone": "Europe/Gibraltar"
}
```

### Set System Defaults
`DO NOT RUN THIS UNLESS YOU HAVE TO*`
There should only ever be one system settings object in the database.
If you want to delete the setting, do get for this setting, then follow through with a delete on that URI that is given.
* This should have already been done on your machine by a migration.

`POST /api/3/system_settings`
``` json
{
    "privateValues": {
        "system": {
            "login": {
                "session": {
                    "idle_logout_time": 18000
                }
            }
        },
        "user": {
            "global": {
                "timezone": {
                    "timezone_type": 3
                }
            }
        }
    },
    "publicValues": {
        "user": {
            "view": {
                "theme": "dark"
            },
            "global": {
                "timezone": {
                    "timezone_type": 3,
                    "timezone": "America/Phoenix"
                }
            }
        }
    }
}
```
