<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\SettingsBundle\Tests\Unit\Utility;


use Cybersponse\SettingsBundle\Utility\Resolver;

class ResolverTest extends \PHPUnit_Framework_TestCase
{
    public function test_Resolve_SetValue_GivenEmptyArray()
    {
        $test = [
            '1',
            '2',
            '3'
        ];

        $array = [];

        $path = 'a/b';

        Resolver::set($array, $path, $test, '/');
        $expectedResult = [
            'a' => [
                'b' => $test
            ]
        ];

        $this->assertEquals($expectedResult, $array);
    }

    public function test_Resolve_SetValue_GivenNonEmptyArray()
    {
        $testData = [
            '1',
            '2',
            '3'
        ];

        $actualArray = [
            'c' => [
                'd'
            ]
        ];

        $path = 'a/b';

        Resolver::set($actualArray, $path, $testData, '/');
        $expectedResult = [
            'a' => [
                'b' => $testData
            ],
            'c' => [
                'd'
            ]
        ];

        $this->assertEquals($expectedResult, $actualArray);
    }

    public function test_Resolve_GetValue_GivenNonEmptyArray()
    {
        $array = [
            'a' => [
                'b' => 'test'
            ]
        ];

        $path = 'a/b';

        $actualResult = Resolver::get($array, $path, '/');
        $expectedResult = 'test';

        $this->assertEquals($expectedResult, $actualResult);
    }

    public function test_Resolve_GetValue_GivenNonEmptyArray_GivenDotDelimiter()
    {
        $array = [
            'a' => [
                'b' => 'test'
            ]
        ];

        $path = 'a.b';

        $actualResult = Resolver::get($array, $path);
        $expectedResult = 'test';

        $this->assertEquals($expectedResult, $actualResult);
    }

    public function test_Resolve_SetValue_GivenNonEmptyArray_GivenDotDelimiter()
    {
        $array = [
            'a' => [
                'b' => 'test'
            ]
        ];

        $path = 'a.b';

        Resolver::set($array, $path, 'new');
        $expectedResult = 'new';

        $this->assertEquals($expectedResult, $array['a']['b']);
    }

    public function test_Resolve_SetValue_GivenIndexedArray_GivenDotDelimiter()
    {
        $array = [
            'a' => [
                0 => 'test'
            ]
        ];

        $path = 'a.0';

        Resolver::set($array, $path, 'new');
        $expectedResult = 'new';

        $this->assertEquals($expectedResult, $array['a'][0]);
    }

    /**
     * @expectedException \Cybersponse\SettingsBundle\Utility\UndefinedIndexException
     */
    public function test_Resolve_GetValue_GivenNonEmptyArray_GivenEmptyPath()
    {
        $testArray = [
            'a' => [
                'b' => 'test'
            ]
        ];

        // This should fail because the empty string key does not exist
        $path = '';

        Resolver::get($testArray, $path, '/');
    }

    public function test_Resolve_GetValue_GivenEmptyKeyInArray_GivenPath()
    {
        $testArray = [
            '' => 'test'
        ];

        $path = '/';

        $actualResult = Resolver::get($testArray, $path, '/');
        $expectedResult = 'test';

        $this->assertEquals($expectedResult, $actualResult);
    }

    public function test_Resolve_GetValue_GivenEmptyKeyInArray_GivenEmptyPath()
    {
        $testArray = [
            '' => 'test'
        ];

        $path = '';

        $actualResult = Resolver::get($testArray, $path, '/');
        $expectedResult = 'test';

        $this->assertEquals($expectedResult, $actualResult);
    }

    public function test_Resolve_GetValue_GivenEmptyKeyInMultiArray_GivenPath()
    {
        $testArray = [
            '' => [
                '' => 'test'
            ]
        ];

        $path = '//';

        $actualResult = Resolver::get($testArray, $path, '/');
        $expectedResult = 'test';

        $this->assertEquals($expectedResult, $actualResult);
    }

    public function test_Resolve_UnsetValue_GivenMultidimenisonalArray()
    {
        $actualArray = [
            'a' => [
                'b' => 'test'
            ]
        ];

        $path = 'a/b';

        Resolver::uset($actualArray, $path, '/');
        $expectedResult = [
            'a' => []
        ];

        $this->assertEquals($expectedResult, $actualArray);
    }

    public function test_Resolve_UnsetOneValue_GivenMultidimenisonalArray()
    {
        $actualArray = [
            'a' => [
                'b' => 'test',
                'c' => 'test2'
            ]
        ];

        $path = 'a/b';

        Resolver::uset($actualArray, $path, '/');
        $expectedResult = [
            'a' => [
                'c' => 'test2'
            ]
        ];

        $this->assertEquals($expectedResult, $actualArray);
    }

    public function test_Resolve_UnsetValue_GivenSimpleArray()
    {
        $actualArray = [
            'a' => 'test'
        ];

        $path = 'a';

        Resolver::uset($actualArray, $path);
        $expectedResult = [];

        $this->assertEquals($expectedResult, $actualArray);
    }

    /**
     * @expectedException \Cybersponse\SettingsBundle\Utility\UndefinedIndexException
     */
    public function test_Resolve_UnsetValue_GivenArrayNotContainingValue()
    {
        $actualArray = [
            'a' => 'test'
        ];

        $path = 'b';

        Resolver::uset($actualArray, $path);
    }
}
