<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\TenantBundle\DataFixtures\Populator\Methods;


use Doctrine\Common\Inflector\Inflector;
use Dunglas\ApiBundle\Mapping\ClassMetadataFactory;
use Nelmio\Alice\Fixtures\Fixture;
use Nelmio\Alice\Instances\Populator\Methods\ArrayAdd;
use Symfony\Component\Serializer\Serializer;

class DunglasAdder extends ArrayAdd
{
    /** @var ClassMetadataFactory */
    protected $classMetadataFactory;

    /** @var Serializer */
    protected $serializer;

    public function canSet(Fixture $fixture, $object, $property, $value)
    {
        return parent::canSet($fixture, $object, $property, $value);
    }

    public function set(Fixture $fixture, $object, $property, $value)
    {
        $classMetadata = $this->classMetadataFactory->getMetadataFor($object);

        $attributes = $classMetadata->getAttributes();

        if (!array_key_exists($property, $attributes)) {
            // dunno if this is right
            return;
        }

        $attributeMetadata = $attributes[$property];

        $method = 'add' . Inflector::ucwords(Inflector::singularize($property));
        foreach ($value as $val) {
            if (is_object($val)) {
                $val = $this->typeHintChecker->check($object, $method, $val);
            }

            if (is_array($val)) {
                $val = $this->serializer->denormalize($val, $attributeMetadata->getTypes()[0]->getCollectionType()->getClass(), 'json');
            }

            $object->{$method}($val);
        }
    }

    /**
     * @param ClassMetadataFactory $classMetadataFactory
     */
    public function setClassMetadataFactory($classMetadataFactory)
    {
        $this->classMetadataFactory = $classMetadataFactory;
    }

    /**
     * @param Serializer $serializer
     */
    public function setSerializer($serializer)
    {
        $this->serializer = $serializer;
    }
}
