<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\UtilsBundle;

use Symfony\Component\Finder\Finder;
use Dunglas\ApiBundle\JsonLd\Response;
use Cybersponse\FilesBundle\Entity\PgFile;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Cybersponse\FilesBundle\Controller\FilesController;
use Dunglas\ApiBundle\Exception\InvalidArgumentException;
use Symfony\Component\HttpKernel\Exception\HttpException;

class ARCUtils
{
    const LINK_KEY = '__link';
    const DE_LINK_KEY = '__unlink';
    const AS_JSON_KEY = "\$asJson";
    const RELATIONSHIP_KEY = "\$relationships";
    const WIDGET_PREVIEW_PATH = '/opt/cyops-ui/widgets_preview/';
    const WIDGET_PATH = '/opt/cyops-ui/widgets/';

    public static function setAsJson($request, $set_asJson = false)
    {
        if (!$request->query->has(self::AS_JSON_KEY) && !$set_asJson) {
            $request->query->set(self::AS_JSON_KEY, true);
            $set_asJson = true;
        } elseif ($set_asJson) {
            $request->query->remove(self::AS_JSON_KEY, true);
            $set_asJson = false;
        }
        return $set_asJson;
    }
    /**
     * @param $request
     * @param bool $set_relation
     * @return bool
     */
    public static function setRelationShip($request, $set_relation = false)
    {
        if (!$request->query->has(self::RELATIONSHIP_KEY) && !$set_relation) {
            $request->query->set(self::RELATIONSHIP_KEY, true);
            $set_relation = true;
        } elseif ($set_relation) {
            $request->query->remove(self::RELATIONSHIP_KEY, true);
            $set_relation = false;
        }
        return $set_relation;
    }

    /**
     * @param $request_body
     * @return mixed
     */
    public static function updateReferenceToIriAndCheckForRelation($request_body)
    {
        foreach ($request_body as $key => $value) {
            if (gettype($value) == "object" && isset($request_body->$key->{'@id'})) {
                $request_body->$key = $request_body->$key->{'@id'};
            }
        }
    }

    /**
     * @param $request_body
     */
    public static function validateRequestData($request_body)
    {
        if (gettype($request_body) != 'object') {
            throw new InvalidArgumentException('Invalid request body. valid Json object request body expected.');
        }
    }

    /**
     * @param $previousObject
     * @param $updatedObject
     * @return null
     */
    public static function extractEntityType($previousObject, $updatedObject, $identity)
    {
        if (array_key_exists($identity, $updatedObject)) {
            return $updatedObject[$identity];
        } elseif (array_key_exists($identity, $previousObject)) {
            return $previousObject[$identity];
        }
        return null;
    }

    /**
     * @param $teams
     * @return array
     */
    public static function getUuidFromIRI($teams)
    {
        $objectTeam = [];
        foreach ($teams as $team) {
            $teamId = explode('/', $team);
            array_push($objectTeam, end($teamId));
        }
        return $objectTeam;
    }

    public static function validateRequestKeys($key, $object)
    {
        if (!array_key_exists($key, $object)) {
            throw new HttpException(Response::HTTP_BAD_REQUEST, $key . ' key does not exist in object');
        }
    }

    // FOR POST REQUEST MERGE LINK AND UNLINK INTO MAIN REQUEST

    /**
     * @param $reqObject
     * @param $preObj
     * @return mixed
     */
    public static function mergePostRequest($reqObject)
    {
        $linkObj = isset($reqObject->{self::LINK_KEY}) ? $reqObject->{self::LINK_KEY} : (object) null;
        unset($reqObject->{self::LINK_KEY});
        unset($reqObject->{self::DE_LINK_KEY});
        return self::mergeLinkPost($reqObject, $linkObj);
    }

    public static function mergeLinkPost($reqObject, $linkObj)
    {
        foreach ($linkObj as $key => $value) {
            if (!is_array($value)) {
                $value = explode(',', $value);
            }
            $referenceToIriObj = [];
            foreach ($value as $innerVal) {
                if (is_object($innerVal) && $innerVal->{'@id'}) {
                    array_push($referenceToIriObj, $innerVal->{'@id'});
                } else {
                    array_push($referenceToIriObj, $innerVal);
                }
            }
            if (count($referenceToIriObj)) {
                $reqObject->$key = $referenceToIriObj;
            }
        }
        return $reqObject;
    }

    /**
     * @param $reqObject
     * @param $preObj
     * @return mixed
     */
    public static function mergeRequest($reqObject, $preObj)
    {
        $linkObj = isset($reqObject[self::LINK_KEY]) ? $reqObject[self::LINK_KEY] : (object) null;
        $unlinkObj = isset($reqObject[self::DE_LINK_KEY]) ? $reqObject[self::DE_LINK_KEY] : (object) null;
        unset($reqObject[self::LINK_KEY]);
        unset($reqObject[self::DE_LINK_KEY]);
        return self::mergeLinkAndUnlink($reqObject, $preObj, $linkObj, $unlinkObj);
    }

    /**
     * @param $reqObject
     * @param $preObj
     * @param $linkObj
     * @param $unlinkObj
     * @return mixed
     */
    public static function mergeLinkAndUnlink($reqObject, $preObj, $linkObj = [], $unlinkObj = [])
    {
        // Handling Linked Object
        foreach ($linkObj as $key => $value) {
            $updatedReq[$key] = [];
            self::validateRequestKeys($key, $preObj);

            // Check if Link object has been send as an string or object then convert to array
            if ($linkObj[$key] && is_string($linkObj[$key])) {
                $linkObj[$key] = array_map('trim', explode(',', $linkObj[$key]));
            } elseif (is_object($linkObj[$key])) {
                $linkObj[$key] = [$linkObj[$key]];
            }

            if (count($preObj[$key])) {
                $updatedReq[$key] = $preObj[$key];
            }

            foreach ($linkObj[$key] as $innerKey) {
                if (gettype($innerKey) == 'array' && array_key_exists('@id', $innerKey)) {
                    array_push($updatedReq[$key], $innerKey['@id']);
                } else {
                    array_push($updatedReq[$key], $innerKey);
                }
            }
            if (gettype($updatedReq[$key]) == "array") {
                $reqObject[$key] = array_map('unserialize', array_unique(array_map('serialize', $updatedReq[$key])));
            } else {
                $reqObject[$key] = array_values(array_unique($updatedReq[$key]));
            }
        }

        // Handling Unlink Object
        foreach ($unlinkObj as $key => $value) {
            self::validateRequestKeys($key, $preObj);
            if (isset($reqObject[$key])) {
                $reqObject[$key] = array_values(array_diff($reqObject[$key], $unlinkObj[$key]));
            } else {
                $reqObject[$key] = array_values(array_diff($preObj[$key], $unlinkObj[$key]));
            }
        }
        return $reqObject;
    }

    public static function extractRequestBody($request, $file=null)
    {
        if ($file !== null) {
            return json_decode(json_encode($file));
        } else {
            $request_body = json_decode($request->getContent());
            self::updateReferenceToIriAndCheckForRelation($request_body);
            return $request_body;
        }
    }

    public function buildWidgetFiles($widget, $path, $entityManager)
    {
        $widgetsPath = sprintf("%s/%s", $path, $widget->getName());
        $widgetsFileSystem = new Filesystem();
        if ($widgetsFileSystem->exists($widgetsPath."/widgetAssets")) {
            $widgetsFileSystem->remove($widgetsPath."/widgetAssets");
        }
        if (!$widgetsFileSystem->exists($widgetsPath)) {
            $widgetsFileSystem->mkdir($widgetsPath, 0755);
            $widgetsFileSystem->mkdir($widgetsPath."/html", 0755);
            $widgetsFileSystem->mkdir($widgetsPath."/js", 0755);
        }

        try {
            foreach ($widget->getFiles() as $key => $value) {
                if ($value->getMetadata() && array_key_exists('path', $value->getMetadata())) {
                    $pgFile = $entityManager->find(PgFile::class, end(explode('/', $value->getFile())));
                    $widgetsFileSystem->dumpFile(sprintf('%s%s/%s', $widgetsPath, $value->getMetadata()['path'], $value->getFilename()), base64_decode(stream_get_contents($pgFile->getFile())));
                }
            }
            $widgetsFileSystem->dumpFile(sprintf('%s/html/view.html', $widgetsPath), $widget->getViewHtml());
            $widgetsFileSystem->dumpFile(sprintf('%s/js/view.controller.js', $widgetsPath), $widget->getViewController());
            $widgetsFileSystem->dumpFile(sprintf('%s/html/edit.html', $widgetsPath), $widget->getEditHtml());
            $widgetsFileSystem->dumpFile(sprintf('%s/js/edit.controller.js', $widgetsPath), $widget->getEditController());
            $infoData['metadata'] = $widget->getMetadata();
            $infoData['name'] = $widget->getName();
            $infoData['title']= $widget->getTitle();
            $infoData['subTitle']= $widget->getSubTitle();
            $infoData['version']= $widget->getVersion();
            if ($widget->getPublishedDate()) {
                $infoData['published_date']= $widget->getPublishedDate();
            }
            $widgetsFileSystem->dumpFile(sprintf('%s/info.json', $widgetsPath), json_encode($infoData, JSON_PRETTY_PRINT));
        } catch (\Exception $e) {
            throw new HttpException(Response::HTTP_BAD_REQUEST, $e->getMessage(), $e);
        }
        return true;
    }
    public static function addWidgetAssests($path, $entityManager, $request)
    {
        $widgetsFileSystem = new Filesystem();
        $filePaths= [];
        $fileIRI =[];
        if ($widgetsFileSystem->exists($path)) {
            try {
                $finder = new Finder();
                $files= $finder->files()->in($path);
                foreach ($files as $file) {
                    array_push($filePaths, $file->getRelativePathname());
                }
                foreach ($filePaths as $key => $value) {
                    $fileController = new FilesController();
                    $request->initialize(
                        $request->query->all(),
                        $request->request->all(),
                        $request->attributes->all(),
                        $request->cookies->all(),
                        $request->files->all(),
                        $request->server->all(),
                    );
                    /** @var UploadedFile $uploadedFile */
                    $uploadedFile = new uploadedFile($path.$value, end(explode('/', $value)));
                    $request->files->set('file', $uploadedFile);
                    $request->request->set('metadata', ["path"=>$value]);
                    $fileObject = $fileController->uploadFile($request);
                    array_push($fileIRI, $fileObject);
                }
            } catch (\Exception $e) {
                throw new HttpException(Response::HTTP_BAD_REQUEST, $e->getMessage(), $e);
            }
            return $fileIRI;
        }
    }

    public function removeWidgetFiles($widget)
    {
        $widgetsPath = sprintf(self::WIDGET_PATH."%s", $widget->getName());
        $widgetsFileSystem = new Filesystem();
        if ($widgetsFileSystem->exists($widgetsPath)) {
            $widgetsFileSystem->remove($widgetsPath);
        }
        return true;
    }
    public function removeWidgetPreviewFiles($widgetName)
    {
        $widgetsPath = sprintf(self::WIDGET_PREVIEW_PATH."%s", $widgetName);
        $widgetsFileSystem = new Filesystem();
        if ($widgetsFileSystem->exists($widgetsPath)) {
            $widgetsFileSystem->remove($widgetsPath);
        }
        return true;
    }
}
