<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\UtilsBundle;

use Ramsey\Uuid\Uuid;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Cybersponse\CoreBundle\Traits\PeerReplicable;
use Cybersponse\AuthorizationBundle\Traits\Ownable;
use Cybersponse\TenantBundle\Entity\Prod\Appliance;
use GuzzleHttp\Psr7\Request as GuzzleRequest;


class TenantUtils extends Controller
{
    const DATA_ROUTING_KEY = 'route.crudhub.data.remoterequest';
    const PLAYBOOK_ALIAS_ROUTING_KEY = 'route.crudhub.instructions.remoterequest.playbookaliaspush';
    const MMD_ROUTING_KEY = 'route.crudhub.instructions.remoterequest.stagingmmdupdate';
    const POSTMAN_SECRET_KEY = 'eW1hSe7mnlIi1a0mmgno1nSe9Rea7pphG2oAdmCeBAoduYnd';
    const MASTER_ROLE_NAME = 'master';
    const IS_DISTRIBUTED_INSTANCE = 'isDistributedInstance';
    const INSTANCE_ROLE = 'instanceRole';

    public function checkForConflicts($contextRef, $objectData, $preUpdatedObject, $isRemoteRequest = false)
    {
        $isDataConflict = false;
        $isDistributedInstance = $this->checkForMsspInstance($contextRef);
        if (!$isDistributedInstance || !$isRemoteRequest) {
            return $isDataConflict;
        }
        $instanceRole = $this->getInstanceRole($contextRef);
        $modifyUserUuidOfRequestedObj = $contextRef->getUser()->getUuid();
        $modifyUserUuidOfExistingObj = end(explode('/', $preUpdatedObject['modifyUser']));
        $modifyDateOfExistingObj = isset($preUpdatedObject['modifyDate']) ? $preUpdatedObject['modifyDate'] : 0;
        $modifyDateOfRequestedObj = isset($objectData['modifyDate']) ? $objectData['modifyDate'] : PHP_INT_MAX;
        if ($instanceRole != self::MASTER_ROLE_NAME && ($modifyUserUuidOfRequestedObj != $modifyUserUuidOfExistingObj) && ($modifyDateOfExistingObj >= $modifyDateOfRequestedObj)) {
            $isDataConflict = true;
        }
        return $isDataConflict;
    }

    public function getDetailsFromDas($contextRef, $cacheClient, $proxyProvider, $isDistributedInstance = false)
    {
        $dasUri = $contextRef->container->getParameter('das.uri');
        $dasClient = $proxyProvider->getDasClient();
        $headers = array(
            "Content-type" => "application/json"
        );
        $dasRequests = new GuzzleRequest('get', $dasUri . '/license/?param=license_details', $headers);
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $dasResponseData = $dasResponse->getBody()->getContents();
            $jsonDecodeResponse = json_decode($dasResponseData, true);
            $resCode = $dasResponse->getStatusCode();
            if ($resCode < 300) {
                $isDistributedInstance = $jsonDecodeResponse['details']['is_distributed'];
                $instanceRole = $isDistributedInstance ? $jsonDecodeResponse['details']['role'][0] : Null;

                if (!is_null($isDistributedInstance)) {
                    $cacheClient->persistToCache(self::IS_DISTRIBUTED_INSTANCE, $isDistributedInstance, $proxyProvider->getCacheLifetime());
                    $cacheClient->persistToCache(self::INSTANCE_ROLE, $instanceRole, $proxyProvider->getCacheLifetime());
                }

                return $isDistributedInstance ? $isDistributedInstance : false;
            }
        } catch (\Exception $error) {
            error_log($error->getMessage());
        }
        return $isDistributedInstance;
    }

    public function checkForMsspInstance($contextRef)
    {
        $isDistributedInstance = false;
        $proxyProvider = $contextRef->get('cybersponse.core.provider.proxy');
        $cacheClient = $proxyProvider->getCache();
        if ($cacheClient->cacheContains(self::IS_DISTRIBUTED_INSTANCE)) {
            $isDistributedInstance = $cacheClient->getFromCache(self::IS_DISTRIBUTED_INSTANCE);
        } else {
            $isDistributedInstance = $this->getDetailsFromDas($contextRef, $cacheClient, $proxyProvider, $isDistributedInstance);
        }
        return $isDistributedInstance;
    }

    public function getInstanceRole($contextRef)
    {
        $proxyProvider = $contextRef->get('cybersponse.core.provider.proxy');
        $cacheClient = $proxyProvider->getCache();
        if ($cacheClient->cacheContains(self::INSTANCE_ROLE)) {
            return $cacheClient->getFromCache(self::INSTANCE_ROLE);
        } else {
            $this->getDetailsFromDas($contextRef, $cacheClient, $proxyProvider);
            return $cacheClient->getFromCache(self::INSTANCE_ROLE);
        }
    }

    public function setTenantObject($contextRef, $resource, $request_body, $object, $peerReplicable = false)
    {
        $isDistributedInstance = $this->checkForMsspInstance($contextRef);
        if (!$isDistributedInstance) {
            return;
        }
        $request_body = json_decode(json_encode($request_body));
        if ($peerReplicable || $this->isPeerReplicable($resource->getEntityClass())) {
            $tenantObject = $this->getTenantObject($contextRef, $request_body);
            if (!$tenantObject) {
                throw new HttpException(Response::HTTP_BAD_REQUEST, "Self TenantId not defined for instance");
            }
            $tenantRecordId = isset($request_body->tenantRecordId) ? $request_body->tenantRecordId : null;
            $object->setTenantRecordId($tenantRecordId);
            $object->setTenant($tenantObject);

            //Set Owners to Records
            if ($this->isOwnable($resource->getEntityClass())) {
                foreach ($contextRef->getUser()->getTeams() as $owner) {
                    $object->addOwner($owner);
                }
                foreach ($tenantObject->getOwners() as $owner) {
                    $object->addOwner($owner);
                }
            }
            $object->setConflict(false);
        }
    }

    public function isPeerReplicable($entity_class)
    {
        return in_array(PeerReplicable::class, class_uses($entity_class));
    }

    public function isOwnable($entity_class)
    {
        return in_array(Ownable::class, class_uses($entity_class));
    }

    public function getTenantObject($contextRef, $request_body = null)
    {
        if (isset($request_body->tenant)) {
            $tenant = $request_body->tenant;
            if (gettype($tenant) == "object") {
                $tenantFilter = ["tenantId" => $tenant->tenantId];
            } else if (substr($tenant, 0, 5) == '/api/') {
                $tenantUuidArray = explode('/', $tenant);
                $tenantUuid = end($tenantUuidArray);
                $tenantFilter = ["uuid" => $tenantUuid];
            } else {
                $tenantFilter = ["tenantId" => $tenant];
            }
        } else {
            $tenantFilter = ["role" => "self"];
        }
        $tenant_resource = $contextRef->identifyResourceByShortName('tenant');
        $tenantObj = $this->findWithWhere($contextRef, $tenant_resource, $tenantFilter);
        if ($tenantObj) {
            return $tenantObj[0];
        } else {
            throw new HttpException(Response::HTTP_BAD_REQUEST, 'No tenant object found.');
        }
    }

    public function replicateData($contextRef, $resource, $request_body, $objects, $method, $conflictingData = false)
    {
        $isDistributedInstance = $this->checkForMsspInstance($contextRef);
        if (!$isDistributedInstance) {
            return;
        }
        $replicateDataList = [];
        $sync = false;
        $replicateData = true;
        $routing_key = '';
        $entityClass = $resource->getEntityClass();
        $entityClassName = explode('\\', $entityClass);

        if (isset($request_body->data) && gettype($request_body->data) == gettype(array())) {
            if (isset($request_body->data[0]->{'__sync'})) {
                $sync = $request_body->data[0]->{'__sync'};
            }
        } elseif (isset($request_body->{'__sync'})) {
            $sync = $request_body->{'__sync'};
        }

        if (isset($request_body->{'__replicateData'})) {
            $replicateData = $request_body->{'__replicateData'};
            if ($conflictingData) {
                $replicateData = true;
            }
        }
        if ($this->isPeerReplicable($entityClass)) {
            $routing_key = self::DATA_ROUTING_KEY;
            $replicateDataList = $objects;
            if ($sync) {
                $replicateDataList = $this->processSyncRequest($objects);
            }
        } elseif (end($entityClassName) == 'Workflow') {
            $routing_key = self::PLAYBOOK_ALIAS_ROUTING_KEY;
            $replicateDataList = $this->processPlaybook($objects, $replicateDataList);;
        } elseif (end($entityClassName) == 'StagingModelMetadata') {
            $routing_key = self::MMD_ROUTING_KEY;
            $replicateDataList = $objects;
        } elseif (end($entityClassName) == 'WorkflowCollection') {
            $routing_key = self::PLAYBOOK_ALIAS_ROUTING_KEY;
            $replicateDataList = $this->processPlaybookCollections($objects, $replicateDataList, $method);
        }
        if ($replicateData && count($replicateDataList) > 0) {
            $this->sendReplicateData($contextRef, $replicateDataList, $method, $routing_key);
        }
    }

    protected function processSyncRequest($objects)
    {
        foreach ($objects as $obj) {
            $obj->new_data['__sync'] = true;
        }
        return $objects;
    }

    protected function processPlaybook($objects, $replicateDataList)
    {
        $remoteExecutableNew = false;
        $remoteExecutableOld = false;
        foreach ($objects as $obj) {
            if ($obj->new_data) {
                $remoteExecutableNew = $obj->new_data['remoteExecutableFlag'];
            }
            if ($obj->old_data) {
                $remoteExecutableOld = $obj->old_data['remoteExecutableFlag'];
            }
            if ($remoteExecutableNew == true || $remoteExecutableOld == true) {
                if ($obj->new_data) {
                    unset($obj->new_data['steps']);
                    unset($obj->new_data['routes']);
                    unset($obj->new_data['versions']);
                }
                if ($obj->old_data) {
                    unset($obj->old_data['steps']);
                    unset($obj->old_data['routes']);
                    unset($obj->new_data['versions']);
                }
                array_push($replicateDataList, $obj);
            }
        }
        return $replicateDataList;
    }

    protected function processPlaybookCollections($objects, $replicateDataList, $method)
    {
        if ($method != 'PUT') {
            foreach ($objects as $wf_collection_obj) {
                if (isset($wf_collection_obj->new_data) && count($wf_collection_obj->new_data)) {
                    $processObject = $wf_collection_obj->new_data;
                } elseif (isset($wf_collection_obj->old_data) && count($wf_collection_obj->old_data)) {
                    $processObject = $wf_collection_obj->old_data;
                } else {
                    return [];
                }
                if (isset($processObject['workflows']) && count($processObject['workflows'])) {
                    foreach ($processObject['workflows'] as $wf_object) {
                        if ($wf_object['remoteExecutableFlag']) {
                            unset($wf_object['steps']);
                            unset($wf_object['routes']);
                            unset($wf_object['versions']);
                            if ($method == 'POST') {
                                $diffObj = $this->prepareReplicationDiffData($wf_object);
                            } elseif ($method == 'DELETE') {
                                $diffObj = $this->prepareReplicationDiffData([], $wf_object);
                            }
                            array_push($replicateDataList, $diffObj);
                        }
                    }
                }
            }
        }
        return $replicateDataList;
    }

    public function prepareReplicationDiffData($updatedObject, $preUpdatedObject = [])
    {
        return (object) [
            "old_data" => $preUpdatedObject,
            "new_data" => $updatedObject,
        ];
    }

    /**
     * @param $contextRef
     * @param $object
     * @param $method
     * @param $routing_key
     */
    public function sendReplicateData($contextRef, $object, $method, $routing_key)
    {
        $groupedObj = (object) null;
        foreach ($object as $obj) {
            $objData = $obj->new_data ? $obj->new_data : $obj->old_data;
            $destinationIds = $this->getDestinationIds($contextRef, $objData, $method);
            if (!$destinationIds) {
                continue;
            }
            foreach ($destinationIds as $id) {
                if (isset($groupedObj->$id)) {
                    $merge = array_merge($groupedObj->$id, [$obj]);
                    $groupedObj->$id = $merge;
                } else {
                    $groupedObj->$id = [$obj];
                }
            }
        }
        foreach ($groupedObj as $key => $value) {
            $replicationData = $this->prepareReplicationData($value, $method, $this->getTenantObject($contextRef)->getTenantId(), $key);
            $this->sendReplicationData($contextRef, $replicationData, $routing_key);
        }
    }

    /**
     * @param $contextRef
     * @param $Object
     * @param $method
     * @return array
     */
    protected function getDestinationIds($contextRef, $Object, $method)
    {
        $ids = array();
        $tenantUuid = null;
        $tenantResource = $contextRef->identifyResourceByShortName('tenant');
        if (isset($Object['tenant']) && $Object['tenant']) {
            $tenant = $Object['tenant'];
            $tenantArray = explode('/', $tenant);
            $tenantUuid = end($tenantArray);
        }
        $tenantObj = $this->getTenantObj($contextRef, $tenantResource, $tenantUuid);
        if (count($tenantObj)) {
            $tenantObj = $tenantObj[0];
            if ($tenantObj->getRole() == 'self' || $Object['@type'] == 'Workflow') {
                $tenantFilter = ['role' => 'master', 'isDedicated' => true];
                $destinationTenants = $this->findWithWhere($contextRef, $tenantResource, $tenantFilter);
                if ($destinationTenants) {
                    foreach ($destinationTenants as $destinationTenant) {
                        $masterId = $destinationTenant->getTenantId();
                        array_push($ids, $masterId);
                    }
                }
            } elseif ($tenantObj->getTenantId() && $tenantObj->getIsDedicated()) {
                array_push($ids, $tenantObj->getTenantId());
            }
        }
        return $ids;
    }


    /**
     * @param $contextRef
     * @param $tenantResource
     * @param $tenantUuid
     * @return array|null|object
     */
    protected function getTenantObj($contextRef, $tenantResource, $tenantUuid)
    {
        if ($tenantUuid) {
            $filter = ['uuid' => $tenantUuid];
        } else {
            $filter = ['role' => 'self'];
        }
        $tenantObj = $this->findWithWhere($contextRef, $tenantResource, $filter);
        if (count($tenantObj)) {
            return $tenantObj;
        }
        return [];
    }

    /**
     * @param $object
     * @param $method
     * @param $sourceId
     * @param $destinationId
     * @return array
     */
    public function prepareReplicationData($object, $method, $sourceId, $destinationId)
    {
        return [
            'data' => $object,
            'method' => $method,
            'sourceId' => $sourceId,
            'destinationId' => $destinationId
        ];
    }

    public function sendReplicationData($contextRef, array $data, $routing_key)
    {
        $contextRef->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->setContentType('application/json');
        $contextRef->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->publish(json_encode($data), $routing_key);
    }

    /**
     * @param $contextRef
     * @param ResourceInterface $resource
     * @param $filters
     * @return array|null|object
     */
    public function findWithWhere($contextRef, $resource, $filters)
    {
        /** @var AggregateDataProviderInterface $aggregateDataProvider */
        $aggregateDataProvider = $contextRef->container->get('cybersponse.query.data_provider');
        $item = $aggregateDataProvider->getItemWithWhere($resource, $filters, true);
        if (!$item) {
            return [];
        }
        return $item;
    }
}
