<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\UtilsBundle;

use Cybersponse\AuthenticationBundle\Hmac\HmacUserToken;
use Ramsey\Uuid\Uuid;
use Cybersponse\UtilsBundle\TenantUtils;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\HttpException;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Lexik\Bundle\JWTAuthenticationBundle\Security\Authentication\Token\JWTUserToken;

class Utils extends TenantUtils
{
    const DATA_ROUTING_KEY = 'route.crudhub.data.remoterequest';
    const PLAYBOOK_ALIAS_ROUTING_KEY = 'route.crudhub.instructions.remoterequest.playbookaliaspush';
    const POSTMAN_SECRET_KEY = 'eW1hSe7mnlIi1a0mmgno1nSe9Rea7pphG2oAdmCeBAoduYnd';
    const MASTER_ROLE_NAME = 'master';

    public static function isPasswordBlock($request)
    {
        $postman_secret_key = self::POSTMAN_SECRET_KEY;
        if (!$request->query->has("\$secretKey")) {
            return true;
        } else {
            $SECRET_KEY = $request->query->get("\$secretKey");
            $request->query->remove("\$secretKey");
            if ($SECRET_KEY != $postman_secret_key) {
                return true;
            }
        }
        return false;
    }

    public function sendData($contextRef, array $data, $key = 'cyops.crudhub.datanotify')
    {
        $contextRef->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->setContentType('application/json');
        $contextRef->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->publish(json_encode($data), $key);
    }

    public function sendSearchAction($contextRef, array $data, $key = 'cyops.crudhub.essearch')
    {
        $request_uuid = Uuid::uuid4()->toString();
        $client = $contextRef->get('old_sound_rabbit_mq.parallel_rpc');
        try {
            $client->addRequest(json_encode($data), 'cyops.crudhub.datanotify', $request_uuid, $key, 50);
            return json_decode($client->getReplies()[$request_uuid]);
        } catch (\Exception $exception) {
            throw new HttpException(Response::HTTP_BAD_REQUEST, 'Search Indexing is in progress, please try after some time');
        }
    }

    public static function prepare_filter_data($requestBody, $filterItems)
    {
        $filter = [];
        foreach ($filterItems as $item) {
            if (is_string($item)) {
                $filter[$item] = $requestBody[$item];
            } else {
                $filter = array_merge($filter, get_object_vars($item));
            }
        }
        return $filter;
    }
    public function broadcastToHA($contextRef, $commandId, $command)
    {
        $dasUri = $contextRef->getParameter('das.uri');
        $proxyProvider = $contextRef->get('cybersponse.core.provider.proxy');
        $dasClient = $proxyProvider->getDasClient();
        $objectData = array(
            "commandId" => $commandId,
            "command" => $command
        );
        $headers = array(
            "Content-type" => "application/json"
        );
        $dasRequests = new GuzzleRequest('post', $dasUri . '/clustercommand/', $headers, json_encode($objectData));
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $resCode = $dasResponse->getStatusCode();
            if ($resCode > 299) {
                return 0;
            }
        } catch (\Exception $error) {
            error_log($error->getMessage());
        }
        return true;
    }

    public static function findRecordWithClassName($contextRef, string $className, string $uuid)
    {
        /** @var EntityManager $entityManager */
        $entityManager = $contextRef->get('doctrine')->getManagerForClass($className);
        $repository = $entityManager->getRepository($className);
        return $repository->findOneBy(['uuid' => $uuid]);
    }

    public static function setRefreshedToken($contextRef, $rawToken, $tokenType='JWT')
    {

        $authenticationManager = $contextRef->get('security.authentication.manager');
        $tokenStorage = $contextRef->get('security.token_storage');
        if ($tokenType == 'JWT') {
            $token = new JWTUserToken();
            $token->setRawToken($rawToken);
        } else {
            $token = new HmacUserToken();
            $token->setPayload($rawToken);
        }
        $authToken = $authenticationManager->authenticate($token);
        $tokenStorage->setToken($authToken);
    }

    /**
     * @return string
     */
    public static function extractUserName($user)
    {
        if (method_exists($user, "getfirstname")) {
            if (method_exists($user, "getlastname")) {
                $userName = $user->getfirstname() . ' ' . $user->getlastname();
            } else {
                $userName = $user->getfirstname();
            }
        } else {
            $userName = $user->getname();
        }
        return $userName;
    }
}
