<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ViewTemplateBundle\Controller;;

use Cybersponse\UtilsBundle\ARCUtils;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Cybersponse\QueryBundle\Query\ExpressionBuilder;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Cybersponse\ViewTemplateBundle\Entity\UserDefaultTemplate;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Cybersponse\ViewTemplateBundle\Provider\GenericTemplateProvider;
use Cybersponse\CoreBundle\Controller\AuthorizationResourceController;

class GenericTemplateController extends AuthorizationResourceController
{

    protected $allowedType = ['dashboard', 'reporting'];
    const RELATIONSHIP_KEY = "\$relationships";
    const ENABLED = 'enabled';

    /**
     * @param $type String
     */
    private function isAllowedType($type)
    {
        if (!in_array($type, $this->allowedType)) {
            throw new BadRequestHttpException("Invalid Type.");
        }
    }

    /**
     * @return GenericTemplateProvider
     */
    private function templateProvider()
    {
        return $this->get('cybersponse.generic_template.provider.generic_template');
    }

    /**
     * @param array genericTemplate
     * @return array template
     */
    private function filterRoles($genericTemplate)
    {
        $genericTemplateProvider = $this->templateProvider();
        if (array_key_exists('roles', $genericTemplate)) {
            $validRoleIds = $genericTemplateProvider->getAllRoles();
            $assignedRoles = $genericTemplate['roles'];
            $genericTemplate['roles'] = array_values(array_intersect($assignedRoles, $validRoleIds));
        }
        return $genericTemplate;
    }


    /**
     * @param String $permissionType
     */
    private function checkAuthorized($permissionType)
    {
        $authorizationChecker = $this->container->get('security.authorization_checker');
        if (!$authorizationChecker->isGranted(['perm:security.' . $permissionType])) {
            return false;
        }
        return true;
    }

    public function createDefaultUserTemplateAction(Request $request, $type, $id)
    {
        $this->isAllowedType($type);
        $genericTemplateProvider = $this->templateProvider();
        $resource = $this->getResource($request);
        $genericTemplate = $this->findOrThrowNotFound($resource, $id);
        $savedTemplate = $genericTemplateProvider->saveDefaultUserTemplate($type, $id, $genericTemplate);
        if ($savedTemplate) {
            return JsonResponse::create('Success');
        } else {
            return new Response(null, Response::HTTP_FORBIDDEN);
        }
    }

    public function getDefaultUserTemplateAction(Request $request, $type)
    {
        $this->isAllowedType($type);
        $genericTemplateProvider = $this->templateProvider();
        $resource = $this->getResource($request);
        $user = $genericTemplateProvider->getCurrentUser()->getUuid();
        ARCUtils::setAsJson($request, false);
        ARCUtils::setRelationShip($request, false);
        $user = $genericTemplateProvider->getCurrentUser()->getUuid();
        $request->query->set("userId", $user);
        /** @var QueryProvider $queryProvider */
        $queryProvider = $this->container->get('cybersponse.query.provider.query_provider');
        $accessibleQueryFilter =
            ExpressionBuilder::logicOr(
                ExpressionBuilder::field('userId')->equals($user)
            );
        $queryProvider->setAdditionalFilters($accessibleQueryFilter->toArray());
        return parent::cgetAction($request);
    }

    public function deleteDefaultUserTemplateAction($type)
    {
        $this->isAllowedType($type);
        $genericTemplateProvider = $this->templateProvider();
        $user = $genericTemplateProvider->getCurrentUser()->getUuid();
        $templateEntityManager = $this->container->get('doctrine.orm.entity_manager');
        $templateRepo = $templateEntityManager->getRepository(UserDefaultTemplate::class);
        $template = $templateRepo->findOneBy(array("userId" => $user));
        if (!is_null($template)) {
            $genericTemplateProvider->deleteTemplate($template);
            return JsonResponse::create('Success');
        } else {
            return new Response(null, Response::HTTP_NOT_FOUND);
        }
    }

    public function resetTemplateAction(Request $request, $type, $id)
    {
        $this->isAllowedType($type);
        $genericTemplateProvider = $this->templateProvider();
        $resource = $this->getResource($request);
        $genericTemplate =  $this->findOrThrowNotFound($resource, $id);
        $parentTemplateId = $genericTemplate->getParentTemplateId();
        if (!empty($parentTemplateId) && !is_null($parentTemplateId)) {
            $parentDashBoard = $this->findOrThrowNotFound($resource, $parentTemplateId);
            $genericTemplate->setConfig($parentDashBoard->getConfig());
            $genericTemplateProvider->saveTemplate($genericTemplate);
        }
        return JsonResponse::create('Success');
    }
}
