<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ViewTemplateBundle\Controller;

use Cybersponse\ViewTemplateBundle\Entity\UserViewTemplate;
use Cybersponse\ViewTemplateBundle\Provider\ViewTemplateProvider;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Class ViewTemplateController
 */
class ViewTemplateController extends Controller
{
    /**
     * @param $name
     * @return JsonResponse
     */
    public function getAction($name)
    {
        $serializer = $this->get('serializer');

        $viewTemplate = $this->findOrThrowNotFound($name);

        $normalizedViewTemplate = $serializer->normalize($viewTemplate, 'json');

        return JsonResponse::create($normalizedViewTemplate);
    }

    /**
     * @param $name
     * @return JsonResponse
     */
    public function setAction($name, Request $request)
    {
        /** @var ViewTemplateProvider $viewTemplateProvider */
        $viewTemplateProvider = $this->get('cybersponse.view_template.provider.view_template');
        $serializer = $this->get('serializer');

        /** @var UserViewTemplate $userViewTemplate */
        $userViewTemplate = $serializer->deserialize(
            $request->getContent(),
            UserViewTemplate::class,
            'json',
            ['object_to_populate' => $this->findOrThrowNotFound($name)]
        );

        $viewTemplateProvider->saveViewTemplate($name, $userViewTemplate);

        $normalizedViewTemplate = $serializer->normalize($userViewTemplate, 'json');

        return JsonResponse::create($normalizedViewTemplate);
    }

    /**
     * @param $name
     * @return mixed
     */
    private function findOrThrowNotFound($name)
    {
        /** @var ViewTemplateProvider $viewTemplateProvider */
        $viewTemplateProvider = $this->get('cybersponse.view_template.provider.view_template');

        $viewTemplate = $viewTemplateProvider->getViewTemplate($name);

        if (is_null($viewTemplate)) {
            throw new NotFoundHttpException();
        }
        return $viewTemplate;
    }
}
