<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ViewTemplateBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Cybersponse\CoreBundle\Entity\BaseEntity;
use Cybersponse\AuthorizationBundle\Entity\Role;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize;
use Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize;
/**
 * @ORM\Table(name="generic_template")
 * @ORM\Entity
 * @ORM\InheritanceType("SINGLE_TABLE")
 * @ORM\DiscriminatorMap({"GenericTemplate"= "GenericTemplate","Dashboard"="Dashboard", "Reporting"="Reporting"})
 * @ORM\DiscriminatorColumn(name="genericType", type="string", length=20)
 * @Doctrine\ORM\Mapping\Entity(repositoryClass="Cybersponse\ViewTemplateBundle\Repository\GenericTemplateRepository")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="generic_template")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="application")
 */
class GenericTemplate extends BaseEntity
{

    use \Cybersponse\CoreBundle\Traits\Trackable;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`displayname`", type="string", length=255, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Display Name")
     */
    protected $displayName;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`type`", type="string", length=255, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Type")
     */
    protected $type;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`templatetype`", type="string", length=255, nullable=true, unique=false)
     * @Symfony\Component\Validator\Constraints\NotBlank()
     * @Symfony\Component\Validator\Constraints\NotNull()
     * @Symfony\Component\Validator\Constraints\Regex(pattern="/^ *$/",
     * match=false,
     * message="Required value should not contain only space.")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Template Type")
     */
    protected $templateType;

    /**
     * @var Role[]|Collection
     *
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="Cybersponse\AuthorizationBundle\Entity\Role", cascade={"persist"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Role Ids")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="roles")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="manyToMany")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     */
    protected $roles ;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`parenttemplateid`", type="string", length=255, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Parent Template Id")
     */
    protected $parentTemplateId;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`config`", type="json_array", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Config")
     */
    protected $config;
    
    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`filterarray`", type="jsonb", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Filter Array")
     */
    protected $filterArray;

    /**
     * @var \DateTime
     *
     * @Doctrine\ORM\Mapping\Column(name="updatedate", type="datetime", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="datetime")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Update Date")
     */
    protected $updateDate;
    
    /**
     * Constructor
     */
    public function __construct()
    {
        $this->roles = new \Doctrine\Common\Collections\ArrayCollection();
    }
    /**
     * Get displayName
     *
     * @return string $displayName
     */
    public function getDisplayName()
    {
        return $this->displayName;
    }

    /**
     *
     * @param string $displayName
     * @return self
     */
    public function setDisplayName($displayName)
    {
        $this->displayName = $displayName;
        return $this;
    }

    /**
     * Get config
     *
     * @return array $config
     */
    public function getConfig()
    {
        return $this->config;
    }

    /**
     * Set config
     *
     * @param array $config
     * @return self
     */
    public function setConfig($config)
    {
        $this->config = $config;
        return $this;
    }

    /**
     *
     * @return array $roles
     */
    public function getRoles()
    {
        return $this->roles;
    }

     /**
     * Add roles
     *
     * @param \Cybersponse\AuthorizationBundle\Entity\Role $roleId
     * @return GenericTemplate
     */
    public function addRole(Role $role)
    {
        if (!$this->roles->contains($role)) {
            $this->roles[] = $role;
           
        }
        return $this;
    }

    /**
     * Remove Roles
     *
     * @param \Cybersponse\AuthorizationBundle\Entity\Role $roles
     */
    public function removeRole(Role $role)
    {
        if ($this->roles->contains($role)) {
            $this->roles->removeElement($role);
        }
    }



    /**
     * Get updateDate
     *
     * @return int $uploadDate
     */
    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * Set updateDate
     *
     * @param int $updateDate
     * @return self
     */
    public function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
        return $this;
    }

    /**
     * @return String
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param String $type
     * @return self
     */
    public function setType($type)
    {
        $this->type = $type;
        return $this;
    }

    /**
     * @return String
     */
    public function getTemplateType()
    {
        return $this->templateType;
    }

    /**
     * @param String $templateType
     * @return self
     */
    public function setTemplateType($templateType)
    {
        $this->templateType = $templateType;
        return $this;
    }

    /**
     * Get the value of filterArray
     *
     * @return  array
     */ 
    public function getFilterArray()
    {
        return $this->filterArray;
    }

    /**
     * Set the value of filterArray
     *
     * @param  array  $filterArray
     *
     * @return  self
     */ 
    public function setFilterArray($filterArray)
    {
        $this->filterArray = $filterArray;

        return $this;
    }

    /**
     * Get the value of parentTemplateId
     *
     * @return  string
     */ 
    public function getParentTemplateId()
    {
        return $this->parentTemplateId;
    }

    /**
     * Set the value of parentTemplateId
     *
     * @param  string  $parentTemplateId
     *
     * @return  self
     */ 
    public function setParentTemplateId($parentTemplateId)
    {
        $this->parentTemplateId = $parentTemplateId;

        return $this;
    }
}
