<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ViewTemplateBundle\Provider;

use Cybersponse\ViewTemplateBundle\Entity\UserPreference;
use Cybersponse\ViewTemplateBundle\Repository\UserPreferenceRepository;
use Dunglas\ApiBundle\Api\IriConverterInterface;
use Doctrine\ORM\EntityManager;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Component\Serializer\SerializerInterface;

class UserPreferenceProvider
{
    /** @var TokenStorageInterface */
    protected $tokenStorage;

    /** @var IriConverterInterface */
    protected $iriConverter;

    /** @var EntityManager */
    protected $entityManager;


    /** @var SerializerInterface */
    protected $serializer;

    /**
     * @param TokenStorageInterface $tokenStorage
     */
    public function setTokenStorage($tokenStorage)
    {
        $this->tokenStorage = $tokenStorage;
    }

    /**
     * @param IriConverterInterface $iriConverter
     */
    public function setIriConverter($iriConverter)
    {
        $this->iriConverter = $iriConverter;
    }

    /**
     * @param EntityManager $entityManager
     */
    public function setViewTemplateEntityManager($entityManager)
    {
        $this->entityManager = $entityManager;
    }


    /**
     * @param SerializerInterface $serializer
     */
    public function setSerializer($serializer)
    {
        $this->serializer = $serializer;
    }

    /**
     * * @param array $userPrefBody
     * @return UserPreference $userPref
     *
     */
    public function saveUserPref($userPrefBody)
    {
        $userPref = $this->getUserPref();
        if (!is_null($userPref)) {
            $userPref->setPreferences($userPrefBody);
        } else {
            $userPref = new UserPreference();
            $userPref->setUserId($this->getCurrentUserAsIri());
            $userPref->setPreferences($userPrefBody);
        }
        $timestamp = new \DateTime('NOW', new \DateTimeZone('UTC'));
        $userPref->setUpdateDate($timestamp);

        $this->entityManager->persist($userPref);
        $this->entityManager->flush();
        return $userPref;
    }

    /**
     * @return UserPreference
     */
    public function getUserPref()
    {
        /** @var UserPreferenceRepository $userPrefRepository */
        $userPrefRepository = $this->entityManager->getRepository(UserPreference::class);
        $userId = $this->getCurrentUserAsIri();
        $template = $userPrefRepository->getUserPreferences($userId);
        return $template;
    }

    /**
     * @return string
     */
    private function getCurrentUserAsIri()
    {
        $user = $this->tokenStorage->getToken()->getUser();
        $iri = $this->iriConverter->getIriFromItem($user);
        return $iri;
    }
}
