<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\ViewTemplateBundle\Provider;


use Cybersponse\ViewTemplateBundle\Entity\SystemViewTemplate;
use Cybersponse\ViewTemplateBundle\Entity\UserViewTemplate;
use Cybersponse\ViewTemplateBundle\Repository\SystemViewTemplateRepository;
use Cybersponse\ViewTemplateBundle\Repository\UserViewTemplateRepository;
use Dunglas\ApiBundle\Api\IriConverterInterface;
use Doctrine\ORM\EntityManager;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Component\Serializer\SerializerInterface;

class ViewTemplateProvider
{
    /** @var TokenStorageInterface */
    protected $tokenStorage;

    /** @var IriConverterInterface */
    protected $iriConverter;

    /** @var EntityManager */
    protected $entityManager;

    /** @var SerializerInterface */
    protected $serializer;

    /**
     * @param TokenStorageInterface $tokenStorage
     */
    public function setTokenStorage($tokenStorage)
    {
        $this->tokenStorage = $tokenStorage;
    }

    /**
     * @param EntityManager $entityManager
     */
    public function setViewTemplateEntityManager($entityManager)
    {
        $this->entityManager = $entityManager;
    }

    /**
     * @param IriConverterInterface $iriConverter
     */
    public function setIriConverter($iriConverter)
    {
        $this->iriConverter = $iriConverter;
    }

    /**
     * @param SerializerInterface $serializer
     */
    public function setSerializer($serializer)
    {
        $this->serializer = $serializer;
    }

    /**
     * @return string
     */
    private function getCurrentUserAsIri()
    {
        $user = $this->tokenStorage->getToken()->getUser();
        $iri = $this->iriConverter->getIriFromItem($user);

        return $iri;
    }

    /**
     * @param $name
     *
     * @return SystemViewTemplate|UserViewTemplate
     */
    public function getViewTemplate($name)
    {
        /** @var UserViewTemplateRepository $userViewTemplateRepository */
        $userViewTemplateRepository = $this->entityManager->getRepository(UserViewTemplate::class);

        $viewTemplate = $userViewTemplateRepository->getUserViewTemplate(
            $this->getCurrentUserAsIri(),
            $name
        );

        if (is_null($viewTemplate)) {
            /** @var SystemViewTemplateRepository $systemViewTemplateRepository */
            $systemViewTemplateRepository = $this->entityManager->getRepository(SystemViewTemplate::class);

            $viewTemplate = $systemViewTemplateRepository->getSystemViewTemplate($name);
        }

        return $viewTemplate;
    }

    /**
     * @param UserViewTemplate $viewTemplate
     */
    public function saveViewTemplate($name, $viewTemplate)
    {
        $viewTemplate->setName($name);
        $viewTemplate->setOwner($this->getCurrentUserAsIri());

        $this->entityManager->persist($viewTemplate);
        $this->entityManager->flush();
    }
}
