<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\WorkflowBundle\Controller;


use Cybersponse\QueryBundle\Provider\QueryProvider;
use Cybersponse\QueryBundle\Query\ExpressionBuilder;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;
use Cybersponse\QueryBundle\Query\Query;
use Cybersponse\WorkflowBundle\Entity\Workflow;
use Dunglas\ApiBundle\Api\ResourceCollectionInterface;
use Dunglas\ApiBundle\JsonLd\Response;
use Cybersponse\MetadataBundle\Entity\StagingModelMetadata;
use Doctrine\Common\Inflector\Inflector;
use Dunglas\ApiBundle\Model\DataProviderInterface;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Serializer\Serializer;

class WorkflowActionsController extends Controller
{

    const CLASSNAME = 'workflow';

    /**
     * @param string $type
     * @return StagingModelMetadata|null
     */
    protected function getClassNameByType($type)
    {
        /** @var ManagerRegistry $managerRegistry */
        $managerRegistry = $this->container->get('doctrine');
        $queryManager = $managerRegistry->getManagerForClass(StagingModelMetadata::class);
        $queryRepository = $queryManager->getRepository(StagingModelMetadata::class);
        return $queryRepository->findOneBy(array('type' => $type));
    }

    /**
     * @param $resourceShortName
     * @return ResourceInterface|null
     */
    protected function getResourceFromShortName($resourceShortName)
    {
        $resourceClassName = Inflector::singularize(Inflector::classify($resourceShortName));
        /** @var ResourceCollectionInterface $resourceCollection */
        $resourceCollection = $this->container->get('api.resource_collection');
        $resource = $resourceCollection->getResourceForShortName($resourceClassName);

        if ($resource === null) {
            $model = $this->getClassNameByType($resourceShortName);
            if ($model === null) {
                return null;
            }
            $singular = $model->getDescriptions()['singular'];
            $escapedSingular = preg_replace('/[^a-z0-9\s]+/i', '', $singular);
            $resourceName = \Doctrine\Common\Util\Inflector::classify($escapedSingular);
            $resource = $resourceCollection->getResourceForShortName($resourceName);
        }
        return $resource;
    }

    /**
     * @param Request $request
     * @return Response|static
     */
    public function getAction(Request $request)
    {
        $resource = $this->getResourceFromShortName(self::CLASSNAME);
        $authorizationProvider = $this->get('cybersponse.authorization.authorization_provider');
        $authorizationProvider->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_EXECUTE);
        $query = $this->buildActionQuery($request);
        /** @var QueryProvider $queryProvider */
        $queryProvider = $this->container->get('cybersponse.query.provider.query_provider');
        $queryProvider->setInitialQuery($query->toArray());

        $workflowResource = $this->getWorkflowResource();
        $workflowActions = $this->getWorkflowActions($request, $workflowResource);

        /** @var Serializer $serializer */
        $serializer = $this->container->get('serializer');

        $context = $workflowResource->getNormalizationContext();
        $context += [
            'ignore_authorization' => true,
            'ignore_field_authorization' => true,
            'ignore_relationship_authorization' => true,
            'request_uri' => $request->getRequestUri()
        ];

        $normalizedActions = $serializer->normalize($workflowActions, 'json-ld', $context);

        return Response::create($normalizedActions);
    }

    /**
     * @return \Dunglas\ApiBundle\Api\ResourceInterface|null
     */
    protected function getWorkflowResource()
    {
        /** @var ResourceCollectionInterface $resourceCollection */
        $resourceCollection = $this->get('api.resource_collection');
        return $resourceCollection->getResourceForEntity(Workflow::class);
    }

    /**
     * @param Request $request
     * @return Query
     */
    protected function buildActionQuery(Request $request)
    {
        $query = ExpressionBuilder::logicAnd()
            ->addFilter(ExpressionBuilder::field('isActive')->equals(true))
            ->addFilter(ExpressionBuilder::field('triggerStep.stepType.name')->equals('cybersponse.action'));

        $type = $request->query->get('type');
        $sort['field'] = "name";
        $sort['direction'] = "ASC";
        $query->addSort($sort);
        if ($type) {
            $query->addFilter(ExpressionBuilder::field('triggerStep.arguments.resources')->exists($type));
            return $query;
        }
        return $query;
    }

    /**
     * @param Request $request
     * @param $workflowResource
     * @return array|\Dunglas\ApiBundle\Model\PaginatorInterface|\Traversable
     */
    protected function getWorkflowActions(Request $request, $workflowResource)
    {
        /** @var DataProviderInterface $dataProvider */
        $dataProvider = $this->container->get('cybersponse.query.data_provider');
        return $dataProvider->getCollection($workflowResource, $request);
    }
}
