<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\WorkflowBundle\Controller;


use Cybersponse\UtilsBundle\Utils;
use Cybersponse\UtilsBundle\ARCUtils;
use Cybersponse\WorkflowBundle\Api\RouteResource;
use Cybersponse\WorkflowBundle\Constants\Triggers;
use Symfony\Component\HttpFoundation\JsonResponse;
use Cybersponse\ProxyBundle\GuzzleHttp\ProxyClient;
use Cybersponse\WorkflowBundle\Subscriber\DataEvent;
use Dunglas\ApiBundle\Api\ResourceCollectionInterface;
use Cybersponse\ProxyBundle\Provider\SamlActorProvider;
use Cybersponse\MetadataBundle\Entity\StagingModelMetadata;
use Cybersponse\CoreBundle\Controller\AuthorizationResourceController;
use Cybersponse\AuthorizationBundle\Permissions\PermissionsDictionaryBuilder;

use Doctrine\Common\Inflector\Inflector;
use Doctrine\Common\Persistence\ManagerRegistry;
use Dunglas\ApiBundle\Api\ResourceInterface;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\HttpFoundation\Request;
use GuzzleHttp\Exception\BadResponseException;


class WorkflowTriggerController extends AuthorizationResourceController
{

    const CLASSNAME = 'workflow';

    const DELTA_DATA = 'data';
    const HTTP_METHOD = 'operation';
    const REMOTE_ADDR = 'source';
    const WEB_SOCKET_SESSION_ID = 'websocketId';
    const COMPONENT = 'component';
    const UUID = 'entityUuid';
    const USER_UUID = 'userId';
    const USER = 'user';
    const TRANSACTION_DATE = 'transactionDate';
    const PLAYBOOK_IRI = 'playbookIri';
    const PLAYBOOK_NAME = 'playbookName';

    /**
     * @param $route
     * @param $action
     * @param Request $request
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function apiAction($route, $action, Request $request)
    {
        $body = $request->getContent();
        $data = json_decode($body, true);
        if ($this->getUser() != null) {
            $resource = $this->getResourceFromShortName(self::CLASSNAME);
            $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_EXECUTE);
            if (method_exists($this->getUser(), "getfirstname")) {
                $headerObj['X-RUNBYUSER'] = '/api/3/people/' . $this->getUser()->getUuid();
            } else {
                $headerObj['X-RUNBYUSER'] = '/api/3/appliance/' . $this->getUser()->getUuid();
            }
        } else if (isset($_SERVER['PHP_AUTH_USER'])) {
            $loginId = $_SERVER['PHP_AUTH_USER'];
            $userId = $this->getUserId($loginId);
            if ($userId != null) {
                $userUuid = $this->getuuid($userId);
                $headerObj['X-RUNBYUSER'] = '/api/3/people/' . $userUuid;
            } else {
                $headerObj['X-RUNBYUSER'] = "";
            }
        } else {
            $headerObj['X-RUNBYUSER'] = "";
        }
        $headerObj['X-TRIGGERTYPE'] = Triggers::API_CALL;
        $routeResource = new RouteResource($route);
        $dataEvent = new DataEvent($routeResource, is_null($data) ? $body : $data, null, $headerObj);
        $this->get(self::EVENT_DISPATCHER)->dispatch(Triggers::API_CALL, $dataEvent);

        return $dataEvent->getResponse();
    }

    /**
     * @param String $loginId
     * @return String
     */
    private function getUserId($loginId)
    {
        $client = $this->findHandlerForRoute('auth');
        $body = ["json" => ["loginid" => $loginId]];
        $userRequestUri = sprintf("%s/%s", $client->getTargetUri(), 'users');
        $response = null;
        try {
            $response = $client->request('POST', $userRequestUri, $body);
            $respBody = json_decode($response->getBody()->getContents(), true);
            return $respBody['uuid'];
        } catch (BadResponseException $exception) {
            return null;
        }
    }

    /**
     * @param $routeIdentifier
     * @return ProxyClient
     */
    public function findHandlerForRoute($routeIdentifier)
    {
        $handler = $this->findHandlerMapForRoute($routeIdentifier);

        return $this->container->get($handler[$routeIdentifier]['handlerName']);
    }


    /**
     * @param $routeIdentifier
     * @return mixed
     * @throws NotFoundHttpException
     */
    private function findHandlerMapForRoute($routeIdentifier)
    {
        $handlerMap = $this->container->getParameter('cybersponse.proxy.handler.map');

        if (!array_key_exists($routeIdentifier, $handlerMap)) {
            throw new NotFoundHttpException();
        }
        return $handlerMap;
    }

    private function getuuid($userId)
    {
        /** @var SamlActorProvider $samlProvider */
        $actorProvider = $this->get('cybersponse.saml_proxy.provider.samlactor');
        $actor = $actorProvider->getActor($userId);
        return $actor->getUuid();
    }

    /**
     * @param ResourceInterface $resource
     * @param string|array $actions
     * @param object|null $object
     */
    protected function denyAccessUnlessGranted(ResourceInterface $resource, $actions, $object = null)
    {
        /** @var AuthorizationProvider $authorizationProvider */
        $authorizationProvider = $this->get('cybersponse.authorization.authorization_provider');
        $authorizationProvider->denyAccessUnlessGranted($resource, $actions, $object);
    }

    /**
     * @param $route
     * @param $action
     * @param Request $request
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function manualAction($route, $action, Request $request)
    {
        $resource = $this->getResourceFromShortName(self::CLASSNAME);
        $this->denyAccessUnlessGranted($resource, PermissionsDictionaryBuilder::PERM_CRUD_EXECUTE);
        $audit_array = [];
        $body = $request->getContent();
        $data = $this->extractDataFromRequest($request, json_decode($body, true));
        $workflowObj = $this->fetch_workflow_data($data, $request);
        $routeResource = new RouteResource($route);
        $trigger = Triggers::ACTION;
        $body = json_decode($body, true);
        $headerObj['X-TRIGGERTYPE'] = Triggers::ACTION;
        if (isset($body['singleRecordExecution']) && $body['singleRecordExecution'] == true) {
            $task_ids = [];
            $records = $data['records'];
            $clonedRecordData = $data;
            unset($clonedRecordData['records']);
            foreach ($records as $record) {
                $headerObj['X-RUNBYUSER'] = '/api/3/people/' . $this->getUser()->getUuid();
                $clonedRecordData['records'] = [$record];
                array_push($audit_array, $this->prepareChangeData(
                    $clonedRecordData['records'],
                    (object)null,
                    'Trigger',
                    $workflowObj['@id'],
                    $workflowObj['name']
                ));
                $dataEvent = new DataEvent($routeResource, is_null($data) ? $body : $clonedRecordData, null, $headerObj);
                $this->get(self::EVENT_DISPATCHER)->dispatch($trigger, $dataEvent);
                array_push($task_ids, json_decode($dataEvent->getResponse()->getContent())->task_id);
            }
            $utilsObj = new Utils();
            $utilsObj->sendData($this, $audit_array);
            return new JsonResponse([
                'task_ids' => $task_ids
            ]);
        } else {
            $headerObj['X-RUNBYUSER'] = '/api/3/people/' . $this->getUser()->getUuid();
            $dataEvent = new DataEvent($routeResource, is_null($data) ? $body : $data, null, $headerObj);
            $this->get(self::EVENT_DISPATCHER)->dispatch($trigger, $dataEvent);
            array_push($audit_array, $this->prepareChangeData(
                $data['records'],
                (object)null,
                'Trigger',
                $workflowObj['@id'],
                $workflowObj['name']
            ));
            $utilsObj = new Utils();
            $utilsObj->sendData($this, $audit_array);
            return $dataEvent->getResponse();
        }
    }

    public function getHeaders($headers) {
        $header_list = array();
        foreach($headers as $header=>$value) {
            $header_list[$header] = $value[0];
        }
        return $header_list;
    }

    /**
     * @inheritdoc
     */
    public function noTriggerExecuteAction(Request $request, $workflowId)
    {
        $body = json_decode($request->getContent(), true);
        $body['request']['method'] = $request->getMethod();
        $body['request']['uri'] = $request->getUri();
        $body['request']['baseUri'] = 'https://' . $request->getHost();
        $body['request']['currentUser'] = '/api/3/people/' . $this->getUser()->getUuid();
        $body['request']['headers'] = $this->getHeaders($request->headers->all());

        $workflowResource = $this->getResourceFromShortName(self::CLASSNAME);
        $this->denyAccessUnlessGranted($workflowResource, PermissionsDictionaryBuilder::PERM_CRUD_EXECUTE);
        $workflowObject = $this->findOrThrowNotFound($workflowResource, $workflowId, $request);
        ARCUtils::setRelationShip($request, false);
        $context = [
            'show_to_many' => true,
            'show_to_one' => true,
            'ignore_authorization' => true,
            'ignore_field_authorization' => true,
            'ignore_relationship_authorization' => true
        ];
        $normalized_data = $this->get('serializer')->normalize(
            $workflowObject,
            self::NORMALIZER_METHOD,
            $workflowResource->getNormalizationContext() + $context
        );

        $triggerStepIri = isset($normalized_data['triggerStep']) ? $normalized_data['triggerStep'] : null;

        foreach($normalized_data['steps'] as &$normalizedStep){
            if ($normalizedStep['@id'] === $triggerStepIri) {
                $normalizedStep['arguments'] = array_merge($normalizedStep['arguments'], $body);
                if (isset($normalizedStep['arguments']['inputVariables']) && count($normalizedStep['arguments']['inputVariables']) > 0) {
                    foreach ($normalizedStep['arguments']['inputVariables'] as $userInput) {
                        $normalizedStep['arguments']['request']['data'][$userInput['name']] = $normalizedStep['arguments'][$userInput['name']];
                    }
                }
                break;
            }
        }

        $token = $this->get('security.token_storage')->getToken();
        $headers = array(
            "Content-type" => "application/json",
        );
        $headers['Authorization'] = "Bearer " . $token->getCredentials();
        $headers['X-RUNBYUSER'] = '/api/3/people/' . $this->getUser()->getUuid();
        $headers['X-TRIGGERTYPE'] = Triggers::NOTRIGGER;

        $container = $this->container;
        $workflowStartUri = $container->getParameter('workflow.uri') . $container->getParameter('workflow_endpoint');

        $body = $this->get('serializer')->encode(
            $normalized_data,
            'json'
        );
        $workflowRequest = new GuzzleRequest('post', $workflowStartUri, $headers, $body);
        $proxyProvider = $this->get('cybersponse.core.provider.proxy');
        $client = $proxyProvider->getProxyClient();
        try {
            $response = $client->send($workflowRequest);
            $statusCode = $response->getStatusCode();
            if ($statusCode >= 400) {
                $errMsg = "Failed to get response from Workflow log";
                return JsonResponse::create($errMsg, $statusCode);
            } else {
                $jsonDecodeResponse = json_decode($response->getBody()->getContents(), true);
                return JsonResponse::create($jsonDecodeResponse, $statusCode);
            }
        } catch (\Exception $exception) {
            $errVar = json_decode($exception->getResponse()->getBody()->read(2048), true);
            $errMsg['message'] = $errVar['Error'] ? isset($errVar['Error']) : $errVar['error'];
            $errCode = $exception->getCode();
            return JsonResponse::create($errMsg, $errCode);
        }
    }

    /**
     * @param $request
     * @param $data
     * @return mixed
     */
    protected function extractDataFromRequest($request, $data)
    {

        if (array_key_exists("__resource", $data)) {
            $resourceShortName = $data['__resource'];
        } else {
            throw $this->createNotFoundException($message = 'Resource Not Found In Request');
        }
        $resource = $this->getResourceFromShortName($resourceShortName);
        if ($resource === null) {
            throw $this->createNotFoundException($message = '"' . $resourceShortName . '"' . ' resource not found');
        }
        foreach ($data['records'] as $index => $record) {
            $explodeObj = explode('/', $record);
            $id = end($explodeObj);
            $object = $this->findOrThrowNotFound($resource, $id);
            $normalized_data = $this->get('serializer')->normalize(
                $object,
                self::NORMALIZER_METHOD,
                $resource->getNormalizationContext()
            );
            $data['records'][$index] = $normalized_data;
        }
        return $data;
    }

    /**
     * @param $resourceShortName
     * @return ResourceInterface|null
     */
    protected function getResourceFromShortName($resourceShortName)
    {
        $resourceClassName = Inflector::singularize(Inflector::classify($resourceShortName));
        /** @var ResourceCollectionInterface $resourceCollection */
        $resourceCollection = $this->container->get('api.resource_collection');
        $resource = $resourceCollection->getResourceForShortName($resourceClassName);

        if ($resource === null) {
            $model = $this->getClassNameByType($resourceShortName);
            if ($model === null) {
                return null;
            }
            $singular = $model->getDescriptions()['singular'];
            $escapedSingular = preg_replace('/[^a-z0-9\s]+/i', '', $singular);
            $resourceName = \Doctrine\Common\Util\Inflector::classify($escapedSingular);
            $resource = $resourceCollection->getResourceForShortName($resourceName);
        }
        return $resource;
    }

    /**
     * @param string $type
     * @return StagingModelMetadata|null
     */
    protected function getClassNameByType($type)
    {
        /** @var ManagerRegistry $managerRegistry */
        $managerRegistry = $this->container->get('doctrine');
        $queryManager = $managerRegistry->getManagerForClass(StagingModelMetadata::class);
        $queryRepository = $queryManager->getRepository(StagingModelMetadata::class);
        return $queryRepository->findOneBy(array('type' => $type));
    }

    /**
     * @param $data
     */
    protected function fetch_workflow_data($data, $request)
    {
        if (array_key_exists("__uuid", $data)) {
            $workflowResourceName = 'workflows';
            $workflowId = $data['__uuid'];
        } else {
            throw $this->createNotFoundException($message = 'Workflow UUID Not Found In Request');
        }
        $workflowResource = $this->getResourceFromShortName($workflowResourceName);
        if ($workflowResource === null) {
            throw $this->createNotFoundException($message = '"' . $workflowResource . '"' . ' resource not found');
        }
        $workflowObject = $this->findOrThrowNotFound($workflowResource, $workflowId, $request);
        $context = [
            'ignore_authorization' => true,
            'ignore_field_authorization' => true,
            'ignore_relationship_authorization' => true
        ];
        return $this->get('serializer')->normalize(
            $workflowObject,
            self::NORMALIZER_METHOD,
            $workflowResource->getNormalizationContext() + $context
        );
    }

    /**
     * @param $updatedObject
     * @param $previousObject
     * @param $method
     * @param array $changeData
     * @return array
     */
    protected function prepareChangeData(
        $updatedObject,
        $previousObject,
        $method,
        $playbook_iri,
        $playbook_name,
        $changeData = []
    ) {
        $delta_data = [];
        $delta_data['oldData'] = $previousObject;
        $delta_data['newData'] = $updatedObject;
        $delta_data['changeData'] = $changeData;
        $userName = $this->extractUserName();
        return [
            self::DELTA_DATA => $delta_data,
            self::HTTP_METHOD => $method,
            self::REMOTE_ADDR => $_SERVER['REMOTE_ADDR'],
            self::COMPONENT => 'crudhub',
            self::USER_UUID => $this->getUser()->getUuid(),
            self::TRANSACTION_DATE => (int)($_SERVER['REQUEST_TIME_FLOAT']*1000),
            self::USER => $userName,
            self::PLAYBOOK_IRI => $playbook_iri,
            self::PLAYBOOK_NAME => $playbook_name
        ];
    }

    /**
     * @return string
     */
    private function extractUserName()
    {
        if (method_exists($this->getUser(), "getfirstname")) {
            if (method_exists($this->getUser(), "getlastname")) {
                $userName = $this->getUser()->getfirstname() . ' ' . $this->getUser()->getlastname();
            } else {
                $userName = $this->getUser()->getfirstname();
            }
        } else {
            $userName = $this->getUser()->getname();
        }
        return $userName;
    }
}
