<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\WorkflowBundle\Entity;


use Cybersponse\CoreBundle\Entity\BaseEntity;
use Cybersponse\WorkflowBundle\Entity\RuleCollection;
use Cybersponse\AuthorizationBundle\Traits\Taggable;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\ORM\Mapping\UniqueConstraint;

/**
 * @ORM\Table(name="rules", uniqueConstraints={@UniqueConstraint(name="name_collection_unique", columns={"name", "collection"})})
 * @Doctrine\ORM\Mapping\Entity
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="rules")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="rules")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ name }}")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="Rule",plural="Rules")
 */
class Rule extends BaseEntity
{
    use Taggable;

    /**
     * @ORM\Column(type="string", length=140)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Name")
     * @Symfony\Component\Validator\Constraints\NotBlank()
     */
    protected $name;

    /**
     * @ORM\Column(type="string", length=140, nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Tags")
     */
    protected $tag;

    /**
     * @ORM\Column(type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Active")"
     */
    protected $isActive = false;

    /**
     * @var RuleCollection
     *
     * @ORM\ManyToOne(targetEntity="RuleCollection", cascade={"persist"}, inversedBy="rules")
     * @ORM\JoinColumn(name="collection", referencedColumnName="uuid", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="rule_collections")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Collection")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="lookup")
     * @Symfony\Component\Validator\Constraints\NotBlank()
     */
    private $collection;

    /**
     * @ORM\Column(type="jsonb", nullable=true)
     */
    protected $trigger;

    /**
     * @ORM\Column(type="jsonb", nullable=true)
     */
    protected $condition;

    /**
     * @ORM\Column(type="jsonb", nullable=true)
     */
    protected $actions;

    /**
     * Set name
     *
     * @param string $name
     * @return self
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Get ruleCollection
     *
     * @return RuleCollection
     */
    public function getCollection()
    {
        return $this->collection;
    }

    /**
     * Set ruleCollection
     *
     * @param RuleCollection $collection
     * @return self
     */
    public function setCollection($collection)
    {
        $this->collection = $collection;

        return $this;
    }

    /**
     * Set tag
     *
     * @param string $tag
     * @return self
     */
    public function setTag($tag)
    {
        $this->tag = $tag;

        return $this;
    }

    /**
     * Get tag
     *
     * @return string
     */
    public function getTag()
    {
        return $this->tag;
    }

    /**
     * Set isActive
     *
     * @param boolean $isActive
     * @return self
     */
    public function setIsActive($isActive)
    {
        $this->isActive = $isActive;

        return $this;
    }

    /**
     * Get isActive
     *
     * @return boolean
     */
    public function getIsActive()
    {
        return $this->isActive;
    }

    /**
     * @return mixed
     */
    public function getTrigger()
    {
        return $this->trigger;
    }

    /**
     * @param mixed $trigger
     * @return self
     */
    public function setTrigger($trigger)
    {
        $this->trigger = $trigger;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getCondition()
    {
        return $this->condition;
    }

    /**
     * @param $condition
     * @return $this
     */
    public function setCondition($condition)
    {
        $this->condition = $condition;

        return $this;
    }


    /**
     * @return mixed
     */
    public function getActions()
    {
        return $this->actions;
    }

    /**
     * @param mixed $actions
     * @return self
     */
    public function setActions($actions)
    {
        $this->actions = $actions;

        return $this;
    }
}
