<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\WorkflowBundle\Entity;

use Cybersponse\CoreBundle\Entity\BaseEntity;
use Cybersponse\AuthorizationBundle\Traits\Taggable;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;

/**
 * RulesCollection
 *
 * @Doctrine\ORM\Mapping\Table(name="rule_collections")
 * @Doctrine\ORM\Mapping\Entity
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="rule_collections")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="rules")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ name }}")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="Rule Collection",plural="Rule Collections")
 */
class RuleCollection extends BaseEntity
{
    use Taggable;

    /**
     * @ORM\Column(type="string", length=140, unique=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Name")
     * @Symfony\Component\Validator\Constraints\NotBlank()
     */
    protected $name;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="description", type="string", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Description")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="textarea")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     */
    protected $description;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`image`", type="string", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="images")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Image")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="image")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     */
    protected $image;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @ORM\OneToMany(targetEntity="Rule", mappedBy="collection", cascade={"persist", "remove"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="rules")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Rule")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="oneToMany")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Exportable\Exportable()
     *
     */
    protected $rules;

    /**
     * RuleCollection constructor.
     */
    public function __construct()
    {
        $this->rules = new ArrayCollection();
    }

    /**
     * Set name
     *
     * @param string $name
     * @return self
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set description
     *
     * @param string $description
     * @return self
     */
    public function setDescription($description)
    {
        $this->description = $description;

        return $this;
    }

    /**
     * Get description
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @return string
     */
    public function getImage()
    {
        return $this->image;
    }

    /**
     * @param string $image
     * @return Workflow
     */
    public function setImage($image)
    {
        $this->image = $image;

        return $this;
    }

    /**
     * Add rule
     *
     * @param Rule $rule
     * @return self
     */
    public function addRule(Rule $rule)
    {
        $this->rules[] = $rule;
        $rule->setCollection($this);
        return $this;
    }

    /**
     * Remove rule
     *
     * @param Rule $rule
     */
    public function removeRule(Rule $rule)
    {
        $this->rules->removeElement($rule);
        $rule->setCollection(null);
    }

    /**
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getRules()
    {
        if (array_key_exists('$workflowCountOnly', $_GET)) {
            return $this->rules->count();
        }
        return $this->rules;
    }
}
