<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\WorkflowBundle\Entity;


use Cybersponse\CoreBundle\Entity\BaseEntity;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Exportable\Exportable;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize;
use Cybersponse\CoreBundle\Traits\Indexable;
use Cybersponse\AuthorizationBundle\Traits\Taggable;
use Cybersponse\CoreBundle\Traits\Trackable;
use Doctrine\ORM\Mapping\UniqueConstraint;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity(repositoryClass="Cybersponse\WorkflowBundle\Repository\WorkflowRepository")
 * @ORM\Table(name="workflows", uniqueConstraints={@UniqueConstraint(name="name_unique", columns={"name", "collection"})})
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="workflows")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="workflows")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ name }}")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="Playbook",plural="Playbooks")
 */
class Workflow extends BaseEntity
{
    use Indexable;
    use Trackable;
    use Taggable;
    use \Cybersponse\AuthorizationBundle\Traits\ConditionalOwnable;

    /**
     * @ORM\Column(name="triggerLimit", type="integer", nullable=true)
     */
    protected $triggerLimit;

    /**
     * @ORM\ManyToOne(targetEntity="WorkflowStep", cascade={"detach"})
     * @ORM\JoinColumn(name="triggerstep_uuid", referencedColumnName="uuid", nullable=true)
     */
    protected $triggerStep;

    /**
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="WorkflowStep", orphanRemoval=true, cascade={"persist", "remove", "detach"})
     * @Doctrine\ORM\Mapping\JoinTable(name="workflow_workflowstep",
     *   joinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="workflow_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   },
     *   inverseJoinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="workflowstep_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   }
     * )
     * @Denormalize()
     * @Normalize()
     * @Exportable()
     */
    protected $steps;

    /**
     * @Doctrine\ORM\Mapping\ManyToMany(targetEntity="WorkflowRoute", cascade={"persist", "remove", "detach"}, orphanRemoval=true)
     * @Doctrine\ORM\Mapping\JoinTable(name="workflow_workflowroute",
     *   joinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="workflow_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   },
     *   inverseJoinColumns={
     *     @Doctrine\ORM\Mapping\JoinColumn(name="workflowroute_uuid", referencedColumnName="uuid", onDelete="CASCADE")
     *   }
     * )
     * @Denormalize()
     * @Normalize()
     * @Exportable()
     */
    protected $routes;

    /**
     * @Doctrine\ORM\Mapping\OneToMany(targetEntity="WorkflowVersion", mappedBy="workflow", cascade={"persist", "remove", "detach"})
     * @Denormalize()
     * @Normalize()
     * @Exportable()
     */
    protected $versions;

    /**
     * @ORM\Column(type="string", length=255)
     * @Symfony\Component\Validator\Constraints\Length(max=255, maxMessage = "NAME cannot be longer than {{ limit }} characters")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Name")
     * @Symfony\Component\Validator\Constraints\NotBlank()
     */
    protected $name;

    /**
     * @ORM\Column(name="`aliasname`", type="string", length=255, nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Visibility\Visibility(visibility=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Alias Name")
     */
    protected $aliasName;

    /**
     * @ORM\Column(type="string", length=140, nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Tag")
     */
    protected $tag;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="description", type="string", length=1024, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Description")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="textarea")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     */
    protected $description;

    /**
     * @ORM\Column(type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DefaultValue\DefaultValue(defaultValue="1")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Active")"
     */
    protected $isActive = false;

    /**
     * @ORM\Column(type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Single Record Execution")"
     */
    protected $singleRecordExecution = false;

    /**
     * @ORM\Column(type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Remote Executable")"
     */
    protected $remoteExecutableFlag = false;

    /**
     * @var WorkflowCollection
     *
     * @ORM\ManyToOne(targetEntity="WorkflowCollection", cascade={"persist"}, inversedBy="workflows")
     * @ORM\JoinColumn(name="collection", referencedColumnName="uuid", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="workflow_collections", query=
     *     @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\Query(logic="AND", filters={{"field"="visible", "operator"="eq", "value"=true}}, sort={{"field"="name", "direction"="ASC"}})
     * )
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Collection")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="lookup")
     * @Symfony\Component\Validator\Constraints\NotBlank()
     */
    private $collection;

    /**
     * @ORM\Column(type="jsonb", nullable=true)
     */
    protected $parameters;

    /**
     * @ORM\Column(type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Synchronous")
     */
    protected $synchronous = false;

    /**
     * @var \DateTime
     *
     * @Doctrine\ORM\Mapping\Column(name="lastmodifydate", type="datetime", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Last Modified On")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Visibility\Visibility(visibility=false)
     */
    protected $lastModifyDate;

    /**
     * @var \Cybersponse\CoreBundle\Entity\Picklist
     *
     * @Doctrine\ORM\Mapping\ManyToOne(targetEntity="Cybersponse\CoreBundle\Entity\Picklist", cascade={"persist"})
     * @Doctrine\ORM\Mapping\JoinColumns({
     *   @Doctrine\ORM\Mapping\JoinColumn(name="priority", referencedColumnName="uuid", onDelete="SET NULL")
     * })
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="picklists", query=
     *     @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\Query(logic="AND", filters={{"field"="listName__name", "operator"="eq", "value"="WorkflowPriority"}}, sort={{"field"="orderIndex", "direction"="ASC"}})
     * )
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Priority")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\BulkAction\BulkAction(allow=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="picklist")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     */
    protected $priority;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->steps = new \Doctrine\Common\Collections\ArrayCollection();
        $this->routes = new \Doctrine\Common\Collections\ArrayCollection();
        $this->versions = new \Doctrine\Common\Collections\ArrayCollection();
    }

    /**
     * Set triggerLimit
     *
     * @param integer $triggerLimit
     * @return Workflow
     */
    public function setTriggerLimit($triggerLimit)
    {
        $this->triggerLimit = $triggerLimit;

        return $this;
    }

    /**
     * Get triggerLimit
     *
     * @return integer
     */
    public function getTriggerLimit()
    {
        return $this->triggerLimit;
    }

    /**
     * Set name
     *
     * @param string $name
     * @return Workflow
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get alias name
     *
     * @return string
     */
    public function getAliasName()
    {
        return $this->aliasName;
    }

    /**
     * Set alias name
     *
     * @param string $aliasName
     * @return Workflow
     */
    public function setAliasName($aliasName)
    {
        $this->aliasName = $aliasName;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set description
     *
     * @param string $description
     * @return Workflow
     */
    public function setDescription($description)
    {
        $this->description = $description;

        return $this;
    }

    /**
     * Get description
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Get workflowCollection
     *
     * @return WorkflowCollection
     */
    public function getCollection()
    {
        return $this->collection;
    }

    /**
     * Set workflowCollection
     *
     * @param WorkflowCollection $collection
     * @return Workflow
     */
    public function setCollection($collection)
    {
        $this->collection = $collection;

        return $this;
    }

    /**
     * Set tag
     *
     * @param string $tag
     * @return Workflow
     */
    public function setTag($tag)
    {
        $this->tag = $tag;

        return $this;
    }

    /**
     * Get tag
     *
     * @return string
     */
    public function getTag()
    {
        return $this->tag;
    }

    /**
     * Set isActive
     *
     * @param boolean $isActive
     * @return Workflow
     */
    public function setIsActive($isActive)
    {
        $this->isActive = $isActive;

        return $this;
    }

    /**
     * Get isActive
     *
     * @return boolean
     */
    public function getIsActive()
    {
        return $this->isActive;
    }

    /**
     * Set singleRecordExecution
     *
     * @param boolean $singleRecordExecution
     * @return Workflow
     */
    public function setSingleRecordExecution($singleRecordExecution)
    {
        $this->singleRecordExecution = $singleRecordExecution;

        return $this;
    }

    /**
     * Get singleRecordExecution
     *
     * @return boolean
     */
    public function getSingleRecordExecution()
    {
        return $this->singleRecordExecution;
    }


    /**
     * Set remoteExecutableFlag
     *
     * @param boolean $remoteExecutableFlag
     * @return Workflow
     */
    public function setRemoteExecutableFlag($remoteExecutableFlag)
    {
        $this->remoteExecutableFlag = $remoteExecutableFlag;
        return $this;
    }

    /**
     * Get getremoteExecutableFlag
     *
     * @return boolean
     */
    public function getRemoteExecutableFlag()
    {
        return $this->remoteExecutableFlag;
    }

    /**
     * Set parameters
     *
     * @param array $parameters
     * @return Workflow
     */
    public function setParameters($parameters)
    {
        $this->parameters = $parameters;

        return $this;
    }

    /**
     * Get parameters
     *
     * @return array
     */
    public function getParameters()
    {
        return $this->parameters;
    }

    /**
     * Set synchronous
     *
     * @param boolean $synchronous
     * @return Workflow
     */
    public function setSynchronous($synchronous)
    {
        $this->synchronous = $synchronous;

        return $this;
    }

    /**
     * Get synchronous
     *
     * @return boolean
     */
    public function getSynchronous()
    {
        return $this->synchronous;
    }

    /**
     * Set triggerStep
     *
     * @param \Cybersponse\WorkflowBundle\Entity\WorkflowStep $triggerStep
     * @return Workflow
     */
    public function setTriggerStep(\Cybersponse\WorkflowBundle\Entity\WorkflowStep $triggerStep = null)
    {
        $this->triggerStep = $triggerStep;

        return $this;
    }

    /**
     * Get triggerStep
     *
     * @return \Cybersponse\WorkflowBundle\Entity\WorkflowStep
     */
    public function getTriggerStep()
    {
        return $this->triggerStep;
    }

    /**
     * Add steps
     *
     * @param \Cybersponse\WorkflowBundle\Entity\WorkflowStep $steps
     * @return Workflow
     */
    public function addStep(\Cybersponse\WorkflowBundle\Entity\WorkflowStep $steps)
    {
        $this->steps[] = $steps;

        return $this;
    }

    /**
     * Remove steps
     *
     * @param \Cybersponse\WorkflowBundle\Entity\WorkflowStep $steps
     */
    public function removeStep(\Cybersponse\WorkflowBundle\Entity\WorkflowStep $steps)
    {
        $this->steps->removeElement($steps);
    }

    /**
     * Get steps
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getSteps()
    {
        if (array_key_exists('$triggerOnly', $_GET)) {
            return array($this->getTriggerStep());
        }
        return $this->steps;
    }

    /**
     * Add routes
     *
     * @param \Cybersponse\WorkflowBundle\Entity\WorkflowRoute $routes
     * @return Workflow
     */
    public function addRoute(\Cybersponse\WorkflowBundle\Entity\WorkflowRoute $routes)
    {
        $this->routes[] = $routes;

        return $this;
    }

    /**
     * Remove routes
     *
     * @param \Cybersponse\WorkflowBundle\Entity\WorkflowRoute $routes
     */
    public function removeRoute(\Cybersponse\WorkflowBundle\Entity\WorkflowRoute $routes)
    {
        $this->routes->removeElement($routes);
    }

    /**
     * Get routes
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getRoutes()
    {
        if (!array_key_exists('$triggerOnly', $_GET)) {
            return $this->routes;
        }
    }

    /**
     * Add versions
     *
     * @param \Cybersponse\WorkflowBundle\Entity\WorkflowVersion $versions
     * @return Workflow
     */
    public function addVersion(\Cybersponse\WorkflowBundle\Entity\WorkflowVersion $versions)
    {
        $this->versions[] = $versions;

        return $this;
    }

    /**
     * Remove versions
     *
     * @param \Cybersponse\WorkflowBundle\Entity\WorkflowVersion $versions
     */
    public function removeVersion(\Cybersponse\WorkflowBundle\Entity\WorkflowVersion $versions)
    {
        $this->versions->removeElement($versions);
    }

    /**
     * Get versions
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getVersions()
    {
        if (array_key_exists('$versions', $_GET)) {
            return $this->versions;
        }
    }

    /**
     * Get the value of lastModifyDate
     *
     * @return  \DateTime
     */
    public function getLastModifyDate()
    {
        return $this->lastModifyDate;
    }

    /**
     * Set the value of lastModifyDate
     *
     * @param  \DateTime  $lastModifyDate
     *
     * @return  self
     */
    public function setLastModifyDate($lastModifyDate)
    {
        $this->lastModifyDate = $lastModifyDate;

        return $this;
    }

    /**
     * Get the value of priority
     *
     * @return  \Cybersponse\CoreBundle\Entity\Picklist
     */ 
    public function getPriority()
    {
        return $this->priority;
    }

    /**
     * Set the value of priority
     *
     * @param  \Cybersponse\CoreBundle\Entity\Picklist  $priority
     *
     * @return  self
     */ 
    public function setPriority(\Cybersponse\CoreBundle\Entity\Picklist $priority = null)
    {
        $this->priority = $priority;

        return $this;
    }
}
