<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\WorkflowBundle\Entity;

use Cybersponse\AuthorizationBundle\Traits\Taggable;
use Cybersponse\CoreBundle\Entity\BaseEntity;
use Cybersponse\CoreBundle\Traits\Indexable;
use Cybersponse\CoreBundle\Traits\Trackable;
use Cybersponse\WorkflowBundle\Entity\Workflow;
use Doctrine\ORM\Mapping as ORM;

/**
 * WorkflowCollection
 *
 * @Doctrine\ORM\Mapping\Table(name="workflow_collections")
 * @Doctrine\ORM\Mapping\Entity
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="workflow_collections")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="workflows")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ name }}")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="Playbook Collection",plural="Playbook Collections")
 */
class WorkflowCollection extends BaseEntity
{
    use Indexable;
    use Trackable;
    use Taggable;

    /**
     * @ORM\Column(type="string", length=255, unique=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Name")
     * @Symfony\Component\Validator\Constraints\NotBlank()
     */
    protected $name;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="description", type="string", length=1024, nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Description")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="textarea")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     */
    protected $description;

    /**
     * @ORM\Column(name="visible", type="boolean")
     */
    protected $visible = true;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="`image`", type="string", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="images")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Image")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="image")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     */
    protected $image;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @ORM\OneToMany(targetEntity="Workflow", mappedBy="collection", cascade={"persist", "remove"})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="workflows")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Workflow")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="oneToMany")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Exportable\Exportable()
     *
     */
    protected $workflows;

    /**
     * Set name
     *
     * @param string $name
     * @return WorkflowCollection
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set description
     *
     * @param string $description
     * @return WorkflowCollection
     */
    public function setDescription($description)
    {
        $this->description = $description;

        return $this;
    }

    /**
     * Get description
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @return mixed
     */
    public function getVisible()
    {
        return $this->visible;
    }

    /**
     * @param mixed $visible
     */
    public function setVisible($visible)
    {
        $this->visible = $visible;
    }

    /**
     * @return string
     */
    public function getImage()
    {
        return $this->image;
    }

    /**
     * @param string $image
     * @return Workflow
     */
    public function setImage($image)
    {
        $this->image = $image;

        return $this;
    }

    /**
     * Add workflow
     *
     * @param Workflow $workflow
     * @return WorkflowCollection
     */
    public function addWorkflow(Workflow $workflow)
    {
        $this->workflows[] = $workflow;
        $workflow->setCollection($this);
        return $this;
    }

    /**
     * Remove workflow
     *
     * @param Workflow $workflow
     */
    public function removeWorkflow(Workflow $workflow)
    {
        $this->workflows->removeElement($workflow);
        $workflow->setCollection(null);
    }

    /**
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getWorkflows()
    {
        if (array_key_exists('$workflowCountOnly', $_GET)) {
            return $this->workflows->count();
        }
        return $this->workflows;
    }
}
