<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\WorkflowBundle\Entity;


use Cybersponse\CoreBundle\Entity\BaseEntity;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity
 * @ORM\Table(name="workflow_steps")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="workflow_steps")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="workflows")
 */
class WorkflowStep extends BaseEntity
{
    use \Cybersponse\CoreBundle\Traits\Indexable;

    /**
     * @ORM\ManyToOne(targetEntity="WorkflowStepType")
     * @ORM\JoinColumn(name="steptype_uuid", referencedColumnName="uuid", nullable=true)
     * @Normalize()
     */
    protected $stepType;

    /**
     * @ORM\Column(type="string", length=140)
     */
    protected $name;

    /**
     * @ORM\Column(name="description", type="string", length=1024, nullable=true, unique=false)
     */
    protected $description;

    /**
     * @ORM\Column(type="jsonb", nullable=true)
     */
    protected $arguments;

    /**
     * @ORM\Column(type="string", length=140, nullable=true)
     */
    protected $status;

    /**
     * @ORM\Column(name="left_position", type="decimal", options={"default":0})
     */
    protected $left = 0;

    /**
     * @ORM\Column(name="top_position", type="decimal", options={"default":0})
     */
    protected $top = 0;

    /**
     * Set name
     *
     * @param string $name
     * @return WorkflowStep
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set description
     *
     * @param string $description
     * @return WorkflowStepType
     */
    public function setDescription($description)
    {
        $this->description = $description;

        return $this;
    }

    /**
     * Get description
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Set arguments
     *
     * @param array $arguments
     * @return WorkflowStep
     */
    public function setArguments($arguments)
    {
        $this->arguments = $arguments;

        return $this;
    }

    /**
     * Get arguments
     *
     * @return array
     */
    public function getArguments()
    {
        return $this->arguments;
    }

    /**
     * Set status
     *
     * @param string $status
     * @return WorkflowStep
     */
    public function setStatus($status)
    {
        $this->status = $status;

        return $this;
    }

    /**
     * Get status
     *
     * @return string
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * Set left
     *
     * @param integer $left
     * @return WorkflowStep
     */
    public function setLeft($left)
    {
        $this->left = $left;

        return $this;
    }

    /**
     * Get left
     *
     * @return integer
     */
    public function getLeft()
    {
        return $this->left;
    }

    /**
     * Set top
     *
     * @param integer $top
     * @return WorkflowStep
     */
    public function setTop($top)
    {
        $this->top = $top;

        return $this;
    }

    /**
     * Get top
     *
     * @return integer
     */
    public function getTop()
    {
        return $this->top;
    }

    /**
     * Set stepType
     *
     * @param \Cybersponse\WorkflowBundle\Entity\WorkflowStepType $stepType
     * @return WorkflowStep
     */
    public function setStepType(\Cybersponse\WorkflowBundle\Entity\WorkflowStepType $stepType = null)
    {
        $this->stepType = $stepType;

        return $this;
    }

    /**
     * Get stepType
     *
     * @return \Cybersponse\WorkflowBundle\Entity\WorkflowStepType
     */
    public function getStepType()
    {
        return $this->stepType;
    }
}
