<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\WorkflowBundle\Entity;


use Cybersponse\CoreBundle\Entity\BaseEntity;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity
 * @ORM\Table(name="workflow_step_types")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="workflow_step_types")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="workflows")
 */
class WorkflowStepType extends BaseEntity
{
    use \Cybersponse\CoreBundle\Traits\Indexable;

    /**
     * @ORM\Column(type="string", length=140, unique=true)
     */
    protected $name;

    /**
     * @ORM\Column(type="string")
     */
    protected $displayName;

    /**
     * @ORM\Column(type="string", length=140, nullable=true)
     */
    protected $widget;

    /**
     * @ORM\ManyToOne(targetEntity="StepTypeCollection", inversedBy="stepTypes")
     * @ORM\JoinColumn(name="collection_uuid", referencedColumnName="uuid", nullable=true)
     */
    protected $collection;

    /**
     * @ORM\Column(type="jsonb", nullable=true)
     */
    protected $arguments;

    /**
     * @ORM\ManyToOne(targetEntity="WorkflowStepType", inversedBy="parent")
     * @ORM\JoinColumn(name="parent_uuid", referencedColumnName="uuid", nullable=true)
     * @Normalize()
     */
    protected $parent;

    /**
     * @ORM\Column(type="string", length=140, options={"default":"fa fa-check-square-o"})
     */
    protected $icon = 'fa fa-check-square-o';

    /**
     * @ORM\Column(type="string", length=7, nullable=true)
     */
    protected $background;

    /**
     * @ORM\Column(type="integer", nullable=true)
     */
    protected $index;

    /**
     * @ORM\Column(type="boolean",options={"default" = true})
     */
    protected $visible = true;

    /**
     * @ORM\Column(name="description", type="string", length=1024, nullable=true, unique=false)
     */
    protected $description;
    
    /**
     * Set name
     *
     * @param string $name
     * @return WorkflowStepType
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set displayName
     *
     * @param string $displayName
     * @return WorkflowStepType
     */
    public function setDisplayName($displayName)
    {
        $this->displayName = $displayName;

        return $this;
    }

    /**
     * Get displayName
     *
     * @return string
     */
    public function getDisplayName()
    {
        return $this->displayName;
    }

    /**
     * Set widget
     *
     * @param string $widget
     * @return WorkflowStepType
     */
    public function setWidget($widget)
    {
        $this->widget = $widget;

        return $this;
    }

    /**
     * Get widget
     *
     * @return string
     */
    public function getWidget()
    {
        return $this->widget;
    }

    /**
     * Set collection
     *
     * @param \Cybersponse\WorkflowBundle\Entity\StepTypeCollection $collection
     * @return WorkflowStepType
     */
    public function setCollection(\Cybersponse\WorkflowBundle\Entity\StepTypeCollection $collection = null)
    {
        $this->collection = $collection;

        return $this;
    }

    /**
     * Get collection
     *
     * @return \Cybersponse\WorkflowBundle\Entity\StepTypeCollection
     */
    public function getCollection()
    {
        return $this->collection;
    }

    /**
     * Set arguments
     *
     * @param array $arguments
     * @return WorkflowStep
     */
    public function setArguments($arguments)
    {
        $this->arguments = $arguments;

        return $this;
    }

    /**
     * Get arguments
     *
     * @return array
     */
    public function getArguments()
    {
        return $this->arguments;
    }

    /**
     * Set parent
     *
     * @param \Cybersponse\WorkflowBundle\Entity\WorkflowStepType $parent
     * @return WorkflowStepType
     */
    public function setParent(\Cybersponse\WorkflowBundle\Entity\WorkflowStepType $parent = null)
    {
        $this->parent = $parent;

        return $this;
    }

    /**
     * Get parent
     *
     * @return \Cybersponse\WorkflowBundle\Entity\WorkflowStepType
     */
    public function getParent()
    {
        return $this->parent;
    }

    /**
     * Set icon
     *
     * @param string $icon
     * @return WorkflowStepType
     */
    public function setIcon($icon)
    {
        $this->icon = $icon;

        return $this;
    }

    /**
     * Get icon
     *
     * @return string
     */
    public function getIcon()
    {
        return $this->icon;
    }

    /**
     * Set background
     *
     * @param string $background
     * @return WorkflowStepType
     */
    public function setBackground($background)
    {
        $this->background = $background;

        return $this;
    }

    /**
     * Get background
     *
     * @return string
     */
    public function getBackground()
    {
        return $this->background;
    }

    /**
     * Set index
     *
     * @param integer $index
     * @return WorkflowStepType
     */
    public function setIndex($index)
    {
        $this->index = $index;

        return $this;
    }

    /**
     * Get index
     *
     * @return integer
     */
    public function getIndex()
    {
        return $this->index;
    }

    /**
     * Set visible
     * @param boolean $visible
     */
    public function setVisible($visible)
    {
        $this->visible = $visible;
    }

    /**
     * Get isVisible
     *
     * @return boolean
     */
    public function getVisible()
    {
        return $this->visible;
    }

    /**
     * Get the value of description
     */ 
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Set the value of description
     *
     * @return  description
     */ 
    public function setDescription($description)
    {
        return $this->description = $description;
    }
}
