<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\WorkflowBundle\Entity;


use Cybersponse\CoreBundle\Entity\BaseEntity;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Denormalize\Denormalize;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Exportable\Exportable;
use Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize;
use Doctrine\ORM\Mapping\UniqueConstraint;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity
 * @ORM\Table(name="workflow_versions")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModelType\ModelType(modelType="workflow_versions")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\ModuleType\ModuleType(moduleType="workflows")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\DisplayName\DisplayName(displayName="{{ note }}")
 * @Cybersponse\MetadataBundle\Generation\Annotations\Classes\Descriptions\Descriptions(singular="Playbook Version",plural="Playbook Versions")
 */
class WorkflowVersion extends BaseEntity
{
    use \Cybersponse\CoreBundle\Traits\Indexable;
    use \Cybersponse\CoreBundle\Traits\Trackable;

    /**
     * @var Actor
     *
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Normalize\Normalize()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Readonly\Readonly()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\DataSource\DataSource(model="actors")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Created By")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="lookup")
     * @Doctrine\ORM\Mapping\ManyToOne(targetEntity="Cybersponse\ActorsBundle\Entity\Actor", cascade={"persist"})
     * @Doctrine\ORM\Mapping\JoinColumns({
     *   @Doctrine\ORM\Mapping\JoinColumn(name="createuser", referencedColumnName="uuid", onDelete="SET NULL")
     * })
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     */
    protected $createUser;

    /**
     * @var \DateTime
     *
     * @Doctrine\ORM\Mapping\Column(name="createdate", type="datetime", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="datetime")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Readonly\Readonly()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Created On")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     */
    protected $createDate;

    /**
     * @ORM\Column(type="string", nullable=true)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="text")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Searchable\Searchable()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Version Notes")
     */
    protected $note;

    /**
     * @var string
     *
     * @Doctrine\ORM\Mapping\Column(name="json", type="jsonb", nullable=true, unique=false)
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="JSON")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="json")
     */
    protected $json;

    /**
     * @ORM\Column(type="boolean", options={"default" = false})
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\GridColumn\GridColumn()
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\FormType\FormType(type="checkbox")
     * @Cybersponse\MetadataBundle\Generation\Annotations\Fields\Title\Title(name="Autosave")"
     */
    protected $autosave = false;

    /**
     * @ORM\ManyToOne(targetEntity="Workflow", inversedBy="versions")
     * @ORM\JoinColumn(name="workflow_uuid", referencedColumnName="uuid", nullable=true)
     * @Normalize()
     */
    protected $workflow;

    /**
     * Set note
     *
     * @param string $note
     * @return WorkflowVersion
     */
    public function setNote($note)
    {
        $this->note = $note;

        return $this;
    }

    /**
     * Get note
     *
     * @return string
     */
    public function getNote()
    {
        return $this->note;
    }

    /**
     * Set json
     *
     * @param string $json
     * @return WorkflowVersion
     */
    public function setJson($json)
    {
        $this->json = $json;
        return $this;
    }


    /**
     * Get json
     *
     * @return string
     */
    public function getJson()
    {
        if (array_key_exists('$includeData', $_GET)) {
            return $this->json;
        }
    }

    /**
     * Set autosave
     *
     * @param boolean $autosave
     * @return WorkflowVersion
     */
    public function setAutosave($autosave)
    {
        $this->autosave = $autosave;

        return $this;
    }

    /**
     * Get autosave
     *
     * @return boolean
     */
    public function getAutosave()
    {
        return $this->autosave;
    }

    /**
     * Get workflow
     *
     * @return Workflow
     */
    public function getWorkflow()
    {
        return $this->workflow;
    }

    /**
     * Set workflow
     *
     * @param Workflow $workflow
     * @return WorkflowVersion
     */
    public function setWorkflow($workflow)
    {
        $this->workflow = $workflow;

        return $this;
    }
}
