<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\WorkflowBundle\Repository;


use Cybersponse\WorkflowBundle\Entity\Workflow;
use Doctrine\ORM\EntityRepository;

class WorkflowRepository extends EntityRepository
{
    /**
     * @param $triggerType
     * @return Workflow[]
     */
    public function getWorkflowsForTriggerType($triggerType, $owners)
    {
        $workflowQueryBuilder = $this->createQueryBuilder('wf');

        if(!$owners){
            /** @var Workflow[] $workflows */
            $workflows = $workflowQueryBuilder
            ->join('wf.triggerStep', 'wfs')
            ->join('wfs.stepType', 'wfst')
            ->where($workflowQueryBuilder->expr()->eq('wfst.name', ':triggerType'))
            ->andWhere($workflowQueryBuilder->expr()->eq('wf.isActive', ':isActive'))
            ->setParameter('triggerType', $triggerType)
            ->setParameter('isActive', true)
            ->getQuery()
            ->execute();
        }else{
            /** @var Workflow[] $workflows */
            $workflows = $workflowQueryBuilder
            ->join('wf.triggerStep', 'wfs')
            ->join('wfs.stepType', 'wfst')
            ->leftJoin('wf.owners', 'wfo')
            ->where($workflowQueryBuilder->expr()->eq('wfst.name', ':triggerType'))
            ->andWhere($workflowQueryBuilder->expr()->eq('wf.isActive', ':isActive'))
            ->andwhere($workflowQueryBuilder->expr()->orX(
                $workflowQueryBuilder->expr()->eq('wf.isPrivate', ':isPrivate'),
                $workflowQueryBuilder->expr()->in('wfo', ':owners'))
            )
            ->setParameter('triggerType', $triggerType)
            ->setParameter('isActive', true)
            ->setParameter('isPrivate', false)
            ->setParameter('owners', $owners)
            ->getQuery()
            ->execute();
        }

        return $workflows;
    }
}
