# Dynamic Variables

## Overview

Dynamic vars are a replacement of the macros system from v3.1.28. Macros allowed for insertion of data based on context using a similar coding format commonly found in MVC templating systems.

These variables can be referenced using dot notation inside a Jinja template.


## Filters

### Filter: `| fromIRI`
Normally relationships are shown as IRI's to reduce the amount of look up in the system. The look up can be achieved by using the fromIRI filter so that data can then be accessed with the dot notation. See examples for more information.

## Examples

### Get all the variables registered

`{{ vars }}`

### Get the current user's email

`{{ (vars.currentUser | fromIRI).email }}`

### Get the current user's theme from settings

`{{ (vars.settings | fromIRI).user.view.theme }}`

### Get the current request's resource name

`{{ vars.resource }}`

### Get the current request's resource data `name` field

`{{ vars.request.data.name }}`

### Get the current request's info

`{{ vars.request.data }}`
`{{ vars.request.method }}`
`{{ vars.request.uri }}`
`{{ vars.request.headers }}`
`{{ vars.request.body }}`
`{{ vars.request.query }}`

### Get the data previously in the database, valid for `UPDATE` triggers

`{{ vars.previous.data }}`