<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\WorkflowBundle\Rule;


use Cybersponse\ProxyBundle\GuzzleHttp\HmacClient;
use Cybersponse\WorkflowBundle\Entity\Rule;
use GuzzleHttp\Psr7\Request;
use Psr\Log\LoggerInterface;
use Symfony\Component\Serializer\Serializer;

class SealabRuleExecutionEngine implements RuleExecutionEngine
{
    /** @var HmacClient */
    protected $sealabClient;

    /** @var Serializer */
    protected $serializer;

    /** @var LoggerInterface */
    protected $logger;

    /** @var int */
    protected $timeout;

    /**
     * @param HmacClient $sealabClient
     */
    public function setSealabClient($sealabClient)
    {
        $this->sealabClient = $sealabClient;
    }

    /**
     * @param Serializer $serializer
     */
    public function setSerializer($serializer)
    {
        $this->serializer = $serializer;
    }

    /**
     * @param LoggerInterface $logger
     */
    public function setLogger($logger)
    {
        $this->logger = $logger;
    }

    /**
     * @param int $timeout
     */
    public function setTimeout($timeout)
    {
        $this->timeout = $timeout;
    }

    /**
     * @inheritdoc
     */
    public function executeRule(Rule $rule, $arguments)
    {
        $this->executeRules([$rule], $arguments);
    }

    /**
     * @inheritdoc
     */
    public function executeRules($rules, $arguments)
    {
        if (empty($rules)) {
            return;
        }

        $normalizedRules = array_map(function ($rule) use ($arguments) {
            $normalizedRule = $this->serializer->normalize($rule, 'json-ld', ['ignore_authorization' => true]);
            $normalizedRule['args'] = $arguments;
            return $normalizedRule;
        }, $rules);

        $serializedRules = $this->serializer->encode($normalizedRules, 'json');
        $ruleExecutionUri = $this->sealabClient->getTargetUri() . '/api/rules/start/';
        $request = new Request('POST', $ruleExecutionUri, ['Content-Type' => 'application/json'], $serializedRules);

        $this->logger->info(sprintf('Waiting for up to %ds for a Sealab Rule Engine response...', $this->timeout));

        try {
            $response = $this->sealabClient->send($request, ['timeout' => $this->timeout]);
            if (($statusCode = $response->getStatusCode()) >= 400) {
                $this->logger->alert(sprintf('An error occurred while sending Rule to Sealab for execution. Returned with status: %s', $statusCode));
            } else {
                $this->logger->info(sprintf('Rule successfully sent to Sealab for execution. Returned with status: %s', $statusCode));
            }
        } catch (\Exception $exception) {
            $this->logger->alert('An error occurred sending data to Sealab.', ['exception' => $exception]);
        }
    }
}
