<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\WorkflowBundle\Serializer;


use Cybersponse\CoreBundle\Serializer\EfficientItemNormalizer;
use Cybersponse\WorkflowBundle\Entity\Workflow;
use Cybersponse\WorkflowBundle\Entity\WorkflowRoute;
use Cybersponse\WorkflowBundle\Entity\WorkflowStep;
use Cybersponse\WorkflowBundle\Entity\WorkflowStepType;
use Dunglas\ApiBundle\Api\ResourceInterface;
use Symfony\Component\Serializer\Serializer;

class WorkflowNormalizer extends EfficientItemNormalizer
{
    const FORMAT_JSON_LD = 'json-ld';

    /** @var Serializer */
    protected $serializer;

    /** @var ResourceInterface */
    protected $workflowResource;

    /**
     * {@inheritdoc}
     */
    public function supportsNormalization($data, $format = null)
    {
        return self::FORMAT_JSON_LD === $format && ($data instanceof Workflow ||
            $data instanceof WorkflowStep ||
            $data instanceof WorkflowRoute ||
            $data instanceof WorkflowStepType);
    }

    /**
     * {@inheritdoc}
     */
    public function supportsDenormalization($data, $type, $format = null)
    {
        return false;
    }

    /**
     * {@inheritdoc}
     */
    public function normalize($object, $format = null, array $context = [])
    {
        $context['ignore_depth'] = true;

        $normalizationContext = $this->workflowResource->getNormalizationContext();

        return parent::normalize(
            $object,
            $format,
            $context + $normalizationContext
        );
    }

    /**
     * @param ResourceInterface $workflowResource
     */
    public function setWorkflowResource($workflowResource)
    {
        $this->workflowResource = $workflowResource;
    }
}
